package com.loit.common.json;

import java.io.Serializable;
import java.util.LinkedHashMap;
import java.util.Map;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.loit.common.mapper.JsonMapper;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;

/**

 - $.ajax后需要接受的JSON
 -
 - @author
 - */
@SuppressWarnings("rawtypes")
public class AjaxJson<T> implements Serializable{
	public static final String CODE_COMMON_FAIL = LoitStatusCode.FAIL.statusCode;
	/**
	 - @author wbh
	 - @date 2018年12月11日
	 */
	private static final long serialVersionUID = -7790919285662399570L;
	private boolean success = true;// 是否成功
	private String code = LoitStatusCode.SUCESS.statusCode;//返回码 100-成功，101失败，其他返回码参见LoitStatusCode
	private String msg = "操作成功";// 提示信息
	private Map<String, T> body = new LinkedHashMap<String, T>();//封装json的map
	public Map<String, T> getBody() {
		return body;
	}
	public void setBody(Map<String, T> body) {
		this.body = body;
	}
	public void put(String key, T value){//向json中添加属性，在js中访问，请调用data.map.key
		body.put(key, value);
	}
	public void remove(String key){
		body.remove(key);
	}
	public String getMsg() {
		return msg;
	}
	public void setMsg(String msg) {//向json中添加属性，在js中访问，请调用data.msg
		this.msg = msg;
	}
	public boolean isSuccess() {
		return success;
	}
	public void setSuccess(boolean success) {
		this.success = success;
	}
	@JsonIgnore//返回对象时忽略此属性
	public String getJsonStr() {//返回json字符串数组，将访问msg和key的方式统一化，都使用data.key的方式直接访问。
		String json = JsonMapper.getInstance().toJson(this);
		return json;
	}
	public String getCode() {
		return code;
	}
	public void setCode(String code) {
		this.code = code;
	}
	@Override
	public String toString() {
		return "AjaxJson [success=" + success + ", code=" + code + ", msg=" + msg + "]";
	}
	/**
	 - 返回普通异常信息。code=101
	 - @author wbh
	 - @date：2018年12月16日
	 - @param info
	 - @return
	 */
	public static AjaxJson returnExceptionInfo(String info) {
		AjaxJson json = new AjaxJson();
		json.setSuccess(false);
		json.setCode(CODE_COMMON_FAIL);
		json.setMsg(info);
		return json;
	}
	/**
	 - 返回普通异常信息。code=101
	 - @author wbh
	 - @date：2018年12月16日
	 - @param info
	 - @return
	 */
	public static AjaxJson returnExceptionInfo(String code, String info) {
		AjaxJson json = new AjaxJson();
		json.setSuccess(false);
		json.setCode(code);
		json.setMsg(info);
		return json;
	}
	/**
	 - 返回普通异常信息。code=101
	 - @author wbh
	 - @date：2018年12月16日
	 - @param info
	 - @return
	 */
	public static AjaxJson returnExceptionInfo(String info, AjaxJson json) {
		json.setSuccess(false);
		json.setCode(CODE_COMMON_FAIL);
		json.setMsg(info);
		return json;
	}
	/**
	 - 返回指定异常信息。
	 - @author wbh
	 - @date：2019年10月31日
	 - @param code 状态码
	 - @param info 返回信息
	 - @return
	 */
	public static AjaxJson returnExceptionInfo(LoitStatusCode info) {
		AjaxJson json = new AjaxJson();
		json.setSuccess(false);
		json.setCode(info.statusCode);
		json.setMsg(info.statusDesc);
		return json;
	}
	/**
	 - 返回指定异常信息。
	 - @author wbh
	 - @date：2019年10月31日
	 - @param code 状态码
	 - @param info 返回信息
	 - @return
	 */
	public static AjaxJson returnExceptionInfo(LoitStatusCode info, AjaxJson json) {
		json.setSuccess(false);
		json.setCode(info.statusCode);
		json.setMsg(info.statusDesc);
		return json;
	}
}
