package com.loit.common.validator;

import java.lang.reflect.Field;

import com.loit.common.annotation.LoitValidateAnnotation.LoitValidate;
import com.loit.common.json.AjaxJson;

import io.swagger.annotations.ApiModelProperty;
/**
 * 属性验证解析器
 * @author wbh
 * @date 2018年12月9日
 */
public class LoitValidaters {
	public static final String LENGTH_INFO = "超过定义长度";
	public static final String IS_NOT_NUL_INFO = "不能为空";
	
	/**
	 * 常用验证（不为空，长度限制）
	 * @author wbh
	 * @date：2018年12月9日
	 * @param obj
	 * @return
	 * @throws Exception
	 */
	public static AjaxJson validateField(Object obj) throws Exception{
    	Class<?> clazz = obj.getClass();
        Field[] fields = clazz.getDeclaredFields();
        for (Field field : fields) {
            //打开私有访问
            field.setAccessible(true);
//            String name = field.getName();
            //获取属性值
            Object value = field.get(obj);
        	String name=null;
        	//获取swagger注释信息做校验
        	ApiModelProperty myApiModelProperty = field.getAnnotation(ApiModelProperty.class);
        	if(myApiModelProperty!=null) {
        		name = myApiModelProperty.value();
        		if(name==null) name = field.getName();
        		if(myApiModelProperty.required()&&value==null) {//验证不为空
        			return AjaxJson.returnExceptionInfo(name+IS_NOT_NUL_INFO);
        		}
        	}
            
            LoitValidate myFieldAnnotation = field.getAnnotation(LoitValidate.class);
            if(myFieldAnnotation!=null) {
            	if(name==null) name = myFieldAnnotation.name();
        		if(name==null) name = field.getName();
                if(myFieldAnnotation.isNotNull()&&value == null) {//验证不为空
                	return AjaxJson.returnExceptionInfo(name+IS_NOT_NUL_INFO);
                }
                if(myApiModelProperty!=null&&myApiModelProperty.required()&&myFieldAnnotation.length()!=0) {//验证长度限制
                	int len = value.toString().length();
                	if(len>myFieldAnnotation.length()) {
                		AjaxJson json = new AjaxJson();
                		json.setSuccess(false);
                		json.setCode("-3");
                		json.setMsg(name+LENGTH_INFO+":"+myFieldAnnotation.length());
                		return json;
                	}
                }
            }
            field.setAccessible(false);
		}
        return null;
    }
	
	public static boolean validateFieldBoolean(Object obj) throws Exception{
    	Class<?> clazz = obj.getClass();
        Field[] fields = clazz.getDeclaredFields();
        for (Field field : fields) {
            
            LoitValidate myFieldAnnotation = field.getAnnotation(LoitValidate.class);
            if(myFieldAnnotation!=null) {
                //打开私有访问
                field.setAccessible(true);
//                String name = field.getName();
                //获取属性值
                Object value = field.get(obj);
                if(myFieldAnnotation.isNotNull()) {//验证不为空
                	if(value == null) {
                		return false;
                	}
                }
                if(myFieldAnnotation.length()!=0) {//验证长度限制
                	int len = value.toString().length();
                	if(len>myFieldAnnotation.length()) {
                		return false;
                	}
                }
            }
            
		}
        return true;
    }
}
