package com.loit.common.cache;


import com.loit.common.cache.exception.CacheException;

import java.util.List;

/**
 * Description： 定义缓存操作接口
 * Author: yangwenbin
 * Date: Created in 2020/5/29 8:43
 * Version: 1.0
 * Modified By:
 */
public interface Cache {
    /**
     * 读取缓存
     * @param key
     * @return
     * @throws CacheException
     */
    Object get(String key) throws CacheException;

    /**
     * 写入缓存
     * @param key
     * @param value
     * @return
     * @throws CacheException
     */
    boolean set(String key, Object value) throws CacheException;

    /**
     * 写入缓存
     * @param key
     * @param value
     * @param expireTime 有效期
     * @return
     * @throws CacheException
     */
    boolean set(String key, Object value,Long expireTime) throws CacheException;

    /**
     * 批量删除对应的key
     * @param keys
     * @throws CacheException
     */
    void remove(String ... keys) throws CacheException;

    /**
     * 删除对应的key
     * @param key
     * @throws CacheException
     */
    void remove(String key) throws  CacheException;

    /**
     * 批量删除匹配的key
     * @param pattern
     * @throws CacheException
     */
    void removePattern(String pattern) throws CacheException;

    /**
     * 判断是否存在key
     * @param key
     * @return
     */
    boolean exists(final String key) throws  CacheException;

    /**
     *  向集合中添加一个元素,从左到右 redis有效
     * @param key
     * @param value
     * @return
     * @throws CacheException
     */
    boolean lpush(final String key,Object value) throws  CacheException;

    /**
     * 向集合中添加一个元素,从左到右,有效期
     * @param key
     * @param value
     * @param expireTime
     * @return
     * @throws CacheException
     */
    boolean lpush(final String key,Object value,Long expireTime) throws CacheException;

    /**
     * 获取List集合
     * @param key
     * @return
     * @throws CacheException
     */

    List<Object> getList(final String key) throws  CacheException;





}
