package com.loit.common.cache.ehcache;

import com.loit.common.cache.Cache;
import com.loit.common.cache.exception.CacheException;
import net.sf.ehcache.CacheManager;
import net.sf.ehcache.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.regex.Pattern;

/**
 * Description： TODO
 * Author: yangwenbin
 * Date: Created in 2020/5/29 17:39
 * Version: 1.0
 * Modified By:
 */
public class EhcacheCache implements Cache {
    private static Logger logger = LoggerFactory.getLogger(EhcacheCache.class);
    public static CacheManager cacheManager;
    private net.sf.ehcache.Cache cache;

    static{
        try {
            cacheManager = CacheManager.getInstance();
            if(cacheManager==null)
                cacheManager = CacheManager.create();
        } catch (CacheException e) {
            logger.error("Initialize cache manager failed.", e);
        }
    }
    public EhcacheCache(String cacheName){
        cache = cacheManager.getCache(cacheName);
        if(cache == null){
            cacheManager.addCache(cacheName);
            cache = cacheManager.getCache(cacheName);
            cache.getCacheConfiguration().setEternal(true);
        }
    }
    @Override
    public Object get(String key) throws CacheException {
        final Element element = cache.get(key);
        return element.getObjectValue();
    }

    @Override
    public boolean set(String key, Object value) throws CacheException {
        final Element element = new Element(key, value);
        cache.put(element);
        return true;
    }

    @Override
    public boolean set(String key, Object value, Long expireTime) throws CacheException {
        final Element element = new Element(key, value);
        element.setTimeToLive(Integer.valueOf(expireTime.toString()));
        cache.put(element);
        return true;
    }

    @Override
    public void remove(String... keys) throws CacheException {
        for(String key : keys){
            cache.remove(key);
        }
    }

    @Override
    public void remove(String key) throws CacheException {
        cache.remove(key);
    }

    @Override
    public void removePattern(String pattern) throws CacheException {
        List keyList = cache.getKeys();
        Pattern p = Pattern.compile(pattern);
        for(Object obj : keyList){
            String key = String.valueOf(obj);
            if(p.matcher(key).find()){
                cache.remove(key);
            }
        }
    }

    @Override
    public boolean exists(String key) throws CacheException {
        Object object = cache.get(key);
        if(object != null){
            return true;
        }
        return false;
    }

    @Override
    public boolean lpush(String key, Object value) throws CacheException {
        final Element element = new Element(key, value);
        cache.put(element);
        return true;

    }

    @Override
    public boolean lpush(String key, Object value, Long expireTime) throws CacheException {
        final Element element = new Element(key, value);
        element.setTimeToLive(Integer.parseInt(expireTime.toString()));
        cache.put(element);
        return true;
    }

    @Override
    public List<Object> getList(String key) throws CacheException {
        final Element element = cache.get(key);
        return (List< Object >) element.getObjectValue();

    }



}
