
package com.loit.common.cache.redis;

import com.loit.common.cache.Cache;
import com.loit.common.cache.exception.CacheException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import java.io.Serializable;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * Description： redis缓存实现
 * Author: yangwenbin
 * Date: Created in 2020/5/29 8:43
 * Version: 1.0
 * Modified By:
 */

public class RedisCache implements Cache {

    private long expireTime = 3600;// 缓存的超时时间，单位为s
    @Autowired
    private RedisTemplate redisTemplate;// 通过构造方法注入该对象

    public RedisCache() {
        super();
    }

    public RedisCache( RedisTemplate redisTemplate) {
        super();
        this.redisTemplate = redisTemplate;

    }


    @Override
    public Object get(String key) throws CacheException {
        Object result = null;
        ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
        result = operations.get(key);
        return result;
    }

    @Override
    public boolean set(String key, Object value) throws CacheException {
        boolean result = false;
        try {
            ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
            operations.set(key, value);
            result = true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public boolean set(String key, Object value, Long expireTime) throws CacheException {
        boolean result = false;
        try {
            ValueOperations<Serializable, Object> operations = redisTemplate.opsForValue();
            operations.set(key, value);
            redisTemplate.expire(key, expireTime, TimeUnit.SECONDS);
            result = true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public void remove(String... keys) throws CacheException {
        for(String key : keys){
            remove(key);
        }

    }

    @Override
    public void remove(String key) {
        if (exists(key)) {
            redisTemplate.delete(key);
        }
    }


    @Override
    public void removePattern(String pattern) throws CacheException {
        Set<Serializable> keys = redisTemplate.keys(pattern);
        if (keys.size() > 0)
            redisTemplate.delete(keys);
    }

    public  boolean lpush(final String key,Object value){
        boolean result = false;
        try {
            ListOperations<Serializable, Object> operations = redisTemplate.opsForList();
            operations.leftPush(key,value);
            result = true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public boolean lpush(String key, Object value, Long expireTime) throws CacheException {
        boolean result = false;
        try {
            ListOperations<Serializable, Object> operations = redisTemplate.opsForList();
            operations.leftPush(key,value);
            redisTemplate.expire(key, expireTime, TimeUnit.SECONDS);
            result = true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return result;
    }


    public  List<Object> getList(final String key){
        List<Object> list=null;
        try {
            ListOperations<Serializable, Object> operations = redisTemplate.opsForList();
            list = operations.range(key,0,-1);

        }catch (Exception e){
            e.printStackTrace();
        }
        return list;
    }


    @Override
    public boolean exists(String key) {
        return redisTemplate.hasKey(key);
    }





}
