package com.loit.common.config;

import com.loit.modules.sys.interceptor.LogInterceptor;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.MediaType;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.ContentNegotiationConfigurer;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.ResourceHandlerRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * 拦截器配置
 *
 * @author wbh
 * @date 2018年12月3日
 */
@Configuration
public class WebMvcConfig implements WebMvcConfigurer {

    protected Logger logger = LoggerFactory.getLogger(getClass());

    @Value("${fileUpload.uploadDir}")
    private String uploadPath;//文件上传的地址

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        // 添加拦截器
        registry.addInterceptor(securityInterceptor()).addPathPatterns("/**");
    }

    private HandlerInterceptor securityInterceptor() {
        return new LogInterceptor();
    }

    @Override
    public void addResourceHandlers(ResourceHandlerRegistry registry) {
        //和页面有关的静态目录都放在项目的static目录下
        registry.addResourceHandler("/static/**").addResourceLocations("classpath:/static/");
        //上传的图片在D盘下的img目录下，访问路径如：http://localhost:7007/image/d3cf0281-bb7f-40e0-ab77-406db95ccf2c.jpg
        //其中image表示访问的前缀。"file:E:/img/"是文件真实的存储路径
        logger.info("本地图片存储路径:" + uploadPath);
        registry.addResourceHandler("/image/**").addResourceLocations("file:" + uploadPath);
    }

    /**
     * 设置SpringBoot默认返回体Content-Type
     *
     * @param configurer
     */
    @Override
    public void configureContentNegotiation(ContentNegotiationConfigurer configurer) {
        configurer.defaultContentType(MediaType.APPLICATION_JSON_UTF8);
    }

}
