package com.loit.common.persistence;

import java.util.Date;

import javax.servlet.http.HttpServletRequest;

import org.hibernate.validator.constraints.Length;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.loit.common.constant.CommonConstant;
import com.loit.common.utils.StringUtils;
import com.loit.loitcasclient.utils.ClientSpringContextUtil;
import com.loit.loitcasclient.utils.ClientTokenUtil;
import com.loit.loitcasclient.utils.ClientUserUtil;
import com.loit.modules.sys.entity.User;

import io.swagger.annotations.ApiModelProperty;

/**
 * 数据Entity类
 * @author loit
 * @version 2014-05-16
 */
public abstract class DataEntity<T> extends BaseEntity<T> {
	
	private static final long serialVersionUID = 1-1082077813040196160L;
	
	protected User createBy;	// 创建者
	@ApiModelProperty(hidden = true)
	protected Date createTime;	// 创建日期
	protected User updateBy;	// 更新者
	@ApiModelProperty(hidden = true)
	protected Date updateTime;	// 更新日期
	protected String delFlag; 	// 删除标记（0：正常；1：删除；）
	
	public DataEntity() {
		super();
		this.delFlag = DEL_FLAG_NORMAL;
	}
	
	/**
	 * 插入之前执行方法，需要手动调用
	 */
	@Override
	public void preInsert(){		
		User user = getUser();
		this.updateBy = user;
		this.createBy = user;	
		this.updateTime = new Date();
		this.createTime = this.updateTime;
	}
	
	/**
	 * 更新之前执行方法，需要手动调用
	 */
	@Override
	public void preUpdate(){
		User user = getUser();
		// DEMON modify by 2019-12-5: 登录后没有产生令牌, 是获取不到当前登录用户的, 但是需要更新用户的最后登录时间, 所以添加判断
		if(user != null) {
			this.updateBy = user;
		}
		this.updateTime = new Date();
	}

	@Override
	public User getCurrentUser() {
		if(currentUser == null){
			currentUser = getUser();
		}
		return currentUser;
	}
	
	/**
	 * 获取当前用户缓存信息
	 * 
	 * @return 取不到返回 new User()
	 */
	protected static User getUser() {
		ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		if(servletRequestAttributes == null) {
			return null;
		}
		HttpServletRequest request = servletRequestAttributes.getRequest();
		return getUser(request);
	}

	/**
     * 获取当前登录用户
	 *
	 * DEMON modify by 2019-11-19
     */
	protected static User getUser(HttpServletRequest request){
		// 获取accessToken
//		String accessToken = ClientTokenUtil.cookieAccessToken();
//		if(null == accessToken || StringUtils.isEmpty(accessToken)){
//			accessToken = request.getHeader("access_token");
//		}
//
//		if(null == accessToken || StringUtils.isEmpty(accessToken)){
//            return null;
//        }
//
//        // access token 换 userId
//        ClientUserUtil clientUserUtil = (ClientUserUtil) ClientSpringContextUtil.getBean("clientUserUtil");
//        Object userId = clientUserUtil.getUserId(accessToken);
//        if(StringUtils.isEmpty((String)userId)){
//            return null;
//        }
		String userId = request.getHeader(CommonConstant.AUTHORIZATION);
        User user = new User();
        user.setUserId(userId);
        return  user;
    }
		
	@JsonIgnore
	public User getCreateBy() {
		return createBy;
	}

	public void setCreateBy(User createBy) {
		this.createBy = createBy;
	}

	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	public Date getCreateTime() {
		return createTime;
	}

	public void setCreateTime(Date createTime) {
		this.createTime = createTime;
	}

	@JsonIgnore
	public User getUpdateBy() {
		return updateBy;
	}

	public void setUpdateBy(User updateBy) {
		this.updateBy = updateBy;
	}

	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	public Date getUpdateTime() {
		return updateTime;
	}

	public void setUpdateTime(Date updateTime) {
		this.updateTime = updateTime;
	}

	@JsonIgnore
	@Length(min=1, max=1)
	public String getDelFlag() {
		return delFlag;
	}

	public void setDelFlag(String delFlag) {
		this.delFlag = delFlag;
	}

}
