package com.loit.loitcasclient.config;

import com.loit.common.constant.LoitHeaderConstant;
import com.loit.common.utils.ListUtil;
import com.loit.loitcasclient.utils.ClientAjaxJson;
import com.loit.loitcasclient.utils.LoitLog;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.*;
import java.util.stream.Collectors;

@Component("clientAppConfig")
public class ClientAppConfig {
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private LoitLog log;
    @Autowired
    private RestTemplate restTemplate;

    /**
     * cas服务器ip+port
     */
    private String casServerUrl;

    /**
     * 应用端后台的ip+port
     */
    private String appServerUrl;



    /**
     * 门户首页地址
     */
    private String portalIndexUrl;

    /**
     * CAS服务器应用注册的client_id
     */
    @Value("${loit.cas.app.client_id:}")
    private String clientId;

    /**
     * CAS服务器应用注册的client_secret
     */
    @Value("${loit.cas.app.client_secret:}")
    private String clientSecret;

    /**
     * CAS服务器应用(前端)注册的client_secret
     */
    @Value("${loit.cas.app.client_url:}")
    private String clientUrl;

    /**
     * 资源鉴权开关
     */
    @Value("${loit.cas.app.resource_auth:false}")
    private Boolean resourceAuth;

    /**
     * 登录鉴权开关
     */
    @Value("${loit.cas.app.login_auth:false}")
    private Boolean loginAuth;

    /**

     * 令牌cookie过期时间
     */
    @Value("${loit.cas.app.access_token_cookie_tim_out:}")
    private Integer accessTokenCookieTimOut;
    /**
     * 门户服务名（网关中门户路由path）
     */
    @Value("${loit.portal_service_name:}")
    private String portalServiceName;
    /**
     * 门户服务id（服务ID一般不会变的，默认，如果有变动，在yml中配置）
     */
    @Value("${loit.portal_service_id:loit-portal}")
    private String portalServiceId;

    /**
     * 单点登录服务名（网关中单点登录路由的path）
     */
    @Value("${loit.cas_service_name:}")
    private String casServiceName;

    /**
     * 单点登录服务（网关中单点登录路由的Id）
     */
    @Value("${loit.cas_service_id:}")
    private String casServiceId;
    @Value("${loit.gate_way_url:}")
    private String gateWayUrl;
    @Value("${loit.cas_server_url:}")
    private String casServerRealUrl;


    /**
     * 转发(不登录, 直接转发, 门户专用)
     */
    private List<String> forwards;
    @Value("#{'${loit.cas.forwards:}'}")
    public void setForwards(String forwards) {
        this.forwards = StringUtils.isEmpty(forwards) ? null : Arrays.asList(forwards.split(","));
    }

    /**
     * 过去onlineUserOuttime长时间内登录过什么用户
     */
    @Value("${loit.cas.online_user_outtime:}")
    private Integer onlineUserOuttime;

    /**
     * 获取项目名称
     */
    @Value("${spring.application.name:}")
    private String springApplicationName;


    /**
     * 单点登录服务器登录页面请求（必须重定向）
     * @return
     */
    public String codeUrl(){
        String codeUrl = "";
        if(!StringUtils.isEmpty(casServerRealUrl)){
            codeUrl = casServerRealUrl + "/cas/oauth2.0/authorize?response_type=code&client_id=portal&redirect_uri=" + getAppServerUrl() + "/cas/accessToken";
        }else{
            codeUrl = getCasServerUrl() + "/cas/oauth2.0/authorize?response_type=code&client_id=portal&redirect_uri=" + getAppServerUrl() + "/cas/accessToken";
        }
    	log.info("base url 单独登录服务器登录页面请求（必须重定向）:"+codeUrl);
    	return codeUrl;
    }

    /**
     * CAS Oauth code模式url
     * 获取token令牌（应用间接口调用时使用）
     * @return
     */
    public String accessTokenUrl(){
    	String accessTokenUrl = "http://"+casServiceId + "/cas/oauth2.0/accessToken?grant_type=authorization_code&client_id=" + clientId + "&client_secret=" + clientSecret + "&code={code}&redirect_uri=" + getAppServerUrl() + "/cas/accessToken";
    	log.info("base url 获取token令牌（应用间接口调用时使用）:"+accessTokenUrl);
    	return accessTokenUrl;
    }

    /**
     * CAS Oauth 密码模式获取令牌url（对接其他单点登录系统） loginType=0 表示app端登录非pc端，不需要验证码时用到
     * @return
     */
    public String accessTokenForPasswordUrl(){
//    	String accessTokenForPasswordUrl = "http://"+casServiceId + "/cas/oauth2.0/accessToken?grant_type=password&client_id=" + clientId + "&client_secret=" + clientSecret + "&username={username}&password={password}";
        String accessTokenForPasswordUrl = "http://"+casServiceId + "/cas/oauth2.0/accessToken?grant_type=password&client_id=" + clientId +  "&username={username}&password={password}&loginType=0";
        log.info("base url CAS Oauth 密码模式获取令牌url（对接其他单点登录系统）:"+accessTokenForPasswordUrl);
        return accessTokenForPasswordUrl;
    }

    /**
     * CAS Oauth 无密模式获取令牌url（对接其他单点登录系统） loginType=0 表示app端登录非pc端， 与有密登录url不同的是传递标识标识位参数nopassword
     * @return
     */
    public String accessTokenForNoPasswordUrl(){
        String accessTokenForPasswordUrl = "http://"+casServiceId + "/cas/oauth2.0/accessToken?grant_type=password&client_id=" + clientId + "&username={username}&password=***&loginType=0&nopassword=1";
        log.info("base url CAS Oauth 密码模式获取令牌url（对接其他单点登录系统）:"+accessTokenForPasswordUrl);
        return accessTokenForPasswordUrl;
    }

    /**
     * 获取用户信息，报901则需登录
     * @return
     */
    public String profileUrl(){
    	String profileUrl = "http://"+casServiceId + "/cas/oauth2.0/profile?access_token={accessToken}";
    	log.info("base url 获取用户信息，报901则需登录:"+profileUrl);
        return profileUrl;
    }

    public String logoutUrl(){
    	String logoutUrl = getCasServerUrl() + "/cas/logout?service=" + getAppServerUrl() + "/cas/login";
    	log.info("base url 获取用户信息，报901则需登录:"+logoutUrl);
        return logoutUrl;
    }


    public String resourceRoleListUrl() {
    	String resourceRoleListUrl = getAppServerUrl() + "/api/v1/cas/resource/role/list?appId={appId}&userId={userId}";
    	log.info("base url resourceRoleListUrl:"+resourceRoleListUrl);
        return resourceRoleListUrl;
    }

    public String appIdUrl(){
    	String appIdUrl = getAppServerUrl() + "/api/v1/cas/application/appId?userId={userId}";
    	log.info("base url appIdUrl:"+appIdUrl);
        return appIdUrl;
    }

    /**
     * 从门户获取无查询条件的全部上下文列表
     * @date: 2020年7月24日
     * add by wbh
     * @return
     */
    public String getContextList() {
        String url = "http://" + portalServiceId + "/api/v1/feign/contextConfig/list";
    	log.info("base url getWebUrl:"+url);
    	return url;
    }

    public Boolean getResourceAuth() {
        return resourceAuth;
    }

    public Boolean getLoginAuth() {
        return loginAuth;
    }

    public String getCasServerUrl() {
        if(StringUtils.isEmpty(gateWayUrl)){
            //获取来源的网关ip+端口+单点服务名的路由path
            casServerUrl = request.getScheme()+"://"+request.getHeader("x-forwarded-host")+"/" +  casServiceName;
        }else{
            casServerUrl = gateWayUrl +"/" +casServiceName;
        }

//        printRequest(request);
        return casServerUrl;
    }

    public String getAppServerUrl() {
        //获取来源的网关ip+端口+门户服务名的路由path
        if(StringUtils.isEmpty(gateWayUrl)){
            appServerUrl = request.getScheme()+"://"+request.getHeader("x-forwarded-host")+"/" + portalServiceName;
        }else{
            appServerUrl = gateWayUrl +"/"+portalServiceName;
        }

        return appServerUrl;
    }

    public String getPortalIndexUrl() {
        portalIndexUrl = clientUrl;
        return portalIndexUrl;
    }

    public String getClientId() {
        return clientId;
    }

    public String getClientSecret() {
        return clientSecret;
    }

    public Integer getAccessTokenCookieTimOut() {
        return accessTokenCookieTimOut;
    }

    public String getClientUrl() {
        try {
            URL url = new URL(clientUrl);
            clientUrl = url.getProtocol() + "://" + url.getHost() + ":" + url.getPort();
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
        return clientUrl;
    }

    /**
     * Demon modify by 2020-05-13： 临时修改，以后删除该逻辑
     * @return
     */
    @SuppressWarnings({ "unchecked", "rawtypes" })
	public String getWebUrl() {
        try {
            // 从门户获取无查询条件的全部上下文列表
            String url = getContextList();
            ResponseEntity<ClientAjaxJson> contextEntity = restTemplate.getForEntity(url, ClientAjaxJson.class);
            ClientAjaxJson json = contextEntity.getBody();

            if(json == null
                    || json.getBody() == null
                    || json.getBody().size() == 0
                    || ListUtil.isEmpty((List)json.getBody().get("list"))){
                return null;
            }

            // 验证springApplicationName的有效性
            if(StringUtils.isEmpty(springApplicationName)){
                return null;
            }

            // loit-xxxx-xxx转化为api-xxxx-xxxxx
            String applicationName = "api"+springApplicationName.substring(springApplicationName.split("-")[0].length());

            // 类型转换
            List<LinkedHashMap<String, String>> contexts = ((List<LinkedHashMap<String, String>>) json.getBody().get("list")).stream().map(e -> (LinkedHashMap<String, String>)e).collect(Collectors.toList());

            // 循环找出前端列表
            for (LinkedHashMap<String, String> context : contexts) {
                // 用配置文件中的application-name匹配上下文列表中的后端上下文，从而找出前端上下文
                String contextValue = context.get("contextValue");
                if(StringUtils.isEmpty(contextValue)){
                    continue;
                }

                // 如果后台上下文匹配成功，则返回其对应的前端上下文
                URI uri = new URI(contextValue);
                if(uri.getPath().contains(applicationName)){
                    return context.get("contextWebValue");
                }
            }

        } catch (URISyntaxException e) {
            e.printStackTrace();
        }

        return null;
    }

    /**
     * 获得门户平台的前端地址信息
     * @return
     */
    public String gePortalUrl(){
        String result = "";
        try {
            // 从门户获取无查询条件的全部上下文列表
            String url = getContextList();
            ResponseEntity<ClientAjaxJson> contextEntity = restTemplate.getForEntity(url, ClientAjaxJson.class);
            ClientAjaxJson json = contextEntity.getBody();

            if(json == null
                    || json.getBody() == null
                    || json.getBody().size() == 0
                    || ListUtil.isEmpty((List)json.getBody().get("list"))){
                return null;
            }

            // 类型转换
            List<LinkedHashMap<Object, Object>> contexts = ((List<LinkedHashMap<Object, Object>>) json.getBody().get("list")).stream().map(e -> (LinkedHashMap<Object, Object>)e).collect(Collectors.toList());
            // 循环找出前端列表
            for (LinkedHashMap<Object, Object> context : contexts) {
                Integer contextId = Integer.valueOf(context.get("contextId").toString());
                if(contextId==1){  //门户平台
                    result =String.valueOf(context.get("contextWebValue"));
                    break;
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

        return result;
    }

    /**
     * 获得用户的日志级别的信息
     * @return
     */
    public Map getUserLogLevel(){
        Map map = new HashMap();


        try {
            // 从门户获取无查询条件的全部上下文列表
            String url = "http://" + portalServiceId + "/api/v1/userLogLeveluserLogLevel/list";
            ResponseEntity<ClientAjaxJson> contextEntity = restTemplate.getForEntity(url, ClientAjaxJson.class);
            ClientAjaxJson json = contextEntity.getBody();

            if(json == null
                    || json.getBody() == null
                    || json.getBody().size() == 0
                    || ListUtil.isEmpty((List)json.getBody().get("list"))){
                return null;
            }

            // 类型转换
            List<LinkedHashMap<Object, Object>> contexts = ((List<LinkedHashMap<Object, Object>>) json.getBody().get("list")).stream().map(e -> (LinkedHashMap<Object, Object>)e).collect(Collectors.toList());
            for (LinkedHashMap<Object, Object> context : contexts) {
                String userid = String.valueOf(context.get("userid"));
                String logLevel = String.valueOf(context.get("logLevel"));
                map.put(userid,logLevel);
            }

        } catch (Exception e) {
            e.printStackTrace();
        }

        return map;
    }

    /**
     * 获得动态配置的URL信息  ,用于cas跳转时重定向的方法
     * @param userid
     * @return
     */
    public Map<String,Object> getConfUrl(String userid){
        Map<String,Object> resultMap = new HashMap<>();
        String result = "";
        String message = "";
        try {
            String url ="http://" + portalServiceId + "/api/v1/feign/applicationManage/getConfUrl";
            HttpHeaders headers = new HttpHeaders();
            headers.add(LoitHeaderConstant.AUTHORIZATION,userid);
            HttpEntity<String> entity = new HttpEntity<String>(null, headers);
//            ResponseEntity<ClientAjaxJson> ajax = restTemplate.getForEntity(url, ClientAjaxJson.class);
            ResponseEntity<ClientAjaxJson> ajax = restTemplate.exchange(url, HttpMethod.GET, entity, ClientAjaxJson.class);
//            URI uri = new URI(url);
//            ClientAjaxJson json = restTemplate.postForObject(url, entity , ClientAjaxJson.class);
            ClientAjaxJson json = ajax.getBody();
            if(json.isSuccess()){
                Map map = json.getBody();
                if(map!=null&&!map.isEmpty()){
                    Object tempUrl = map.get("url");
                    if(tempUrl!=null){
                        result = String.valueOf(tempUrl);
                    }
                }
            }else{
                message = json.getMsg();
            }
        } catch (Exception e) {
            message = "访问首页逻辑处理异常，请联系管理员";
            log.error("访问首页逻辑处理异常，请联系管理员",e);
        }
        resultMap.put("resultUrl",result);
        resultMap.put("message",message);
        return resultMap;
    }



    /**
     * 获取门户上下文配置
     * @return
     */
    public Map contextConfig(){
        Map result = null;
        try {
            String url ="http://" + portalServiceId + "/api/v1/feign/contextConfig";
            ResponseEntity<ClientAjaxJson> ajax = restTemplate.getForEntity(url, ClientAjaxJson.class);
            ClientAjaxJson json = ajax.getBody();
            if(json.isSuccess()){
                result = (Map<String, Object>) json.getBody().get("obj");
            }
        } catch (Exception e) {
            log.error("获取门户上下文配置异常",e);
            return null;
        }
        return result;
    }



    public List<String> getForwards() {
        return forwards;
    }

    public Integer getOnlineUserOuttime() {
        return onlineUserOuttime;
    }

 /*   @Bean
    public RestTemplate getRestTemplate() {
        return new RestTemplate();
    }*/

    public String getPortalServiceId() {
        return portalServiceId;
    }

    private void  printRequest(HttpServletRequest request){
        System.out.println("Protocol: " + request.getProtocol());
        System.out.println("Scheme: " + request.getScheme());
        System.out.println("Server Name: " + request.getServerName());
        System.out.println("Server Port: " + request.getServerPort());
        System.out.println("Protocol: " + request.getProtocol());
        System.out.println("Remote Addr: " + request.getRemoteAddr());
        System.out.println("Remote Host: " + request.getRemoteHost());
        System.out.println("Character Encoding: " + request.getCharacterEncoding());
        System.out.println("Content Length: " + request.getContentLength());
        System.out.println("Content Type: " + request.getContentType());
        System.out.println("Auth Type: " + request.getAuthType());
        System.out.println("HTTP Method: " + request.getMethod());
        System.out.println("Path Info: " + request.getPathInfo());
        System.out.println("Path Trans: " + request.getPathTranslated());
        System.out.println("Query String: " + request.getQueryString());
        System.out.println("Remote User: " + request.getRemoteUser());
        System.out.println("Session Id: " + request.getRequestedSessionId());
        System.out.println("Request URI: " + request.getRequestURI());
        System.out.println("Servlet Path: " + request.getServletPath());
        System.out.println("Accept: " + request.getHeader("Accept"));
        System.out.println("Host: " + request.getHeader("Host"));
        System.out.println("Referer : " + request.getHeader("Referer"));
        System.out.println("x-forwarded-host:" +request.getHeader("x-forwarded-host"));
        System.out.println("Accept-Language : " + request.getHeader("Accept-Language"));
        System.out.println("Accept-Encoding : " + request.getHeader("Accept-Encoding"));
        System.out.println("User-Agent : " + request.getHeader("User-Agent"));
        System.out.println("Connection : " + request.getHeader("Connection"));
        System.out.println("Cookie : " + request.getHeader("Cookie"));
        System.out.println("Created : " + request.getSession().getCreationTime());
        System.out.println("LastAccessed : " + request.getSession().getLastAccessedTime());

    }

}
