package com.loit.loitcasclient.controller;

import com.loit.common.constant.LoitHeaderConstant;
import com.loit.common.utils.CacheUtils;
import com.loit.common.utils.CookieUtils;
import com.loit.common.utils.StringUtils;
import com.loit.loitcasclient.config.ClientAppConfig;
import com.loit.loitcasclient.service.ClientLoginService;
import com.loit.loitcasclient.utils.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.JdkSerializationRedisSerializer;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.client.RestTemplate;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping(value = "cas")

public class ClientLoginController {
    protected Logger logger = LoggerFactory.getLogger(getClass());
    private RedisTemplate cacheTemplate = new RedisTemplate();

    @Autowired
    private RestTemplate restTemplate;
    @Autowired(required = false)
    private ClientAppConfig clientAppConfig;
    @Autowired
    private ClientUserUtil userUtil;
    @Autowired
    private ClientLoginService clientLoginService;
    @Autowired
    private LoitLog log;
    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private ClientAppConfig config = new ClientAppConfig();
    @Autowired
    private ClientUserUtil clientUserUtil  = new ClientUserUtil();

    /**
     * 登录入口
     * 采用预登录，预登录链接可被收藏
     */
    @RequestMapping(value = "login/index", method = RequestMethod.GET)
    public void loginFree(
            HttpServletResponse response) throws IOException {
        String loginUrl = clientAppConfig.getCasServerUrl() + "/cas/login?service="+clientAppConfig.getAppServerUrl()+"/cas/login";
        response.sendRedirect(loginUrl);
    }

    /**
     * 登录入口
     * 登录页面需要手动输入用户名密码验证码
     */
    @RequestMapping(value = "login", method = RequestMethod.GET)
    public void login(@RequestParam(value="url",required = false) String url, String appId,
            HttpServletResponse response,HttpServletRequest request) {
        clientLoginService.login(url,appId, request, response);
    }

    /**
     * 获取令牌
     */
    @RequestMapping(value = "accessToken", method = RequestMethod.GET)
    public void accessToken(
            @RequestParam("code") String code,
            HttpServletResponse response,
            HttpServletRequest request) throws IOException {

    	String accessTokenUrl = clientAppConfig.accessTokenUrl().replace("{code}", code);
        log.info("ClientLoginController accessToken() accessTokenUrl:" + accessTokenUrl);
        String accessToken = ClientTokenUtil.accessToken(accessTokenUrl, restTemplate);
        log.info("accessToken:"+accessToken);

        response.addCookie(ClientCookieUtil.instance("accessToken", accessToken, clientAppConfig.getAccessTokenCookieTimOut()));
        String url = ClientBase64Util.decoder(ClientCookieUtil.get(request.getCookies(), "url"));
        log.info("ClientLoginController accessToken() Redirect url 获取令牌后重定向至:"+url);
        response.sendRedirect(url);

        //add by ymm 20200727 start
//        String userid = clientUserUtil.getUserId(accessToken);
//        Map<String,Object> map  = config.getConfUrl(userid);
//        if(map!=null&&!map.isEmpty()){
//            String resultUrl = String.valueOf(map.get("resultUrl"));
//            if(StringUtils.isNotBlank(resultUrl)){
//                log.info("ClientLoginController accessToken() Redirect url 获取令牌后重定向至:"+resultUrl);
//                response.sendRedirect(resultUrl);
//            }else{
//                String message  = String.valueOf(map.get("message"));
//                response.setCharacterEncoding("UTF-8");
//                response.setContentType("text/html; charset=UTF-8");
//                response.setHeader("Cache-Control", "no-store");
//                response.setHeader("Pragma", "no-cache");
//                response.setDateHeader("Expires", 0);
//                PrintWriter out = response.getWriter();
//                out.println(message);
//                out.flush();
//                out.close();
//            }
//        }
    }

    /**
     * 登出
     */
    @ResponseBody
    @RequestMapping(value = "logout", method = RequestMethod.GET)
    public void logout(HttpServletRequest request,HttpServletResponse response) throws IOException {
        // cookie 里设置门户首页url
    	log.info("cookie 里设置门户首页url"+clientAppConfig.getPortalIndexUrl());
        response.addCookie(ClientCookieUtil.instance("url", ClientBase64Util.encoder(clientAppConfig.getPortalIndexUrl()), clientAppConfig.getAccessTokenCookieTimOut()));
        String accessToekn = CookieUtils.getCookie(request,"accessToken");
        if(StringUtils.isNotEmpty(accessToekn)){
            //清除accessToken缓存
            CacheUtils.remove(accessToekn);
            //因为单点ticket存储无序列化，需要将其设置成无序列化再将其key删除
            RedisSerializer stringSerializer = new StringRedisSerializer();
            cacheTemplate.setDefaultSerializer(stringSerializer);
            cacheTemplate.setKeySerializer(stringSerializer);
            cacheTemplate.setValueSerializer(stringSerializer);
            cacheTemplate.setHashKeySerializer(stringSerializer);
            cacheTemplate.setHashValueSerializer(stringSerializer);
            RedisConnectionFactory redisConnectionFactory = redisTemplate.getConnectionFactory();
            cacheTemplate.setConnectionFactory(redisConnectionFactory);
            cacheTemplate.afterPropertiesSet();
            cacheTemplate.delete("CAS_TICKET:"+accessToekn);

        }
        CookieUtils.setCookie(response,"accessToken","");
        // 跳转到登陆接口
        log.info("ClientLoginController logout() Redirect logoutUrl 登出后重定向至:"+clientAppConfig.logoutUrl());
        response.sendRedirect(clientAppConfig.logoutUrl());
//        clientLoginService.logout(response);
    }

//    /**
//     * 登出后跳转到登录页, 登录成功后跳转到门户首页
//     */
//    @RequestMapping(value = "logoutIndex", method = RequestMethod.GET)
//    public void logoutIndex(HttpServletResponse response) throws IOException {
//        // cookie 里设置门户首页url
//        response.addCookie(ClientCookieUtil.instance("url", ClientBase64Util.encoder(clientAppConfig.getPortalIndexUrl()), clientAppConfig.getAccessTokenCookieTimOut()));
//
//        // 跳转到登陆接口
//        String codeUrl = clientAppConfig.getCasServerUrl() + "/cas/login";
//        response.sendRedirect(codeUrl);
//    }

    /**
     * 获取登录用户名-从cookie中获取
     */
    @ResponseBody
    @RequestMapping(value = "userId/cookie", method = RequestMethod.GET)
    public Map userId(HttpServletRequest request) {
        String accessToken = ClientCookieUtil.get(request.getCookies(), "accessToken");
        String userName = userUtil.getUserId(accessToken);
        Map<String, Object> map = new HashMap<>();
        map.put("userName", userName);
        return map;
    }

    /**
     * 获取登录用户名-从登录服务器中获取
     */
    @ResponseBody
    @RequestMapping(value = "userId", method = RequestMethod.GET)
    public Map userId(@RequestParam("accessToken") String accessToken) {
        String userName = userUtil.getUserId(accessToken);
        Map<String, Object> map = new HashMap<>();
        map.put("userName", userName);
        return map;
    }

    /**
     * 测试方法
     */
    @ResponseBody
    @RequestMapping(value = "test", method = RequestMethod.GET)
    public Map<String, Object> cookieAccessToken() {
        Map<String, Object> map = new HashMap<>();
        map.put("value", "Success.");
        return map;
    }
}
