package com.loit.loitcasclient.service;

import com.loit.common.constant.LoitHeaderConstant;
import com.loit.common.utils.ListUtil;
import com.loit.loitcasclient.config.ClientAppConfig;
import com.loit.loitcasclient.utils.ClientBase64Util;
import com.loit.loitcasclient.utils.ClientCookieUtil;
import com.loit.loitcasclient.utils.LoitLog;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;

@Service
public class ClientLoginService {

    @Autowired
    private LoitLog logger;
    
    @Autowired(required = false)
    private ClientAppConfig clientAppConfig;

    /**
     * 一般登录行为
     */
    public void login(String url, String appId, HttpServletRequest request, HttpServletResponse response){
        try {
            // 不需要登录的页面, 直接转发
            List<String> forwards = clientAppConfig.getForwards();
            if(ListUtil.isNotEmpty(forwards) && !StringUtils.isEmpty(url)){
                String _url = ClientBase64Util.decoder(url);
                for (String forward : forwards) {
                    String path = forward.substring(0, forward.indexOf("-"));
                    if(_url.contains(path)){
                        String uri = forward.substring(forward.indexOf("-") + 1);
                        response.sendRedirect(uri);
                        logger.info("ClientLoginService login() Redirect uri:"+uri); 
                        return;
                    }
                }
            }

            // 如果没有url参数, 就使用portal门户的主页地址(表示这是给portal项目配置的)
            if(StringUtils.isEmpty(url)){
                url = clientAppConfig.getPortalIndexUrl();
            }else{
                // Demon modify by 2020-05-13： 临时修改为getWebUrl，以后删除该逻辑
                url = clientAppConfig.getWebUrl() + ClientBase64Util.decoder(url);
            }

            logger.info("前端地址："+url);
            String userId = request.getHeader(LoitHeaderConstant.AUTHORIZATION);
            logger.info("登录时如果存在userId,则直接跳转前端页面，userId: "+userId); 
            if(userId!=null) {
                response.addCookie(ClientCookieUtil.instance("appId", appId, clientAppConfig.getAccessTokenCookieTimOut()));
            	response.sendRedirect(url);
            }else {
	            url = ClientBase64Util.encoder(url);
	            response.addCookie(ClientCookieUtil.instance("url", url, clientAppConfig.getAccessTokenCookieTimOut()));
	            
	            String codeUrl = clientAppConfig.codeUrl();
	            logger.info("ClientLoginService login() Redirect codeUrl 登录后重定向至:"+codeUrl); 
	            response.sendRedirect(codeUrl);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
