package com.loit.loitcasclient.utils;

import com.loit.loitcasclient.dto.ClientAccessTokenDTO;
import com.loit.loitcasclient.dto.ClientAccessTokenUserDTO;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import javax.servlet.http.HttpServletRequest;

/**
 * 令牌工具类
 */
public class ClientTokenUtil {
    /**
     * 根据code获取登录服务器上的accessToken
     * @return accessToken
     * {
     *     "access_token": "AT-6-cRCq3wj9BBvDPj7V0dON6-m7hAB0vWgY",
     *     "token_type": "bearer",
     *     "expires_in": 28800,
     *     "refresh_token": "RT-6-LGG-eRoW-QNDDsfaHWF7LvrpQIDPlHoN"
     * }
     */
    public static String accessToken(String url, RestTemplate restTemplate){
        try {
            ResponseEntity<ClientAccessTokenDTO> accessTokenEntity = restTemplate.postForEntity(url, null, ClientAccessTokenDTO.class);
            ClientAccessTokenDTO accessTokenDTO = accessTokenEntity.getBody();
            return accessTokenDTO.getAccess_token();
        } catch (RestClientException e) {
            return null;
        }
    }

    /**
     * 根据accessToken获取userId
     */
    public static String profile(String url, RestTemplate restTemplate){
        try {
            ResponseEntity<ClientAccessTokenUserDTO> userEntity = restTemplate.getForEntity(url, ClientAccessTokenUserDTO.class);
            ClientAccessTokenUserDTO accessTokenUserDTO = userEntity.getBody();
            return accessTokenUserDTO.getId();
        } catch (RestClientException e) {
            return null;
        }
    }

    /**
     * 从第三方应用的cookie中获取令牌
     */
    public static String cookieAccessToken(){
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = servletRequestAttributes.getRequest();

        // 从cookie中获取令牌
        String accessToken = ClientCookieUtil.get(request.getCookies(), "accessToken");
        return accessToken;
    }
}
