package com.loit.loitcasclient.utils;

import com.loit.common.constant.CommonConstant;
import com.loit.common.constant.LoitHeaderConstant;
import com.loit.common.utils.CacheUtils;
import com.loit.common.utils.CookieUtils;
import com.loit.common.utils.StringUtils;
import com.loit.loitcasclient.config.ClientAppConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;
import org.slf4j.spi.LocationAwareLogger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Controller;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.RequestContextListener;
import org.springframework.web.context.request.ServletRequestAttributes;
import sun.misc.JavaLangAccess;
import sun.misc.SharedSecrets;

import javax.servlet.http.HttpServletRequest;
import java.net.InetAddress;
import java.util.Map;

@Component
@Configuration
@Controller
public class LoitLog {

	/**空数组*/
	private static final Object[] EMPTY_ARRAY = new Object[] {};
	/**全类名*/
	private static final String FQCN = LoitLog.class.getName();

	private static final int DEPTH = 2;

    @Value("${loit.login_debug_out:}")
    private Boolean loginDebugOut; //还未使用
    @Value("${loit.login_debug_level:}")
	private String loggingLevel; //还未使用
	private final static String USER_ID = "userId";
	private final static String QUERY_STRING = "queryString";
	private final static String HOST_IP = "hostIp";
	private final static String HOST_PORT = "hostPort";
	private final static String REQ_URI="reqUri";
	private final static String REQ_TYPE="reqType";

	@Autowired
	private ClientAppConfig config = new ClientAppConfig();
	/**
	 * redis 缓存用户日志级别的信息
	 */
	public static final String USER_LOG_LEVEL = "user_log_level";
	@Autowired
	private ClientUserUtil clientUserUtil  = new ClientUserUtil();


	private static Logger  getLogger(int stackDepth){
		//通过堆栈信息获取调用当前方法的类名和方法名
		JavaLangAccess access = SharedSecrets.getJavaLangAccess();
		Throwable throwable = new Throwable();
		StackTraceElement frame = access.getStackTraceElement(throwable, stackDepth);
		return LoggerFactory.getLogger(frame.getClassName());
	}

	/**
	 * @desc INFO级别信息
	 * @param info
	 */
	public void info(String info) {
		HttpServletRequest httpServletRequest = getHttpServletRequest();
		insertIntoMDC(httpServletRequest);
//		getLogger(DEPTH).info(info);
		commonLog("info",info,httpServletRequest,null);
	}

	/**
	 * INFO级别信息
	 * @param info
	 * @param var2
	 */
	public void info(String info, Object... var2){
		HttpServletRequest httpServletRequest = getHttpServletRequest();
		insertIntoMDC(httpServletRequest);
//		getLogger(DEPTH).info(info,var2);
		commonLog("info",info,httpServletRequest,var2);
	}

	/**
	 * @desc 跟踪级别
	 * @param info
	 */
    public void trace(String info){
		HttpServletRequest httpServletRequest = getHttpServletRequest();
		insertIntoMDC(httpServletRequest);
//		getLogger(DEPTH).trace(info);
		commonLog("trace",info,httpServletRequest,null);
	}

	/**
	 * @desc 跟踪级别
	 * @param info
	 * @param var2
	 */
	public void trace(String info,Object... var2){
		HttpServletRequest httpServletRequest = getHttpServletRequest();
		insertIntoMDC(httpServletRequest);
//		getLogger(DEPTH).trace(info,var2);
		commonLog("trace",info,httpServletRequest,var2);
	}

	/**
	 * @desc Debug级别
	 * @param info
	 */
	public void debug(String info){
		HttpServletRequest httpServletRequest = getHttpServletRequest();
		insertIntoMDC(httpServletRequest);
//		getLogger(DEPTH).debug(info);
		commonLog("debug",info,httpServletRequest,null);
	}

	/**
	 * @desc Debug级别
	 * @param info
	 * @param var2
	 */
	public void debug(String info,Object... var2){
		HttpServletRequest httpServletRequest = getHttpServletRequest();
		insertIntoMDC(httpServletRequest);
//		getLogger(DEPTH).debug(info,var2);
		commonLog("debug",info,httpServletRequest,var2);
	}

	/**
	 * @desc Warn级别
	 * @param info
	 *
	 */
	public void warn(String info){
		HttpServletRequest httpServletRequest = getHttpServletRequest();
		insertIntoMDC(httpServletRequest);
//		getLogger(DEPTH).warn(info);
		commonLog("warn",info,httpServletRequest,null);
	}
	/**
	 * @desc Warn级别
	 * @param info
	 * @param var2
	 */
	public void warn(String info,Object... var2){
		HttpServletRequest httpServletRequest = getHttpServletRequest();
		insertIntoMDC(httpServletRequest);
//		getLogger(DEPTH).warn(info,var2);
		commonLog("warn",info,httpServletRequest,var2);
	}

	/**
	 * @desc error级别
	 * @param info
	 */
	public void error(String info){
		HttpServletRequest httpServletRequest = getHttpServletRequest();
		insertIntoMDC(httpServletRequest);
		getLogger(DEPTH).error(info);
	}

	/**
	 * error级别
	 * @param info
	 * @param var2
	 */
	public void error(String info,Object... var2){
		HttpServletRequest httpServletRequest = getHttpServletRequest();
		insertIntoMDC(httpServletRequest);
		getLogger(DEPTH).error(info,var2);
	}



	public Boolean getLoginDebugOut() {
		return loginDebugOut;
	}

	public void setLoginDebugOut(Boolean loginDebugOut) {
		this.loginDebugOut = loginDebugOut;
	}

	private HttpServletRequest getHttpServletRequest(){
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        //如果获取不到可能调用者是工具类或者任务等，一般Controller，service都能获取到
        if(servletRequestAttributes == null){
            return null;
        }else{
            return servletRequestAttributes.getRequest();
        }
	}
	private void insertIntoMDC(HttpServletRequest request) {
	    if(request != null){
            //获取服务IP
            String hostIp = getHostIp();
            //获取服务端口
            String hostPort = String.valueOf(request.getLocalPort());
            //获取参数
            String queryString = request.getQueryString();
            //请求地址
            String reqUri = request.getRequestURI();
            //请求方式
            String reqType = request.getMethod();
            String accessToken = CookieUtils.getCookie(request, CommonConstant.TOKEN_NAME);
            String userId = "";
            //用户ID
            if(StringUtils.isNotEmpty(accessToken)){
                userId = (String) CacheUtils.get(accessToken);
            }
            //以下是存放相关信息到MDC中
            MDC.put(HOST_IP,hostIp);
            MDC.put(HOST_PORT,hostPort);
            MDC.put(QUERY_STRING,queryString);
            MDC.put(REQ_URI,reqUri);
            MDC.put(REQ_TYPE,reqType);
            MDC.put(USER_ID,userId);
        }



	}

	/**
	 * @Description: TODO(获取服务IP地址)
	 * @author yangwenbin
	 * @date
	 */
	private String getHostIp() {
		try {
			InetAddress address = InetAddress.getLocalHost();
			return address.getHostAddress();

		} catch (Exception e) {
			info("获取服务IP地址异常:"+e);
		}
		return null;
	}


	public String getLoggingLevel() {
		return loggingLevel;
	}

	public void setLoggingLevel(String loggingLevel) {
		this.loggingLevel = loggingLevel;
	}

	/**
	 * 用于动态的输出日志的信息
	 * @param info  日志的信息
	 * @param var2  具体的参数
	 * @param defaultInfo  默认显示的级别
	 */
	public void commonLog(String defaultInfo,String info,HttpServletRequest request,Object... var2){
		//用于判断用户的级别，然后确定日志输出的级别
		String userLevel = "";
		if(request != null){
			Object obj = CacheUtils.get(USER_LOG_LEVEL);
			Map map =null;
			if(obj==null){
				map = config.getUserLogLevel();
				if (map!=null){
					CacheUtils.set(USER_LOG_LEVEL,map,300L);
				}
			}else{
				map = (Map)obj;
			}
			if(map!=null){
				// 每次请求时，网关使用令牌去登录服务器获取userId并放入Header中
				String  userId = request.getHeader(LoitHeaderConstant.AUTHORIZATION);
				userLevel = map.get(userId)==null?"":map.get(userId).toString();
			}
		}
		if(StringUtils.isNotBlank(userLevel)){
			commmomLogPrint(userLevel,info,var2);
		}else{
			commmomLogPrint(defaultInfo,info,var2);
		}
	}

	/**
	 * 用于动态的输出日志的信息
	 * @param info  日志的信息
	 * @param var2  具体的参数
	 * @param defaultInfo  默认显示的级别
	 */
	public void commmomLogPrint(String defaultInfo,String info,Object... var2){
		if("trace".equals(defaultInfo)) {
			if(var2!=null){
				getLogger(DEPTH).trace(info,var2);
			}else{
				getLogger(DEPTH).trace(info);
			}
		}else if("info".equals(defaultInfo)){
			if(var2!=null){
				getLogger(DEPTH).info(info,var2);
			}else{
				getLogger(DEPTH).info(info);
			}
		}else if("debug".equals(defaultInfo)){
			if(var2!=null){
				getLogger(DEPTH).debug(info,var2);
			}else{
				getLogger(DEPTH).debug(info);
			}
		}else if("warn".equals(defaultInfo)){
			if(var2!=null){
				getLogger(DEPTH).warn(info,var2);
			}else{
				getLogger(DEPTH).warn(info);
			}
		}

	}

}
