package com.loit.modules.sys.interceptor;

import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import com.loit.common.constant.CommonConstant;
import com.loit.common.constant.LoitPageParamEnum;

import feign.RequestInterceptor;
import feign.RequestTemplate;

/**
 * 设置请求上游服务的请求header
 * @author wbh
 * @date 2018年12月1日
 */
@Component
public class FeignRequestInterceptor implements RequestInterceptor {
	private Logger logger = LoggerFactory.getLogger(getClass());
 
    @Value("${spring.application.name}")
    private String serviceName;
 
    @Override
    public void apply(RequestTemplate requestTemplate) {
       requestTemplate.header("from-service", serviceName);
       ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
       if (attributes != null) {
           HttpServletRequest request = attributes.getRequest();
           String cookie = request.getHeader("cookie");
           String authorization = request.getHeader(CommonConstant.AUTHORIZATION);
           logger.info("FeignRequestInterceptor cookie:"+cookie+" | userId: " + authorization);
           requestTemplate.header("cookie", cookie);
           requestTemplate.header(CommonConstant.AUTHORIZATION, authorization);
//           Enumeration<String> headerNames = request.getHeaderNames();
//           if (headerNames != null) {
//               while (headerNames.hasMoreElements()) {
//                   String name = headerNames.nextElement();
//                   String value = request.getHeader(name);
//                   /**
//                    * 遍历请求头里面的属性字段，将logId和token添加到新的请求头中转发到下游服务
//                    * */
//                   if ("cookie".equalsIgnoreCase(name)) {
//                       logger.debug("将原session设置到请求头中 cookie:"+value);
//                       requestTemplate.header(name, value);
//                   }
//               }
//           } else {
//               logger.warn("FeignHeadConfiguration", "获取请求头失败！");
//           }
           List<String> list = LoitPageParamEnum.toList();
           for (String string : list) {
    		   String values = request.getParameter(string);
    		   if(StringUtils.isEmpty(values)&&request.getAttribute(string)!=null)
    		   	  values = request.getAttribute(string).toString();
    		   if(!StringUtils.isEmpty(values)) {
	    		   logger.info("FeignRequestInterceptor 参数转发：name: {} value: {}",string,values);
	    		   requestTemplate.header(string, values);
    		   }
           }
       }
       
    }
}
