package com.loit.getway.controller;

import com.loit.common.json.AjaxJson;
import com.loit.getway.config.GateWayConfig;
import com.loit.getway.constant.CommonConstant;
import com.loit.getway.utils.RedisUtil;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Set;

@RestController
@RequestMapping(value = "/online/user")
@Api(tags = "在线用户数量统计",  value = "OnlineUserCountController")
public class OnlineUserCountController {

    @Autowired
    private GateWayConfig gateWayConfig;

    /**
     * 登录，设置用户在线时间
     * @return
     */
    @GetMapping(value = "/save")
    public void saveOnline(@RequestParam(value = "userId", required = true) String userId){
        Calendar now = Calendar.getInstance();
        now.set(Calendar.SECOND, now.get(Calendar.SECOND) + gateWayConfig.getKillSeconds());
        RedisUtil.zAdd(CommonConstant.ONLINE_USERS, userId, now.getTimeInMillis());
    }

    /**
     * 登出，删除在线用户信息
     * @return
     */
    @GetMapping(value = "/delete")
    public void deleteOnline(@RequestParam(value = "userId", required = true) String userId){
        RedisUtil.zRemove(CommonConstant.ONLINE_USERS, userId);
    }

    /**
     * 获取在线用户数量
     * @return
     */
    @GetMapping(value = "/count")
    public AjaxJson<Long> onlineUserCount(){
        AjaxJson json = new AjaxJson();
        Long count = RedisUtil.zCard(CommonConstant.ONLINE_USERS);
        json.put("count", count);
        return json;
    }

    /**
     * 获取在线用户列表
     * @return
     */
    @GetMapping(value = "/list")
    public AjaxJson<List<String>> onlineUserList(){
        AjaxJson json = new AjaxJson();
        Set<String> ids = RedisUtil.zRange(CommonConstant.ONLINE_USERS);
        json.put("list", new ArrayList(ids));
        return json;
    }
}
