package com.loit.getway.filter;
import com.loit.getway.config.GateWayConfig;
import com.loit.getway.constant.CommonConstant;
import com.loit.getway.utils.CacheUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.Ordered;
import org.springframework.http.HttpCookie;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.net.InetAddress;

@Configuration
public class GetwayLogFilter implements GlobalFilter, Ordered {
	protected Logger logger = LoggerFactory.getLogger(GetwayLogFilter.class);
    private static final String REQUEST_TIME_BEGIN = "beginTime";
	private final static String USER_ID = "userId";
	private final static String QUERY_STRING = "queryString";
	private final static String HOST_IP = "hostIp";
	private final static String HOST_PORT = "hostPort";
	private final static String REQ_URI="reqUri";
	private final static String REQ_TYPE="reqType";
	@Override
	public int getOrder() {
		// TODO Auto-generated method stub
		return -1;
	}

	@Override
	public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
		exchange.getAttributes().put(REQUEST_TIME_BEGIN, System.currentTimeMillis());
		insertIntoMDC(exchange.getRequest());
        return chain.filter(exchange).then(
                Mono.fromRunnable(() -> {
                    Long startTime = exchange.getAttribute(REQUEST_TIME_BEGIN);
                    long timeSlot = System.currentTimeMillis() - startTime;
        			String timeDesc = "time consuming";
        			if(timeSlot>1000&&timeSlot<= 5000) {
        				timeDesc = "long time consuming";
        			}else if(timeSlot>5000) {
        				timeDesc = "super long time consuming";
        			}
                    if (startTime != null) {
                    	logger.info("URI: {}  最大内存: {}m  已分配内存: {}m  已分配内存中的剩余空间: {}m  最大可用内存: {}m  "+timeDesc+": {}ms",
                    			exchange.getRequest().getURI().getRawPath(), Runtime.getRuntime().maxMemory()/1024/1024, Runtime.getRuntime().totalMemory()/1024/1024, Runtime.getRuntime().freeMemory()/1024/1024,
                                (Runtime.getRuntime().maxMemory()-Runtime.getRuntime().totalMemory()+Runtime.getRuntime().freeMemory())/1024/1024,timeSlot);
                    }
                })
        );


	}

	/**
	 * @Description: TODO(将请求相关信息存放到MDC中，以便在logback日志中输出)
	 * @author yangwenbin
	 * @date
	 */
	private void insertIntoMDC(ServerHttpRequest request) {

		//获取服务IP
		String hostIp = GateWayConfig.hostIp;
		//获取服务端口
		String hostPort = String.valueOf(request.getURI().getPort());
		//获取参数
		String queryString=request.getQueryParams().toString();
		//请求地址
		String reqUri=request.getURI().getPath();
		//请求方式
		String reqType = request.getMethodValue();
		HttpCookie httpCookie = request.getCookies().getFirst(CommonConstant.TOKEN_NAME);
		String accessToken="";
		if(null == httpCookie || org.springframework.util.StringUtils.isEmpty(httpCookie)) {
			accessToken = request.getHeaders().getFirst(CommonConstant.TOKEN_NAME);
		}else {
			accessToken = httpCookie.getValue();
		}
		String userId = "";
		//用户ID
		if(StringUtils.isNotEmpty(accessToken)){
			userId = (String) CacheUtils.get(accessToken);
		}
		//以下是存放相关信息到MDC中
		MDC.put(HOST_IP,hostIp);
		MDC.put(HOST_PORT,hostPort);
		MDC.put(QUERY_STRING,queryString);
		MDC.put(REQ_URI,reqUri);
		MDC.put(REQ_TYPE,reqType);
		MDC.put(USER_ID,userId);

	}

//	/**
//	 * @Description: TODO(获取服务IP地址)
//	 * @author yangwenbin
//	 * @date
//	 */
//	private String getHostIp() {
//		try {
//			InetAddress address = InetAddress.getLocalHost();
//			return address.getHostAddress();
//
//		} catch (Exception e) {
//			logger.error("获取服务IP地址异常{}",e);
//		}
//		return null;
//	}

}
