package com.loit.getway.schedule;

import com.loit.getway.constant.CommonConstant;
import com.loit.getway.utils.RedisUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

/**
 * @description: 在线用户定时器
 * @date: 2020/7/6 9:43
 * @author: wubj
 * @version v1.0
 */
@Component
public class OnlineUserSchedule {

    private Logger log = LoggerFactory.getLogger(this.getClass());

    /**
     * 定时任务：删除不在线用户数据
     */
    @Scheduled(cron = "0 0/3 * * * ?")
    public void scheduledTask() {
        log.info("删除不在线用户数据开始执行");
        long begin = System.currentTimeMillis();

        // 删除过了有效期的数据
        RedisUtil.removeRangeByScore(CommonConstant.ONLINE_USERS, 0.0, Double.valueOf(System.currentTimeMillis()));

        long end = System.currentTimeMillis();
        log.info("删除不在线用户数据执行结束，花费时间为" + (end - begin) + "毫秒");
    }
}
