/**
 * Copyright &copy; 2015-2020 isaac All rights reserved.
 */
package com.loit.modules.sys.service;

import com.loit.common.service.CrudService;
import com.loit.modules.sys.dao.DictDataDao;
import com.loit.modules.sys.entity.DictData;
import com.loit.modules.sys.utils.DictDataUtils;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * 系统字典数据项Service
 * @author xutao
 * @date 2019-10-17
 */
@Service
@Transactional(readOnly = true)
public class DictDataService extends CrudService<DictDataDao, DictData> {

    /**
     * 删除数据
     * @author xutao
     * @date 2019-10-18
     * @param ids
     */
    @Transactional(readOnly = false)
    public void deleteAll(String ids) {
        String idArray[] =ids.split(",");
        StringBuilder dictIds = new StringBuilder();
        for(String dataId : idArray){
            DictData dict = get(dataId);
            if(dict != null) {
            	dictIds.append(dataId).append(",");
            	 super.delete(dict);
            }            
        }        
        //更新缓存
        if(dictIds.length()>0) {
        	String[] dictIdArr = dictIds.toString().split(",");
        	for(String dictId : dictIdArr) {
        		DictDataUtils.setDictDataList(Integer.valueOf(dictId));
        	}
        	
        }
    }
    
    /**
    * @Description: 保存/更新数据并更新缓存
    * @author: gzx
    * @date: 2019年11月3日 下午3:55:33 
    * @param @param dictData 参数 
    * @return void 返回类型 
    * @throws
     */
    @Transactional(readOnly = false)
	public void save(DictData dictData) {
		super.save(dictData, dictData.getDataId());
		DictDataUtils.setDictDataList(dictData.getDictId());
	}
    
    /**
	* @Description: 判断名称的唯一性
	* @author: xutao
	* @date: 2019年11月19日 下午6:59:32 
	* @param dictId
	* @param dataId
	* @param dataName
	* @return boolean 返回类型  名称重复，返回false；不重复，返回true
	* @throws
	 */
	public boolean checkNameUnique(Integer dictId,Integer dataId,String dataName) {
		int count = dao.checkNameUnique(dataName,dataId,dictId);
		if(count>0) {
			return false;
		}
		return true;
	}

}
