package com.loit.modules.sys.service;

import com.loit.common.persistence.Page;
import com.loit.common.service.CrudService;
import com.loit.modules.sys.dao.DictDao;
import com.loit.modules.sys.dao.DictDataDao;
import com.loit.modules.sys.entity.Dict;
import com.loit.modules.sys.utils.DictDataUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author xutao
 * @version 2019-11-07
 * @description 字典Service
 */
@Service
public class DictService extends CrudService<DictDao, Dict> {

    @Autowired
    private DictDataDao dictDataDao;


    /**
     * 新增数据
     *
     * @param dictList
     */
    @Transactional(readOnly = false)
    public void insertResultBatch(List<Dict> dictList) {
        dao.insertResultBatch(dictList);
    }

    /**
     * 批量删除数据
     *
     * @param dictIdList
     */
    @Transactional(readOnly = false)
    public void deleteResultBatch(List<Integer> dictIdList) {
        dao.deleteResultBatch(dictIdList);
    }

    /**
     * 批量更新数据
     *
     * @param dictList
     */
    @Transactional(readOnly = false)
    public void updateResultBatch(List<Dict> dictList) {
        dao.updateResultBatch(dictList);
    }


    /**
     * @param dict 字典实体
     * @description 保存或更新字典
     */
    @Transactional(readOnly = false)
    public void save(Dict dict) {

        if (dict.getpId() == null) {
            dict.setpId(0);
        }
        super.save(dict, dict.getDictId());
    }

    /**
     * @param dict 字典ID
     * @description 删除字典数据
     */
    @Transactional(readOnly = false)
    public void delete(Dict dict) {
        //删除该字典对应的字典项
        super.delete(dict);
    }

    /**
     * 单个或批量删除系统字典，多ID时，以逗号分隔.并递归删除子节点
     *
     * @param dictIds
     */
    @Transactional(readOnly = false)
    public void deleteAll(String dictIds) {
        String[] dictIdArr = dictIds.split(",");
        Dict dict = new Dict();
        for (String dictId : dictIdArr) {
            //删除父级数据
            dict.setDictId(Integer.valueOf(dictId));
            super.delete(dict);
            //查询子级数据
            Dict dict1 = new Dict();
            dict1.setpId(Integer.valueOf(dictId));
            List<Dict> list = dao.findList(dict1);
            //删除存在的子级数据
            if (list != null && list.size() > 0) {
                for (Dict d : list) {
                    super.delete(d);
                }
            }
            //删除该字典对应的字典项
            dictDataDao.deleteByDictId(dict.getDictId());
            //从缓存中移除对应的字典项
            DictDataUtils.removeDictDataList(dict.getDictId());
        }
    }

    /**
     * @param page   分页对象
     * @param entity 字典实体
     * @return
     * @description 查询字典树形分页数据
     */
    public Page<Dict> findTreePage(Page<Dict> page, Dict entity) {
        entity.setPage(page);
        page.setList(dao.getDictTreeList(entity));
        return page;
    }

    /**
     * @param dictId
     * @param dictName
     * @return boolean 返回类型  名称重复，返回false；不重复，返回true
     * @throws
     * @Description: 判断名称的唯一性
     * @author: xutao
     * @date: 2019年11月19日 下午6:59:32
     */
    public boolean checkNameUnique(Integer dictId, String dictName) {
        int count = dao.checkNameUnique(dictName, dictId);
        if (count > 0) {
            return false;
        }
        return true;
    }

}
