package com.loit.modules.sys.stressTest;

import com.loit.common.exception.BusinessException;
import com.loit.common.json.AjaxJson;
import com.loit.common.web.BaseController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author chenshiying
 * @description 压力测试Controller
 */
@RestController
@RequestMapping(value = "api/v1/test/base")
@Api(value = "StressTestController", tags = {"压力测试Controller"})
public class StressTestController<T> extends BaseController {

    /**
     * @description 返回json
     */
    @GetMapping(value = "json")
    @ApiOperation(value = "json", notes = "返回json")
    public AjaxJson testJson() {
        AjaxJson json = new AjaxJson();
        return json;
    }


    @GetMapping(value = "void")
    @ApiOperation(value = "void", notes = "void")
    public void testVoid() {

    }


    /**
     * @description 系统字典详情
     */
    @GetMapping(value = "exception")
    @ApiOperation(value = "异常", notes = "BusinessException")
    public AjaxJson testException() {
        throw new BusinessException("测试异常: 出错了");
    }


    /**
     * @description 延迟测试, 默认延迟1秒
     */
    @GetMapping(value = "sleep")
    @ApiOperation(value = "sleep", notes = "延迟测试")
    @ApiImplicitParam(name = "sleep", value = "系统字典ID", dataType = "Integer")
    public AjaxJson testSleep(@PathVariable(required = false) Integer time) {

        if (time == null) {
            time = 1000;
        }

        try {
            Thread.sleep(time);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        AjaxJson json = new AjaxJson();
        json.setMsg("睡眠：" + String.valueOf(time) + "毫秒");
        return json;
    }

}
