package com.loit.modules.sys.utils;

import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.common.collect.Lists;
import com.loit.common.annotation.LoitDictAnnotation.LoitDict;
import com.loit.common.utils.CacheUtils;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.SpringContextHolder;
import com.loit.modules.sys.dao.DictDataDao;
import com.loit.modules.sys.entity.DictData;
/**
 * @Description: 字典数据项工具类
 * @author: gzx
 * @date: 2019年11月3日 下午1:31:11
 * @version: V1.0
 */
public class DictDataUtils {

    protected static Logger logger = LoggerFactory.getLogger(DictDataUtils.class);
    private static DictDataDao dictDataDao;

	public static final String CACHE_DICT_KEY = "portal:dictdata:";
	public static final Long cacheSecond = 300L;
	
	public static DictDataDao getDictDataDao() {
        if(dictDataDao==null) {
        	dictDataDao = SpringContextHolder.getBean(DictDataDao.class);
        }
        return dictDataDao;
    }
	
	public static String getDictDataName(Integer dataId, String dictCode, String defaultDataName){
		if (dictCode!= null && dataId != null){
			for (DictData dict : getDictDataList(dictCode)){
				if (dataId.equals(dict.getDataId())){
					return dict.getDataName();
				}
			}
		}
		return defaultDataName;
	}
	
	public static String getDataNames(String dataIds, String dictCode, String defaultDataName){
		if (dictCode != null && StringUtils.isNotBlank(dataIds)){
			List<String> dataNameList = Lists.newArrayList();
			for (String value : StringUtils.split(dataIds, ",")){
				dataNameList.add(getDictDataName(Integer.valueOf(value), dictCode, defaultDataName));
			}
			return StringUtils.join(dataNameList, ",");
		}
		return defaultDataName;
	}

	public static Integer getDataId(String dataName, String dictCode, Integer defaultDataId){
		if (dictCode != null && StringUtils.isNotBlank(dataName)){
			for (DictData dictData : getDictDataList(dictCode)){
				if (dataName.equals(dictData.getDataName())){
					return dictData.getDataId();
				}
			}
		}
		return defaultDataId;
	}
	
	@SuppressWarnings("unchecked")
	public static List<DictData> getDictDataList(String dictCode){
		if(dictCode == null) return new ArrayList<>();
		List<DictData> dictDataList =null;
		boolean sign = true;
		try {
			dictDataList = (List<DictData>)CacheUtils.get(CACHE_DICT_KEY+dictCode);
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("errorCode:{},info:字典工具类，{}",LoitStatusCode.REDIS_LINK_FAIL.statusCode,LoitStatusCode.REDIS_LINK_FAIL.statusDesc);
			sign = false;
		}
		if (dictDataList==null){
			dictDataList = getDictDataDao().findDictDataByDictCode(dictCode);
			//redis无法连接，无需缓存至redis
			if(sign) {
				CacheUtils.set(CACHE_DICT_KEY+dictCode, dictDataList, cacheSecond);
			}
		}
		return dictDataList;
	}
	/**
	* @Description: 移出缓存
	* @author: gzx
	* @date: 2019年11月3日 下午3:30:21 
	* @param @param dictId 参数 
	* @return void 返回类型 
	* @throws
	 */
	public static void removeDictDataList(Integer dictId) {
		CacheUtils.remove(CACHE_DICT_KEY+dictId);
	}
	/**
	* @Description: 将字典数据放入缓存
	* @author: gzx
	* @date: 2019年11月3日 下午3:32:16 
	* @param @param dictId 参数 
	* @return void 返回类型 
	* @throws
	 */
	public static void setDictDataList(Integer dictId) {
		DictData dictData = new DictData();
		dictData.setDictId(dictId);
		List<DictData> dictDatalist = getDictDataDao().findList(dictData);
		//先删除后插入
		CacheUtils.remove(CACHE_DICT_KEY+dictId);
		CacheUtils.set(CACHE_DICT_KEY+dictId, dictDatalist, cacheSecond);
	}
	/**
	* @Description: 对实体字典数据根据ID查出相应的数据名称。 注：实体字典字段名称起名的时候，必须以“Name”结尾，才能用该功能
	* @author: wbh 
	* @date: 2019年11月3日 下午4:56:32 
	* @param @param obj
	* @param @return 参数 
	* @return Object 返回类型 
	* @throws
	 */
	public static Object transfor(Object obj) {
		if(obj==null) return obj;
    	Class<?> clazz = obj.getClass();
        Field[] fields = clazz.getDeclaredFields();
        try {
			for (Field field : fields) {
			    //打开私有访问
			    field.setAccessible(true);
			    //获取属性值
			    Object value = field.get(obj);
				String name=field.getName();
				LoitDict loitDict = field.getAnnotation(LoitDict.class);
			    if(loitDict!=null&&StringUtils.isNotBlank(loitDict.dictCode())) {
			    	if("loit_dict_data".equals(loitDict.tableName())) {
			    		List<DictData> list = null;
			        	list = DictDataUtils.getDictDataList(loitDict.dictCode());
			        	if(value!=null) {
			        		String dataCode = value.toString();
			            	for (DictData dictData : list) {
								if(StringUtils.equals(dictData.getDataCode(), dataCode)) {
									 setValue(obj, name+"Name", dictData.getDataName()); 
								}
							}
			        	}
			    	}else {
			    		throw new Exception("该字典表未配置");
			    	}
			    }
			    field.setAccessible(false);
			}
		} catch (Exception e) {
			logger.info("字典解析失败："+e.getMessage());
		}
        return obj;
	}
	
	public static List transforList(List list) {
		for (Object obj : list) {
			transfor(obj);
		}
		return list;
	}
	
	/*
	 * 反射根据对象和属性名获取属性值
	 */
	public static Object getValue(Object obj, String filed) {
		try {
			Class clazz = obj.getClass();
			PropertyDescriptor pd = new PropertyDescriptor(filed, clazz);
			Method getMethod = pd.getReadMethod();//获得get方法

			if (pd != null) {

				Object o = getMethod.invoke(obj);//执行get方法返回一个Object
				return o;

			}
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IntrospectionException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		
		return null;
	}
	
	/**
	 * 反射根据对象和属性名设置属性值
	 * @author wbh
	 * @date：2019年1月7日
	 * @param obj 对象
	 * @param field 属性名称
	 * @param value 属性值
	 * @throws Exception
	 */
	public static void setValue(Object obj, String field, Object value) throws Exception {
		Class<?> clazz = obj.getClass();
		PropertyDescriptor pd = new PropertyDescriptor(field, clazz);
		Method getMethod = pd.getWriteMethod();//获得get方法
		if (pd != null) {
			getMethod.invoke(obj,value);//执行get方法返回一个Object
		}
	}
}
