package com.loit.modules.sys.web;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.api.dto.ContextConfigurationDTO;
import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.modules.sys.entity.ContextConfiguration;
import com.loit.modules.sys.service.ContextConfigurationService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @description 上下文配置Controller
 * @author xutao
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/contextConfig")
@Api(value = "ContextConfigurationController", tags = {"上下文配置Controller"})
public class ContextConfigurationController extends BaseController {

	@Autowired
	private ContextConfigurationService contextConfigurationService;    
	/**
     * 获取项目名称
     */
    @Value("${spring.application.name:}")
    private String springApplicationName;

	/**
	 * @description 上下文配置列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @param contextConfiguration 上下文配置实体类	
	 * @return
	 */
	@GetMapping(value = "list")
	@ApiOperation(value="列表", notes="获取上下文配置列表")	
	public AjaxJson list(@ApiIgnore ContextConfiguration contextConfiguration) {
		List<ContextConfiguration> list = contextConfigurationService.findAllList(contextConfiguration);
		//logger.debug("获取上下文配置列表信息成功");
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}
	
	
	/**
	 * @description 上下文配置分页列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @param contextConfiguration 上下文配置实体类
	 * @param request
	 * @param response
	 * @return
	 */
	@GetMapping(value = "page")
	@ApiOperation(value="分页列表", notes="获取上下文配置分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "contextValue", value = "上下文值", required = false, dataType = "String",
				paramType = "query"),
		@ApiImplicitParam(name = "contextDesc", value = "描述", required = false, dataType = "String",
				paramType = "query")
	})
	public AjaxJson page(@ApiIgnore ContextConfiguration contextConfiguration, HttpServletRequest request, HttpServletResponse response) {
		Page<ContextConfiguration> page = contextConfigurationService.findPage(new Page<ContextConfiguration>(request, response), contextConfiguration);
		//logger.debug("获取上下文配置分页信息成功");
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}


	/**
	 * @description 上下文配置详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param contextId
	 * @return
	 */
	@GetMapping(value = "{contextId}")
	@ApiOperation(value="详情", notes="根据contextId获取上下文配置详情")
	@ApiImplicitParam(name = "contextId", value = "上下文配置ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String contextId) {	
		if(StringUtils.isBlank(contextId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		ContextConfiguration contextConfiguration = contextConfigurationService.get(contextId);
		//logger.debug("获取上下文配置详情成功");
		AjaxJson json = new AjaxJson();
		json.put("obj", contextConfiguration);
		return json;
	}
	/**
	 * @description 根据服务ID获取上下文配置详情
	 * @author wbh
	 * @date：2020年07月30日
	 * @param serviceId
	 * @return
	 */
	@GetMapping(value = "{serviceId}")
	@ApiOperation(value="详情", notes="根据服务ID获取上下文配置")
	@ApiImplicitParam(name = "serviceId", value = "服务ID", required = true, dataType = "String")
	public AjaxJson<ContextConfiguration> getByserviceId(@PathVariable(required=false) String serviceId) {	
		if(StringUtils.isBlank(serviceId)) {
			return AjaxJson.returnExceptionInfo("服务ID为空");
		}
		ContextConfiguration contextConfiguration = contextConfigurationService.getByserviceId(serviceId);
		//logger.debug("获取上下文配置详情成功");
		AjaxJson<ContextConfiguration> json = new AjaxJson<>();
		json.put("obj", contextConfiguration);
		return json;
	}
	/**
	 * @description 获取门户上下文配置详情
	 * @author wbh
	 * @date：2020年07月30日
	 * @param serviceId
	 * @return
	 */
	@GetMapping()
	@ApiOperation(value="详情", notes="获取门户上下文配置")
	public AjaxJson<ContextConfiguration> getPortalContextConf() {
		logger.debug("springApplicationName: "+springApplicationName);
		ContextConfiguration contextConfiguration = contextConfigurationService.getByserviceId(springApplicationName);
		//logger.debug("获取上下文配置详情成功");
		AjaxJson<ContextConfiguration> json = new AjaxJson<>();
		json.put("obj", contextConfiguration);
		return json;
	}
	/**
	 * @description 新增上下文配置
	 * @author xutao
	 * @date：2019年10月18日
	 * @param contextConfigurationDTO
	 * @return
	 */
	@PostMapping(value = "save")
	@ApiOperation(value = "保存", notes = "新增上下文配置")
	@ApiImplicitParam(name = "contextConfigurationDTO", value = "上下文配置数据传输实体类", required = true, dataType = "ContextConfigurationDTO")
	public AjaxJson save(@RequestBody ContextConfigurationDTO contextConfigurationDTO) {
		AjaxJson json = new AjaxJson();
		ContextConfiguration contextConfiguration = ModelMapperUtils.map(contextConfigurationDTO, ContextConfiguration.class);		
		boolean uniqueFlag = contextConfigurationService.checkDescUnique(contextConfiguration.getContextId(), contextConfiguration.getContextDesc());
		if(uniqueFlag == false){
			return AjaxJson.returnExceptionInfo(LoitStatusCode.CONTEXT_IS_EXIST);
		}else {
			try {
				contextConfigurationService.insert(contextConfiguration);
				json.setMsg("保存成功");
				return json;
			} catch (Exception e) {
				logger.error("info:上下文配置保存失败,class:{},message:{}",this.getClass(),e.getMessage());
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
			}
		}	
		
	}
	

	/**
	 * @description 修改上下文配置
	 * @param contextId
	 * @param contextConfigurationDTO
	 * @return
	 */
	@PostMapping(value = "{contextId}")
	@ApiOperation(value = "修改", notes = "修改上下文配置")
	public AjaxJson update(@PathVariable int contextId,@RequestBody ContextConfigurationDTO contextConfigurationDTO) {
		AjaxJson json = new AjaxJson();
		ContextConfiguration configuration = ModelMapperUtils.map(contextConfigurationDTO, ContextConfiguration.class);
		try {
			configuration.setContextId(contextId);
			contextConfigurationService.update(configuration);
			json.setMsg("修改成功");
			return json;
		} catch (Exception e) {
			logger.error("info:上下文配置修改失败,class:{},message:{}",this.getClass(),e.getMessage());			
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.UPDATE_EXCEPTION);
		}		
	}

	/**
	 * @description 上下文配置删除
	 * @author xutao
	 * @date：2019年10月18日
	 * @param ids 多ID时，以逗号分隔
	 * @return
	 */
	@ApiOperation(value = "删除", notes = "单个或批量删除上下文配置，多ID时，以逗号分隔")
	@DeleteMapping(value = "{contextId}")
	public AjaxJson deleteAll(@PathVariable("contextId")String ids) {
		if(StringUtils.isBlank(ids)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		AjaxJson json = new AjaxJson();
		try {
			ContextConfiguration contextConfiguration = new ContextConfiguration();
			contextConfigurationService.deleteAll(ids,contextConfiguration);
			json.setMsg("删除成功");
			return json;
		} catch (Exception e) {
			logger.error("info:删除异常,class:{},message:{}",this.getClass(),e.getMessage());
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.DELETE_EXCEPTION);			
		}
	}
	
	/**
	 * @description 检查上下文描述是否重复
	 * @param contextId
	 * @param contextDesc
	 * @return
	 */
	@ApiOperation(value = "检查上下文描述是否重复", notes = "检查上下文描述是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "contextId", value = "上下文ID", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "contextDesc", value = "上下文描述", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkDescUnique")
	public AjaxJson checkDictNameUnique(Integer contextId,String contextDesc) {
		boolean uniqueFlag = contextConfigurationService.checkDescUnique(contextId, contextDesc);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("上下文描述不重复");
		} else {
			json.setMsg("上下文描述重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}
}
