package com.loit.v1.api.controller.casserver;

import com.loit.v1.modules.entity.ResourceManage;
import com.loit.v1.modules.service.ResourceManageService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;

/**
 * 资源管理[对外接口]
 */
@RestController
@RequestMapping(value = "api/v1/cas/resource")
@Api(tags = "资源管理", value = "ResourceManageApiController")
public class ResourceManageApiController {

    @Autowired
    private ResourceManageService resourceManageService;

    /**
     * 根据应用获取资源列表
     */
    @RequestMapping(value = "list", method = RequestMethod.GET)
    public List<ResourceManage> resourceList(Integer appId) {
        long one = System.currentTimeMillis(); //获取开始时间

        // 参数类型转换
        if (appId == null) {
            return new ArrayList<>();
        }

        ResourceManage query = new ResourceManage();
        query.setAppId(appId);
        List<ResourceManage> resourceManages = resourceManageService.findAllList(query);

        long two = System.currentTimeMillis();
        System.out.println("Resource $$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$1111: " + (two - one));
        return resourceManages;
    }

    /**
     * 根据应用获取用户权限对应的资源列表
     */
    @RequestMapping(value = "role/list", method = RequestMethod.GET)
    public List<ResourceManage> resourceRoleList(String userId, Integer appId) {
        // 参数类型转换
        if (appId == null) {
            return new ArrayList<>();
        }

        List<ResourceManage> resourceManages = resourceManageService.getResourceManagesByUserId(userId, appId);
        return resourceManages;
    }
}
