package com.loit.v1.api.controller.feign;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.api.dto.ApplicationManageDTO;
import com.loit.v1.modules.entity.AppResourceRelation;
import com.loit.v1.modules.entity.ApplicationManage;
import com.loit.v1.modules.entity.MenuManage;
import com.loit.v1.modules.entity.RoleResMenu;
import com.loit.v1.modules.service.ApplicationManageService;
import com.loit.v1.modules.service.ResourceManageService;
import com.loit.v1.modules.service.RightShortcutService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description 应用管理 - 开发临时使用没有loit-portal-api的service
 * @author xutao
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/feign/applicationManage")
@Api(value = "FeignApplicationManageController", tags = {"Feign 应用注册管理"})
public class FeignApplicationManageController extends BaseController {

	@Autowired
	private ApplicationManageService applicationManageService;
	@Autowired
	private RightShortcutService rightShortcutService;

	/**
	 * @description 获取当前用户所拥有的应用列表及菜单树形列表及当前应用的主题，皮肤，菜单布局方式等
	 * @author xutao
	 * @date 2019年10月18日
	 * @param roleResMenu 用户角色资源菜单应用关系对象	
	 * @return
	 */
	@GetMapping(value = "getCurrentUserAppMenu")
	@ApiOperation(value="获取当前用户所拥有的应用列表及菜单树形列表及当前应用的主题，皮肤，菜单布局方式等", notes="获取当前用户所拥有的应用列表及菜单树形列表及当前应用的主题，皮肤，菜单布局方式等")	
	@ApiImplicitParams({
			@ApiImplicitParam(name = "accountId", value = "用户账号ID", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "accountType", value = "账号类型 0:普通帐号 1:应用管理员账号2:应用管理帐号 3:超级管理帐号", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "appId", value = "应用ID", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "appCode", value = "应用编码", required = false, dataType = "String", paramType = "query")
	})
	public AjaxJson getCurrentUserAppMenu(@ApiIgnore RoleResMenu roleResMenu) {
		//获取当前用户
		User user = UserUtils.getUser();

		// appId和appCode不能同时存在
		if(StringUtils.isNotEmpty(roleResMenu.getAppCode()) && roleResMenu.getAppId() != null){
			return AjaxJson.returnExceptionInfo(LoitStatusCode.APPLICATION_APP_ID_CODE_SIMULTANEOUSLYEXIST_ERRO);
		}

		// 无用户信息
		if(user == null || user.getAccountId() == null) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.CURRENT_USER_ERROR);
		}

		return applicationManageService.getCurrentUserAppMenu(roleResMenu, user);
	}

	/**
	 * 获得拼装的URL信息
	 * @return
	 */
	@GetMapping(value = "getConfUrl")
	@ApiOperation(value="获得拼装的URL信息", notes="获得拼装的URL信息")
	public AjaxJson getConfUrl(){
		return applicationManageService.getConfUrl();
	}

}
