package com.loit.v1.api.controller.feign;

import com.loit.common.json.AjaxJson;
import com.loit.v1.modules.entity.CommonEntrance;
import com.loit.v1.modules.service.CommonEntranceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @description: 常用入口管理
 * @date: 2020/6/19 18:42
 * @author: wubj
 * @version v1.0
 */
@RestController
@RequestMapping(value = "api/v1/feign/commonEntrance")
@Api(tags = "Feign 常用入口管理",  value = "FeignCommonEntranceController")
public class FeignCommonEntranceController {

    /**
     * 日志工具
     */
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private CommonEntranceService commonEntranceService;

    /**
     * 根据用户登录账号获取常用入口列表
     */
    @ApiOperation(value = "根据用户登录账号获取常用入口列表", notes = "根据用户登录账号获取常用入口列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "用户登录账号", required = false, dataType = "String", paramType = "query")
    })
    @GetMapping(value = "/getListByUserId")
    public AjaxJson getListByUserId(@RequestParam(value = "userId", required = false) String userId){
        AjaxJson json = new AjaxJson();
        List<CommonEntrance> list = commonEntranceService.getListByUserId(userId);
        json.put("list", list);
        return json;
    }
}
