package com.loit.v1.api.controller.feign;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Dict;
import com.loit.modules.sys.service.DictService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @description 字典controller
 * @author: gzx
 * @date: 2019年11月4日 上午9:40:29
 * @version: V1.0
 */
@RestController
@RequestMapping(value = "api/v1/feign/dict")
@Api(value = "FeignDictController", tags = {"Feign 系统字典"})
public class FeignDictController {
	protected Logger logger = LoggerFactory.getLogger(FeignDictController.class);
	
	@Autowired
	private DictService dictService;
	
	/**
	 * @description 系统字典分页列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @return
	 */
	@GetMapping(value = "/list")
	@ApiOperation(value="分页列表", notes="获取系统字典分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "dictCode", value = "字典编码", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "dictName", value = "字典名称", required = false, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "baseFlag", value = "1、系统内置 2、用户自定义 ", required = false, dataType = "String", paramType = "query")
	})
	public AjaxJson list(
			@RequestParam(value = "pageNo", required = true) Integer pageNo,
			@RequestParam(value = "pageSize", required = true) Integer pageSize,
			@RequestParam(value = "dictCode", required = false)String dictCode,
			@RequestParam(value = "dictName", required = false)String dictName,
			@RequestParam(value = "baseFlag", required = false)String baseFlag) {
		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = attributes.getRequest();
		HttpServletResponse response = attributes.getResponse();

		Dict dict = new Dict();
		dict.setDictCode(dictCode);
		dict.setDictName(dictName);
		dict.setBaseFlag(baseFlag);

		Page<Dict> page = dictService.findTreePage(new Page<Dict>(request, response), dict);
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}


	/**
	 * @description 系统字典详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @return
	 */
	@GetMapping(value = "{dictId}")
	@ApiOperation(value="详情", notes="根据dictId获取系统字典详情")
	@ApiImplicitParam(name = "dictId", value = "系统字典ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String dictId) {
		if(StringUtils.isBlank(dictId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		Dict dict = dictService.get(dictId);
		AjaxJson json = new AjaxJson();
		json.put("obj", dict);
		return json;
	}

	/**
	 * @description 检查字典名称是否重复
	 * @param dictId
	 * @param dictName
	 * @return
	 */
	@ApiOperation(value = "检查字典名称是否重复", notes = "检查字典名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "dictId", value = "字典ID", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "dictName", value = "字典名称", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkDictNameUnique")
	public AjaxJson checkDictNameUnique(
			@RequestParam(value = "dictId", required = false) Integer dictId,
			@RequestParam(value = "dictName", required = true) String dictName) {
		boolean uniqueFlag = dictService.checkNameUnique(dictId, dictName);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("字典名称不重复");
		} else {
			json.setMsg("字典名称重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}
}
