package com.loit.v1.api.controller.feign;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.client.ServiceInstance;
import org.springframework.cloud.client.discovery.DiscoveryClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;

/**
 * 消息管理
 * @author wbh12
 * @date: 2020年5月8日
 * add by wbh
 */
@RestController
@RequestMapping(value = "api/v1/feign/notice")
@Api(tags = "Feign 消息管理",  value = "FeignNoticeController")
public class FeignNoticeController {


	@Autowired
	private DiscoveryClient discoveryClient;

	/**
	 * 获取服务实例
	 */
	@GetMapping("/serviceurl")
	public Map<String, List<ServiceInstance>> serviceUrl() {
	    Map<String, List<ServiceInstance>> msl = new HashMap<>();
	    List<String> services = discoveryClient.getServices();
	    for (String service : services) {
	        List<ServiceInstance> sis = discoveryClient.getInstances(service);
	        msl.put(service, sis);
	    }
	    return msl;
	}
	
	/**
	 * 获取服务实例
	 */
	@GetMapping("/isOnline/{instanceId}")
	@ApiOperation(value = "根据服务id获取该服务是在线于注册中心", notes = "根据服务id获取该服务是在线于注册中心（success为true则该服务在线，success为false，该服务未在线）")
	@ApiImplicitParam(name = "instanceId", value = "instanceId", required = true, dataType = "String", paramType = "path")
	public AjaxJson isOnline(@PathVariable("instanceId")String instanceId) {
		if(instanceId==null) return AjaxJson.returnExceptionInfo("instanceId为空！");
		Map<String, List<ServiceInstance>> msl = new HashMap<>();
	    List<String> services = discoveryClient.getServices();
	    for (String service : services) {
	        List<ServiceInstance> sis = discoveryClient.getInstances(service);
	        msl.put(service, sis);
	    }
	    List<ServiceInstance> list = msl.get(instanceId);
	    if(list==null) return AjaxJson.returnExceptionInfo("服务'"+instanceId+"'未在线");
		if(list.size()>0) {
			return new AjaxJson();
		}
		return AjaxJson.returnExceptionInfo("服务'"+instanceId+"'未在线");
	}
}
