package com.loit.v1.api.controller.feign;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.utils.ListUtil;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.api.controller.handler.OrgListHandler;
import com.loit.v1.api.dto.OrgDTO;
import com.loit.v1.api.dto.OrgExtattrDTO;
import com.loit.v1.modules.activemq.MessageModel;
import com.loit.v1.modules.activemq.Producer;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.entity.ApporgShortcut;
import com.loit.v1.modules.entity.Label;
import com.loit.v1.modules.entity.Org;
import com.loit.v1.modules.service.ApporgShortcutService;
import com.loit.v1.modules.service.LabelService;
import com.loit.v1.modules.service.OrgExtattrService;
import com.loit.v1.modules.service.OrgService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.activemq.command.ActiveMQQueue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import springfox.documentation.annotations.ApiIgnore;

import javax.jms.Destination;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**  
* @ClassName: OrgController  
* @Description:   
* @author zhaokz  
* @date 2019年10月22日 下午5:41:41 
* @version V1.0  
*/
@RestController
@RequestMapping(value = "api/v1/feign/org")
@Api(tags = "Feign 组织管理",  value = "FeignOrgController")
public class FeignOrgController {
	
	private static Logger logger = LoggerFactory.getLogger(FeignOrgController.class);

	@Autowired
	private OrgService orgService;
	@Autowired
	private OrgExtattrService orgExtattrService;
	@Autowired
	private LabelService labelService;

	/**
	 * 获取组织树形列表
	 * @return
	 */
	@ApiOperation(value = "获取组织树形列表", notes = "获取组织树形列表")
	@GetMapping(value = "/treeList")
	public AjaxJson treeList(){
		Org org = new Org();
		org.setDeltag(Org.DEL_FLAG_NORMAL);
		List<OrgDTO> treeList = orgService.findOrgTreeList(org);
		AjaxJson json = new AjaxJson();
		json.put("treeList", treeList);
		return json;
	}
	
	/**
	 * 获取当前应用组织树形列表
	 * @return
	 */
	@ApiOperation(value = "获取当前应用组织树形列表", notes = "获取当前应用组织树形列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用id", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/currentApp/treeList1")
	public AjaxJson treeList1(
			@RequestParam(value = "appId", required = true) Integer appId){
		AjaxJson json = new AjaxJson();
		Org org = new Org();
		org.setAppId(Integer.valueOf(appId));
		org.setDeltag(Org.DEL_FLAG_NORMAL);
		User user = UserUtils.getUser();
		//判断是否组织用户
		if(BusinessConstants.ORG_USER.equals(user.getUserType())) {
			//查询所属及下属组织
			org.setOrgId(user.getOrgId());
			List<OrgDTO> treeList = orgService.getOwnOrgTree(org);
			json.put("treeList", treeList);
			return json;
		}
		if(BusinessConstants.ACCOUNT_TYPE_SUPERADMIN.equals(user.getAccountType())) {
			List<OrgDTO> treeList = orgService.findOrgTreeList(org);
			json.put("treeList", treeList);
			return json;
		}
		List<OrgDTO> treeList = orgService.findCuAppOrgTree(org);
		json.put("treeList", treeList);
		return json;
	}
	
	/**
	 * @description 根据组织编码获取当前应用组织树形列表（带用户）
	 * @return
	 */
	@ApiOperation(value = "根据组织编码获取当前应用组织树形列表（带用户）", notes = "")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用id", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "orgCode", value = "组织编码", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/currentApp/treeList2")
	public AjaxJson treeList2(
			@RequestParam(value = "appId", required = true) String appId,
			@RequestParam(value = "orgCode", required = false) String orgCode){
		AjaxJson json = new AjaxJson();
		Org org = new Org();
		org.setAppId(Integer.valueOf(appId));
		org.setOrgCode(orgCode);
		org.setDeltag(Org.DEL_FLAG_NORMAL);
		List<OrgDTO> treeList = orgService.findOrgTreeByOrgCode(org);
		json.put("treeList", treeList);
		return json;
	}
	
	/**
	 * @description 根据组织编码获取当前应用组织树形列表
	 * @return
	 */
	@ApiOperation(value = "根据组织编码获取当前应用组织树形列表,orgId为空时则查询该用户下的组织树", notes = "")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用id", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "orgId", value = "组织id", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/currentApp/treeListByOrgId")
	public AjaxJson treeListByOrgId(
			@RequestParam(value = "appId", required = true) String appId,
			@RequestParam(value = "orgId", required = true) Integer orgId){
		AjaxJson json = new AjaxJson();
		Org org = new Org();
		org.setAppId(Integer.valueOf(appId));
		if(orgId==null||orgId==0) {
			User user = UserUtils.getUser();
			org.setOrgId(user.getOrgId());
		}else {
			org.setOrgId(orgId);
		}
		org.setDeltag(Org.DEL_FLAG_NORMAL);
		List<OrgDTO> treeList = orgService.getOwnOrgTree(org);
		json.put("treeList", treeList);
		return json;
	}
	
	/**
	 * 获取组织分页列表
	 * @return
	 */
	@ApiOperation(value = "获取组织分页列表", notes = "获取组织分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "appId", value = "应用id", required = false, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "orgId", value = "组织ID", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "orgName", value = "组织名称", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "orgCode", value = "组织编码", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "deltag", value = "删除标记", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "queryLabelName", value = "标签名称", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/pageList")
	public AjaxJson list(
			@RequestParam(value = "pageNo", required = true) Integer pageNo,
			@RequestParam(value = "pageSize", required = true) Integer pageSize,
			@RequestParam(value = "appId", required = false) Integer appId,
			@RequestParam(value = "orgId", required = false) Integer orgId,
			@RequestParam(value = "orgName", required = false) String orgName,
			@RequestParam(value = "orgCode", required = false) String orgCode,
			@RequestParam(value = "deltag", required = true) String deltag,
            @RequestParam(value = "queryLabelName", required = false) String queryLabelName){
		return _orgList(appId, orgId, orgName, orgCode, deltag, queryLabelName, (Org org) -> {
			//判断是否组织用户
			if(org.getOrgId() == null) {
				User user = UserUtils.getUser();
				if(BusinessConstants.ORG_USER.equals(user.getUserType())) {
					org.setOrgId(user.getOrgId());
				}
			}
		});
	}

	/**
	 * 获取组织分页列表-无登录用户
	 * @return
	 */
	@ApiOperation(value = "获取组织分页列表-无登录用户", notes = "获取组织分页列表-无登录用户")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "appId", value = "应用id", required = false, dataType = "String", paramType = "query") ,
			@ApiImplicitParam(name = "orgId", value = "组织ID", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "orgName", value = "组织名称", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "orgCode", value = "组织编码", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "deltag", value = "删除标记", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "queryLabelName", value = "标签名称", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/nouser/pageList")
	public AjaxJson noUserList(
			@RequestParam(value = "pageNo", required = true) Integer pageNo,
			@RequestParam(value = "pageSize", required = true) Integer pageSize,
			@RequestParam(value = "appId", required = false) Integer appId,
			@RequestParam(value = "orgId", required = false) Integer orgId,
			@RequestParam(value = "orgName", required = false) String orgName,
			@RequestParam(value = "orgCode", required = false) String orgCode,
			@RequestParam(value = "deltag", required = true) String deltag,
			@RequestParam(value = "queryLabelName", required = false) String queryLabelName){
		return _orgList(appId, orgId, orgName, orgCode, deltag, queryLabelName, (Org org) -> {});
	}

	/**
	 * 获取组织分页列表的内部方法，外部有不同的实现调用
	 */
	private AjaxJson _orgList(
			Integer appId,
			Integer orgId,
			String orgName,
			String orgCode,
			String deltag,
			String queryLabelName,
			OrgListHandler orgListHandler){
		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = attributes.getRequest();
		HttpServletResponse response = attributes.getResponse();

		Org org = new Org();
		org.setAppId(appId);
		org.setOrgId(orgId);
		org.setOrgName(orgName);
		org.setOrgCode(orgCode);
		org.setDeltag(deltag);
		org.setQueryLabelName(queryLabelName);

		// 外部调用时候又其他逻辑处理
		orgListHandler.invok(org);

		Page<Org> pageList = orgService.findPage(new Page<Org>(request, response),org);

		// 获取组织标签并放入组织对象中
		Label label = new Label();
		label.setLabelDictCode("orgLabel");
		List<Label> labels = labelService.findList(label);
		if(ListUtil.isNotEmpty(pageList.getList()) && ListUtil.isNotEmpty(labels)){
			for (Org _org : pageList.getList()) {
				_org.setLabels(labels.stream().filter(e -> e.getLabelModuleId().equals(_org.getOrgId())).collect(Collectors.toList()));
			}
		}

		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
	}
	
	/**
	 * 获取组织详细信息
	 * @param orgId
	 * @return
	 */
	@ApiOperation(value = "获取组织详细信息", notes = "根据id来获取Org详细信息")
	@ApiImplicitParam(name = "orgId", value = "orgId", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/{orgId}")
	public AjaxJson detail(@PathVariable String orgId) {
		AjaxJson json = new AjaxJson();
		Org newOrg = new Org();
		newOrg.setOrgId(Integer.valueOf(orgId));
		Org org = orgService.get(newOrg);
		if(org != null) {
			OrgDTO orgDTO = ModelMapperUtils.map(org, OrgDTO.class);
//			OrgExtattr orgExtattr = new OrgExtattr();
//			orgExtattr.setOrgId(orgDTO.getOrgId());
//			List<OrgExtattr> orgExtattrs = orgExtattrService.findExtattrList(orgExtattr);
//			orgDTO.setOrgExtattrs(orgExtattrs);
			OrgExtattrDTO orgExtattr = new OrgExtattrDTO();
			orgExtattr.setOrgId(orgDTO.getOrgId());
			List<OrgExtattrDTO> orgExtattrs = orgExtattrService.findExtattrListDTO(orgExtattr);
			orgDTO.setOrgExtattrs(orgExtattrs);

			// 获取组织标签并放入组织对象中
			Label label = new Label();
			label.setLabelDictCode("orgLabel");
			label.setLabelModuleId(Integer.valueOf(orgId));
			List<Label> labels = labelService.findList(label);
			orgDTO.setLabels(labels);

			json.put("org", orgDTO);
		}
		return json;
	}

	/**
	 * 验证组织唯一
	 * @param
	 * @return
	 */
	@ApiOperation(value = "组织唯一验证", notes = "组织唯一验证")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "orgName", value = "组织名称", required = false, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "orgShortName", value = "组织简称", required = false, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "parentId", value = "父组织id", required = true, dataType = "String",paramType = "query"),
	})
	@PostMapping(value = "/validateOrgName")
	public AjaxJson validateOrgName(@RequestBody Org org) {
		AjaxJson json = new AjaxJson();
		boolean result = orgService.validateOrgName(org);
		if(result) {
			json.setMsg("该组织名称可用!");
			json.setSuccess(true);
		}else {
			json.setSuccess(false);
			return AjaxJson.returnExceptionInfo(LoitStatusCode.ORG_NAME_IS_EXIST);
		}
		return json;
	}

}
