package com.loit.v1.api.controller.feign;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Role;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.modules.activemq.Producer;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.entity.AccountPolicy;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.entity.PersonBase;
import com.loit.v1.modules.service.AccountPolicyService;
import com.loit.v1.modules.service.ApporgShortcutService;
import com.loit.v1.modules.service.PersonBaseService;
import com.loit.v1.modules.service.RoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
* @ClassName: PersonBaseController  
* @Description:   人员基本信息管理
* @author zhaokz  
* @date 2019年10月24日 下午3:23:26 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/feign/person")
@Api(tags = "Feign 人员管理",  value = "FeignPersonController")
public class FeignPersonController {
	@Autowired
	private PersonBaseService personBaseService;
	@Autowired
	private AccountPolicyService accountPolicyService;
	@Autowired
	private RoleService roleService;

	/**
	 * 获取人员信息列表
	 * @return
	 */
	@ApiOperation(value = "获取人员信息列表", notes = "获取人员信息列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "delFlag", value = "是否删除", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "appId", value = "应用Id", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "personName", value = "人员名称", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "userId", value = "用户id", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/pageList")
	public AjaxJson list(
			@RequestParam(value="pageNo", required = true) Integer pageNo,
			@RequestParam(value="pageSize", required = true) Integer pageSize,
			@RequestParam(value="delFlag", required = true) String delFlag,
			@RequestParam(value="appId", required = false) Integer appId,
			@RequestParam(value="personName", required = false) String personName,
			@RequestParam(value="userId", required = false) String userId){
		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = attributes.getRequest();
		HttpServletResponse response = attributes.getResponse();

		PersonBase personBase = new PersonBase();
		personBase.setDelFlag(delFlag);
		personBase.setAppId(appId);
		personBase.setPersonName(personName);
		personBase.setUserId(userId);

		AjaxJson json = new AjaxJson();
		User user = UserUtils.getUser();
		if(null == user){
			return AjaxJson.returnExceptionInfo("用户id不能为空", json);
		}

		if(BusinessConstants.ORG_USER.equals(user.getUserType())) {
			personBase.setOrgId(user.getOrgId());
		}
		if(BusinessConstants.ACCOUNT_TYPE_SUPERADMIN.equals(user.getAccountType())) {
			personBase.setAppId(null);
		}
		Page<PersonBase> pageList = personBaseService.findPage(new Page<PersonBase>(request, response),personBase);
		json.put("page", pageList);
		return json;
	}
	
	/**
	 * 根据组织id获取人员信息列表
	 * @return
	 */
	@ApiOperation(value = "根据组织id获取人员信息列表", notes = "根据组织id获取人员信息列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "orgId", value = "组织id", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "delFlag", value = "是否删除", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/orgPersonList")
	public AjaxJson getPersonByOrgId(
			@RequestParam(value = "pageNo", required = true) Integer pageNo,
			@RequestParam(value = "pageSize", required = true) Integer pageSize,
			@RequestParam(value = "delFlag", required = true) String delFlag,
			@RequestParam(value = "orgId", required = true) String orgId){
		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = attributes.getRequest();
		HttpServletResponse response = attributes.getResponse();
		PersonBase personBase = new PersonBase();
		personBase.setDelFlag(delFlag);
		Page<PersonBase> pageList = personBaseService.findPageByOrgId(new Page<PersonBase>(request, response),personBase,orgId);
		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
	}
	
	/**
     * @description 根据用户id获取该用户所在组织下的人员列表
     * @return
     */
    @ApiOperation(value = "根据组织id获取人员信息列表", notes = "根据组织id获取人员信息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "accountId", value = "用户id", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "delFlag", value = "是否删除", required = true, dataType = "String", paramType = "query")
    })
    @GetMapping(value = "api/v1/feign/person/ownOrgPersonList")
    public AjaxJson getListByUserId(
    		 @RequestParam(value = "pageNo", required = true) Integer pageNo,
             @RequestParam(value = "pageSize", required = true) Integer pageSize,
             @RequestParam(value = "accountId", required = true) String accountId,
             @RequestParam(value = "delFlag", required = true) String delFlag)
    {
    	ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
    	HttpServletRequest request = attributes.getRequest();
		HttpServletResponse response = attributes.getResponse();
		PersonBase personBase = new PersonBase();
		personBase.setAccountId(Integer.valueOf(accountId));
		Page<PersonBase> pageList = personBaseService.findPageByAccountId(new Page<PersonBase>(request, response),personBase);
		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
    }

	/**
	 * 获取人员详细信息
	 * @param accountId
	 * @return
	 */
	@ApiOperation(value = "获取人员详细信息", notes = "根据id来获取人员详细信息")
	@ApiImplicitParam(name = "accountId", value = "id", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/{accountId}")
	public AjaxJson detail(@PathVariable String accountId) {
		AjaxJson json = new AjaxJson();
		PersonBaseDTO personBaseDTO = personBaseService.getDetailInfo(accountId);
		//密码策略
		AccountPolicy accountPolicy = accountPolicyService.getOne();
		json.put("accountPolicy", accountPolicy);
		json.put("accout", personBaseDTO);
		return json;
	}

	/**
	 * 获取密码策略
	 * @param
	 * @return
	 */
	@ApiOperation(value = "获取密码策略", notes = "获取密码策略")
	@GetMapping(value = "/passwordApolicy")
	public AjaxJson passwordApolicy() {
		AjaxJson json = new AjaxJson();
		//密码策略
		AccountPolicy accountPolicy = accountPolicyService.getOne();
		json.put("accountPolicy", accountPolicy);
		return json;
	}
	
	/**
	 * 验证账号唯一
	 * @param
	 * @return
	 */
	@ApiOperation(value = "账号唯一验证", notes = "账号唯一验证")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "userId", value = "账号", required = true, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "accountId", value = "账号id", required = false, dataType = "Integer",paramType = "query")
	})
	@GetMapping(value = "/validateUserId")
	public AjaxJson validateUserId(
			@RequestParam(value = "userId", required = true) String userId,
			@RequestParam(value = "accountId", required = false) Integer accountId) {
		AjaxJson json = new AjaxJson();

		AccoutInfo accoutInfo = new AccoutInfo();
		accoutInfo.setUserId(userId);
		accoutInfo.setAccountId(accountId);

		boolean result = personBaseService.validateUserId(accoutInfo);
		if(result) {
			json.setMsg("该账号可用!");
			json.setSuccess(true);
		}else {
			json.setSuccess(false);
			return AjaxJson.returnExceptionInfo(LoitStatusCode.USER_ACCOUNT_EXISTED);
		}
		return json;
	}
	
	/**
	 * 验证人员名称唯一
	 * @param
	 * @return
	 */
	@ApiOperation(value = "人员名称唯一验证", notes = "人员名称唯一验证")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "personName", value = "人员名称", required = true, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "personBaseId", value = "人员基本信息id", required = false, dataType = "Integer",paramType = "query")
	})
	@GetMapping(value = "/validatePersonName")
	public AjaxJson validatePersonName(
			@RequestParam(value = "personName", required = true) String personName,
			@RequestParam(value = "personBaseId", required = false) Integer personBaseId) {
		AjaxJson json = new AjaxJson();

		PersonBase personBase = new PersonBase();
		personBase.setPersonName(personName);
		personBase.setPersonBaseId(personBaseId);

		boolean result = personBaseService.validatePersonName(personBase);
		if(result) {
			json.setMsg("该名称可用!");
			json.setSuccess(true);
		}else {
			json.setSuccess(false);
			return AjaxJson.returnExceptionInfo(LoitStatusCode.USER_NAME_EXISTED);
		}
		return json;
	}

	/**
	 * 密码策略校验
	 * @param
	 * @return
	 */
	@ApiOperation(value = "密码策略校验", notes = "密码策略校验")
	@ApiImplicitParam(name = "passWord", value = "密码", required = true, dataType = "String",paramType = "query")
	@PostMapping(value = "/validatePassword")
	public AjaxJson validatePassword(@RequestParam(value = "passWord", required = true) String passWord) {
		AjaxJson json = accountPolicyService.pwdPolicyValidate(passWord);
		return json;
	}
	
	/**
	 * 获取人员详细信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "获取当前登录用户", notes = "获取当前登录用户")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userId", value = "用户登录账号", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/getCurretUser")
	public AjaxJson getCurretUser(@RequestParam(value = "userId", required = false) String userId){
		AjaxJson json = new AjaxJson();

		// 如果userId为空，则走之前的逻辑，从cookie中获取userId或令牌换取userId
		if(StringUtils.isNotEmpty(userId)){
			json.put("accout", UserUtils.get(userId));
		}else{
			json.put("accout", UserUtils.getUser());
		}
		return json;
	}

	/**
	 * 获取所有的账号信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "获取所有的账号信息", notes = "获取所有的账号信息")
	@GetMapping(value = "/getAccoutInfoList")
	public AjaxJson getAccoutInfoList() {
		AjaxJson json = new AjaxJson();
		List<AccoutInfo> list = personBaseService.getAccoutInfoList();
		json.put("list", list);
		return json;
	}

	/**
	 * 根据登录账号列表获取账号信息列表
	 * @param
	 * @return
	 */
	@ApiOperation(value = "根据登录账号列表获取账号信息列表", notes = "根据登录账号列表获取账号信息列表")
	@GetMapping(value = "/getAccountByUserIds")
	public AjaxJson getAccountByUserIds(@RequestParam List<String> userIds) {
		AjaxJson json = new AjaxJson();
		List<PersonBaseDTO> list = personBaseService.getAccountByUserIds(userIds);
		json.put("list", list);
		return json;
	}

	/**
	 * @description 根据角色获取人员分页列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @return
	 */
	@GetMapping(value = "findPersonPageByRole")
	@ApiOperation(value="根据角色获取人员分页列表", notes="根据角色获取人员分页列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "roleId", value = "角色ID（和角色编码2选1）", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "roleCode", value = "角色编码（和角色ID2选1）", required = false, dataType = "String", paramType = "query")
	})
	public AjaxJson findPersonPageByRole(
			@RequestParam(value = "pageNo", required = true) Integer pageNo,
			@RequestParam(value = "pageSize", required = true) Integer pageSize,
			@RequestParam(value = "roleId", required = false) Integer roleId,
			@RequestParam(value = "roleCode", required = false) String roleCode) {
		// 角色ID和角色编码2选1,参数必传
		if(roleId == null && StringUtils.isEmpty(roleCode)){
			return AjaxJson.returnExceptionInfo(LoitStatusCode.ROLE_ID_CODE_PARAM_ISNULL);
		}

		// 角色ID和角色编码2选1，不能同时选择
		if(roleId != null && StringUtils.isNotEmpty(roleCode)){
			return AjaxJson.returnExceptionInfo(LoitStatusCode.ROLE_ID_CODE_PARAM_ISNULL);
		}

		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
		HttpServletRequest request = attributes.getRequest();
		HttpServletResponse response = attributes.getResponse();

		Role role = null;

		if(StringUtils.isNotEmpty(roleCode)){
			role = roleService.getRoleByRoleCode(roleCode);
		}

		if(roleId != null){
			role = roleService.get(roleId);
		}

		if(role == null ){
			AjaxJson json = new AjaxJson();
			json.put("page", new Page<>());
			return json;
		}

		Page<PersonBase> page = personBaseService.findPersonPageByRole(new Page<PersonBase>(request, response), new PersonBase().roleId(role.getRoleId()).appId(role.getAppId()));

		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}

	/**
	 * 根据人员名称获取人员信息列表
	 * @param
	 * @return
	 */
	@ApiOperation(value = "根据人员名称获取人员信息列表", notes = "根据人员名称获取人员信息列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "personName", value = "人员名称", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/getPersonByPersonName")
	public AjaxJson getPersonByPersonName(@RequestParam(value = "personName", required = true) String personName){
		AjaxJson json = new AjaxJson();
		json.put("list", personBaseService.getPersonByPersonName(personName));
		return json;
	}
	
}
