package com.loit.v1.api.controller.feign;

import com.loit.common.json.AjaxJson;
import com.loit.common.utils.RedisUtil;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.handler.MyWebSocketHandler;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.task.TaskExecutor;
import org.springframework.web.bind.annotation.*;

import java.util.Map;
import java.util.concurrent.ExecutorCompletionService;

/**
 * @description: websocket推送消息
 * @date: 2020/5/7 18:03
 * @author: wubj
 * @version v1.0
 */
@RestController
@RequestMapping(value = "api/v1/feign/websocket")
@Api(tags = "Feign 消息推送",  value = "FeignWebSocketController")
public class FeignWebSocketController {

    /**
     * 日志工具
     */
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private TaskExecutor taskExecutor;

    /**
     * 发送给指定用户
     * Map<String, String> sendMap（key:userId, value:消息）
     * @return
     */
    @PostMapping(value = "/toUsers")
    @ApiOperation(value = "发送给指定用户")
    public AjaxJson sendMessageToUsers(@RequestBody Map<String, String> sendMap) {
        AjaxJson json = new AjaxJson();
        try {
            long startTime=System.currentTimeMillis();
            ExecutorCompletionService<String> completionService = new ExecutorCompletionService<String>(taskExecutor);
            for (String receiveId : sendMap.keySet()) {
                completionService.submit(() -> {
                    // 服务端发送消息给指定客户
                    MyWebSocketHandler.sendMessageToUsers(receiveId, sendMap.get(receiveId));
                    return receiveId;
                });
            }
            long endTime=System.currentTimeMillis();
            float excTime=(float)(endTime-startTime)/1000;
            logger.info("====================WebSocket推送消息，用时：" + excTime + "秒====================");
            json.setMsg("推送成功");
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxJson.returnExceptionInfo("推送异常");
        }
        return json;
    }

    /**
     * 广播所有在线用户
     * @return
     */
    @PostMapping(value = "/websocketServer")
    @ApiOperation(value = "广播所有在线用户")
    public AjaxJson sendMessageBroadcast(@RequestBody String message){
        AjaxJson json = new AjaxJson();
        try {
            MyWebSocketHandler.sendMessageBroadcast(message);
            json.setMsg("推送成功");
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxJson.returnExceptionInfo("推送异常");
        }
        return json;
    }

    /**
     * 获取用户连接的websocket服务器信息
     * @return
     */
    @GetMapping(value = "/websocketServer")
    @ApiOperation(value = "获取用户连接的websocket服务器信息")
    public AjaxJson getWebsocketServer(){
        AjaxJson json = new AjaxJson();
        String serverInfo = (String) RedisUtil.get(BusinessConstants.WEBSOCKET_CATCH_KEY);
        json.put("serverInfo", serverInfo);
        return json;
    }
}
