package com.loit.v1.api.controller.open;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.modules.entity.*;
import com.loit.v1.modules.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * @description 应用Controller
 * @author DEMON
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/open/applicationManage")
@Api(value = "OpenApplicationManageController", tags = {"Open 应用管理"})
public class OpenApplicationManageController extends BaseController {
	
	@Autowired
	private ApplicationManageService applicationManageService;
	@Autowired
	private RightShortcutService rightShortcutService;
	@Autowired
	private ThemeManageService themeManageService;
	@Autowired
	private MenuManageService menuManageService;

	@GetMapping(value = "applications")
	@ApiOperation(value="获取当前用户所拥有的应用列表", notes="获取当前用户所拥有的应用列表")
	public AjaxJson applications() {
		//获取当前用户
		User user = UserUtils.getUser();
		if(user == null || user.getAccountId() == null) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.CURRENT_USER_ERROR);
		}

		AjaxJson json = new AjaxJson();
		json.put("list", applications(user));
		return json;
	}

	@GetMapping(value = "application")
	@ApiOperation(value="根据应用id获取应用信息", notes="根据应用id获取应用信息, 如果不传应用id则获取当前用户的第一个应用的信息")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "appId", value = "应用ID", required = false, dataType = "Integer", paramType = "query"),
			@ApiImplicitParam(name = "appCode", value = "应用编码", required = false, dataType = "String", paramType = "query")
	})
	public AjaxJson application(
			@RequestParam(value = "appId", required = false) Integer appId,
			@RequestParam(value = "appCode", required = false) String appCode) {
		// appId和appCode不能同时存在
		if(StringUtils.isNotEmpty(appCode) && appId != null){
			return AjaxJson.returnExceptionInfo(LoitStatusCode.APPLICATION_APP_ID_CODE_SIMULTANEOUSLYEXIST_ERRO);
		}

		Integer defaultAppId = applicationManageService.defaultAppId(appCode, appId);
		ApplicationManage application = applicationManageService.get(defaultAppId);

		AjaxJson json = new AjaxJson();
		json.put("application", application);
		return json;
	}

	@GetMapping(value = "application/theme")
	@ApiOperation(value="获取当前用户的系统主题", notes="获取当前用户的系统主题")
	public AjaxJson applicationTheme() {
		AjaxJson json = new AjaxJson();
		//获取当前用户
		User user = UserUtils.getUser();
		if(user == null || user.getAccountId() == null) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.CURRENT_USER_ERROR);
		}

		List<Map<String,Object>> list = applications(user);
		Integer appId = (Integer) list.get(0).get("appId");

		ApplicationManage application = applicationManageService.get(appId);
		if(null != application && null != application.getThemeManage()){
			json.put("themeManage",  application.getThemeManage());
		}

		return json;
	}

	@GetMapping(value = "application/menus")
	@ApiOperation(value="根据应用id获取当前用户所拥有的菜单列表", notes="根据应用id获取当前用户所拥有的菜单列表, 如果不传应用id则获取当前用户的第一个应用")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "appId", value = "应用ID", required = false, dataType = "Integer", paramType = "query"),
			@ApiImplicitParam(name = "appCode", value = "应用编码", required = false, dataType = "String", paramType = "query")
	})
	public AjaxJson applicationMenus(
			@RequestParam(value = "appId", required = false) Integer appId,
			@RequestParam(value = "appCode", required = false) String appCode) {
		// appId和appCode不能同时存在
		if(StringUtils.isNotEmpty(appCode) && appId != null){
			return AjaxJson.returnExceptionInfo(LoitStatusCode.APPLICATION_APP_ID_CODE_SIMULTANEOUSLYEXIST_ERRO);
		}

		// 根据多种情况获取最终的appId，并查询菜单
		Integer defaultAppId = applicationManageService.defaultAppId(appCode, appId);
		List<MenuManage> menus = applicationManageService.getMenuListForUser(UserUtils.getUser(), defaultAppId);

		AjaxJson json = new AjaxJson();
		json.put("menus", menus);
		return json;
	}

	/**
	 * 公共方法-获取用户下的应用列表
	 * @param user
	 * @return
	 */
	private List<Map<String,Object>> applications(User user){
		return rightShortcutService.getAppList(
				new RoleResMenu().accountId(user.getAccountId()).accountType(user.getAccountType())
		);
	}

}
