package com.loit.v1.api.controller.open;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LogUtil;
import com.loit.common.utils.AesEncryptUtil;
import com.loit.v1.api.dto.OpenAccessTokenDTO;
import com.loit.v1.api.util.DateTotalUtils.DateTotalUtils;
import com.loit.v1.oauth.data.CasStatus;
import com.loit.v1.oauth.service.RestLoginOauthService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@RestController
@RequestMapping(value = "api/v1/open/oauth")
@Api(value = "OpenOauthController", tags = "Open 登录")
public class OpenOauthController {
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    protected RestLoginOauthService restLoginOauthService;

    /**
     * rest 登录接口
     * <p>
     * 调用方法
     * 1.端直接调用，端需要保存cookie并在调用接口时候传回cookie
     * 2.端调用接口后需要从头部或cookie中获取并保存accessToken
     *
     * @return：登录成功则返回登录用户对象信息
     */
    @PostMapping(value = "login")
    @ApiOperation(value = "Rest 登录", notes = "登录成功则返回登录用户对象信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "账号", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "password", value = "密码", required = true, dataType = "String", paramType = "query")
    })
    public AjaxJson login(
            @RequestParam(value = "userId", required = true) String userId,
            @RequestParam(value = "password", required = true) String password,
            HttpServletResponse response) {

        // 登录验证
        CasStatus loginCastatus = DateTotalUtils.printR(
                "1.登录验证 restLoginOauthService.login(userId, password)",
                CasStatus.class,
                () -> restLoginOauthService.login(userId, password)
        );

        // rest登录验证成功后的令牌处理程序
        //@TODO
        AjaxJson json = DateTotalUtils.printR(
                "2.rest登录验证成功后的令牌处理程序 restLoginOauthService.restAccessTokenHandler(userId, loginCastatus, response)",
                AjaxJson.class,
                () -> restLoginOauthService.restAccessTokenHandler(userId, password,loginCastatus,"1", response)
        );

        return json;

    }

    /**
     * rest 无密码登录接口
     * <p>
     * 调用方法
     * 1.端直接调用，端需要保存cookie并在调用接口时候传回cookie
     * 2.端调用接口后需要从头部或cookie中获取并保存accessToken
     *
     * @return：登录成功则返回登录用户对象信息
     */
    @PostMapping(value = "nopassword/login")
    @ApiOperation(value = "Rest 无密码登录", notes = "登录成功则返回登录用户对象信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "账号", required = true, dataType = "String", paramType = "query")
    })
    public AjaxJson nopasswordLogin(
            @RequestParam(value = "userId", required = true) String userId,
            HttpServletRequest request,
            HttpServletResponse response) {

        // 登录验证
        CasStatus loginCastatus = DateTotalUtils.printR(
                "1.登录验证 restLoginOauthService.login(userId)",
                CasStatus.class,
                () -> restLoginOauthService.login(userId)
        );

        // rest登录验证成功后的令牌处理程序
        AjaxJson json = DateTotalUtils.printR(
                "2.rest登录验证成功后的令牌处理程序 restLoginOauthService.restAccessTokenHandler(userId, loginCastatus, response)",
                AjaxJson.class,
                () -> restLoginOauthService.restAccessTokenHandler(userId,"", loginCastatus, "0",response)
        );

        return json;
    }

    /**
     * sdk rest 令牌获取接口-密文密码-SDK专用
     * @return：登录成功则返回accessToken对象
     */
    @PostMapping(value = "sdk/encrypt/accessToken")
    @ApiOperation(value = "Rest 登录", notes = "登录成功则返回登录用户对象信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userId", value = "账号", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "password", value = "密码", required = true, dataType = "String", paramType = "query")
    })
    public AjaxJson encryptAccessToken(
            @RequestParam(value = "userId", required = true) String userId,
            @RequestParam(value = "password", required = true) String password,
            HttpServletResponse response) {
        try {
            password = AesEncryptUtil.desEncrypt(password).trim();
            logger.debug(LogUtil.print("解密的密码：{}"), password);
        } catch (Exception e) {
            return AjaxJson.returnExceptionInfo(CasStatus.STATUS_908.getValue(), CasStatus.STATUS_908.getMsg());
        }

        // 登录验证
        final String finalPassword = password;
        CasStatus loginCastatus = DateTotalUtils.printR(
                "1.登录验证 restLoginOauthService.login(userId, password)",
                CasStatus.class,
                () -> restLoginOauthService.login(userId, finalPassword)
        );

        // rest登录验证成功后的令牌处理程序
        OpenAccessTokenDTO openAccessTokenDTO = DateTotalUtils.printR(
                "2.rest登录验证成功后的令牌处理程序 restLoginOauthService.restAccessTokenHandler(userId, loginCastatus, response)",
                OpenAccessTokenDTO.class,
                () -> restLoginOauthService.restAccessTokenNoUserHandler(userId, finalPassword, "1", response)
        );

        // 返回令牌信息
        AjaxJson ajaxJson = new AjaxJson();
        ajaxJson.put("obj", openAccessTokenDTO);
        return ajaxJson;
    }

}
