package com.loit.v1.api.controller.open;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.utils.ListUtil;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.api.dto.OrgDTO;
import com.loit.v1.api.dto.OrgExtattrDTO;
import com.loit.v1.modules.activemq.MessageModel;
import com.loit.v1.modules.activemq.Producer;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.entity.ApporgShortcut;
import com.loit.v1.modules.entity.Label;
import com.loit.v1.modules.entity.Org;
import com.loit.v1.modules.service.ApporgShortcutService;
import com.loit.v1.modules.service.LabelService;
import com.loit.v1.modules.service.OrgExtattrService;
import com.loit.v1.modules.service.OrgService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.activemq.command.ActiveMQQueue;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.jms.Destination;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**  
* @ClassName: OrgController  
* @Description:   
* @author zhaokz  
* @date 2019年10月22日 下午5:41:41 
* @version V1.0  
*/
@RestController
@RequestMapping(value = "api/v1/open/org")
@Api(value = "OpenOrgController", tags = "Open 组织管理")
public class OpenOrgController {


	public static final Destination QUEUE_SYNC_ORG_SAVE = new ActiveMQQueue(BusinessConstants.QUEUE_SYNC_ORG_SAVE);
	public static final Destination QUEUE_SYNC_ORG_DELETE = new ActiveMQQueue(BusinessConstants.QUEUE_SYNC_ORG_DELETE);

	@Autowired
	private OrgService orgService;
	@Autowired
	private OrgExtattrService orgExtattrService;
	@Autowired
	private Producer producer;
    @Autowired
    private ApporgShortcutService apporgShortcutService;
	@Autowired
	private LabelService labelService;

	
	/**
	 * 当前应用组织树形列表
	 * 根据当前应用id获取组织, 如果是超级管理员则返回整棵组织树形列表
	 * @param org
	 * @return
	 */
	@ApiOperation(value = "获取当前应用组织树形列表", notes = "获取当前应用组织树形列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用id", required = true, dataType = "String", paramType = "query") 
	})
	@GetMapping(value = "/currentApp/treeList")
	public AjaxJson treeList1(@ApiIgnore Org org){
		AjaxJson json = new AjaxJson();
		org.setDeltag(Org.DEL_FLAG_NORMAL);
		User user = UserUtils.getUser();
		if(BusinessConstants.ACCOUNT_TYPE_SUPERADMIN.equals(user.getAccountType())) {
			List<OrgDTO> treeList = orgService.findOrgTreeList(org);
			json.put("treeList", treeList);
			return json;
		}
		List<OrgDTO> treeList = orgService.findCuAppOrgTree(org);
		json.put("treeList", treeList);
		return json;
	}
	
	/**
	 * 获取组织分页列表
	 * @param request
	 * @param response
	 * @param org
	 * @return
	 */
	@ApiOperation(value = "获取组织分页列表", notes = "获取组织分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用id", required = true, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "orgId", value = "组织ID", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "deltag", value = "删除标记", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "queryLabelName", value = "标签名称", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/pageList")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response,@ApiIgnore Org org){
		AjaxJson json = new AjaxJson();
		Page<Org> pageList = orgService.findPage(new Page<Org>(request, response),org);

		// 获取组织标签并放入组织对象中
		Label label = new Label();
		label.setLabelDictCode("orgLabel");
		List<Label> labels = labelService.findList(label);
		if(ListUtil.isNotEmpty(pageList.getList()) && ListUtil.isNotEmpty(labels)){
			for (Org _org : pageList.getList()) {
				_org.setLabels(labels.stream().filter(e -> e.getLabelModuleId().equals(_org.getOrgId())).collect(Collectors.toList()));
			}
		}

		json.put("page", pageList);
		return json;
	}

	/**
	 * 查看已删除组织列表
	 * @param request
	 * @param response
	 * @param
	 * @return
	 */
	@ApiOperation(value = "查看已删除组织列表", notes = "查看已删除组织列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "orgName", value = "组织名称", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "orgCode", value = "组织编码", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/deletedList")
	public AjaxJson deletedList(HttpServletRequest request, HttpServletResponse response,@ApiIgnore Org org){
		org.setDeltag(Org.DEL_FLAG_DELETE);
		Page<Org> pageList = orgService.findPage(new Page<Org>(request, response),org);

		// 获取组织标签并放入组织对象中
		Label label = new Label();
		label.setLabelDictCode("orgLabel");
		List<Label> labels = labelService.findList(label);
		if(ListUtil.isNotEmpty(pageList.getList()) && ListUtil.isNotEmpty(labels)){
			for (Org _org : pageList.getList()) {
				_org.setLabels(labels.stream().filter(e -> e.getLabelModuleId().equals(_org.getOrgId())).collect(Collectors.toList()));
			}
		}

		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
	}
	
	/**
	 * 获取组织详细信息
	 * @param orgId
	 * @return
	 */
	@ApiOperation(value = "获取组织详细信息", notes = "根据id来获取Org详细信息")
	@ApiImplicitParam(name = "orgId", value = "orgId", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/{orgId}")
	public AjaxJson detail(@PathVariable String orgId) {
		AjaxJson json = new AjaxJson();
		Org newOrg = new Org();
		newOrg.setOrgId(Integer.valueOf(orgId));
		Org org = orgService.get(newOrg);
		if(org != null) {
			OrgDTO orgDTO = ModelMapperUtils.map(org, OrgDTO.class);
//			OrgExtattr orgExtattr = new OrgExtattr();
//			orgExtattr.setOrgId(orgDTO.getOrgId());
//			List<OrgExtattr> orgExtattrs = orgExtattrService.findExtattrList(orgExtattr);
//			orgDTO.setOrgExtattrs(orgExtattrs);
			OrgExtattrDTO orgExtattr = new OrgExtattrDTO();
			orgExtattr.setOrgId(orgDTO.getOrgId());
			List<OrgExtattrDTO> orgExtattrs = orgExtattrService.findExtattrListDTO(orgExtattr);
			orgDTO.setOrgExtattrs(orgExtattrs);

			// 获取组织标签并放入组织对象中
			Label label = new Label();
			label.setLabelDictCode("orgLabel");
			label.setLabelModuleId(Integer.valueOf(orgId));
			List<Label> labels = labelService.findList(label);
			orgDTO.setLabels(labels);

			json.put("org", orgDTO);
		}
		return json;
	}

	
	/**
	 * 保存
	 * @param org
	 * @return
	 */
	@ApiOperation(value = "保存", notes = "保存对象信息")
	@ApiImplicitParam(name = "org", value = "实体Org详情", required = true, dataType = "Org")
	@PostMapping(value = "/save")
	public AjaxJson save(@RequestBody Org org) {
		AjaxJson json = new AjaxJson();
		//发送消息到队列
		Map<String, Object> map = new HashMap<>();
		map.put("isInsert", org.getOrgId() == null ? true : false);//判断是否是新增或是修改
		try {
			//保存组织
			orgService.saveOrg(org);
			json.setMsg("保存成功!");
			json.setSuccess(true);
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
		}
		//发送消息到队列
		map.put("org",org);
		MessageModel message = new MessageModel();
		message.setMap(map);
		producer.send(QUEUE_SYNC_ORG_SAVE,message);
		return json;
	}

	/**
	 * 验证组织唯一
	 * @param
	 * @return
	 */
	@ApiOperation(value = "组织唯一验证", notes = "组织唯一验证")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "orgName", value = "组织名称", required = false, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "orgShortName", value = "组织简称", required = false, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "parentId", value = "父组织id", required = true, dataType = "String",paramType = "query"),
	})
	@PostMapping(value = "/validateOrgName")
	public AjaxJson validateOrgName(@RequestBody Org org) {
		AjaxJson json = new AjaxJson();
		boolean result = orgService.validateOrgName(org);
		if(result) {
			json.setMsg("该组织名称可用!");
			json.setSuccess(true);
		}else {
			json.setSuccess(false);
			return AjaxJson.returnExceptionInfo(LoitStatusCode.ORG_NAME_IS_EXIST);
		}
		return json;
	}
	
	/**
	 * 彻底删除
	 *@param 
	 */
	@ApiOperation(value = "彻底删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "orgIds", value = "orgIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/{orgIds}/{appId}")
	public AjaxJson delete(@PathVariable String orgIds,@PathVariable Integer appId) {
		if(StringUtils.isBlank(orgIds)) {
			return AjaxJson.returnExceptionInfo("组织id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(orgIds)) {
			try {
				//取出应用组织关系列表
				String[] orgArr = orgIds.split(",");
				List<ApporgShortcut> apporgShortcutList = apporgShortcutService.findByOrgIdList(Arrays.asList(orgArr));
				List<Org> orgList = orgService.findByIdList(Arrays.asList(orgArr));
				//删除
				orgService.deleteOrg(orgIds);
				json.setMsg("删除成功!");

				if(apporgShortcutList != null){
					//发送消息到队列
					Map<String, Object> map = new HashMap<>();
					map.put("apporgShortcutList", apporgShortcutList);
					map.put("orgList", orgList);
					MessageModel message = new MessageModel();
					message.setMap(map);
//					System.out.println("\033[31;0m" + "【message】" + message.toString() + "\033[0m");
					producer.send(QUEUE_SYNC_ORG_DELETE,message);//发送消息
				}
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
			}
		}
		return json;
	}
	
	/**
	 * 逻辑删除
	 * @param orgIds
	 * @return
	 */
	@ApiOperation(value = "逻辑删除", notes = "根据id来指定逻辑删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "orgIds", value = "orgIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/deleteByLogic/{orgIds}")
	public AjaxJson deleteByLogic(@PathVariable String orgIds) {
		if(StringUtils.isBlank(orgIds)) {
			return AjaxJson.returnExceptionInfo("机构id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(orgIds)) {
			Org org = new Org();
			try {
				orgService.deleteByLogicAll(orgIds, org);
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
			}
		}
		return json;
	}
	
	/**
	 *  还原
	 * @param
	 * @return
	 */
	@ApiOperation(value = "还原", notes = "支持还原多个,用逗号隔开")
	@ApiImplicitParam(name = "orgIds", value = "组织id", required = true, dataType = "String",paramType = "path")
	@GetMapping(value = "/revert/{orgIds}")
	public AjaxJson revert(@PathVariable String orgIds) {
		 AjaxJson json = orgService.revert(orgIds);
		return json;
	}

}
