package com.loit.v1.api.controller.open;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.service.AccoutInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

/**
 * 移动端人员信息和密码接口
 *
 * Demon add by 20200508
 */
@RestController
@RequestMapping(value = "api/v1/open/password")
@Api(value = "OpenPasswordController", tags = {"Open 密码"})
public class OpenPasswordController {

    @Autowired
    private AccoutInfoService accoutInfoService;

    @ApiOperation(value = "修改当前登录用户的密码", notes = "根据旧密码设置登录用户的新密码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "oldPassWord", value = "旧密码", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "newPassWord", value = "新密码", required = true, dataType = "String", paramType = "query")
    })
    @GetMapping(value = "/passwordUp")
    public AjaxJson setPassword(@RequestParam(value = "oldPassWord", required = true) String oldPassWord,
                                @RequestParam(value = "newPassWord", required = true) String newPassWord) {
        AjaxJson json = new AjaxJson();
        try {
            // 验证旧密码，如果旧密码输入错误，则返回错误信息
            if(!accoutInfoService.validatePassword(UserUtils.getUser().getUserId(), oldPassWord)){
                return AjaxJson.returnExceptionInfo(LoitStatusCode.APP_MOBILE_OLD_PASSWORD_ERR);
            }

            // 修改新密码
            accoutInfoService.setPassword(
                    new AccoutInfo().passWord(newPassWord).accountId(UserUtils.getUser().getAccountId())
            );

            json.setMsg("密码修改成功!");
            json.setSuccess(true);
        } catch (Exception e) {
            e.printStackTrace();
            return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
        }
        return json;
    }
}
