package com.loit.v1.api.controller.open;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.modules.service.ResourceOperationService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @Description: 资源操作权限验证
 * @Author: yangwenbin
 * @Date: 2020/4/28 11:28
 */
@RestController
@RequestMapping(value = "api/v1/open/permission")
@ApiIgnore
public class OpenPermissionController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());
    @Autowired
    private ResourceOperationService resourceOperationService;

    /**
     * 根据url相对地址判断用户是否拥有资源操作权限
     * 1.feign的checkUrlPermission接口SDK专用版
     * 2.此版不需要userId参数，SDK会向请求头放入令牌，网关从请求头中获取令牌后转换为userId重新放入请求头中。
     * 3.UserUtils.getUser()会从请求头中获取userId
     *
     * @param actUrl 请求路径
     * @param method 请求方式
     *
     * Demon add by 2020-07-29
     */
    @GetMapping(value = "sdk/checkUrlPermission")
    public AjaxJson sdkCheckUrlPermission(
            @RequestParam(value = "actUrl",required = true) String actUrl,
            @RequestParam(value = "method",required = true) String method){
        AjaxJson json = new AjaxJson();
        try{
            User user = UserUtils.getUser();
            //本方法鉴权如果controller方法写如@GetMapping(value = "{resourceId}") 操作注册会写为/**方式匹配，如有另一个方法@GetMapping(value = "list"),此时如果操作注册中存在/** 则会匹配两个请求无法做过颗粒度最细化
            boolean flag = resourceOperationService.checkPermission(user.getUserId(),actUrl,method);
            logger.info("鉴权信息结果：用户:"+user.getUserId()+" 操作地址:"+actUrl+" 请求方式:"+method+" 是否有权限:"+flag);
            json.put("flag",flag);
            return json;
        }catch (Exception e){
            logger.error("根据url相对地址判断用户是否拥有资源操作权限异常",e);
            return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
        }
    }
}
