package com.loit.v1.api.controller.open;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.v1.api.dto.ActRightDTO;
import com.loit.v1.modules.entity.ActRight;
import com.loit.v1.modules.entity.ResourceTypeOperation;
import com.loit.v1.modules.service.ActRightService;
import com.loit.v1.modules.service.ResourceTypeOperationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;

/**  
* @ClassName: OpenResourceRightController
* @Description:   
* @author zhaokz  
* @date 2019年10月28日 下午3:56:15 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/open")
@Api(value = "资源操作权限管理Controller", tags = "Open 资源操作权限管理")
public class OpenResourceRightController {

	@Autowired
	private ActRightService actRightService;
	@Autowired
	private ResourceTypeOperationService resourceTypeOperationService;

	/**
	 * @description 应用资源类别操作管理列表
	 * @author xutao
	 * @date 2019年10月18日
	 * @param resourceTypeOperation 应用资源类别操作管理实体类
	 * @return
	 */
	@GetMapping(value = "resourceTypeOperation/list")
	@ApiOperation(value="列表", notes="获取资源类别操作列表")
	@ApiImplicitParam(name = "typeId", value = "资源类别ID", required = true, dataType = "String", paramType = "query")
	public AjaxJson list(@ApiIgnore ResourceTypeOperation resourceTypeOperation) {
		List<ResourceTypeOperation> list = resourceTypeOperationService.findAllList(resourceTypeOperation);
		//logger.debug("获取资源类别操作列表成功");
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}
	
	@ApiOperation(value = "获取资源操作权限列表-不分页", notes = "获取资源操作权限列表-不分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "roleId", value = "角色ID", required = false, dataType = "int", paramType = "query") ,
		@ApiImplicitParam(name = "resId", value = "资源ID", required = false, dataType = "int", paramType = "query") 
	})
	@GetMapping(value = "actRight/allList")
	public AjaxJson list(@ApiIgnore ActRight actRight){
		List<ActRight> actRightList = actRightService.findAllList(actRight);
		AjaxJson json = new AjaxJson();
		json.put("list", actRightList);
		return json;
	}
	
	@ApiOperation(value = "获取角色资源操作权限IDs", notes = "获取角色资源操作权限IDs")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "roleId", value = "角色ID", required = true, dataType = "int", paramType = "query") ,
		@ApiImplicitParam(name = "resId", value = "资源ID", required = true, dataType = "int", paramType = "query") 
	})
	@GetMapping(value = "actRight/getIdsByRoleIdAndResId")
	public AjaxJson getIdsByRoleIdAndResId(@ApiIgnore ActRight actRight){
		List<Integer> list = actRightService.getIdsByRoleIdAndResId(actRight);
		AjaxJson json = new AjaxJson();
		json.put("actIdArr", list);
		return json;
	}
	
	/**
	 * 获取资源操作权限资源操作权限详细信息
	 * @param resActId
	 * @return
	 */
	@ApiOperation(value = "获取资源操作权限详细信息", notes = "根据id来获取ActRight详细信息")
	@ApiImplicitParam(name = "actRight/resActId", value = "resActId", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/{resActId}")
	public AjaxJson detail(@PathVariable String resActId) {
		AjaxJson json = new AjaxJson();
		ActRight newActRight = new ActRight();
		newActRight.setResActId(Integer.valueOf(resActId));
		ActRight actRight = actRightService.get(newActRight);
		ActRightDTO actRightDTO = null;
		if(actRight != null) {
			actRightDTO = ModelMapperUtils.map(actRight, ActRightDTO.class);
		}
		json.put("actRight", actRightDTO);
		return json;
	}

}
