package com.loit.v1.api.controller.open;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.modules.activemq.MessageModel;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.entity.Org;
import com.loit.v1.modules.entity.OrgPerson;
import com.loit.v1.modules.entity.SynTarget;
import com.loit.v1.modules.service.AccoutInfoService;
import com.loit.v1.modules.service.OrgService;
import com.loit.v1.modules.service.PersonBaseService;
import com.loit.v1.modules.service.SynTargetService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**  
* @ClassName: DataSyncController
* @Description:  数据同步controller
* @author zhangyl
* @date 2019年11月12日 上午15:37:51
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/open/sync")
@Api(value = "OpenSyncController", tags = {"Open 数据同步"})
public class OpenSyncController extends BaseController {

	@Autowired
	private OrgService orgService;
	@Autowired
	private PersonBaseService personBaseService;
	@Autowired
	private AccoutInfoService accoutInfoService;

	/**
	 * 组织机构保存
	 */
	@ApiOperation(value = "组织机构同步", notes = "组织机构保存")
	@ApiImplicitParam(name = "org", value = "实体Org详情", required = true, dataType = "Org")
	@PostMapping(value = "/org")
	public AjaxJson orgSave(@RequestBody Org org) {
		AjaxJson json = new AjaxJson();
		try {
			orgService.saveOrg(org);
			json.setMsg("保存成功!");
			json.setSuccess(true);
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
		}
		return json;
	}

	/**
	 * 组织机构逻辑删除
	 * @param orgIds
	 * @return
	 */
	@ApiOperation(value = "组织机构删除", notes = "根据组织机构id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "orgIds", value = "orgIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/org/{orgIds}")
	public AjaxJson orgDelete(@PathVariable String orgIds) {
		if(StringUtils.isBlank(orgIds)) {
			return AjaxJson.returnExceptionInfo("机构id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(orgIds)) {
			Org org = new Org();
			try {
				orgService.deleteByLogicAll(orgIds, org);
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
			}
		}
		return json;
	}

	/**
	 * 用户信息保存
	 * @param
	 * @return
	 */
	@ApiOperation(value = "用户信息同步", notes = "用户信息保存")
	@ApiImplicitParam(name = "personBaseDTO", value = "实体PersonBaseDTO详情", required = true, dataType = "PersonBaseDTO")
	@PostMapping(value = "/user")
	public AjaxJson save(@RequestBody PersonBaseDTO personBaseDTO) {
		AjaxJson json = new AjaxJson();
		try {
			personBaseService.addPerson(personBaseDTO);
			json.setMsg("保存成功!");
			json.setSuccess(true);
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
		}
		return json;
	}

	/**
	 * 用户信息逻辑删除
	 * @param
	 * @return
	 */
	@ApiOperation(value = "用户信息删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "accountIds", value = "accoutInfoIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/user/{accountIds}")
	public AjaxJson deleteByLogic(@PathVariable String accountIds) {
		if(StringUtils.isBlank(accountIds)) {
			return AjaxJson.returnExceptionInfo("id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(accountIds)) {
			AccoutInfo accoutInfo = new AccoutInfo();
			try {
				accoutInfoService.deleteByLogicAll(accountIds, accoutInfo);
				json.setMsg("删除成功!");
				json.setSuccess(true);
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
			}
		}
		return json;
	}


}
