package com.loit.v1.api.controller.open;

import com.alibaba.fastjson.JSONObject;
import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.api.dto.OrgDTO;
import com.loit.v1.api.dto.UserGroupDTO;
import com.loit.v1.modules.entity.GrpFrom;
import com.loit.v1.modules.entity.Org;
import com.loit.v1.modules.entity.PersonBase;
import com.loit.v1.modules.entity.UserGroup;
import com.loit.v1.modules.service.GrpFromService;
import com.loit.v1.modules.service.OrgService;
import com.loit.v1.modules.service.PersonBaseService;
import com.loit.v1.modules.service.UserGroupService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**  
* @ClassName: UserGroupController  
* @Description:  用户组管理 
* @author zhaokz  
* @date 2019年10月26日 下午4:30:19 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/open/userGroup")
@Api(value = "OpenUserGroupController", tags = "Open 用户组管理")
public class OpenUserGroupController extends BaseController{

	@Autowired
	private UserGroupService userGroupService;
	@Autowired
	private GrpFromService grpFromService;
	@Autowired
	private PersonBaseService personBaseService;
	@Autowired
	private OrgService orgService;
	
	/**
	 * 获取用户组列表
	 * @param request
	 * @param response
	 * @param userGroup
	 * @return
	 */
	@ApiOperation(value = "获取用户组列表", notes = "获取用户组列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用Id", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "grpName", value = "用户组名称", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/pageList")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response,@ApiIgnore UserGroup userGroup){
		Page<UserGroup> pageList = userGroupService.findPage(new Page<UserGroup>(request, response),userGroup);
		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
	}

	/**
	 * 获取用户组用户组详细信息
	 * @param grpId
	 * @return
	 */
	@ApiOperation(value = "获取用户组详细信息", notes = "根据id来获取UserGroup详细信息")
	@ApiImplicitParam(name = "grpId", value = "grpId", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/{grpId}")
	public AjaxJson detail(@PathVariable String grpId) {
		AjaxJson json = new AjaxJson();
		UserGroup newUserGroup = new UserGroup();
		newUserGroup.setGrpId(Integer.valueOf(grpId));
		UserGroup userGroup = userGroupService.get(newUserGroup);
		UserGroupDTO userGroupDTO = null;
		if(userGroup != null) {
			userGroupDTO = ModelMapperUtils.map(userGroup, UserGroupDTO.class);
		}
		json.put("userGroup", userGroupDTO);
		return json;
	}

	
	/**
	 * 保存
	 * @param userGroup
	 * @return
	 */
	@ApiOperation(value = "保存", notes = "保存对象信息")
	@ApiImplicitParam(name = "userGroupDTO", value = "实体UserGroupDTO", required = true, dataType = "UserGroupDTO")
	@PostMapping(value = "/save")
	public AjaxJson save(@RequestBody UserGroupDTO userGroupDTO) {
		AjaxJson json = new AjaxJson();
		try {
			UserGroup userGroup = ModelMapperUtils.map(userGroupDTO, UserGroup.class);
			userGroupService.saveUserGroup(userGroup);
			GrpFrom grpFrom = ModelMapperUtils.map(userGroupDTO, GrpFrom.class);
			grpFromService.save(grpFrom, grpFrom.getGrpFromId());
			json.setMsg("保存成功!");
			json.setSuccess(true);
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存用户组异常" + e.getMessage());
		}
		return json;
	}
	
	/**
	 * 用户组名称唯一性验证(仅当前应用内)
	 * @param userGroup
	 * @return
	 */
	@ApiOperation(value = "用户组名称唯一性验证", notes = "用户组名称唯一性验证")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "grpId", value = "用户组id", required = false, dataType = "String", paramType = "query"), 
		@ApiImplicitParam(name = "grpName", value = "用户组名称", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "appId", value = "应用id", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/grpNameUnique")
	public AjaxJson grpNameUnique(@ApiIgnore UserGroup userGroup) {
		AjaxJson json = new AjaxJson();
		boolean result = userGroupService.validateGrpName(userGroup);
		if(result) {
			json.setMsg("用户组名称可用");
			json.setSuccess(true);
		}else {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.GRP_NAME_EXISTED);
		}
		return json;
	}

	/**
	 * 删除
	 *@param 
	 */
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "grpIds", value = "grpIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/{grpIds}")
	public AjaxJson delete(@PathVariable String grpIds) {
		if(StringUtils.isBlank(grpIds)) {
			return AjaxJson.returnExceptionInfo("机构id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(grpIds)) {
			try {
				userGroupService.deleteGrp(grpIds);
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除用户组失败" + e.getMessage());
			}
		}
		return json;
	}
	
	/**
	 * 用户组添加人员
	 * @param grpId  用户组id
	 * @param userIds 用户id,支持多个,以逗号隔开
	 * @return
	 */
	@ApiOperation(value = "用户组添加人员", notes = "支持批量添加,以逗号隔开")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "grpId", value = "用户组id", required = false, dataType = "String", paramType = "query"), 
		@ApiImplicitParam(name = "userIds", value = "用户id", required = false, dataType = "String", paramType = "query")
	})
	@PostMapping(value = "/addPerson")
	public AjaxJson addPerson(@RequestBody String params) {
		AjaxJson json = new AjaxJson();
		JSONObject jsonObject = JSONObject.parseObject(params);
	    String grpId = jsonObject.getString("grpId");
		String userIds = jsonObject.getString("userIds");
		UserGroup userGroup = userGroupService.get(grpId);
		if(userGroup != null) {
			try {
				userGroupService.addperson(grpId,userIds);
				json.setMsg("添加成功!");
				json.setSuccess(true);
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("添加失败 " + e.getMessage());
			}
		}else {
			json.setSuccess(false);
			json.setMsg("该用户组不存在!");
			return json;
		}
		
		return json;
	}
	
	/**
	 * 用户组添加组织
	 * @param grpId  用户组id
	 * @param orgIds 组织id,支持多个,以逗号隔开
	 * @return
	 */
	@ApiOperation(value = "用户组添加组织", notes = "支持批量添加,以逗号隔开")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "grpId", value = "用户组id", required = false, dataType = "String", paramType = "query"), 
		@ApiImplicitParam(name = "orgIds", value = "组织id", required = false, dataType = "String", paramType = "query")
	})
	@PostMapping(value = "/addOrg")
	public AjaxJson addOrg(@RequestBody String params) {
		AjaxJson json = userGroupService.addOrg(params);
		return json;
	}
	
	/**
	 * 未添加人员列表
	 * @param request
	 * @param response
	 * @param userGroup
	 * @return
	 */
	@ApiOperation(value = "未添加人员列表", notes = "未添加人员列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用Id", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "grpId", value = "用户组Id", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "orgName", value = "所属组织名称", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "账号", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/unAddPersons/pageList")
	public AjaxJson unAddPersons(HttpServletRequest request, HttpServletResponse response,@ApiIgnore PersonBase personBase){
		Page<PersonBase> pageList = personBaseService.findUnAddPersons(new Page<PersonBase>(request, response),personBase);
		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
	}
	
	
	/**
	 * 用户组已添加人员列表
	 * @param request
	 * @param response
	 * @param userGroup
	 * @return
	 */
	@ApiOperation(value = "用户组已添加人员列表", notes = "用户组已添加人员列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "grpId", value = "用户组Id", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "orgName", value = "所属组织名称", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "userId", value = "账号", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/grpPersons/pageList")
	public AjaxJson grpPersons(HttpServletRequest request, HttpServletResponse response,@ApiIgnore PersonBase personBase){
		Page<PersonBase> pageList = personBaseService.findGrpPersonPage(new Page<PersonBase>(request, response),personBase);
		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
	}
	
	
	/**
	 * 未添加组织列表
	 * @param request
	 * @param response
	 * @param userGroup
	 * @return
	 */
	@ApiOperation(value = "未添加组织列表", notes = "未添加组织列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用Id", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "grpId", value = "用户组Id", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/unAddOrgs/pageList")
	public AjaxJson unAddOrgs(@ApiIgnore Org org){
		AjaxJson json = new AjaxJson();
		List<OrgDTO> list = orgService.findUnAddOrgs(org);
		json.put("treeList", list);
		return json;
	}
	
	/**
	 * 已添加组织列表
	 * @param request
	 * @param response
	 * @param userGroup
	 * @return
	 */
	@ApiOperation(value = "已添加组织列表", notes = "已添加组织列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "grpId", value = "用户组Id", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/grpOrgs/pageList")
	public AjaxJson grpOrgs(HttpServletRequest request, HttpServletResponse response,@ApiIgnore Org org){
		AjaxJson json = new AjaxJson();
		Page<Org> pageList = orgService.findGrpOrgs(new Page<Org>(request, response),org);
		json.put("page", pageList);
		return json;
	}
	
	/**
	 * 用户组删除组织
	 * @param request
	 * @param response
	 * @param userGroup
	 * @return
	 */
	@ApiOperation(value = "用户组删除组织", notes = "用户组删除组织")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "orgIds", value = "orgIds", required = true, dataType = "String",paramType = "path"),
		@ApiImplicitParam(name = "grpId", value = "grpId", required = true, dataType = "Integer",paramType = "path")
	})
	@DeleteMapping(value = "/removeOrg/{orgIds}/{grpId}")
	public AjaxJson removeOrg(@PathVariable String orgIds,@PathVariable String grpId){
		AjaxJson json = new AjaxJson();
		userGroupService.removeOrg(orgIds,grpId);
		json.setSuccess(true);
		json.setMsg("删除成功!");
		return json;
	}
	
	/**
	 * 用户组删除人员
	 * @param request
	 * @param response
	 * @param userGroup
	 * @return
	 */
	@ApiOperation(value = "用户组删除人员", notes = "用户组删除人员")
	@ApiImplicitParam(name = "accountIds", value = "accountIds", required = true, dataType = "String",paramType = "path")
	@DeleteMapping(value = "/removePerson/{accountIds}/{grpId}")
	public AjaxJson grpOrgs(@PathVariable String accountIds,@PathVariable String grpId){
		AjaxJson json = new AjaxJson();
		userGroupService.removePerson(accountIds,grpId);
		json.setSuccess(true);
		json.setMsg("删除成功!");
		return json;
	}
}
