package com.loit.v1.api.dto;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import org.springframework.format.annotation.DateTimeFormat;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.loit.common.annotation.LoitDictAnnotation.LoitDict;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

/**  
* @ClassName: PersonBaseDTO  
* @Description:   
* @author zhaokz  
* @date 2019年10月24日 下午3:30:32 
* @version V1.0  
*/
@ApiModel(value = "PersonBaseDTO", description="人员基本信息表数据传输类")
public class PersonBaseDTO implements Serializable {

	private static final long serialVersionUID = 1L;


	public PersonBaseDTO() {

	}

	public PersonBaseDTO(Integer accountId) {
		this.accountId = accountId;
	}

	@ApiModelProperty(value = "基本信息id",example = "1")
	private Integer personBaseId;


	@ApiModelProperty(value = "人员名称",example = "丁力")
	private String personName;


	@ApiModelProperty(value = "人员统一编码",example = "3243")
	private String personCode;


	@ApiModelProperty(value = "证件号",example = "130897456787654321")
	private String idNum;


	@ApiModelProperty(value = "证件类型",example = "1")
	@LoitDict(dictCode = "idType")
	private String idType;
	
	@ApiModelProperty(value = "证件类型名称",example = "身份证")
	private String idTypeName;

	/**
	 * (0保密, 1男, 2女)
	 */
	@ApiModelProperty(value = "性别",example = "1")
	private Integer pbSex;


	@ApiModelProperty(value = "婚姻状况",example = "0")
	private String marryCode;


	@ApiModelProperty(value = "政治面貌",example = "3")
	@LoitDict(dictCode = "politicsStatus")
	private String pbPcode;
	
	@ApiModelProperty(value = "政治面貌名称",example = "党员")
	private String pbPcodeName;


	@ApiModelProperty(value = "家庭电话",example = "86-2343222")
	private String homeTel;


	@ApiModelProperty(value = "办公电话",example = "010-7832366")
	private String officeTel;


	@ApiModelProperty(value = "家庭传真",example = "23214324")
	private String homeFax;


	@ApiModelProperty(value = "办公传真",example = "785677866")
	private String officeFax;


	@ApiModelProperty(value = "手机号",example = "15628963467")
	private String pbMobile;


	@ApiModelProperty(value = "电子邮件",example = "3037877@163.com")
	private String pbEmail;


	@ApiModelProperty(value = "国家",example = "0")
	private String pbCountry;

	@ApiModelProperty(value = "国家名称",example = "中国")
	private String pbCountryName;
	
	@ApiModelProperty(value = "省份",example = "23")
	private String provinceCode;

	@ApiModelProperty(value = "省份名称",example = "福建")
	private String provinceCodeName;
	
	@ApiModelProperty(value = "市区",example = "33")
	private String cityCode;

	@ApiModelProperty(value = "市区名称",example = "北京市")
	private String cityCodeName;
	
	@ApiModelProperty(value = "街道",example = "12")
	private String streetCode;

	@ApiModelProperty(value = "街道名称",example = "路源街道办事处")
	private String streetCodeName;
	
	@ApiModelProperty(value = "联系地址",example = "北京市朝阳区来广营西路66号")
	private String connectAddr;


	@ApiModelProperty(value = "邮编",example = "010000")
	private String postCode;


	@ApiModelProperty(value = "最高学历",example = "1")
	@LoitDict(dictCode = "education")
	private String eduCode;

	@ApiModelProperty(value = "最高学历名称",example = "本科")
	private String eduCodeName;
	
	@ApiModelProperty(value = "最高学位",example = "3")
	@LoitDict(dictCode = "degreeCode")
	private String degreeCode;

	@ApiModelProperty(value = "最高学位名称",example = "学士")
	private String degreeCodeName;
	
	@ApiModelProperty(value = "其他信息",example = "")
	private String otherInfo;


	@ApiModelProperty(value = "排序号",example = "10")
	private Integer serialIndex;


	@ApiModelProperty(value = "图片地址",example = "/src/webapp/dsfasdfassag.jpg")
	private String photoUrl;


	@ApiModelProperty(value = "民族",example = "3")
	@LoitDict(dictCode = "nation")
	private String pbNation;
	
	@ApiModelProperty(value = "民族名称",example = "3")
	private String pbNationName;

	@ApiModelProperty(value = "身份证特征码",example = "2132")
	private String idCode;


	@ApiModelProperty(value = "是否发放证书",example = "0")
	private String caStatus;


	@ApiModelProperty(value = "现任职务",example = "34")
	private String pbJob;

	@ApiModelProperty(value = "人员唯一ID",example = "1")
	private Integer accountId;
	
	/**
	 * （0,访客1,普通用户2,一般管理员）
	 */
	@ApiModelProperty(value = "账号类别",example = "1")
	private String userType;


	@ApiModelProperty(value = "账号",example = "1")
	private String userId;


	@ApiModelProperty(value = "未加密密码",example = "123456")
	private String unPassword;


	@ApiModelProperty(value = "密码",example = "f@sdgfd%d$dsfs")
	private String passWord;

	/**
	 * （0,正常1,锁定2,一般管理员禁止）
	 */
	@ApiModelProperty(value = "账号状态",example = "0")
	private String userStat;


	@ApiModelProperty(value = "失败登录次数",example = "2")
	private Integer loginFailnum;


	@ApiModelProperty(value = "最近登录IP",example = "17.34.56.213")
	private String lastLoginIp;


	@ApiModelProperty(value = "最近登录日期",example = "2019-10-24 15:45:23")
	@DateTimeFormat(pattern = "yyyy-MM-ddHH:mm:ss")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date lastLoginDate;


	@ApiModelProperty(value = "密码问题",example = "我是谁")
	private String passQuestion;


	@ApiModelProperty(value = "密码答案",example = "小马哥")
	private String passAnswer;

	/**
	 * （0:普通帐号1:临时账号2:应用管理帐号3:超级管理帐号）
	 */
	@ApiModelProperty(value = "账号类型",example = "0")
	private String accountType;


	@ApiModelProperty(value = "账号有效期限",example = "4")
	private Integer accountTimelimit;

	@ApiModelProperty(value = "密码修改时间",example = "2019-10-24 15:45:23")
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	@JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date passUpdateTime;


	@ApiModelProperty(value = "创建者类型",example = "2")
	private String createBy;
	
	@ApiModelProperty(value = "组织id",example = "2")
	private Integer orgId;

	@ApiModelProperty(value = "组织名称",example = "北京市城管局")
	private String orgName;
	
	@ApiModelProperty(value = "组织编码",example = "beijing")
	private String orgCode;

	@ApiModelProperty(name = "enableDel", value = "是否允许删除（0：否 1：是）")
	private String enableDel;


//	 * 扩展属性和值
//	 * @param personBaseId
//	 */
//	List<PersonExtattr> personExtattr;
	/**
	 * 扩展属性和值
	 * @param personBaseId
	 */
	List<PersonExtattrDTO> personExtattr;
//	/**
//	 * 扩展实例表
//	 * @param personBaseId
//	 */
//	List<PersonExtattrInst> personExtattrInst;

	public Integer getPersonBaseId() {
		return personBaseId;
	}


	public void setPersonBaseId(Integer personBaseId) {
		this.personBaseId = personBaseId;
	}


	public String getPersonName() {
		return personName;
	}


	public void setPersonName(String personName) {
		this.personName = personName;
	}


	public String getPersonCode() {
		return personCode;
	}


	public void setPersonCode(String personCode) {
		this.personCode = personCode;
	}


	public String getIdNum() {
		return idNum;
	}


	public void setIdNum(String idNum) {
		this.idNum = idNum;
	}


	public String getIdType() {
		return idType;
	}


	public void setIdType(String idType) {
		this.idType = idType;
	}


	public Integer getPbSex() {
		return pbSex;
	}


	public void setPbSex(Integer pbSex) {
		this.pbSex = pbSex;
	}


	public String getMarryCode() {
		return marryCode;
	}


	public void setMarryCode(String marryCode) {
		this.marryCode = marryCode;
	}


	public String getPbPcode() {
		return pbPcode;
	}


	public void setPbPcode(String pbPcode) {
		this.pbPcode = pbPcode;
	}


	public String getHomeTel() {
		return homeTel;
	}


	public void setHomeTel(String homeTel) {
		this.homeTel = homeTel;
	}


	public String getOfficeTel() {
		return officeTel;
	}


	public void setOfficeTel(String officeTel) {
		this.officeTel = officeTel;
	}


	public String getHomeFax() {
		return homeFax;
	}


	public void setHomeFax(String homeFax) {
		this.homeFax = homeFax;
	}


	public String getOfficeFax() {
		return officeFax;
	}


	public void setOfficeFax(String officeFax) {
		this.officeFax = officeFax;
	}


	public String getPbMobile() {
		return pbMobile;
	}


	public void setPbMobile(String pbMobile) {
		this.pbMobile = pbMobile;
	}


	public String getPbEmail() {
		return pbEmail;
	}


	public void setPbEmail(String pbEmail) {
		this.pbEmail = pbEmail;
	}


	public String getPbCountry() {
		return pbCountry;
	}


	public void setPbCountry(String pbCountry) {
		this.pbCountry = pbCountry;
	}


	public String getProvinceCode() {
		return provinceCode;
	}


	public void setProvinceCode(String provinceCode) {
		this.provinceCode = provinceCode;
	}


	public String getCityCode() {
		return cityCode;
	}


	public void setCityCode(String cityCode) {
		this.cityCode = cityCode;
	}


	public String getStreetCode() {
		return streetCode;
	}


	public void setStreetCode(String streetCode) {
		this.streetCode = streetCode;
	}


	public String getConnectAddr() {
		return connectAddr;
	}


	public void setConnectAddr(String connectAddr) {
		this.connectAddr = connectAddr;
	}


	public String getPostCode() {
		return postCode;
	}


	public void setPostCode(String postCode) {
		this.postCode = postCode;
	}


	public String getEduCode() {
		return eduCode;
	}


	public void setEduCode(String eduCode) {
		this.eduCode = eduCode;
	}


	public String getDegreeCode() {
		return degreeCode;
	}


	public void setDegreeCode(String degreeCode) {
		this.degreeCode = degreeCode;
	}


	public String getOtherInfo() {
		return otherInfo;
	}


	public void setOtherInfo(String otherInfo) {
		this.otherInfo = otherInfo;
	}


	public Integer getSerialIndex() {
		return serialIndex;
	}


	public void setSerialIndex(Integer serialIndex) {
		this.serialIndex = serialIndex;
	}


	public String getPhotoUrl() {
		return photoUrl;
	}


	public void setPhotoUrl(String photoUrl) {
		this.photoUrl = photoUrl;
	}


	public String getPbNation() {
		return pbNation;
	}


	public void setPbNation(String pbNation) {
		this.pbNation = pbNation;
	}


	public String getIdCode() {
		return idCode;
	}


	public void setIdCode(String idCode) {
		this.idCode = idCode;
	}


	public String getCaStatus() {
		return caStatus;
	}

	public void setCaStatus(String caStatus) {
		this.caStatus = caStatus;
	}


	public String getPbJob() {
		return pbJob;
	}


	public void setPbJob(String pbJob) {
		this.pbJob = pbJob;
	}

	public String getUserId() {
		return userId;
	}

	public void setUserId(String userId) {
		this.userId = userId;
	}


	public String getUserType() {
		return userType;
	}


	public void setUserType(String userType) {
		this.userType = userType;
	}


	public String getUnPassword() {
		return unPassword;
	}


	public void setUnPassword(String unPassword) {
		this.unPassword = unPassword;
	}

	public String getUserStat() {
		return userStat;
	}


	public void setUserStat(String userStat) {
		this.userStat = userStat;
	}


	public Integer getLoginFailnum() {
		return loginFailnum;
	}


	public void setLoginFailnum(Integer loginFailnum) {
		this.loginFailnum = loginFailnum;
	}


	public String getLastLoginIp() {
		return lastLoginIp;
	}


	public void setLastLoginIp(String lastLoginIp) {
		this.lastLoginIp = lastLoginIp;
	}


	public Date getLastLoginDate() {
		return lastLoginDate;
	}


	public void setLastLoginDate(Date lastLoginDate) {
		this.lastLoginDate = lastLoginDate;
	}


	public String getPassQuestion() {
		return passQuestion;
	}


	public void setPassQuestion(String passQuestion) {
		this.passQuestion = passQuestion;
	}


	public String getPassAnswer() {
		return passAnswer;
	}


	public void setPassAnswer(String passAnswer) {
		this.passAnswer = passAnswer;
	}


	public String getAccountType() {
		return accountType;
	}


	public void setAccountType(String accountType) {
		this.accountType = accountType;
	}


	public Integer getAccountTimelimit() {
		return accountTimelimit;
	}


	public void setAccountTimelimit(Integer accountTimelimit) {
		this.accountTimelimit = accountTimelimit;
	}


	public Date getPassUpdateTime() {
		return passUpdateTime;
	}


	public void setPassUpdateTime(Date passUpdateTime) {
		this.passUpdateTime = passUpdateTime;
	}


	public String getCreateBy() {
		return createBy;
	}

	public void setCreateBy(String createBy) {
		this.createBy = createBy;
	}

	public Integer getAccountId() {
		return accountId;
	}


	public void setAccountId(Integer accountId) {
		this.accountId = accountId;
	}


	public Integer getOrgId() {
		return orgId;
	}


	public void setOrgId(Integer orgId) {
		this.orgId = orgId;
	}


	public String getIdTypeName() {
		return idTypeName;
	}


	public void setIdTypeName(String idTypeName) {
		this.idTypeName = idTypeName;
	}


	public String getPbPcodeName() {
		return pbPcodeName;
	}


	public void setPbPcodeName(String pbPcodeName) {
		this.pbPcodeName = pbPcodeName;
	}


	public String getPbCountryName() {
		return pbCountryName;
	}


	public void setPbCountryName(String pbCountryName) {
		this.pbCountryName = pbCountryName;
	}


	public String getProvinceCodeName() {
		return provinceCodeName;
	}


	public void setProvinceCodeName(String provinceCodeName) {
		this.provinceCodeName = provinceCodeName;
	}


	public String getCityCodeName() {
		return cityCodeName;
	}


	public void setCityCodeName(String cityCodeName) {
		this.cityCodeName = cityCodeName;
	}


	public String getStreetCodeName() {
		return streetCodeName;
	}


	public void setStreetCodeName(String streetCodeName) {
		this.streetCodeName = streetCodeName;
	}


	public String getEduCodeName() {
		return eduCodeName;
	}


	public void setEduCodeName(String eduCodeName) {
		this.eduCodeName = eduCodeName;
	}


	public String getDegreeCodeName() {
		return degreeCodeName;
	}


	public void setDegreeCodeName(String degreeCodeName) {
		this.degreeCodeName = degreeCodeName;
	}


	public String getPbNationName() {
		return pbNationName;
	}


	public void setPbNationName(String pbNationName) {
		this.pbNationName = pbNationName;
	}


	public String getOrgName() {
		return orgName;
	}


	public void setOrgName(String orgName) {
		this.orgName = orgName;
	}

	public List<PersonExtattrDTO> getPersonExtattr() {
		return personExtattr;
	}

	public void setPersonExtattr(List<PersonExtattrDTO> personExtattr) {
		this.personExtattr = personExtattr;
	}

	public String getPassWord() {
		return passWord;
	}


	public void setPassWord(String passWord) {
		this.passWord = passWord;
	}

	public String getOrgCode() {
		return orgCode;
	}

	public void setOrgCode(String orgCode) {
		this.orgCode = orgCode;
	}


	public String getEnableDel() {
		return enableDel;
	}

	public void setEnableDel(String enableDel) {
		this.enableDel = enableDel;
	}
}
