package com.loit.v1.aspect;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.CacheUtils;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.api.dto.ApplicationManageDTO;
import com.loit.v1.modules.config.OptimizeConfig;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.entity.ApplicationManage;
import com.loit.v1.modules.entity.MenuManage;
import com.loit.v1.modules.entity.RoleResMenu;
import com.loit.v1.modules.entity.ThemeManage;
import com.loit.v1.modules.service.ApplicationManageService;
import com.loit.v1.modules.service.RoleService;
import com.loit.v1.modules.service.ThemeManageService;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Description： 应用初次进来对加载到资源设置缓存切面
 * Author: yangwenbin
 * Date: Created in 2020/5/27 15:03
 * Version: 1.0
 * Modified By:
 */
@Aspect
@Configuration
public class ApplicationFirstLoadAspect {
    protected static Logger logger = LoggerFactory.getLogger(ApplicationFirstLoadAspect.class);

    @Autowired
    private RoleService roleService;
    @Autowired
    private ApplicationManageService applicationManageService;
    @Autowired
    private ThemeManageService themeManageService;
    @Autowired
    private OptimizeConfig optimizeConfig;

    @Pointcut("execution(* com.loit.v1.modules.controller.ApplicationManageController.update(..))")
    public void excudeApplicationUpdateService() {

    }
    @Pointcut("execution(* com.loit.v1.modules.controller.ApplicationManageController.save(..))")
    public void excudeApplicationSaveService() {

    }
    @Pointcut("execution(* com.loit.v1.modules.controller.ApplicationManageController.deleteAll(..))")
    public void excudeApplicationDeleteService() {

    }

    @Pointcut("execution(* com.loit.v1.modules.service.RightShortcutService.getAppList(..))")
    public void excudeGetAppListService() {
    }
    @Pointcut("execution(* com.loit.v1.modules.service.ApplicationManageService.getApplicationManage(..))")
    public void excudegetApplicationManageService() {
    }

    @Pointcut("execution(* com.loit.v1.modules.controller.ThemeManageController.themeSet(..))")
    public void excudeThemeSetUpdateService() {
    }

    @Pointcut("execution(* com.loit.v1.modules.service.ThemeManageService.get(..))")
    public void excudeGetThemeSetService(){

    }



    @Around("excudeApplicationUpdateService()")
    public Object doApplicationUpdateAround(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson)pjp.proceed();
        if(result.isSuccess()) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    Object[] args =pjp.getArgs();
                    Integer appId = (Integer) args[0];
                    Role appRole = new Role();
                    appRole.setAppId(appId);
                    //查询当前操作影响到的角色
                    List<Role> roleList = roleService.findList(appRole);
                    //更新缓存
                    applicationManageService.resetRoleApplicationCache(roleList);
                }
            }).start();
        }
        return result;
    }

    @Around("excudeApplicationSaveService()")
    public Object doApplicationSaveAround(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson)pjp.proceed();
        if(result.isSuccess()) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    Object[] args =pjp.getArgs();
                    ApplicationManageDTO applicationManageDTO = (ApplicationManageDTO) args[0];
                    ApplicationManage applicationManage = ModelMapperUtils.map(applicationManageDTO, ApplicationManage.class);
                    Role appRole = new Role();
                    appRole.setAppId(applicationManage.getAppId());
                    //查询当前操作影响到的角色
                    List<Role> roleList = roleService.findList(appRole);
                    //更新缓存
                    applicationManageService.resetRoleApplicationCache(roleList);
                }
            }).start();
        }
        return result;
    }

    @Around("excudeApplicationDeleteService()")
    public Object doApplicationDeleteAround(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson)pjp.proceed();
        if(result.isSuccess()) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    Role role = new Role();
                    List<Role> roleList = roleService.findList(role);
                    //更新缓存
                    applicationManageService.resetRoleApplicationCache(roleList);
                }
            }).start();
        }
        return result;
    }

    @Around("excudeGetAppListService()")
    public Object doGetAppListAround(ProceedingJoinPoint pjp) throws Throwable {
        if(optimizeConfig.getOptimizeFirstLoad().equals("true")){
            Object[] args =pjp.getArgs();
            RoleResMenu roleResMenu = (RoleResMenu) args[0];
            if(roleResMenu != null){
                List<Integer> roleIdList = roleResMenu.getRoleList();
                List<Map<String,Object>> allMapAppList = new ArrayList<>();
                if(roleIdList != null && roleIdList.size() > 0){
                    for(Integer roleId:roleIdList){
                        List< Map<String,Object> > appList = (List< Map< String, Object > >) CacheUtils.get(BusinessConstants.ROLE_APPLICATION_LIST+roleId);
                        //如果是超级管理员
                        if(roleId.compareTo(1)==0 && appList != null && appList.size()>0){
                            return appList;
                        }

                        if(appList == null || appList.size()<=0){
                            List< Map<String,Object> > result = ( List< Map<String,Object> >) pjp.proceed();
                            //TODO 更新缓存
                            CacheUtils.set(BusinessConstants.ROLE_APPLICATION_LIST+roleId,result,0L);
                            if(roleId.compareTo(1)==0){
                                return result;
                            }
                            allMapAppList.addAll(result);
                        }else {
                            allMapAppList.addAll(appList);
                        }

                    }

                    //去重
                    for(int i = 0; i < allMapAppList.size();i++){
                        String appId = String.valueOf(allMapAppList.get(i).get("appId"));
                        for(int j = i+1; j < allMapAppList.size();j++){
                            String appIdCompare = String.valueOf(allMapAppList.get(j).get("appId"));
                            if(appId.equals(appIdCompare)){
                                allMapAppList.remove(j);
                            }
                        }

                    }

                    return allMapAppList;
                }else{
                    List< Map<String,Object> > result = ( List< Map<String,Object> >) pjp.proceed();
                    return result;
                }


            }else{
                List< Map<String,Object> > result = ( List< Map<String,Object> >) pjp.proceed();
                return result;
            }
        }else{
            List< Map<String,Object> > result = ( List< Map<String,Object> >) pjp.proceed();
            return result;
        }

    }

    @Around("excudegetApplicationManageService()")
    public Object dogetApplicationManageAround(ProceedingJoinPoint pjp) throws Throwable {
        if(optimizeConfig.getOptimizeFirstLoad().equals("true")) {
            Object[] args = pjp.getArgs();
            Integer appId = (Integer) args[0];
            if (appId != null) {
                ApplicationManage applicationManage = (ApplicationManage) CacheUtils.get(BusinessConstants.APPLICATION_THEME_CACHE + appId);
                if (applicationManage == null) {
                    ApplicationManage applicationM = (ApplicationManage) pjp.proceed();
                    CacheUtils.set(BusinessConstants.APPLICATION_THEME_CACHE + appId, applicationM,0L);
                    return applicationM;
                } else {
                    return applicationManage;
                }
            } else {
                ApplicationManage applicationManage = (ApplicationManage) pjp.proceed();
                return applicationManage;
            }
        }else{
            ApplicationManage applicationManage = (ApplicationManage) pjp.proceed();
            return applicationManage;
        }
    }

    @Around("excudeThemeSetUpdateService()")
    public Object doThemeSetUpdateAround(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson)pjp.proceed();
        if(optimizeConfig.getOptimizeFirstLoad().equals("true")) {
            if (result.isSuccess()) {
                Object[] args = pjp.getArgs();
                Integer appId = (Integer) args[0];
                if (appId != null) {
                    themeManageService.resetApplicationThemeCache(appId);
                }
            }
        }
        return result;
    }


    @Around("excudeGetThemeSetService()")
    public Object doGetThemeSetAround(ProceedingJoinPoint pjp) throws Throwable {
        if(optimizeConfig.getOptimizeFirstLoad().equals("true")) {
            Object[] args = pjp.getArgs();
            String appId = (String) args[0];
            if (StringUtils.isNotEmpty(appId)) {
                ThemeManage themeManage = (ThemeManage) CacheUtils.get(BusinessConstants.APPLICATION_THEME_CACHE+appId);
                return themeManage;
            }else{
                ThemeManage themeManage = (ThemeManage) pjp.proceed();
                return themeManage;
            }
        }else{
            ThemeManage themeManage = (ThemeManage) pjp.proceed();
            return themeManage;
        }


    }

}
