package com.loit.v1.aspect;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import com.loit.common.utils.CacheUtils;
import com.loit.v1.modules.config.OptimizeConfig;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;

import com.loit.common.utils.RedisUtil;
import com.loit.modules.sys.entity.Role;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.entity.MenuManage;
import com.loit.v1.modules.entity.RoleResMenu;
import com.loit.v1.modules.service.MenuManageService;

/**  
* @ClassName: ResourceMenuAspect  
* @Description:   获取应用资源菜单切面类
* @author zhaokz  
* @date 2020年3月17日 下午4:10:50 
* @version V1.0  
*/
@Aspect
@Configuration
public class MenuListAspect {
	
	protected static Logger logger = LoggerFactory.getLogger(MenuListAspect.class);
	
	@Autowired
	private MenuManageService menuManageService;
	@Autowired
	private OptimizeConfig optimizeConfig;
	
	//两个..代表所有子目录,最后括号里面的..指所有参数
	@Pointcut("execution(* com.loit.v1.modules.service.ApplicationManageService.getMenuList(..))") 
    public void excudeService() {
    }
	
	@SuppressWarnings("unchecked")
	@Around("excudeService()")
    public Object doAround(ProceedingJoinPoint pjp) throws Throwable {
		if(optimizeConfig.getOptimizeFirstLoad().equals("true")){
			Object[] args =pjp.getArgs();
			RoleResMenu roleResMenu = (RoleResMenu) args[0];
//		Integer accountId = roleResMenu.getAccountId();
			Integer appId = roleResMenu.getAppId();
			//appId为空默认取应用列表的第一个
			if(appId == null) {
				List<Map<String,Object>> appList = (List<Map<String, Object>>) CacheUtils.get(BusinessConstants.APP_CACHE);
				if(appList != null && appList.size() >0) {
					appId = (Integer) appList.get(0).get("appId");
				}
			}
			User user = UserUtils.getUser();
			List<Role> roleList = user.getRoleList();
			if(roleList != null && roleList.size() > 0 ) {
				List<MenuManage> menuListAll = new ArrayList<MenuManage>();
				for (Role role : roleList) {
					if(role.getAppId().equals(appId)) {
						List<MenuManage> menuList = (List<MenuManage>) CacheUtils.get(BusinessConstants.USER_MENU_CACHE + role.getRoleId() + appId);
						//超级管理员查出来是所有菜单
						if(role.getRoleId() == 1 && menuList != null && menuList.size()>0) {
							return menuList;
						}
						if(menuList == null || menuList.size() < 1) {
							List<MenuManage> result = (List<MenuManage>) pjp.proceed();
							//TODO 更新缓存
							CacheUtils.set(BusinessConstants.USER_MENU_CACHE + role.getRoleId() + appId,result,0L);
							menuListAll.addAll(result);
						}else{
							menuListAll.addAll(menuList);
						}

					}
				}
				if(menuListAll.size() > 0) {
					//排序
					List<MenuManage> listsorted = menuListAll.stream().sorted(Comparator.comparing(MenuManage::getMenuId)).collect(Collectors.toList());
					//去重
					List<MenuManage> listDistinct = listsorted.stream().distinct().collect(Collectors.toList());
					List<MenuManage> listDistinctNew= new ArrayList<MenuManage>();
					Set<Integer> set = new HashSet<Integer>();
					for (MenuManage menu : listDistinct) {
						if (set.add(menu.getMenuId())) {
							listDistinctNew.add(menu);
						}
					}
					for (Integer integer : set) {

					}
					List<MenuManage> resultList = new ArrayList<MenuManage>();
					HashMap<String, MenuManage> map = new HashMap<String, MenuManage>();
					//获取根菜单目录
					List<MenuManage> rootMenuList = (List<MenuManage>) CacheUtils.get(BusinessConstants.ROOT_MENU_CACHE);
					if(rootMenuList != null && rootMenuList.size() > 0) {
						for (MenuManage menuManage1 : rootMenuList) {
							map.put(menuManage1.getMenuId().toString(), menuManage1);
						}
					}
					if(listDistinctNew != null && listDistinctNew.size() > 0) {
						for (MenuManage menuManage : listDistinctNew) {
//						System.out.println("菜单名称------------------"+menuManage.getMenuName());
							map.put(menuManage.getMenuId().toString(), menuManage);
						}
						resultList = getMenuTreeList(listDistinctNew, new HashMap<String, Object>(),new ArrayList<MenuManage>(),map);
						return resultList;
					}
				}

			}
			// result的值就是被拦截方法的返回值
			List<MenuManage> result = (List<MenuManage>) pjp.proceed();
			return result;
		}else{
			List<MenuManage> result = (List<MenuManage>) pjp.proceed();
			return result;
		}

    }
	
	/**
	 * @description 通过子菜单查询菜单的树形列表集合
	 * @param menuList 各级子菜单集合（从叶子菜单到根菜单，初始传入的为叶子菜单集合）
	 * @param map 用于控制父菜单的唯一性
	 * @param resultMenuList  树形菜单集合
	 * @return resultMenuList 树形菜单集合
	 */
	public List<MenuManage> getMenuTreeList(List<MenuManage> menuList,HashMap<String, Object> map,List<MenuManage> resultMenuList,HashMap<String, MenuManage> mapAll){
		if(menuList.size() > 0) {
			//初始化父菜单集合
			List<MenuManage> pList = new ArrayList<MenuManage>();
			//遍历各级子菜单（从叶子菜单到根菜单）
			for (MenuManage menuManage : menuList) {
				//已遍历到根菜单
				if(menuManage.getParentId() == 0) {	
					resultMenuList.add(menuManage);
					continue;
				}
				//获取父菜单对象
				MenuManage pMenuManage = (MenuManage) map.get(menuManage.getParentId().toString());
				if(pMenuManage == null) {
					pMenuManage = mapAll.get(menuManage.getParentId().toString());
					if(pMenuManage == null) {
						pMenuManage = menuManageService.get(menuManage.getParentId());
					}
				}
				//获取父菜单的子菜单集合
				List<MenuManage> childrenlist = pMenuManage.getChildren();
				if(childrenlist == null) {
					childrenlist = new ArrayList<MenuManage>();
				}
				//将子菜单集合添加到父菜单
				if(!childrenlist.contains(menuManage)) {
					childrenlist.add(menuManage);
				}
				
				pMenuManage.setChildren(childrenlist);
				//控制父菜单的唯一性
				if(!map.containsKey(menuManage.getParentId() + "")) {
					map.put(menuManage.getParentId()+"", pMenuManage);
					
					pList.add(pMenuManage);
				}
			}
			//递归调用
			getMenuTreeList(pList,map,resultMenuList,mapAll);
		}
		return resultMenuList.stream().distinct().collect(Collectors.toList());
	}
}
