package com.loit.v1.aspect;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;

import com.loit.common.json.AjaxJson;
import com.loit.common.utils.RedisUtil;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.entity.MenuManage;
import com.loit.v1.modules.service.MenuManageService;
import com.loit.v1.modules.service.ResourceManageService;

/**  
* @ClassName: AppMenuUpdateAspect  
* @Description:   应用资源菜单主题等新增/修改/删除方法切面,用于redis缓存更新
* @author zhaokz  
* @date 2020年3月18日 下午4:05:01 
* @version V1.0  
*/

@Aspect
@Configuration
public class ResourceMenuUpdateAspect {

	protected static Logger logger = LoggerFactory.getLogger(ResourceMenuUpdateAspect.class);
	
	@Autowired
	private MenuManageService menuManageService;
	@Autowired
	private ResourceManageService resourceManageService;
	
	//两个..代表所有子目录,最后括号里面的..指所有参数
	@Pointcut("execution(* com.loit.v1.modules.controller.MenuManageController.update(..))") 
    public void excudeUpdateService() {
		
    }
	@Pointcut("execution(* com.loit.v1.modules.controller.MenuManageController.deleteAll(..))") 
    public void excudeDeleteService() {
		
    }
	
	@Pointcut("execution(* com.loit.v1.modules.controller.ResourceManageController.update(..))") 
    public void resourceUpdateService() {
		
    }
	@Pointcut("execution(* com.loit.v1.modules.controller.ResourceManageController.deleteAll(..))") 
    public void resourceDeleteService() {
		
    }
	
	@Around("excudeUpdateService()")
    public Object doAround1(ProceedingJoinPoint pjp) throws Throwable {
		AjaxJson result = (AjaxJson)pjp.proceed();
		if(result.isSuccess()) {
			new Thread(new Runnable() {
				@Override
				public void run() {
					menuManageService.resetMenuCache();
//					Object[] args =pjp.getArgs();
//					Integer menuId = (Integer) args[0];
//					通过菜单id获取拥有该菜单的角色
//					List<Role> roleList = menuManageService.getRoleIdsByMenuId(menuId);
//					if(roleList.size() > 0) {
//						for (Role role : roleList) {
//							RedisUtil.remove(BusinessConstants.USER_MENU_CACHE + role.getRoleId() + role.getAppId());
//							// 更新缓存
//							List<MenuManage> menuList = menuManageService.findLeafMenuListByRoleId(role.getRoleId(), role.getAppId());
//							RedisUtil.set(BusinessConstants.USER_MENU_CACHE + role.getRoleId() + role.getAppId(), menuList);
//						}
//					}
				}
			}).start();
		}
		return result;
	}
    
	@Around("excudeDeleteService()")
    public Object doAround2(ProceedingJoinPoint pjp) throws Throwable {
//		List<Role> list = new ArrayList<Role>();
//		Object[] args =pjp.getArgs();
//		String menuIds = (String) args[0];
//		if(StringUtils.isNotBlank(menuIds)) {
//			String[] menuIdArray = menuIds.split(",");
//			for (int i = 0; i < menuIdArray.length; i++) {
//				String menuId = menuIdArray[i];
//				//通过菜单id获取拥有该菜单的人员
//				List<Role> roleList = menuManageService.getRoleIdsByMenuId(Integer.valueOf(menuId));
//				list.addAll(roleList);
//			}
//		}
		AjaxJson result = (AjaxJson)pjp.proceed();
		if(result.isSuccess()) {
			new Thread(new Runnable() {
				@Override
				public void run() {
					menuManageService.resetMenuCache();
//					List<Role> listDistinct=(List<Role>) list.stream().distinct().collect(Collectors.toList());
//					if(listDistinct.size() > 0) {
//						for (Role role : listDistinct) {
//							RedisUtil.remove(BusinessConstants.USER_MENU_CACHE + role.getRoleId() + role.getAppId());
//							// 更新缓存
//							List<MenuManage> menuList = menuManageService.findLeafMenuListByRoleId(role.getRoleId(), role.getAppId());
//							RedisUtil.set(BusinessConstants.USER_MENU_CACHE + role.getRoleId() + role.getAppId(), menuList);
//						}
//					}
				}
			}).start();
		}
		return result;
	}
	
	@Around("resourceUpdateService()")
    public Object doAround3(ProceedingJoinPoint pjp) throws Throwable {
		AjaxJson result = (AjaxJson)pjp.proceed();
		if(result.isSuccess()) {
			new Thread(new Runnable() {
				@Override
				public void run() {
					menuManageService.resetMenuCache();
//					Object[] args =pjp.getArgs();
//					Integer resourceId = (Integer) args[0];
//					//通过资源id获取拥有该资源的角色
//					List<Role> roleList = resourceManageService.getRolesByResourceId(resourceId);
//					if(roleList.size() > 0) {
//						for (Role role : roleList) {
//							RedisUtil.remove(BusinessConstants.USER_MENU_CACHE + role.getRoleId() + role.getAppId());
//							// 更新缓存
//							List<MenuManage> menuList = menuManageService.findLeafMenuListByRoleId(role.getRoleId(), role.getAppId());
//							RedisUtil.set(BusinessConstants.USER_MENU_CACHE + role.getRoleId() + role.getAppId(), menuList);
//						}
//					}
				}
			}).start();
		}
		return result;
	}
    
	@Around("resourceDeleteService()")
    public Object doAround4(ProceedingJoinPoint pjp) throws Throwable {
//		List<Role> list = new ArrayList<Role>();
//		Object[] args =pjp.getArgs();
//		String resourceIds = (String) args[0];
//		if(StringUtils.isNotBlank(resourceIds)) {
//			String[] menuIdArray = resourceIds.split(",");
//			for (int i = 0; i < menuIdArray.length; i++) {
//				String resourceId = menuIdArray[i];
//				//通过资源id获取拥有该资源的人员
//				List<Role> roleList = resourceManageService.getRolesByResourceId(Integer.valueOf(resourceId));
//				list.addAll(roleList);
//			}
//		}
		AjaxJson result = (AjaxJson)pjp.proceed();
		if(result.isSuccess()) {
			new Thread(new Runnable() {
				@Override
				public void run() {
					menuManageService.resetMenuCache();
//					List<Role> listDistinct=(List<Role>) list.stream().distinct().collect(Collectors.toList());
//					if(listDistinct.size() > 0) {
//						for (Role role : listDistinct) {
//							RedisUtil.remove(BusinessConstants.USER_MENU_CACHE + role.getRoleId() + role.getAppId());
//							// 更新缓存
//							List<MenuManage> menuList = menuManageService.findLeafMenuListByRoleId(role.getRoleId(), role.getAppId());
//							RedisUtil.set(BusinessConstants.USER_MENU_CACHE + role.getRoleId() + role.getAppId(), menuList);
//						}
//					}
				}
			}).start();
		}
		return result;
	}
	
}
