package com.loit.v1.aspect;

import com.loit.common.json.AjaxJson;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.modules.entity.ResourceOperation;
import com.loit.v1.modules.service.ResourceManageService;
import com.loit.v1.modules.service.ResourceOperationService;
import com.loit.v1.modules.service.RoleService;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description: 资源权限等新增/修改/删除方法切面,用于redis缓存更新
 * @Author: yangwenbin
 * @Date: 2020/5/11 14:01
 */
@Aspect
@Configuration
public class ResourceOperationUpdateAspect {
    protected static Logger logger = LoggerFactory.getLogger(ResourceOperationUpdateAspect.class);
    @Autowired
    private ResourceOperationService resourceOperationService;
    @Autowired
    private ResourceManageService resourceManageService;
    @Autowired
    private RoleService roleService;
    //两个..代表所有子目录,最后括号里面的..指所有参数
    @Pointcut("execution(* com.loit.v1.modules.controller.ResourceManageController.update(..))")
    public void excudeResourceUpdateService() {

    }

    //两个..代表所有子目录,最后括号里面的..指所有参数
    @Pointcut("execution(* com.loit.v1.modules.controller.ResourceManageController.deleteAll(..))")
    public void excudeResourceDeleteService() {

    }


    //两个..代表所有子目录,最后括号里面的..指所有参数
    @Pointcut("execution(* com.loit.v1.modules.controller.ResourceOperationController.save(..))")
    public void excudeOperationSaveService() {

    }

    //两个..代表所有子目录,最后括号里面的..指所有参数
    @Pointcut("execution(* com.loit.v1.modules.controller.ResourceOperationController.update(..))")
    public void excudeOperationUpdateService() {

    }

    //两个..代表所有子目录,最后括号里面的..指所有参数
    @Pointcut("execution(* com.loit.v1.modules.controller.ResourceOperationController.deleteAll(..))")
    public void excudeOperationDeleteService() {

    }

    //两个..代表所有子目录,最后括号里面的..指所有参数
    @Pointcut("execution(* com.loit.v1.modules.controller.ResourceTypeController.update(..))")
    public void excudeResourceTypeUpdateService() {

    }

    @Around("excudeResourceUpdateService()")
    public Object doAround1(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson)pjp.proceed();
        if(result.isSuccess()) {
            new Thread(new Runnable() {
                @Override
                public void run() {
					Object[] args =pjp.getArgs();
					Integer resourceId = (Integer) args[0];
                    //查询当前操作影响到的角色
                    List<Role> roleList = resourceManageService.getRolesByResourceId(resourceId);
                    //更新缓存
                    resourceOperationService.resetOperationCache(roleList);
                }
            }).start();
        }
        return result;
    }

    @Around("excudeResourceDeleteService()")
    public Object doAround2(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson)pjp.proceed();
        if(result.isSuccess()) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    Object[] args =pjp.getArgs();
                    String resourceIds = (String) args[0];
                    //查询当前操作影响到的角色
                    String[] resourceIdArr = resourceIds.split(",");
                    List<Role> allRoleList = new ArrayList<>();
                    for(String resourceId : resourceIdArr){
                        List<Role> roleList = resourceManageService.getRolesByResourceId(Integer.parseInt(resourceId));
                        allRoleList.addAll(roleList);
                    }
                    //去重
                    List newRoleList = allRoleList.stream().distinct().collect(Collectors.toList());
                    //更新缓存
                    resourceOperationService.resetOperationCache(newRoleList);
                }
            }).start();
        }
        return result;
    }

    @Around("excudeOperationSaveService()")
    public Object doAround3(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson)pjp.proceed();
        if(result.isSuccess()) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    Object[] args =pjp.getArgs();
                    ResourceOperation resourceOperation = (ResourceOperation) args[0];
                    if(resourceOperation != null){
                        //查询当前操作影响到的角色
                        List<Role> roleList = resourceManageService.getRolesByResourceId(resourceOperation.getResId());
                        //更新缓存
                        resourceOperationService.resetOperationCache(roleList);
                    }

                }
            }).start();
        }
        return result;
    }

    @Around("excudeOperationUpdateService()")
    public Object doAround4(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson)pjp.proceed();
        if(result.isSuccess()) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    Object[] args =pjp.getArgs();
                    ResourceOperation resourceOperation = (ResourceOperation) args[1];
                    if(resourceOperation != null) {
                        //查询当前操作影响到的角色
                        List<Role> roleList = resourceManageService.getRolesByResourceId(resourceOperation.getResId());
                        //更新缓存
                        resourceOperationService.resetOperationCache(roleList);
                    }
                }
            }).start();
        }
        return result;
    }

    @Around("excudeOperationDeleteService()")
    public Object doAround5(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson)pjp.proceed();
        if(result.isSuccess()) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    Role role = new Role();
                    List<Role> roleList = roleService.findList(role);
                    //更新缓存
                    resourceOperationService.resetOperationCache(roleList);
                }
            }).start();
        }
        return result;
    }

    @Around("excudeResourceTypeUpdateService()")
    public Object doAround6(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson)pjp.proceed();
        if(result.isSuccess()) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    Role role = new Role();
                    List<Role> roleList = roleService.findList(role);
                    //更新缓存
                    resourceOperationService.resetOperationCache(roleList);
                }
            }).start();
        }
        return result;
    }
}
