package com.loit.v1.aspect;

import com.loit.common.json.AjaxJson;
import com.loit.common.utils.CacheUtils;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.entity.UserLogLevel;
import com.loit.v1.modules.service.UserLogLevelService;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *用于获得用户的自定义日志级别的内容
 */
@Aspect
@Configuration
public class UserLogLevelAspect {

    @Autowired
    private UserLogLevelService userLogLevelService;

    //两个..代表所有子目录,最后括号里面的..指所有参数
    @Pointcut("execution(* com.loit.v1.modules.controller.UserLogLevelController.save(..))")
    public void levelSaveService() {

    }
    @Pointcut("execution(* com.loit.v1.modules.controller.UserLogLevelController.update(..))")
    public void levelUpdateService() {

    }
    @Pointcut("execution(* com.loit.v1.modules.controller.UserLogLevelController.delete(..))")
    public void levelDeleteService() {
    }

    @Around("levelSaveService()")
    public Object doAround1(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson) pjp.proceed();
        commonInfo(result);
        return result;
    }
    @Around("levelUpdateService()")
    public Object doAround2(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson) pjp.proceed();
        commonInfo(result);
        return result;
    }
    @Around("levelDeleteService()")
    public Object doAround3(ProceedingJoinPoint pjp) throws Throwable {
        AjaxJson result = (AjaxJson) pjp.proceed();
        commonInfo(result);
        return result;
    }

    /**
     * 用于定义启动的方法
     * @param result
     */
    public void commonInfo(AjaxJson result){
        if (result.isSuccess()) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    commonCache();
                }
            }).start();
        }
    }

    /**
     * 用于更新用户的日志信息
     */
    public void  commonCache(){
        List<UserLogLevel> list = userLogLevelService.findList(new UserLogLevel());
        if(list!=null&&!list.isEmpty()){
            Map<String,String> map= new HashMap<String,String>();
            for(UserLogLevel level:list){
                map.put(level.getUserid(),level.getLogLevel());
            }
            CacheUtils.remove(BusinessConstants.USER_LOG_LEVEL);  //先进行移除，然后再进行添加
            CacheUtils.set(BusinessConstants.USER_LOG_LEVEL , map,300L);
        }
    }
}
