package com.loit.v1.audit.alarm;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.stereotype.Service;

/**
 * @Description: email警告
 * @author: Administrator
 * @date: 2019年10月31日 下午3:29:12
 * @version: V1.0
 */
@Service
public class EmailAlarm {
	
	private final static Logger logger = LoggerFactory.getLogger(EmailAlarm.class);
	
	@Autowired
    private JavaMailSender mailSender;
	@Value("${spring.mail.username}")
	private String fromEmail;
	@Value("${spring.mail.subject}")
	private String emailSubject;
	
	/**
	* @Description: 邮件警告
	* @author: gzx
	* @date: 2019年10月28日 下午7:23:06 
	* @param @param alertContent 警告内容
	* @param @param alertTarget  警告对象
	* @param @return 参数 
	* @return boolean 返回类型 
	* @throws
	 */
	public void sendEmail(String alertContent,String alertTarget) {
		
		// 构造Email消息
	    SimpleMailMessage message = new SimpleMailMessage();
	    message.setFrom(fromEmail);
	    message.setTo(alertTarget);
	    message.setSubject(emailSubject);
	    message.setText(alertContent);
	    mailSender.send(message);
	    logger.info("发送邮件,内容：{},接受邮箱：{}",alertContent,alertTarget);
	}
}
