package com.loit.v1.audit.service;

import java.util.Collections;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.springframework.stereotype.Service;
import com.loit.common.service.CrudService;
import com.loit.v1.audit.dao.AuditStrategyDao;
import com.loit.v1.audit.entity.AuditStrategy;

/**
 * @ClassName: AuditStrategyService
 * @Description: 行为审核策略表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Service
public class AuditStrategyService extends CrudService<AuditStrategyDao, AuditStrategy> {
	
	//加载审计信息
	private static boolean isLoaded = false;
	/**
	 * 缓存的配置项
	 */
	private static Map<String,AuditStrategy> auditMapItems =  Collections.synchronizedMap(new Hashtable<String,AuditStrategy>());

	/**
	* @Description: 根据类名和方法名获取审计策略及相应的审计等级、警告类型
	* @author: gzx
	* @date: 2019年10月27日 下午5:50:32 
	* @param @param className  类名
	* @param @param method     方法
	* @param @return 参数 
	* @return AuditStrategy 返回类型 
	* @throws
	 */
	public AuditStrategy getStrategyByClassAndMethod(String className,String method) {
		if(!isLoaded) {
			loadAuditStrategy();
		}
		String key = new StringBuffer(className.trim()).append(method.trim()).toString();
		return auditMapItems.get(key);
	}
	
	/**
	* @Description: 缓存审计策略信息
	* @author: gzx
	* @date: 2019年10月28日 上午10:58:28 
	* @param  参数 
	* @return void 返回类型 
	* @throws
	 */
	public void loadAuditStrategy() {
		if (isLoaded) {
			return;
		}
		auditMapItems.clear();
		AuditStrategy auditStrategy = new AuditStrategy();
		auditStrategy.setEnabled("1");
		List<AuditStrategy> strategyList = dao.findList(auditStrategy);
		if(strategyList != null && strategyList.size() > 0) {
			for (int i = 0; i < strategyList.size(); i++) {
				AuditStrategy strategyTemp = strategyList.get(i);
				auditMapItems.put((new StringBuffer(String
						.valueOf(strategyTemp.getActClass().trim())))
						.append(strategyTemp.getActMethod().trim()).toString(),
						strategyTemp);
			}
		}
		isLoaded = true;
	}
}