package com.loit.v1.modules.config;

import cn.jiguang.common.ClientConfig;
import cn.jpush.api.JPushClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;

/**
 * @description: 极光推送参数配置
 * @date: 2020/5/12 15:26
 * @author: wubj
 * @version v1.0
 */
@Configuration
@ConfigurationProperties(prefix = "jpush")
public class JPushConfig {

    /**
     * 极光申请的app key
     */
    private String appKey;

    /**
     * 极光申请的secret
     */
    private String masterSecret;

    /**
     * 失效时间，单位:秒
     */
    private Long liveTime;

    /**
     * 是否生成环境，true表示生成环境
     */
    private Boolean apnsProduction;

    /**
     * jPushClient
     */
    private JPushClient jPushClient;

    // 推送客户端
    @PostConstruct
    public void initJPushClient() {
        if (masterSecret != null && appKey != null && liveTime != null && apnsProduction != null) {
            ClientConfig clientConfig = ClientConfig.getInstance();
            clientConfig.setTimeToLive(liveTime);
            clientConfig.setApnsProduction(apnsProduction);
            jPushClient = new JPushClient(masterSecret, appKey, null, clientConfig);
        }
    }

    // 获取推送客户端
    public JPushClient getJPushClient() {
        return jPushClient;
    }

    public String getAppKey() {
        return appKey;
    }

    public void setAppKey(String appKey) {
        this.appKey = appKey;
    }

    public String getMasterSecret() {
        return masterSecret;
    }

    public void setMasterSecret(String masterSecret) {
        this.masterSecret = masterSecret;
    }

    public Long getLiveTime() {
        return liveTime;
    }

    public void setLiveTime(Long liveTime) {
        this.liveTime = liveTime;
    }

    public void setApnsProduction(Boolean apnsProduction) {
        this.apnsProduction = apnsProduction;
    }

    public Boolean getApnsProduction() {
        return apnsProduction;
    }
}
