package com.loit.v1.modules.config;

import com.loit.v1.modules.handler.HandshakeInterceptor;
import com.loit.v1.modules.handler.MyWebSocketHandler;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.socket.config.annotation.EnableWebSocket;
import org.springframework.web.socket.config.annotation.WebSocketConfigurer;
import org.springframework.web.socket.config.annotation.WebSocketHandlerRegistration;
import org.springframework.web.socket.config.annotation.WebSocketHandlerRegistry;
import org.springframework.web.socket.server.standard.ServletServerContainerFactoryBean;

/**
 * @description: 添加拦截地址以及相应的websocket消息处理器
 * @date: 2020/4/26 15:24
 * @author: wubj
 * @version v1.0
 */
@Configuration
@EnableWebSocket
public class WebSocketConfig implements WebSocketConfigurer {

    private static final int MAX_MESSAGE_SIZE = 200 * 1024;
    private static final long MAX_IDLE = 60 * 60 * 1000;

    @Override
    public void registerWebSocketHandlers(WebSocketHandlerRegistry registry) {
        // 添加拦截地址以及相应的websocket消息处理器
        WebSocketHandlerRegistration registration = registry.addHandler(new MyWebSocketHandler(), "/websocket").setAllowedOrigins("*");
        // 添加拦截器
        registration.addInterceptors(new HandshakeInterceptor());
    }

    @Bean
    public ServletServerContainerFactoryBean createWebSocketContainer() {
        ServletServerContainerFactoryBean container = new ServletServerContainerFactoryBean();
        // Websocket通信时，只能传输小于8K的消息，大于8K的消息都不能进行传输
        // ws传输数据的时候，数据过大有时候会接收不到，所以在此处设置bufferSize
        container.setMaxTextMessageBufferSize(MAX_MESSAGE_SIZE);
        container.setMaxBinaryMessageBufferSize(MAX_MESSAGE_SIZE);
        container.setMaxSessionIdleTimeout(MAX_IDLE);
        return container;
    }
}
