package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.utils.StringUtils;
import com.loit.v1.modules.entity.AccountPolicy;
import com.loit.v1.modules.service.AccountPolicyService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

@RestController
@RequestMapping(value = "api/v1/accountPolicy")
@Api(tags = "密码策略管理",  value = "AccountPolicyController")
public class AccountPolicyController {
	@Autowired
	private AccountPolicyService accountPolicyService;
	
	@ApiOperation(value = "获取密码策略分页列表", notes = "获取密码策略分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "mixLevel", value = "密码登记", required = false, dataType = "String", paramType = "query")
	})
	
	@GetMapping(value = "all")
	public AjaxJson all(@ApiIgnore AccountPolicy accountPolicy){
		AjaxJson json = new AjaxJson();
		List<AccountPolicy> list = accountPolicyService.findList(accountPolicy);
		json.put("list", list);
		return json;
	}
	
	@ApiOperation(value = "获取密码策略分页列表", notes = "获取密码策略分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "mixLevel", value = "密码登记", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response,@ApiIgnore AccountPolicy accountPolicy){
		AjaxJson json = new AjaxJson();
		Page<AccountPolicy> pageList = accountPolicyService.findPage(new Page<AccountPolicy>(request, response),accountPolicy);
		json.put("page", pageList);
		return json;
	}
	
	/**
	 * 获取审核策略详细信息
	 * @param orgId
	 * @return
	 */
	@ApiOperation(value = "获取密码策略详细信息", notes = "根据id来获取密码策略详细信息")
	@GetMapping(value = "/{passPolicyId}")
	public AjaxJson get(@PathVariable("passPolicyId") String passPolicyId) {
		if(StringUtils.isBlank(passPolicyId)) {
			return AjaxJson.returnExceptionInfo("获取密码策略详情异常，请联系管理员!");
		}
		AccountPolicy accountPolicy = accountPolicyService.get(passPolicyId);
		AjaxJson json = new AjaxJson();
		json.put("obj", accountPolicy);
		return json;
	}

	
	/**
	 * 保存
	 * @param org
	 * @return
	 */
	@ApiOperation(value = "保存/修改", notes = "保存/修改密码策略")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "passPolicyId", value = "密码策略ID。只有在修改时有值", required = false, dataType = "int", paramType = "query") ,
		@ApiImplicitParam(name = "minPass", value = "密码长度最小值", required = true, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "maxTime", value = "密码最长期限", required = true, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "minTime", value = "密码最短期限", required = true, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "mixLevel", value = "密码复杂度级别", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "passDesc", value = "策略描述", required = false, dataType = "String", paramType = "query")
	})
	@PostMapping(value = "/save")
	public AjaxJson save(@RequestBody @ApiIgnore AccountPolicy accountPolicy) {
		AjaxJson json = new AjaxJson();
		try {
			accountPolicyService.save(accountPolicy, accountPolicy.getPassPolicyId());
			json.setMsg("保存密码策略成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存密码策略异常");
		}
		return json;
	}
	
	/**
	 * 删除
	 *@param 
	 */
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "passPolicyIds", value = "passPolicyIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "{passPolicyIds}")
	public AjaxJson delete(@PathVariable("passPolicyIds") String passPolicyIds) {
		AjaxJson json = new AjaxJson();
		if(StringUtils.isBlank(passPolicyIds)) {
			return AjaxJson.returnExceptionInfo("审核策略id为空,请选择相应的列表!");
		}else {
			try {
				accountPolicyService.deleteAll(passPolicyIds, new AccountPolicy());
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除审核策略失败" + e.getMessage());
			}
		}
		return json;
		
	}
}
