package com.loit.v1.modules.controller;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.v1.api.dto.ActRightDTO;
import com.loit.v1.modules.entity.ActRight;
import com.loit.v1.modules.service.ActRightService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**  
* @ClassName: ActRightController  
* @Description:   
* @author zhaokz  
* @date 2019年10月28日 下午3:56:15 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/actRight")
@Api(tags = "资源操作权限管理",  value = "资源操作权限管理Controller")
public class ActRightController {

	@Autowired
	private ActRightService actRightService;
	
	/**
	 * 获取资源操作权限列表
	 * @param request
	 * @param response
	 * @param actRight
	 * @return
	 */
	@ApiOperation(value = "获取资源操作权限列表-分页", notes = "获取资源操作权限列表-分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "actRight", value = "详细实体ActRight的参数值可设为查询条件", required = false, dataType = "ActRight", paramType = "query") 
	})
	@GetMapping(value = "")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response,@ApiIgnore ActRight actRight){
		Page<ActRight> pageList = actRightService.findPage(new Page<ActRight>(request, response),
				actRight);
		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
	}

	@ApiOperation(value = "获取资源操作权限列表-不分页", notes = "获取资源操作权限列表-不分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "roleId", value = "角色ID", required = false, dataType = "int", paramType = "query") ,
		@ApiImplicitParam(name = "resId", value = "资源ID", required = false, dataType = "int", paramType = "query") 
	})
	@GetMapping(value = "allList")
	public AjaxJson list(@ApiIgnore ActRight actRight){
		List<ActRight> actRightList = actRightService.findAllList(actRight);
		AjaxJson json = new AjaxJson();
		json.put("list", actRightList);
		return json;
	}
	
	@ApiOperation(value = "获取角色资源操作权限IDs", notes = "获取角色资源操作权限IDs")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "roleId", value = "角色ID", required = true, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "resId", value = "资源ID", required = true, dataType = "int", paramType = "query") 
	})
	@GetMapping(value = "getIdsByRoleIdAndResId")
	public AjaxJson getIdsByRoleIdAndResId(@ApiIgnore ActRight actRight){
		List<Integer> list = actRightService.getIdsByRoleIdAndResId(actRight);
		AjaxJson json = new AjaxJson();
		json.put("actIdArr", list);
		return json;
	}
	
	/**
	 * 获取资源操作权限详细信息
	 * @param resActId
	 * @return
	 */
	@ApiOperation(value = "获取资源操作权限详细信息", notes = "根据id来获取ActRight详细信息")
	@ApiImplicitParam(name = "resActId", value = "resActId", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/{resActId}")
	public AjaxJson detail(@PathVariable String resActId) {
		AjaxJson json = new AjaxJson();
		ActRight newActRight = new ActRight();
		newActRight.setResActId(Integer.valueOf(resActId));
		ActRight actRight = actRightService.get(newActRight);
		ActRightDTO actRightDTO = null;
		if(actRight != null) {
			actRightDTO = ModelMapperUtils.map(actRight, ActRightDTO.class);
		}
		json.put("actRight", actRightDTO);
		return json;
	}

	
	/**
	 * 保存资源操作权限
	 * @param actRight
	 * @return
	 */
	@ApiOperation(value = "保存角色资源操作权限", notes = "保存角色资源操作权限")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "roleId", value = "角色ID", required = true, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "resId", value = "资源ID", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "actIds", value = "操作ID,逗号隔开", required = true, dataType = "String", paramType = "query")
	})
	@PostMapping(value = "/save")
	public AjaxJson save(@RequestBody @ApiIgnore Map<String,Object> map) {
		AjaxJson json = new AjaxJson();
		try {
			if(map == null || !map.containsKey("roleId") || map.get("roleId") == null) {
				return AjaxJson.returnExceptionInfo("请先选择角色！");
			}
			if(map == null || !map.containsKey("resId") || map.get("resId") == null) {
				return AjaxJson.returnExceptionInfo("请先选择资源！");
			}
			actRightService.dealActRight(map);
			json.setMsg("保存成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存资源操作权限异常" + e.getMessage());
		}
		return json;
	}
	

	@ApiOperation(value = "根据菜单ID获取当前用户对本资源的操作权限", notes = "根据菜单ID获取当前用户对本资源的操作权限")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "menuId", value = "菜单ID", required = true, dataType = "int", paramType = "query") 
	})
	@GetMapping(value = "getActRightByMenuId")
	public AjaxJson getActRightByMenuId(@ApiIgnore Integer menuId){
		AjaxJson json = new AjaxJson();
		json.put("rightJsonStr", actRightService.getActRightByMenuId(menuId));
		return json;
	}

}
