package com.loit.v1.modules.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.fastjson.JSONObject;
import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.api.dto.AppOrgDTO;
import com.loit.v1.modules.entity.AppOrg;
import com.loit.v1.modules.service.AppOrgService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;

/**  
* @ClassName: AppOrgController  
* @Description:   
* @author zhaokz  
* @date 2019年10月24日 下午1:38:27 
* @version V1.0  
*/
@RestController
@RequestMapping(value = "api/v1/appOrg")
@Api(tags = "应用组织管理Controller",  value = "AppOrgController")
public class AppOrgController extends BaseController{

	@Autowired
	AppOrgService appOrgService;

	/**
	 * 获取已分配列表  
	 * @param org
	 * @return
	 */
	@ApiOperation(value = "获取已分配列表", notes = "获取已分配列表")
	@ApiImplicitParam(name = "appId", value = "应用id", required = true, dataType = "String")
	@GetMapping(value = "/orgAttrList")
	public AjaxJson orgAttrList(@RequestParam String appId){
		AppOrg appOrg = new AppOrg();
		appOrg.setAppId(Integer.valueOf(appId));
		List<AppOrgDTO> list = appOrgService.findAppOrgList(appOrg);
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}
	
	
	/**
	 * 添加应用权限
	 * @param org
	 * @return
	 */
	@ApiOperation(value = "添加应用权限", notes = "保存对象信息")
	@ApiImplicitParam(name = "appOrg", value = "应用组织", required = false, dataType = "AppOrg")
	@PostMapping(value = "/save")
	public AjaxJson save(@RequestBody AppOrg appOrg) {
		AjaxJson json = new AjaxJson();
		if(appOrg.getOrgId() == null) {
			json.setSuccess(false);
			json.setMsg("请选择组织");
			return json;
		}
		boolean result =  appOrgService.check(appOrg);
		if(result) {
			try {
				appOrgService.insertAppOrg(appOrg);
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
			}
		}else {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.APP_ORG_ASSIGN);
		}
		json.setMsg("权限设置成功!");
		json.setSuccess(true);
		return json;
	}
	
	
	/**
	 * 删除
	 *@param 
	 */
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "relIds", value = "relIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/{relIds}")
	public AjaxJson delete(@PathVariable String relIds) {
		if(StringUtils.isBlank(relIds)) {
			return AjaxJson.returnExceptionInfo("id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(relIds)) {
			AppOrg appOrg = new AppOrg();
			try {
				appOrgService.deleteAppOrg(relIds, appOrg);
				json.setMsg("删除成功!");
				json.setSuccess(true);
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
			}
		}
		return json;
	}
	
	public static void main(String[] args) {
		String appOrgsStr = "[{appId=1, orgId=5, orgName=北京市环境监测管理局, orgFlagName=所选及下属, orgFlag=3}]";
		JSONObject jsonObject = JSONObject.parseObject(appOrgsStr);
		List<AppOrg> appOrgs = jsonObject.parseArray(jsonObject.toJSONString(), AppOrg.class);
		for (AppOrg appOrg : appOrgs) {
			System.out.println(appOrg.getAppId());
		}
	}
}
