package com.loit.v1.modules.controller;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.api.dto.ApplicationManageDTO;
import com.loit.v1.modules.entity.AppResourceRelation;
import com.loit.v1.modules.entity.ApplicationManage;
import com.loit.v1.modules.entity.MenuManage;
import com.loit.v1.modules.entity.RoleResMenu;
import com.loit.v1.modules.service.ApplicationManageService;
import com.loit.v1.modules.service.MenuManageService;
import com.loit.v1.modules.service.ResourceManageService;
import com.loit.v1.modules.service.RightShortcutService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;
/**
 * @description 应用管理Controller
 * @author xutao
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/applicationManage")
@Api(value = "ApplicationManageController", tags = {"应用注册管理Controller"})
public class ApplicationManageController extends BaseController {
	
	@Autowired
	private ApplicationManageService applicationManageService;
	@Autowired
	private ResourceManageService resourceManageService;
	@Autowired
	private RightShortcutService rightShortcutService;

	/**
	 * @description 应用管理列表
	 * @author xutao
	 * @date 2019年10月18日
	 * @param applicationManage 应用管理实体类	
	 * @return
	 */
	@GetMapping(value = "list")
	@ApiOperation(value="列表", notes="获取应用管理列表")	
	@ApiImplicitParam(name = "appId", value = "应用ID", required = false, dataType = "String",paramType = "query")
	public AjaxJson list(@ApiIgnore ApplicationManage applicationManage) {
		List<ApplicationManage> list = applicationManageService.findAllList(applicationManage);
		//logger.debug("获取应用管理列表成功");
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}
	
	/**
	 * @description 应用注册管理分页列表
	 * @author xutao
	 * @date 2019年10月18日
	 * @param applicationManage 应用管理实体类	
	 * @return
	 */
	@GetMapping(value = "page")
	@ApiOperation(value="分页列表", notes="获取应用注册管理分页列表")	
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "appName", value = "应用系统名称", required = false, dataType = "String",paramType = "query")
	})
	public AjaxJson page(@ApiIgnore ApplicationManage applicationManage, HttpServletRequest request, HttpServletResponse response) {
		Page<ApplicationManage> page = applicationManageService.findPage(new Page<ApplicationManage>(request, response), applicationManage);
		//logger.debug("获取应用注册管理分页列表成功");
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}
	
	/**
	 * @description 获取应用分配资源的列表
	 * @author xutao
	 * @date 2019年10月18日
	 * @return
	 */
	@GetMapping(value = "distributeList/{appId}")
	@ApiOperation(value="获取应用分配资源的列表", notes="获取应用分配资源的列表")	
	@ApiImplicitParam(name = "appId", value = "应用ID", required = true, dataType = "String")
	public AjaxJson distributeList(@PathVariable(required=false) Integer appId) {
		List<Map<String, Object>> treeList = resourceManageService.treeData(appId);
		List<AppResourceRelation> selectedResList = applicationManageService.getAppResIds(appId);
		//logger.debug("获取应用注册管理分页列表成功");
		AjaxJson json = new AjaxJson();
		json.put("treeList", treeList);
		json.put("selectedResList", selectedResList);
		return json;
	}
	

	/**
	 * @description 应用详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param appId
	 * @return
	 */
	@GetMapping(value = "{appId}")
	@ApiOperation(value="详情", notes="根据appId获取应用详情")
	@ApiImplicitParam(name = "appId", value = "应用ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String appId) {	
		if(StringUtils.isBlank(appId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		ApplicationManage applicationManage = applicationManageService.get(appId);
		//logger.debug("获取应用详情成功");
		AjaxJson json = new AjaxJson();
		json.put("obj", applicationManage);
		return json;
	}


	/**
	 * @description 新增应用
	 * @author xutao
	 * @date：2019年10月18日
	 * @param applicationManageDTO
	 * @return
	 */
	@PostMapping(value = "save")
	@ApiOperation(value = "保存", notes = "新增应用")
	@ApiImplicitParam(name = "applicationManageDTO", value = "应用注册管理数据传输实体类", required = true, dataType = "ApplicationManageDTO")
	public AjaxJson save(@RequestBody ApplicationManageDTO applicationManageDTO) {
		AjaxJson json = new AjaxJson();	
		ApplicationManage applicationManage = ModelMapperUtils.map(applicationManageDTO, ApplicationManage.class);
		try {
			boolean uniqueFlag = applicationManageService.checkNameUnique(applicationManage.getAppId(), applicationManage.getAppName());
			if(uniqueFlag == false) {
				return AjaxJson.returnExceptionInfo(LoitStatusCode.APP_IS_EXIST);
			}else {
				// Demon add by 20200509: 是否在应用切换列表工具条中显示（0：否 1：是）,如果不设置则默认是
				applicationManage.setShowFlag(StringUtils.isEmpty(applicationManage.getShowFlag()) ? "1" : applicationManage.getShowFlag());

				applicationManageService.insert(applicationManage);
				json.setMsg("应用保存成功");
			}			
			return json;
		} catch (Exception e) {
			logger.error("info:应用保存失败,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
		}		
	}
	
	
	/**
	 * @description 修改应用
	 * @param appId
	 * @param applicationManageDTO
	 * @return
	 */
	@PostMapping(value = "{appId}")
	@ApiOperation(value = "修改", notes = "修改应用")
	public AjaxJson update(@PathVariable int appId,@RequestBody ApplicationManageDTO applicationManageDTO) {
		AjaxJson json = new AjaxJson();
		ApplicationManage applicationManage = ModelMapperUtils.map(applicationManageDTO, ApplicationManage.class);
		try {
			applicationManage.setAppId(appId);
			applicationManageService.update(applicationManage);
			json.setMsg("应用修改成功");
			return json;
		} catch (Exception e) {
			logger.error("info:应用修改失败,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.UPDATE_EXCEPTION);
		}		
	}

	/**
	 * @description 应用删除
	 * @author xutao
	 * @date：2019年10月18日
	 * @param ids 多ID时，以逗号分隔
	 * @return
	 */
	@ApiOperation(value = "删除", notes = "单个或批量删除应用，多ID时，以逗号分隔")
	@DeleteMapping(value = "{appId}")
	public AjaxJson deleteAll(@PathVariable("appId")String ids) {
		if(StringUtils.isBlank(ids)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		AjaxJson json = new AjaxJson();
		try {
			String message = applicationManageService.deleteAllByLogic(ids);			
			json.setMsg(message);
			return json;
		} catch (Exception e) {
			logger.error("info:删除异常,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.DELETE_EXCEPTION);
		}
	}
	
	/**
	 * @description 分配资源
	 * @author xutao
	 * @date：2019年10月18日
	 * @return
	 */
	@ApiOperation(value = "分配资源", notes = "给应用分配单个或者多个资源，多ID时，以逗号分隔")
	@PostMapping(value = "distributeResources")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "resourceIds", value = "资源ID,多ID时，以逗号分隔", required = true, dataType = "String",
				paramType = "query"),
		@ApiImplicitParam(name = "appId", value = "应用ID", required = true, dataType = "String",
		paramType = "query")		
	})
	public AjaxJson distributeResources(@RequestBody String mapStr) {		
		AjaxJson json = new AjaxJson();
		try {
			applicationManageService.distributeResources(mapStr);
			json.setMsg("资源分配成功");
			return json;
		} catch (Exception e) {
			logger.error("info:资源分配异常,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.DISTRIBUTE_RESOURCES_EXCEPTION);		
		}
	}
	
	/**
	 * @description 获取当前用户所拥有的应用列表及菜单树形列表及当前应用的主题，皮肤，菜单布局方式等
	 * @author xutao
	 * @date 2019年10月18日
	 * @param roleResMenu 用户角色资源菜单应用关系对象	
	 * @return
	 */
	@GetMapping(value = "getCurrentUserAppMenu")
	@ApiOperation(value="获取当前用户所拥有的应用列表及菜单树形列表及当前应用的主题，皮肤，菜单布局方式等", notes="获取当前用户所拥有的应用列表及菜单树形列表及当前应用的主题，皮肤，菜单布局方式等")	
	@ApiImplicitParams({
		@ApiImplicitParam(name = "accountId", value = "用户账号ID", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "accountType", value = "账号类型 0:普通帐号 1:应用管理员账号2:应用管理帐号 3:超级管理帐号", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "appId", value = "应用ID", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "appCode", value = "应用编码", required = false, dataType = "String", paramType = "query")
	})
	public AjaxJson getCurrentUserAppMenu(@ApiIgnore RoleResMenu roleResMenu) {
		//获取当前用户
		User user = UserUtils.getUser();

		// appId和appCode不能同时存在
		if(StringUtils.isNotEmpty(roleResMenu.getAppCode()) && roleResMenu.getAppId() != null){
			return AjaxJson.returnExceptionInfo(LoitStatusCode.APPLICATION_APP_ID_CODE_SIMULTANEOUSLYEXIST_ERRO);
		}

		// 无用户信息
		if(user == null || user.getAccountId() == null) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.CURRENT_USER_ERROR);
		}

		return applicationManageService.getCurrentUserAppMenu(roleResMenu, user);
	}
	
	/**
	 * @description 检查应用系统名称是否重复
	 * @param appId
	 * @param appName
	 * @return
	 */
	@ApiOperation(value = "检查应用系统名称是否重复", notes = "检查应用系统名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用id", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "appName", value = "应用名称", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkAppNameUnique")
	public AjaxJson checkAppNameUnique(Integer appId,String appName) {
		boolean uniqueFlag = applicationManageService.checkNameUnique(appId, appName);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("应用系统名称不重复");
		} else {
			json.setMsg("应用系统名称重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}
	
	/**
	 * @description 检查应用系统代码是否重复
	 * @param appId
	 * @param appCode
	 * @return
	 */
	@ApiOperation(value = "检查应用系统代码是否重复", notes = "检查应用系统代码是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用id", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "appCode", value = "应用代码", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkAppCodeUnique")
	public AjaxJson checkAppCodeUnique(Integer appId,String appCode) {
		boolean uniqueFlag = applicationManageService.checkCodeUnique(appId, appCode);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("应用系统代码不重复");
		} else {
			json.setMsg("应用系统代码重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}

}
