package com.loit.v1.modules.controller;

import java.util.List;
import java.util.stream.Collectors;
import com.loit.common.utils.ListUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.loit.common.json.AjaxJson;
import com.loit.v1.modules.entity.AreaCode;
import com.loit.v1.modules.service.AreaCodeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;

/**  
* @ClassName: OrgController  
* @Description:   
* @author zhaokz  
* @date 2019年10月22日 下午5:41:41 
* @version V1.0  
*/
@RestController
@RequestMapping(value = "api/v1/area")
@Api(tags = "行政区域管理Controller",  value = "AreaCodeController")
public class AreaCodeController {
	
	@Autowired
	private AreaCodeService areaCodeService;
	
	@ApiOperation(value = "行政区域列表-根据父id", notes = "行政区域列表-根据父id")
	@ApiImplicitParam(name = "pcode", value = "父id", required = true, dataType = "String",paramType = "query")
	@GetMapping(value = "/areaList")
	public AjaxJson areaCodeList(@RequestParam(value = "pcode", required = true) Long pcode) {
		AjaxJson json = new AjaxJson();
		AreaCode areaCode = new AreaCode();
		areaCode.setPcode(pcode);
		List<AreaCode> areaList = areaCodeService.findList(areaCode);
		json.put("areaList", areaList);
		json.setSuccess(true);
		return json;
	}

	@ApiOperation(value = "行政区域树-根据父id", notes = "行政区域树-根据父id")
	@ApiImplicitParam(name = "pcode", value = "id", required = true, dataType = "String",paramType = "query")
	@GetMapping(value = "/areaList/tree")
	public AjaxJson areaCodeListTree(@RequestParam(value = "pcode", required = true) Long pcode) {
		AjaxJson json = new AjaxJson();
		AreaCode areaCode = new AreaCode();
		areaCode.setArealevel(String.valueOf(pcode));
		List<AreaCode> areaList = areaCodeService.findList(areaCode);
		json.put("areaList", tree(areaList));
		json.setSuccess(true);
		return json;
	}

	/**
	 * 递归构建父子结构树
	 * @param areaList
	 * @return
	 */
	private List<AreaCode> tree(List<AreaCode> areaList){
		if(ListUtil.isEmpty(areaList)){
			return areaList;
		}

		for (AreaCode areaCode : areaList) {
			List<AreaCode> children = areaList.stream().filter(e -> e.getPcode().equals(areaCode.getCode())).collect(Collectors.toList());
			areaCode.setChildren(tree(children));
		}

		return areaList;
	}

}
