package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.loit.v1.audit.entity.AuditLevel;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.audit.entity.AuditActCode;
import com.loit.v1.audit.service.AuditActCodeService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;
/**
 * @Description: 审核策略模块——审核行为代码维护
 * @author: gzx
 * @date: 2019年11月1日 下午5:40:15
 * @version: V1.0
 */
@RestController
@RequestMapping(value = "api/v1/actCode")
@Api(value = "AuditActCodeController", tags = {"行为代码Controller"})
public class AuditActCodeController  extends BaseController {
	
	@Autowired
	private  AuditActCodeService auditActCodeService;
	
	/**
	 * @description 审核行为代码分页列表
	 * @author gzx
	 * @date 2019年11月1日
	 * @param auditActCode 行为代码实体类	
	 * @return
	 */
	@GetMapping(value = "")
	@ApiOperation(value="分页列表", notes="获取行为对象类别列表")	
	public AjaxJson page(@ApiIgnore AuditActCode auditActCode, HttpServletRequest request, HttpServletResponse response) {
		Page<AuditActCode> page = auditActCodeService.findPage(new Page<AuditActCode>(request, response), auditActCode);
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}
	

	/**
	 * @description 行为代码详情
	 * @author gzx
	 * @date：2019年10月18日
	 * @param actCodeId
	 * @return
	 */
	@GetMapping(value = "/{actCodeId}")
	@ApiOperation(value="详情", notes="根据objectTypeId获取行为代码详情")
	@ApiImplicitParam(name = "actCodeId", value = "行为代码ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String actCodeId) {	
		if(StringUtils.isBlank(actCodeId)) {
			return AjaxJson.returnExceptionInfo("获取行为代码详情异常，请联系管理员");
		}
		AuditActCode auditActCode = auditActCodeService.get(actCodeId);
		AjaxJson json = new AjaxJson();
		json.put("obj", auditActCode);
		return json;
	}


	/**
	 * @description 新增/修改行为代码
	 * @author gzx
	 * @date：2019年11月1日
	 * @param auditActCode
	 * @return
	 */
	@PostMapping(value = "/save")
	@ApiOperation(value = "新增/修改", notes = "新增/修改行为代码")
	@ApiImplicitParam(name = "auditActCode", value = "行为代码", required = true, dataType = "AuditActCode")
	public AjaxJson save(@RequestBody AuditActCode auditActCode) {
		AjaxJson json = new AjaxJson();
		try {
			auditActCodeService.save(auditActCode, auditActCode.getActCodeId());
			json.setMsg("保存行为代码成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存行为代码异常");
		}
		return json;	
	}

	/**
	 * 删除
	 *@param
	 */
	@DeleteMapping(value = "/{actCodeId}")
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "actCodeId", value = "行为代码ID", required = true, dataType = "String", paramType = "path")
	public AjaxJson delete(@PathVariable("actCodeId") String actCodeId) {
		AjaxJson json = new AjaxJson();
		if(StringUtils.isBlank(actCodeId)) {
			return AjaxJson.returnExceptionInfo("行为代码id为空,请选择相应的列表!");
		}else {
			try {
				auditActCodeService.deleteAll(actCodeId, new AuditActCode());
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除行为代码失败" + e.getMessage());
			}
		}
		return json;

	}
	/**
	 * 下拉框列表展示
	 * @param auditActCode
	 * @return
	 */
	@GetMapping(value = "/all")
	@ApiOperation(value="根据行为对象类别获取所有行为代码", notes="根据行为对象类别获取所有行为代码")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "auditActCode ", value = "行为代码ID", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "appCode", value = "应用编码", required = false, dataType = "String")
	})
	public AjaxJson get(@ApiIgnore AuditActCode auditActCode) {
		AjaxJson json = new AjaxJson();
		List<AuditActCode> auditActCodeList = auditActCodeService.findList(auditActCode);
		json.put("auditActCodeList", auditActCodeList);
		return json;
	}
}
