package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.v1.audit.entity.AuditAlert;
import com.loit.v1.audit.service.AuditAlertService;
import com.loit.v1.modules.enums.AlertTypeEnum;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @Description: 审核策略模块——行为审核警告
 * @author: gzx
 * @date: 2019年10月31日 下午2:39:04
 * @version: V1.0
 */
@RestController
@RequestMapping(value = "api/v1/auditAlert")
@Api(tags = "行为审核警告",  value = "AuditAlertController")
public class AuditAlertController {
	
	@Autowired
	private AuditAlertService auditAlertService;
	
	@ApiOperation(value = "获取行为审核警告分页列表", notes = "获取行为审核警告分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "alertName", value = "警告名称", required = false, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "alertClass", value = "警告类名", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response,@ApiIgnore AuditAlert auditAlert){
		Page<AuditAlert> page = auditAlertService.findPage(new Page<AuditAlert>(request, response),auditAlert);
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}
	
	/**
	 * 获取组织详细信息
	 * @param
	 * @return
	 */
	@GetMapping(value = "/{auditAlertId}")
	@ApiOperation(value = "获取组织详细信息", notes = "根据id来获取Org详细信息")
	@ApiImplicitParam(name = "auditAlertId", value = "alertId", required = true, dataType = "String", paramType = "path")
	public AjaxJson get(@PathVariable String auditAlertId) {
		if(StringUtils.isBlank(auditAlertId)) {
			return AjaxJson.returnExceptionInfo("获取行为审核警告详情异常，请联系管理员");
		}
		AuditAlert auditAlert = auditAlertService.get(auditAlertId);
		AjaxJson json = new AjaxJson();
		json.put("obj", auditAlert);
		return json;
	}

	
	/**
	 * 保存
	 * @param
	 * @return
	 */
	@PostMapping(value = "/save")
	@ApiOperation(value = "保存/修改", notes = "保存/修改审核警告")
	@ApiImplicitParam(name = "auditAlertDTO", value = "实体auditAlertDTO详情", required = true, dataType = "AuditAlertDTO")
	public AjaxJson save(@RequestBody AuditAlert auditAlert) {
		AjaxJson json = new AjaxJson();
		try {
			auditAlertService.save(auditAlert, auditAlert.getAlertId());
			json.setMsg("保存成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存审核警告异常");
		}
		return json;
	}
	
	/**
	 * 删除
	 *@param 
	 */
	@DeleteMapping(value = "/{auditAlertId}")
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "auditAlertId", value = "审核警告ID", required = true, dataType = "String", paramType = "path")
	public AjaxJson delete(@PathVariable String auditAlertId) {
		if(StringUtils.isBlank(auditAlertId)) {
			return AjaxJson.returnExceptionInfo("审核警告id为空,请选择相应的列表!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(auditAlertId)) {
			try {
				auditAlertService.deleteAll(auditAlertId, new AuditAlert());
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除审核警告失败" + e.getMessage());
			}
		}
		return json;
	}
	/**
	 * 下拉框列表展示
	 * @param auditAlert
	 * @return
	 */
	@GetMapping(value = "/all")
	@ApiOperation(value="所有行为级别", notes="获取所有行为级别列表")
	public AjaxJson get(@ApiIgnore AuditAlert auditAlert) {
		AjaxJson json = new AjaxJson();
		List<AuditAlert> auditAlertList = auditAlertService.findList(auditAlert);
		List<AlertTypeEnum> alertTypeEnumList = AlertTypeEnum.getAllEnum();
		json.put("auditAlertList", auditAlertList);
		json.put("alertTypeEnumList", alertTypeEnumList);
		return json;
	}

	@ApiOperation(value = "检查审核警告名称是否重复", notes = "检查审核警告名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "ipPolicyId", value = "账号id", required = false, dataType = "int", paramType = "path"),
		@ApiImplicitParam(name = "ipPolicyName", value = "账号名称", required = true, dataType = "String", paramType = "path")
	})
	@GetMapping(value = "/checkAlertNameUnique")
	public AjaxJson checkNameUnique( Integer ipPolicyId,String ipPolicyName) {
		boolean uniqueFlag = auditAlertService.checkAlertNameUnique(ipPolicyId, ipPolicyName);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("策略名称不重复！");
		} else {
			json.setMsg("策略名称重复！");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}
}
