package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.loit.v1.audit.entity.AuditStrategy;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.utils.StringUtils;
import com.loit.v1.audit.entity.AuditLevel;
import com.loit.v1.audit.service.AuditLevelService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @Description: 审核策略模块——审核行为级别维护
 * @author: gzx
 * @date: 2019年11月1日 下午5:40:15
 * @version: V1.0
 */
@RestController
@RequestMapping(value = "api/v1/auditLevel")
@Api(value = "AuditLevelController", tags = {"审核行为级别Controller"})
public class AuditLevelController {
	@Autowired
	private  AuditLevelService auditLevelService;
	
	/**
	 * @description 审核行为级别分页列表
	 * @author gzx
	 * @date 2019年11月1日
	 * @param auditLevel 行为级别实体类	
	 * @return
	 */
	@GetMapping(value = "")
	@ApiOperation(value="分页列表", notes="获取行为级别列表")	
	public AjaxJson page(@ApiIgnore AuditLevel auditLevel, HttpServletRequest request, HttpServletResponse response) {
		AjaxJson json = new AjaxJson();
		Page<AuditLevel> page = auditLevelService.findPage(new Page<AuditLevel>(request, response), auditLevel);
		json.put("page", page);
		return json;
	}
	

	/**
	 * @description 行为级别详情
	 * @author gzx
	 * @date：2019年10月18日
	 * @param auditLevelId
	 * @return
	 */
	@GetMapping(value = "/{auditLevelId}")
	@ApiOperation(value="详情", notes="根据objectTypeId获取行为级别详情")
	@ApiImplicitParam(name = "auditLevelId", value = "行为级别ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String auditLevelId) {
		AjaxJson json = new AjaxJson();
		if(StringUtils.isBlank(auditLevelId)) {
			return AjaxJson.returnExceptionInfo("获取行为级别详情异常，请联系管理员");
		}
		AuditLevel auditLevel = auditLevelService.get(auditLevelId);
		json.put("obj", auditLevel);
		return json;
	}


	/**
	 * @description 新增新增/修改行为级别
	 * @author gzx
	 * @date：2019年11月1日
	 * @param auditLevel
	 * @return
	 */
	@PostMapping(value = "/save")
	@ApiOperation(value = "保存", notes = "新增/修改行为级别")
	@ApiImplicitParam(name = "auditLevel", value = "行为级别", required = true, dataType = "AuditLevel")
	public AjaxJson save(@RequestBody AuditLevel auditLevel) {
		AjaxJson json = new AjaxJson();
		try {
			auditLevelService.save(auditLevel, auditLevel.getLevelId());
			json.setMsg("保存行为级别成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存行为级别异常");
		}
		return json;	
	}

	/**
	 * 删除
	 *@param
	 */
	@DeleteMapping(value = "/{auditLevelId}")
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "auditLevelIds", value = "行为级别ID", required = true, dataType = "String", paramType = "path")
	public AjaxJson delete(@PathVariable("auditLevelId") String auditLevelId) {
		AjaxJson json = new AjaxJson();
		if(StringUtils.isBlank(auditLevelId)) {
			return AjaxJson.returnExceptionInfo("行为级别id为空,请选择相应的列表!");
		}else {
			try {
				auditLevelService.deleteAll(auditLevelId, new AuditLevel());
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除行为级别失败" + e.getMessage());
			}
		}
		return json;

	}
	/**
	 * 下拉框列表展示
	 * @param auditLevel
	 * @return
	 */
	@GetMapping(value = "/all")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "query", name = "appCode", value = "应用编码", required = false, dataType = "String")
	})
	@ApiOperation(value="所有行为级别", notes="获取所有行为级别列表")
	public AjaxJson get(@ApiIgnore AuditLevel auditLevel) {	
		AjaxJson json = new AjaxJson();
		List<AuditLevel> auditLevelList = auditLevelService.findList(auditLevel);
		json.put("auditLevelList", auditLevelList);
		return json;
	}
}
