package com.loit.v1.modules.controller;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.v1.audit.entity.AuditLog;
import com.loit.v1.audit.service.AuditLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @Description: 审核策略模块——行为审核日志
 * @author: wubj
 * @date: 2020年6月15日 下午13:40:15
 * @version: V1.0
 */
@RestController
@RequestMapping(value = "api/v1/auditLog")
@Api(value = "AuditLogController", tags = {"行为审核日志Controller"})
public class AuditLogController {
	@Autowired
	private AuditLogService auditLogService;
	
	/**
	 * @description 行为审核日志分页列表
	 * @author wubj
	 * @date 2019年11月1日
	 * @param auditLog 行为审核日志实体类
	 * @return
	 */
	@GetMapping(value = "")
	@ApiOperation(value="分页列表", notes="行为审核日志表")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "query", name = "pageNo", value = "当前页码", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "pageSize", value = "分页大小", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "appCode", value = "应用编码", required = false, dataType = "String")
	})
	public AjaxJson page(@ApiIgnore AuditLog auditLog, HttpServletRequest request, HttpServletResponse response) {
		AjaxJson json = new AjaxJson();
		Page<AuditLog> page = auditLogService.findPage(new Page<AuditLog>(request, response), auditLog);
		json.put("page", page);
		return json;
	}

}
