package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.ApiImplicitParams;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.audit.entity.AuditLevel;
import com.loit.v1.audit.entity.AuditObjectType;
import com.loit.v1.audit.service.AuditObjectTypeService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;
/**
 * @Description: 审核策略模块——行为对象类别维护
 * @author: gzx
 * @date: 2019年10月31日 上午9:56:35
 * @version: V1.0
 */
@RestController
@RequestMapping(value = "api/v1/objectType")
@Api(value = "AuditObjectTypeController", tags = {"行为对象类别Controller"})
public class AuditObjectTypeController  extends BaseController {
	
	@Autowired
	private AuditObjectTypeService auditObjectTypeService;


	
	/**
	 * @description 行为对象类别分页列表
	 * @author gzx
	 * @date 2019年10月18日
	 * @param auditObjectType 行为对象类别	
	 * @return
	 */
	@GetMapping(value = "")
	@ApiOperation(value="分页列表", notes="获取行为对象类别列表")	
	public AjaxJson page(@ApiIgnore AuditObjectType auditObjectType, HttpServletRequest request, HttpServletResponse response) {
		Page<AuditObjectType> page = auditObjectTypeService.findPage(new Page<AuditObjectType>(request, response), auditObjectType);
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}
	

	/**
	 * @description 行为对象类别详情
	 * @author gzx
	 * @date：2019年11月1日
	 * @param objectTypeId
	 * @return
	 */
	@GetMapping(value = "/{objectTypeId}")
	@ApiOperation(value="详情", notes="根据objectTypeId获取行为对象类别详情")
	@ApiImplicitParam(name = "objectTypeId", value = "行为对象类别ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String objectTypeId) {	
		if(StringUtils.isBlank(objectTypeId)) {
			return AjaxJson.returnExceptionInfo("获取行为对象类别详情异常，请联系管理员");
		}
		AuditObjectType auditObjectType = auditObjectTypeService.get(objectTypeId);
		AjaxJson json = new AjaxJson();
		json.put("obj", auditObjectType);
		return json;
	}


	/**
	 * @description 新增/修改对象类别
	 * @author gzx
	 * @date：2019年11月1日
	 * @param auditObjectType
	 * @return
	 */
	@PostMapping(value = "/save")
	@ApiOperation(value = "保存", notes = "新增/修改行为对象类别")
	@ApiImplicitParam(name = "auditObjectType", value = "行为对象类别", required = true, dataType = "auditObjectType")
	public AjaxJson save(@RequestBody AuditObjectType auditObjectType) {
		AjaxJson json = new AjaxJson();
		try {
			auditObjectTypeService.save(auditObjectType, auditObjectType.getObjectTypeId());
			json.setMsg("保存行为对象类别成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存行为对象类别异常");
		}
		return json;	
	}

	/**
	 * 删除
	 *@param
	 */
	@DeleteMapping(value = "/{objectTypeId}")
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "objectTypeIds", value = "行为对象类别", required = true, dataType = "String", paramType = "path")
	public AjaxJson delete(@PathVariable("objectTypeId") String objectTypeId) {
		AjaxJson json = new AjaxJson();
		if(StringUtils.isBlank(objectTypeId)) {
			return AjaxJson.returnExceptionInfo("行为对象id为空,请选择相应的列表!");
		}else {
			try {
				auditObjectTypeService.deleteAll(objectTypeId, new AuditObjectType());
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除行为对象类别失败" + e.getMessage());
			}
		}
		return json;

	}

	/**
	 * 下拉框列表展示
	 * @param auditObjectType
	 * @return
	 */
	@GetMapping(value = "/all")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "query", name = "appCode", value = "应用编码", required = false, dataType = "String")
	})
	@ApiOperation(value="所有行为对象类别", notes="获取所有行为对象类别列表")
	public AjaxJson get(@ApiIgnore  AuditObjectType auditObjectType) {	
		AjaxJson json = new AjaxJson();
		List<AuditObjectType> auditObjectTypeList = auditObjectTypeService.findList(auditObjectType);
		json.put("auditObjectList", auditObjectTypeList);
		return json;
	}

}
