package com.loit.v1.modules.controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.utils.StringUtils;
import com.loit.v1.audit.entity.AuditStrategy;
import com.loit.v1.audit.service.AuditStrategyService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;
/**
 * @Description: 行为审核策略
 * @author: gzx
 * @date: 2019年10月31日 下午8:23:11
 * @version: V1.0
 */
@RestController
@RequestMapping(value = "api/v1/auditStrategy")
@Api(tags = "行为审核策略",  value = "AuditStrategyController")
public class AuditStrategyController {
	@Autowired
	private AuditStrategyService auditStrategyService;
	
	@ApiOperation(value = "获取行为审核策略分页列表", notes = "获取行为审核策略分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "actClass", value = "审核类名称", required = false, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "actMethod", value = "类方法", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "levelId", value = "审核级别", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "objectTypeId", value = "对象类别", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response,@ApiIgnore AuditStrategy auditStrategy){
		AjaxJson json = new AjaxJson();
		Page<AuditStrategy> pageList = auditStrategyService.findPage(new Page<AuditStrategy>(request, response),auditStrategy);
		json.put("page", pageList);
		return json;
	}
	
	/**
	 * 获取审核策略详细信息
	 * @param strategyId
	 * @return
	 */
	@ApiOperation(value = "获取审核策略详细信息", notes = "根据id来获取审核策略详细信息")
	@ApiImplicitParam(name = "strategyId", value = "审核策略ID", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/{strategyId}")
	public AjaxJson get(@PathVariable String strategyId) {
		if(StringUtils.isBlank(strategyId)) {
			return AjaxJson.returnExceptionInfo("获取行为审核策略详情异常，请联系管理员!");
		}
		AuditStrategy auditStrategy = auditStrategyService.get(strategyId);
		AjaxJson json = new AjaxJson();
		json.put("obj", auditStrategy);
		return json;
	}

	
	/**
	 * 保存
	 * @param
	 * @return
	 */
	@PostMapping(value = "/save")
	@ApiOperation(value = "保存/修改", notes = "保存/修改审核策略")
	@ApiImplicitParam(name = "AuditStrategyDTO", value = "实体AuditStrategyDTO详情", required = true, dataType = "AuditStrategyDTO")
	public AjaxJson save(@RequestBody AuditStrategy auditStrategy) {
		AjaxJson json = new AjaxJson();
		try {
			auditStrategyService.save(auditStrategy, auditStrategy.getStrategyId());
			json.setMsg("保存成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存审核策略异常");
		}
		return json;
	}
	
	/**
	 * 删除
	 *@param 
	 */
	@DeleteMapping(value = "/{strategyId}")
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "strategyId", value = "审核策略ID", required = true, dataType = "String", paramType = "path")
	public AjaxJson delete(@PathVariable("strategyId") String strategyId) {
		AjaxJson json = new AjaxJson();
		if(StringUtils.isBlank(strategyId)) {
			return AjaxJson.returnExceptionInfo("审核策略id为空,请选择相应的列表!");
		}else {
			try {
				auditStrategyService.deleteAll(strategyId, new AuditStrategy());
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除审核策略失败" + e.getMessage());
			}
		}
		return json;
		
	}
}
