package com.loit.v1.modules.controller;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.v1.modules.entity.CommonEntrance;
import com.loit.v1.modules.service.CommonEntranceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;                                                                                                                       
                                                                                                                                             
                                                                                                                                             


/**
 * 常用入口管理Controller
 * Created by wubj on 2020-06-15.
 **/
@RestController                                                                                                                              
@RequestMapping(value = "/api/v1/commonEntrance")                                                                                                         
@Api(tags = "常用入口管理Controller",  value = "CommonEntranceController")
public class CommonEntranceController {                                                                                                            
                                                                                                                                             
	/**                                                                                                                                      
	 * 日志工具                                                                                                                              
	 */                                                                                                                                      
	private Logger logger = LoggerFactory.getLogger(this.getClass());                                                                        
	                                                                                                                                         
	@Autowired                                                                                                                               
	private CommonEntranceService commonEntranceService;                                                                                                 

	/**                                                                                                                                      
	 * 常用入口管理分页列表                                                                                                                          
	 */                                                                                                                                      
	@ApiOperation(value = "常用入口管理分页列表", notes = "常用入口管理分页列表")                                                                        
	@ApiImplicitParams({                                                                                                                     
	        @ApiImplicitParam(paramType = "query", name = "pageNo", value = "当前页码", required = true, dataType = "String"),  
	        @ApiImplicitParam(paramType = "query", name = "pageSize", value = "分页大小", required = true, dataType = "String")
	})
	@RequestMapping(value = "/page", method = RequestMethod.GET)                                                                           
	public AjaxJson findPage(@ApiIgnore CommonEntrance commonEntrance, HttpServletRequest request, HttpServletResponse response) {                                  
		AjaxJson json = new AjaxJson();                                                                                                      
		Page<CommonEntrance> page = commonEntranceService.findPage(new Page<CommonEntrance>(request, response), commonEntrance);                                     
		json.put("page", page);                                                                                                            
		return json;                                                                                                                         
	}                                                                                                                                        
                                                                                                                                             
	/**                                                                                                                                      
	 * 常用入口管理详情                                                                                                                              
	 */                                                                                                                                    
	@ApiOperation(value = "常用入口管理详情", notes = "根据id获取常用入口管理详情信息")                                                                  
	@ApiImplicitParam(paramType = "path", name = "id", value = "常用入口管理id", required = true, dataType = "Long")                         
	@RequestMapping(value = "/{id}", method = RequestMethod.GET)                                                                           
	public AjaxJson get(@PathVariable("id") Integer id) {                                                                              
		CommonEntrance commonEntrance = commonEntranceService.get(id);                                                                                         
		AjaxJson json = new AjaxJson();                                                                                                      
		json.put("obj", commonEntrance);                                                                                                         
		return json;                                                                                                                         
	}                                                                                                                                        
                                                                                                                                             
	/**                                                                                                                                      
	 * 常用入口管理保存                                                                                                                              
	 */                                                                                                                                      
	@ApiOperation(value = "常用入口管理保存", notes = "常用入口管理保存")                                                                                
	@RequestMapping(value = "/save", method = RequestMethod.POST)                                                                          
	public AjaxJson save(@RequestBody CommonEntrance commonEntrance) {                                                                                   
		AjaxJson json = new AjaxJson();                                                                                                      
		try {                                                                                                                                
			commonEntranceService.save(commonEntrance);
			json.setMsg("保存成功");                                                                                                       
		} catch (Exception e) {                                                                                                              
			e.printStackTrace();                                                                                                             
			return AjaxJson.returnExceptionInfo("保存异常");                                                                               
		}                                                                                                                                    
		return json;                                                                                                                         
	}                                                                                                                                        
	                                                                                                                                         
	/**                                                                                                                                      
	 * 常用入口管理修改                                                                                                                              
	 */                                                                                                                                      
	@ApiOperation(value = "常用入口管理修改", notes = "常用入口管理修改")                                                                                
	@RequestMapping(value = "/update", method = RequestMethod.POST)                                                                        
	public AjaxJson update(@RequestBody CommonEntrance commonEntrance) {                                                                                 
		AjaxJson json = new AjaxJson();                                                                                                      
		try {                                                                                                                                
			commonEntranceService.save(commonEntrance);
			json.setMsg("修改成功");                                                                                                       
		} catch (Exception e) {                                                                                                              
			e.printStackTrace();                                                                                                             
			return AjaxJson.returnExceptionInfo("修改异常");                                                                               
		}                                                                                                                                    
		return json;                                                                                                                         
	}                                                                                                                                        
	                                                                                                                                         
	/**                                                                                                                                      
	 * 常用入口管理删除                                                                                                                              
	 */                                                                                                                                      
	@ApiOperation(value = "常用入口管理删除", notes = "常用入口管理删除")                                                                                
	@ApiImplicitParam(paramType = "path", name = "id", value = "常用入口管理id", required = true, dataType = "Long")                         
	@RequestMapping(value = "/{id}", method = RequestMethod.DELETE)                                                                        
	public AjaxJson delete(@PathVariable("id") String ids) {                                                                               
		AjaxJson json = new AjaxJson();                                                                                                      
		try {                                                                                                                                
			commonEntranceService.deleteAll(ids, new CommonEntrance());                                                                                  
			json.setMsg("删除成功!");                                                                                                      
		} catch (Exception e) {                                                                                                              
			e.printStackTrace();                                                                                                             
			return AjaxJson.returnExceptionInfo("删除失败" + e.getMessage());                                                              
		}                                                                                                                                    
		return json;                                                                                                                         
	}

	/**
	 * 根据用户登录账号获取常用入口列表
	 */
	@ApiOperation(value = "根据用户登录账号获取常用入口列表", notes = "根据用户登录账号获取常用入口列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "userId", value = "用户登录账号", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/getListByUserId")
	public AjaxJson getListByUserId(@RequestParam(value = "userId", required = false) String userId){
		AjaxJson json = new AjaxJson();
		List<CommonEntrance> list = commonEntranceService.getListByUserId(userId);
		json.put("list", list);
		return json;
	}

	/**
	 * 常用入口管理菜单名称校验
	 */
	@ApiOperation(value = "常用入口管理菜单名称校验", notes = "常用入口管理菜单名称校验")
	@GetMapping(value = "/validateMenuId")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "menuId", value = "菜单id", required = true, dataType = "String",paramType = "query"),
			@ApiImplicitParam(name = "entranceId", value = "常用入口id", required = false, dataType = "String",paramType = "query")
	})
	public AjaxJson validateMenuId(@ApiIgnore CommonEntrance commonEntrance) {
		AjaxJson json = new AjaxJson();
		boolean result = commonEntranceService.validateMenuId(commonEntrance);
		if(result) {
			json.setMsg("菜单名称可用!");
			json.setSuccess(true);
		}else {
			json.setSuccess(false);
			return AjaxJson.returnExceptionInfo("菜单名称已存在");
		}
		return json;
	}

}                                                                                                                                            
