package com.loit.v1.modules.controller;

import com.loit.api.service.loituser.PersonApiService;
import com.loit.common.json.AjaxJson;
import com.loit.common.web.BaseController;
import com.loit.v1.modules.service.FileRestService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @description 文件上传下载管理Controller
 * @author xutao
 * @date 2019-10-28
 * @version V1.0
 */
@RestController
@RequestMapping(value = "api/v1/fileManage")
@Api(value = "FileRestController", tags = {"图标文件管理"})
public class FileRestController extends BaseController {

	@Autowired
	private PersonApiService personApiService;
	@Autowired
	private FileRestService fileRestService;


	
	/**
	 * @description 上传图标文件
	 * @param file
	 * @param request
	 * @return
	 */
	@PostMapping(value = "upload",consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @ApiOperation(value="图标文件上传", notes="适用于应用及菜单等图标文件上传")
	public AjaxJson fileUpload(@RequestPart(value = "file", required = true) MultipartFile file, HttpServletRequest request){
		return fileRestService.uploadIcon(file);
	}
	
	/**
	 * @description 删除已上传的图标文件
	 * @param fileName
	 * @return
	 * @throws IOException 
	 */
	@GetMapping(value="delete")
	@ApiOperation(value="图标文件删除", notes="删除已上传的图标文件")
    public AjaxJson deleteFile(@RequestParam(value = "fileName",required = true) String fileName) throws IOException {
    	return fileRestService.delFile(fileName);
    }
	
	@GetMapping(value="downLoad")
    @ApiOperation(value="图标文件下载", notes="图标文件下载")
	public AjaxJson download(@RequestParam(value = "fileName",required = true) String fileName,HttpServletResponse response) {
	    return fileRestService.downLoad(response,fileName);
    }
}
