package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.v1.modules.entity.IpPolicy;
import com.loit.v1.modules.service.IpPolicyService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**  
* @ClassName: IpPolicyController  
* @Description:   
* @author zhaokz  
* @date 2019年10月31日 下午2:49:10 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/ipPolicy")
@Api(tags = "ip策略管理Controller",  value = "IpPolicyController")
public class IpPolicyController {
	
	@Autowired
	private IpPolicyService ipPolicyService;
	
	/**
	 * 获取ip策略列表
	 * @param request
	 * @param response
	 * @param ipPolicy
	 * @return
	 */
	@ApiOperation(value = "获取ip策略列表", notes = "获取ip策略列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "ipPolicyName", value = "策略名称", required = false, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "policyIp", value = "ip", required = false, dataType = "String", paramType = "query") 
	})
	@GetMapping(value = "")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response,@ApiIgnore IpPolicy ipPolicy){
		AjaxJson json = new AjaxJson();
		Page<IpPolicy> pageList = ipPolicyService.findPage(new Page<IpPolicy>(request, response),ipPolicy);
		json.put("page", pageList);
		return json;
	}
	
	/**
	 * 获取ip策略列表(不带分页)
	 * @param request
	 * @param response
	 * @param ipPolicy
	 * @return
	 */
	@ApiOperation(value = "获取ip策略列表(不带分页)", notes = "获取ip策略列表(不带分页)")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "ipPolicyName", value = "策略名称", required = false, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "policyIp", value = "ip", required = false, dataType = "String", paramType = "query") 
	})
	@GetMapping(value = "/list1")
	public AjaxJson list1(@ApiIgnore IpPolicy ipPolicy){
		AjaxJson json = new AjaxJson();
		List<IpPolicy> list = ipPolicyService.findList(ipPolicy);
		json.put("list", list);
		return json;
	}

	/**
	 * 获取ip策略详细信息
	 * @param ipPolicyId
	 * @return
	 */
	@ApiOperation(value = "获取ip策略详细信息", notes = "根据id来获取IpPolicy详细信息")
	@ApiImplicitParam(name = "ipPolicyId", value = "账号id", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/{ipPolicyId}")
	public AjaxJson detail(@PathVariable String ipPolicyId) {
		if(StringUtils.isBlank(ipPolicyId)) {
			return AjaxJson.returnExceptionInfo("获取密码策略详情异常，请联系管理员!");
		}
		IpPolicy ipPolicy = ipPolicyService.get(ipPolicyId);
//		DictDataUtils.transfor(ipPolicy);
		AjaxJson json = new AjaxJson();
		json.put("obj", ipPolicy);
		return json;
	}

	
	/**
	 * 保存/修改
	 * @param ipPolicy
	 * @return
	 */
	@ApiOperation(value = "保存/修改", notes = "保存修改对象信息")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "ipPolicyId", value = "IP策略ID", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "ipPolicyName", value = "IP策略名称", required = true, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "ipType", value = "IP策略类型", required = true, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "serialIndex", value = "排序号", required = true, dataType = "int", paramType = "query") ,
		@ApiImplicitParam(name = "ipDesc", value = "描述", required = false, dataType = "String", paramType = "query") ,
		@ApiImplicitParam(name = "policyIp", value = "IP", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "ipStatus", value = "状态1、允许 2、禁用", required = false, dataType = "String", paramType = "query") 
	})
	@PostMapping(value = "/save")
	public AjaxJson save(@RequestBody @ApiIgnore IpPolicy ipPolicy) {
		AjaxJson json = new AjaxJson();
		try {
			boolean uniqueFlag = ipPolicyService.checkNameUnique(ipPolicy.getIpPolicyId(), ipPolicy.getIpPolicyName());
			if(!uniqueFlag) {
				return AjaxJson.returnExceptionInfo("策略名称重复!");
			}
			ipPolicyService.save(ipPolicy,ipPolicy.getIpPolicyId());
			json.setMsg("保存IP策略成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存IP策略失败!" + e.getMessage());
		}
		return json;
	}

	/**
	 * 删除
	 *@param 
	 */
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "ipPolicyIds", value = "ipPolicyIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/{ipPolicyIds}")
	public AjaxJson delete(@PathVariable String ipPolicyIds) {
		if(StringUtils.isBlank(ipPolicyIds)) {
			return AjaxJson.returnExceptionInfo("请至少选择一条ip策略!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(ipPolicyIds)) {
			IpPolicy ipPolicy = new IpPolicy();
			try {
				ipPolicyService.deleteAll(ipPolicyIds, ipPolicy);
				json.setMsg("删除ip策略成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除ip策略失败!" + e.getMessage());
			}
		}
		return json;
	}
	
	@ApiOperation(value = "检查策略名称是否重复", notes = "检查策略名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "ipPolicyId", value = "账号id", required = false, dataType = "int", paramType = "path"),
		@ApiImplicitParam(name = "ipPolicyName", value = "账号名称", required = true, dataType = "String", paramType = "path")
	})
	@GetMapping(value = "/checkNameUnique")
	public AjaxJson checkNameUnique( Integer ipPolicyId,String ipPolicyName) {
		boolean uniqueFlag = ipPolicyService.checkNameUnique(ipPolicyId, ipPolicyName);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("策略名称不重复！");
		} else {
			json.setMsg("策略名称重复！");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}
	
}
