package com.loit.v1.modules.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.modules.entity.MenuLayout;
import com.loit.v1.modules.service.MenuLayoutService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @description 菜单布局管理Controller
 * @author xutao
 * @date 2019-10-27
 */
@RestController
@RequestMapping(value = "api/v1/menuLayout")
@Api(value = "MenuLayoutController", tags = {"应用菜单布局管理Controller"})
public class MenuLayoutController extends BaseController {

	@Autowired
	private MenuLayoutService menuLayoutService;
	
	/**
	 * @description 获取菜单布局列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @param menuManage 菜单布局管理实体类	
	 * @return
	 */
	@GetMapping(value = "")
	@ApiOperation(value="列表", notes="获取菜单布局列表")
	public AjaxJson getMenuTreeList(@ApiIgnore MenuLayout menuLayout) {
		List<MenuLayout> list = menuLayoutService.findAllList(menuLayout);
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}
	
	
	/**
	 * @description 获取菜单布局详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param layoutId
	 * @return
	 */
	@GetMapping(value = "{layoutId}")
	@ApiOperation(value="详情", notes="根据layoutId获取菜单布局详情")
	@ApiImplicitParam(name = "layoutId", value = "菜单布局ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String layoutId) {
		if(StringUtils.isBlank(layoutId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		MenuLayout menuLayout = menuLayoutService.get(layoutId);
		AjaxJson json = new AjaxJson();
		json.put("obj", menuLayout);
		return json;
	}
	
		
	/**
	 * @description 新增或修改菜单布局
	 * @author xutao
	 * @date：2019年10月18日
	 * @param menuLayout 菜单布局实体
	 * @return
	 */
	@PostMapping(value = "")
	@ApiOperation(value = "保存（新增或修改）", notes = "新增或修改菜单布局")
	@ApiImplicitParam(name = "menuLayout", value = "菜单布局实体", required = true, dataType = "MenuLayout")
	public AjaxJson save(@RequestBody MenuLayout menuLayout) {
		AjaxJson json = new AjaxJson();
		try {
			List<MenuLayout> list = menuLayoutService.findList(menuLayout);
			if(list.size()>0) {
				return AjaxJson.returnExceptionInfo(LoitStatusCode.MENULAYOUT_IS_EXIST);
			}else {
				menuLayoutService.save(menuLayout,menuLayout.getLayoutId());
				json.setMsg("保存成功");
				return json;
			}
		} catch (Exception e) {
			logger.error("erroCode:{},info:保存菜单布局异常",LoitStatusCode.FAIL.statusCode);
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
		}
	}

	/**
	 * @description 菜单布局删除
	 * @author xutao
	 * @date：2019年10月18日
	 * @param ids 多ID时，以逗号分隔
	 * @return
	 */
	@ApiOperation(value = "删除", notes = "单个或批量删除菜单布局，多ID时，以逗号分隔")
	@DeleteMapping(value = "{layoutId}")
	public AjaxJson deleteAll(@PathVariable("layoutId")String ids) {
		if(StringUtils.isBlank(ids)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		AjaxJson json = new AjaxJson();
		try {
			MenuLayout menuLayout = new MenuLayout();
			menuLayoutService.deleteAll(ids,menuLayout);
			json.setMsg("删除菜单布局成功");
			return json;
		} catch (Exception e) {
			logger.error("erroCode:{},info:删除菜单布局异常",LoitStatusCode.REDIS_LINK_FAIL.statusCode);
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.REDIS_LINK_FAIL);
		}
	}


}
