package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.api.dto.MenuManageDTO;
import com.loit.v1.modules.entity.MenuManage;
import com.loit.v1.modules.service.MenuManageService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @description 应用菜单管理Controller
 * @author xutao
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/menuManage")
@Api(value = "MenuManageController", tags = {"应用菜单管理Controller"})
public class MenuManageController extends BaseController {

	@Autowired
	private MenuManageService menuManageService;

	/**
	 * @description 获取菜单树状列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @param menuManage 应用菜单管理实体类	
	 * @return
	 */
	@GetMapping(value = "getMenuTreeList")
	@ApiOperation(value="菜单树状列表", notes="获取菜单树状列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用ID", required = true, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "isEnabled", value = "是否可用（0：否 1：是）", required = false, dataType = "String",paramType = "query")				
	})
	
	public AjaxJson getMenuTreeList(@ApiIgnore MenuManage menuManage) {
		List<MenuManage> list = menuManageService.findAllTreeList(menuManage);
		//logger.debug("获取菜单树状列表成功");
		AjaxJson json = new AjaxJson();
		json.put("menuTreeList", list);
		return json;
	}	
	
	/**
	 * @description 获取应用菜单树形分页列表
	 * @author xutao
	 * @date 2019年10月18日
	 * @param menuManage 应用菜单管理实体类	
	 * @return
	 */
	@GetMapping(value = "treePage")
	@ApiOperation(value="树形分页列表", notes="获取应用菜单树形分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "appId", value = "应用ID", required = true, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "isEnabled", value = "是否可用（0：否 1：是）", required = false, dataType = "String",paramType = "query")				
	})
	public AjaxJson page(@ApiIgnore  MenuManage menuManage, HttpServletRequest request, HttpServletResponse response) {
		Page<MenuManage> treePage = menuManageService.findTreePage(new Page<MenuManage>(request, response), menuManage);
		AjaxJson json = new AjaxJson();
		json.put("treePage", treePage);
		return json;
	}
	
	/**
	 * @description 新增菜单
	 * @author xutao
	 * @date：2019年10月18日
	 * @param menuManageDTO 菜单数据传输类
	 * @return
	 */
	@PostMapping(value = "save")
	@ApiOperation(value = "保存", notes = "新增菜单")
	@ApiImplicitParam(name = "menuManageDTO", value = "应用菜单管理数据传输实体类", required = true, dataType = "MenuManageDTO")
	public AjaxJson save(@RequestBody MenuManageDTO menuManageDTO) {
		AjaxJson json = new AjaxJson();	
		MenuManage manage = ModelMapperUtils.map(menuManageDTO, MenuManage.class);
		try {
//			List<MenuManage> list = menuManageService.findList(manage);
//			if(list.size()>0) {
//				return AjaxJson.returnExceptionInfo(LoitStatusCode.MENU_IS_EXIST);
//			}else {
//				menuManageService.insert(manage);
//				json.setMsg("保存成功");
//			}
			menuManageService.insert(manage);
			json.setMsg("保存成功");
			return json;
		} catch (Exception e) {
			logger.error("info:菜单保存失败,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
		}		
	}
	
	/**
	 * @description 菜单详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param menuId 菜单ID
	 * @return
	 */
	@GetMapping(value = "{menuId}")
	@ApiOperation(value="详情", notes="根据menuId获取菜单详情")
	@ApiImplicitParam(name = "menuId", value = "菜单ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String menuId) {	
		if(StringUtils.isBlank(menuId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		MenuManage manage = menuManageService.get(menuId);		
		AjaxJson json = new AjaxJson();
		json.put("obj", manage);
		return json;
	}
	
	/**
	 * @description 修改菜单
	 * @param menuId 菜单ID
	 * @param menuManageDTO 菜单数据传输实体类
	 * @return
	 */
	@PostMapping(value = "{menuId}")
	@ApiOperation(value = "修改", notes = "修改菜单")
	public AjaxJson update(@PathVariable int menuId,@RequestBody MenuManageDTO menuManageDTO) {
		AjaxJson json = new AjaxJson();
		MenuManage manage = ModelMapperUtils.map(menuManageDTO, MenuManage.class);
		try {
			manage.setMenuId(menuId);
			menuManageService.update(manage);
			json.setMsg("修改成功");
			return json;
		} catch (Exception e) {
			logger.error("info:菜单修改失败,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.UPDATE_EXCEPTION);
		}		
	}
	
	/**
	 * @Description 应用菜单删除
	 * @author xutao
	 * @date：2019年10月18日
	 * @param ids 菜单ID，多ID时，以逗号分隔
	 * @return
	 */
	@ApiOperation(value = "删除", notes = "单个或批量删除应用菜单，多ID时，以逗号分隔")
	@DeleteMapping(value = "{menuId}")
	public AjaxJson deleteAll(@PathVariable("menuId")String ids) {
		AjaxJson json = new AjaxJson();
		if(StringUtils.isBlank(ids)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}		
		try {
			MenuManage manage = new MenuManage();
			menuManageService.deleteAll(ids,manage);
			json.setMsg("删除成功");
			return json;
		} catch (Exception e) {
			logger.error("info:删除异常,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.DELETE_EXCEPTION);			
		}
	}
	
	/**
	 * @description 检查菜单名称是否重复
	 * @param manage
	 * @return
	 */
	@ApiOperation(value = "检查菜单名称是否重复", notes = "检查菜单名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用ID", required = true, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "menuId", value = "菜单ID", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "menuName", value = "菜单名称", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkMenuNameUnique")
	public AjaxJson checkAppNameUnique(@ApiIgnore MenuManage manage) {
		AjaxJson json = new AjaxJson();
		List<MenuManage> list = menuManageService.findList(manage);
		if(list.size()>0) {
			json.setMsg("菜单名称重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}else {
			json.setMsg("菜单名称不重复");
		}
		return json;
	}

}
