package com.loit.v1.modules.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.modules.entity.MenuStyle;
import com.loit.v1.modules.service.MenuStyleService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @description 菜单样式管理Controller
 * @author xutao
 * @date 2020-02-21
 */
@RestController
@RequestMapping(value = "api/v1/menuStyle")
@Api(value = "MenuStyleController", tags = {"应用菜单样式管理Controller"})
public class MenuStyleController extends BaseController {

	@Autowired
	private MenuStyleService menuStyleService;
	
	/**
	 * @description 获取菜单样式列表
	 * @author xutao
	 * @date：2020-02-21
	 * @param menuStyle 菜单样式管理实体类	
	 * @return
	 */
	@GetMapping(value = "")
	@ApiOperation(value="列表", notes="获取菜单样式列表")
	public AjaxJson getMenuTreeList(@ApiIgnore MenuStyle menuStyle) {
		List<MenuStyle> list = menuStyleService.findAllList(menuStyle);
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}
	
	
	/**
	 * @description 获取菜单样式详情
	 * @author xutao
	 * @date：2020-02-21
	 * @param layoutId
	 * @return
	 */
	@GetMapping(value = "{styleId}")
	@ApiOperation(value="详情", notes="根据styleId获取菜单样式详情")
	@ApiImplicitParam(name = "styleId", value = "菜单样式ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String styleId) {
		if(StringUtils.isBlank(styleId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		MenuStyle menuStyle = menuStyleService.get(styleId);
		AjaxJson json = new AjaxJson();
		json.put("obj", menuStyle);
		return json;
	}
	
		
	/**
	 * @description 新增或修改菜单样式
	 * @author xutao
	 * @date：2020-02-21
	 * @param menuStyle 菜单样式实体
	 * @return
	 */
	@PostMapping(value = "")
	@ApiOperation(value = "保存（新增或修改）", notes = "新增或修改菜单样式")
	@ApiImplicitParam(name = "MenuStyle", value = "菜单样式实体", required = true, dataType = "MenuStyle")
	public AjaxJson save(@RequestBody MenuStyle menuStyle) {
		AjaxJson json = new AjaxJson();
		try {
			menuStyleService.save(menuStyle,menuStyle.getstyleId());
			json.setMsg("保存成功");
				return json;	
		} catch (Exception e) {
			logger.error("erroCode:{},info:保存菜单样式异常",LoitStatusCode.FAIL.statusCode);
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
		}
	}

	/**
	 * @description 菜单样式删除
	 * @author xutao
	 * @date：2020-02-21
	 * @param ids 多ID时，以逗号分隔
	 * @return
	 */
	@ApiOperation(value = "删除", notes = "单个或批量删除菜单样式，多ID时，以逗号分隔")
	@DeleteMapping(value = "{styleId}")
	public AjaxJson deleteAll(@PathVariable("styleId")String ids) {
		if(StringUtils.isBlank(ids)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		AjaxJson json = new AjaxJson();
		try {
			MenuStyle menuStyle = new MenuStyle();
			menuStyleService.deleteAll(ids,menuStyle);
			json.setMsg("删除菜单样式成功");
			return json;
		} catch (Exception e) {
			logger.error("erroCode:{},info:删除菜单样式异常",LoitStatusCode.REDIS_LINK_FAIL.statusCode);
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.REDIS_LINK_FAIL);
		}
	}

}