package com.loit.v1.modules.controller;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.v1.modules.entity.Notice;
import com.loit.v1.modules.service.NoticeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;


/**
 * 通知管理
 * Created by WBJ on 2020-04-23.
 **/
@RestController
@RequestMapping(value = "/api/v1/notice")
@Api(tags = "通知管理",  value = "NoticeController")
public class NoticeController {

	/**
	 * 日志工具                                                                                                                              
	 */
	private Logger logger = LoggerFactory.getLogger(this.getClass());

	@Autowired
	private NoticeService noticeService;

	/**
	 * 通知管理分页列表
	 */
	@ApiOperation(value = "通知管理分页列表", notes = "通知管理分页列表")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "query", name = "pageNo", value = "当前页码", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "pageSize", value = "分页大小", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeTopic", value = "通知标题", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeType", value = "通知类型", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "publishStatus", value = "发布状态（0草稿，1已发布）", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "createTimeStart", value = "开始时间", required = false, dataType = "Date"),
			@ApiImplicitParam(paramType = "query", name = "createTimeEnd", value = "结束时间", required = false, dataType = "Date"),
			@ApiImplicitParam(paramType = "query", name = "orderByName", value = "排序字段名称", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "orderBy", value = "排序（asc/desc）", required = false, dataType = "String")
	})
	@RequestMapping(value = "/page", method = RequestMethod.GET)
	public AjaxJson findPage(@ApiIgnore Notice notice, HttpServletRequest request, HttpServletResponse response) {
		AjaxJson json = new AjaxJson();
		Page<Notice> page = noticeService.findPage(new Page<Notice>(request, response), notice);
		json.put("page", page);
		return json;
	}

	/**
	 * 通知详情
	 */
	@ApiOperation(value = "通知详情", notes = "根据id获取通知详情信息")
	@ApiImplicitParam(paramType = "path", name = "id", value = "通知id", required = true, dataType = "Long")
	@RequestMapping(value = "/{id}", method = RequestMethod.GET)
	public AjaxJson get(@PathVariable("id") Integer id) {
		Notice notice = noticeService.get(id);
		AjaxJson json = new AjaxJson();
		json.put("obj", notice);
		return json;
	}

	/**
	 * 通知发布
	 */
	@ApiOperation(value = "通知发布", notes = "通知发布")
	@PostMapping("/publish")
	public AjaxJson publish(@RequestBody Notice notice) {
		AjaxJson json = new AjaxJson();
		try {
			noticeService.publish(notice);
			json.setMsg("发布成功");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("发布异常");
		}
		return json;
	}

	/**
	 * 通知保存
	 */
	@ApiOperation(value = "通知保存", notes = "通知保存")
	@RequestMapping(value = "/save", method = RequestMethod.POST)
	public AjaxJson save(@RequestBody Notice notice) {
		AjaxJson json = new AjaxJson();
		try {
			noticeService.save(notice);
			json.setMsg("保存成功");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存异常");
		}
		return json;
	}

	/**
	 * 通知修改
	 */
	@ApiOperation(value = "通知修改", notes = "通知修改")
	@RequestMapping(value = "/update", method = RequestMethod.POST)
	public AjaxJson update(@RequestBody Notice notice) {
		AjaxJson json = new AjaxJson();
		try {
			noticeService.save(notice);
			json.setMsg("修改成功");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("修改异常");
		}
		return json;
	}

	/**
	 * 通知删除
	 */
	@ApiOperation(value = "通知删除", notes = "通知删除")
	@ApiImplicitParam(paramType = "path", name = "id", value = "通知id", required = true, dataType = "Long")
	@RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
	public AjaxJson delete(@PathVariable("id") String ids) {
		AjaxJson json = new AjaxJson();
		try {
			noticeService.deleteAll(ids);
			json.setMsg("删除成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("删除失败" + e.getMessage());
		}
		return json;
	}

	/**
	 * 通知修改为已发布状态
	 */
	@ApiOperation(value = "通知修改为已发布状态", notes = "通知修改为已发布状态")
	@PostMapping("/publishStatus")
	public AjaxJson publishStatus(@RequestBody Notice notice) {
		AjaxJson json = new AjaxJson();
		try {
			noticeService.publishStatus(notice);
			json.setMsg("发布成功");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("发布异常");
		}
		return json;
	}
}                                                                                                                                            
