package com.loit.v1.modules.controller;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.v1.modules.entity.NoticeReceive;
import com.loit.v1.modules.service.NoticeReceiveService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;


/**
 * 通知接收管理
 * Created by WBJ on 2020-04-23.
 **/
@RestController                                                                                                                              
@RequestMapping(value = "/api/v1/noticeReceive")
@Api(tags = "通知接收管理",  value = "NoticeReceiveController")
public class NoticeReceiveController {                                                                                                            
                                                                                                                                             
	/**                                                                                                                                      
	 * 日志工具                                                                                                                              
	 */                                                                                                                                      
	private Logger logger = LoggerFactory.getLogger(this.getClass());                                                                        
	                                                                                                                                         
	@Autowired                                                                                                                               
	private NoticeReceiveService noticeReceiveService;
                                                                                                                                             
	/**                                                                                                                                      
	 * 通知接收分页列表                                                                                                                          
	 */                                                                                                                                      
	@ApiOperation(value = "通知接收分页列表", notes = "通知接收分页列表")                                                                        
	@ApiImplicitParams({                                                                                                                     
	        @ApiImplicitParam(paramType = "query", name = "pageNo", value = "当前页码", required = true, dataType = "String"),  
	        @ApiImplicitParam(paramType = "query", name = "pageSize", value = "分页大小", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeTopic", value = "通知标题", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeType", value = "通知类型", required = false, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "receiveId", value = "接收人", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "noticeStatus", value = "阅读状态（0未读，1已读）", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "createTimeStart", value = "开始时间", required = false, dataType = "Date"),
			@ApiImplicitParam(paramType = "query", name = "createTimeEnd", value = "结束时间", required = false, dataType = "Date"),
			@ApiImplicitParam(paramType = "query", name = "orderByName", value = "排序字段名称", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "orderBy", value = "排序（asc/desc）", required = false, dataType = "String")
	})
	@RequestMapping(value = "/page", method = RequestMethod.GET)                                                                           
	public AjaxJson findPage(@ApiIgnore NoticeReceive noticeReceive, HttpServletRequest request, HttpServletResponse response) {                                  
		AjaxJson json = new AjaxJson();                                                                                                      
		Page<NoticeReceive> page = noticeReceiveService.findPage(new Page<NoticeReceive>(request, response), noticeReceive);                                     
		json.put("page", page);                                                                                                            
		return json;                                                                                                                         
	}

	/**
	 * 通知接收详情
	 */
	@ApiOperation(value = "通知接收详情", notes = "根据id获取通知接收详情信息")
	@ApiImplicitParam(paramType = "path", name = "id", value = "通知接收id", required = true, dataType = "Long")
	@RequestMapping(value = "/{id}", method = RequestMethod.GET)
	public AjaxJson get(@PathVariable("id") Integer id) {
		NoticeReceive noticeReceive = noticeReceiveService.get(id);
		AjaxJson json = new AjaxJson();
		json.put("obj", noticeReceive);
		return json;
	}

	/**
	 * 更新通知为已读
	 */
	@ApiOperation(value = "更新通知为已读", notes = "更新通知为已读")
	@PostMapping("/updateStatus")
	public AjaxJson updateStatus(@RequestParam(value = "noticeIds", required = true) List<Long> noticeIds,
								 @RequestParam(value = "receiveId", required = false) String receiveId) {
		AjaxJson json = new AjaxJson();
		try {
			noticeReceiveService.updateStatus(noticeIds, receiveId);
			json.setMsg("已读成功");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("已读异常");
		}
		return json;
	}

}                                                                                                                                            
