package com.loit.v1.modules.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.v1.api.dto.OrgExtattrDTO;
import com.loit.v1.modules.entity.OrgExtattr;
import com.loit.v1.modules.service.OrgExtattrService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**  
* @ClassName: OrgExtattrController  
* @Description:   组织扩展属性管理
* @author zhaokz  
* @date 2019年10月24日 下午5:00:12 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/orgExtattr")
@Api(tags = "组织扩展属性管理",  value = "OrgExtattrController")
public class OrgExtattrController {

	@Autowired
	private OrgExtattrService orgExtattrService;
	
	/**
	 * 获取扩展属性列表  
	 * @param request
	 * @param response
	 * @param org
	 * @return
	 */
	@ApiOperation(value = "获取扩展属性列表", notes = "获取扩展属性列表")
	@GetMapping(value = "/list")
	public AjaxJson list(@ApiIgnore OrgExtattr orgExtattr){
		List<OrgExtattr> list = orgExtattrService.findList(orgExtattr);
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}

	/**
	 * 获取扩展属性列表  
	 * @param org
	 * @return
	 */
	@ApiOperation(value = "获取扩展属性列表", notes = "获取扩展属性列表")
	@GetMapping(value = "/orgAttrList")
	public AjaxJson orgAttrList(@ApiIgnore OrgExtattr orgExtattr){
		List<OrgExtattr> list = orgExtattrService.findList(orgExtattr);
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}
	
	/**
	 * 获取组织扩展详细信息
	 * @param orgId
	 * @return
	 */
	@ApiOperation(value = "获取组织扩展详细信息", notes = "根据id来获取OrgExtattr详细信息")
	@ApiImplicitParam(name = "orgAttrId", value = "orgAttrId", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/{orgAttrId}")
	public AjaxJson detail(@PathVariable String orgAttrId) {
		AjaxJson json = new AjaxJson();
		OrgExtattr newOrgExtattr = new OrgExtattr();
		newOrgExtattr.setOrgAttrId(Integer.valueOf(orgAttrId));
		OrgExtattr orgExtattr = orgExtattrService.get(newOrgExtattr);
		OrgExtattrDTO orgExtattrDTO = null;
		if(orgExtattr != null) {
			orgExtattrDTO = ModelMapperUtils.map(orgExtattr, OrgExtattrDTO.class);
		}
		json.put("orgExtattr", orgExtattrDTO);
		return json;
	}

	
	/**
	 * 保存
	 * @param org
	 * @return
	 */
	@ApiOperation(value = "保存", notes = "保存对象信息")
	@ApiImplicitParam(name = "orgExtattrDTO", value = "实体OrgExtattrDTO详情", required = true, dataType = "OrgExtattrDTO")
	@PostMapping(value = "/save")
	public AjaxJson save(@RequestBody OrgExtattrDTO orgExtattrDTO) {
		AjaxJson json = new AjaxJson();
		try {
			OrgExtattr orgExtattr = ModelMapperUtils.map(orgExtattrDTO, OrgExtattr.class);
			orgExtattrService.save(orgExtattr,orgExtattr.getOrgAttrId());
			json.setMsg("保存成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存组织异常" + e.getMessage());
		}
		return json;
	}

	/**
	 * 删除
	 *@param 
	 */
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "orgAttrIds", value = "orgAttrIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/{orgAttrIds}")
	public AjaxJson delete(@PathVariable String orgAttrIds) {
		if(StringUtils.isBlank(orgAttrIds)) {
			return AjaxJson.returnExceptionInfo("扩展字典id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(orgAttrIds)) {
			try {
				orgExtattrService.deleteBatch(orgAttrIds);
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除组织失败" + e.getMessage());
			}
		}
		return json;
	}
	

}
