package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.v1.api.dto.OrgExtattrInstDTO;
import com.loit.v1.modules.entity.OrgExtattrInst;
import com.loit.v1.modules.service.OrgExtattrInstService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**  
* @ClassName: OrgExtattrInstController  
* @Description:   组织扩展实例管理
* @author zhaokz  
* @date 2019年10月26日 上午11:25:11 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/orgExtattrInst")
@Api(tags = "组织扩展实例管理",  value = "OrgExtattrInstController")
public class OrgExtattrInstController {

	
	@Autowired
	private OrgExtattrInstService orgExtattrInstService;
	
	/**
	 * 获取组织扩展实例列表
	 * @param request
	 * @param response
	 * @param orgExtattrInst
	 * @return
	 */
	@ApiOperation(value = "获取组织扩展实例列表", notes = "获取组织扩展实例列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "orgExtattrInst", value = "详细实体OrgExtattrInst的参数值可设为查询条件", required = false, dataType = "OrgExtattrInst", paramType = "query") })
	@GetMapping(value = "")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response,@ApiIgnore OrgExtattrInst orgExtattrInst){

		Page<OrgExtattrInst> pageList = orgExtattrInstService.findPage(new Page<OrgExtattrInst>(request, response),orgExtattrInst);
		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
	}
	
	/**
	 * 获取组织扩展实例详细信息
	 * @param orgExtattrInstId
	 * @return
	 */
	@ApiOperation(value = "获取组织扩展实例详细信息", notes = "根据id来获取OrgExtattrInst详细信息")
	@ApiImplicitParam(name = "instId", value = "id", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/orgExtattrInst/{instId}")
	public AjaxJson detail(@PathVariable String instId) {
		AjaxJson json = new AjaxJson();
		OrgExtattrInst newOrgExtattrInst = new OrgExtattrInst();
		newOrgExtattrInst.setInstId(Integer.valueOf(instId));
		OrgExtattrInst orgExtattrInst = orgExtattrInstService.get(newOrgExtattrInst);
		OrgExtattrInstDTO orgExtattrInstDTO = null;
		if(orgExtattrInst != null) {
			orgExtattrInstDTO = ModelMapperUtils.map(orgExtattrInst, OrgExtattrInstDTO.class);
		}
		json.put("accout", orgExtattrInstDTO);
		return json;
	}

	
	/**
	 * 保存
	 * @param orgExtattrInst
	 * @return
	 */
	@ApiOperation(value = "保存", notes = "保存对象信息")
	@ApiImplicitParam(name = "orgExtattrInstDTO", value = "实体OrgExtattrInstDTO详情", required = true, dataType = "OrgExtattrInstDTO")
	@PostMapping(value = "/save")
	public AjaxJson save(@RequestBody OrgExtattrInstDTO orgExtattrInstDTO) {
		AjaxJson json = new AjaxJson();
		try {
			OrgExtattrInst orgExtattrInst = ModelMapperUtils.map(orgExtattrInstDTO, OrgExtattrInst.class);
			orgExtattrInstService.save(orgExtattrInst,orgExtattrInst.getInstId());
			json.setMsg("保存成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存失败!" + e.getMessage());
		}
		return json;
	}

	/**
	 * 删除
	 *@param 
	 */
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "instIds", value = "orgExtattrInstIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/orgExtattrInst/{instIds}")
	public AjaxJson delete(@PathVariable String instIds) {
		if(StringUtils.isBlank(instIds)) {
			return AjaxJson.returnExceptionInfo("机构id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(instIds)) {
			OrgExtattrInst orgExtattrInst = new OrgExtattrInst();
			try {
				orgExtattrInstService.deleteAll(instIds, orgExtattrInst);
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除失败" + e.getMessage());
			}
		}
		return json;
	}
	
	
}
