package com.loit.v1.modules.controller;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jms.Destination;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.loit.common.constant.LoitHeaderConstant;
import com.loit.loitcasclient.config.ClientAppConfig;
import org.apache.activemq.command.ActiveMQQueue;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.loit.api.service.loituser.PersonApiService;
import com.loit.common.constant.CommonConstant;
import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.utils.CookieUtils;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.modules.activemq.MessageModel;
import com.loit.v1.modules.activemq.Producer;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.entity.AccountPolicy;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.entity.ApporgShortcut;
import com.loit.v1.modules.entity.IpPolicy;
import com.loit.v1.modules.entity.IpPolicyAccount;
import com.loit.v1.modules.entity.PersonBase;
import com.loit.v1.modules.service.AccountPolicyService;
import com.loit.v1.modules.service.AccoutInfoService;
import com.loit.v1.modules.service.ApporgShortcutService;
import com.loit.v1.modules.service.IpPolicyService;
import com.loit.v1.modules.service.PersonBaseService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
* @ClassName: PersonBaseController  
* @Description:   人员基本信息管理
* @author zhaokz  
* @date 2019年10月24日 下午3:23:26 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/person")
@Api(tags = "人员管理Controller",  value = "PersonController")
public class PersonController{


	public static final Destination QUEUE_SYNC_PERSON_SAVE = new ActiveMQQueue(BusinessConstants.QUEUE_SYNC_PERSON_SAVE);
	public static final Destination QUEUE_SYNC_PERSON_DELETE = new ActiveMQQueue(BusinessConstants.QUEUE_SYNC_PERSON_DELETE);

	@Autowired
	private PersonBaseService personBaseService;
	@Autowired
	private AccoutInfoService accoutInfoService;
	@Autowired
	AccountPolicyService accountPolicyService;
    @Autowired
    PersonApiService personApiService;
	@Autowired
	Producer producer;
    @Autowired
    ApporgShortcutService apporgShortcutService;
    @Autowired
    IpPolicyService ipPolicyService;
	@Autowired
	private ClientAppConfig clientAppConfig;

	/**
	 * 获取人员信息列表
	 * @param request
	 * @param response
	 * @param personBase
	 * @return
	 */
	@ApiOperation(value = "获取人员信息列表", notes = "获取人员信息列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "delFlag", value = "是否删除", required = true, dataType = "String", paramType = "query"), 
			@ApiImplicitParam(name = "appId", value = "应用Id", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "personName", value = "人员名称", required = false, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "userId", value = "账号", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/pageList")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response,@ApiIgnore PersonBase personBase){
		//TODO 根据当前登录人员角色查询数据
		AjaxJson json = new AjaxJson();
		User user = UserUtils.getUser();
		if(BusinessConstants.ORG_USER.equals(user.getUserType())) {
			personBase.setOrgId(user.getOrgId());
		}
//		if(BusinessConstants.ACCOUNT_TYPE_SUPERADMIN.equals(user.getAccountType())) {
//			personBase.setAppId(null);
//		}
		Page<PersonBase> pageList = personBaseService.findPage(new Page<PersonBase>(request, response),personBase);
		json.put("page", pageList);
		return json;
	}
	
	/**
	 * 根据组织id获取人员信息列表
	 * @param request
	 * @param response
	 * @param personBase
	 * @return
	 */
	@ApiOperation(value = "根据组织id获取人员信息列表", notes = "根据组织id获取人员信息列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "orgId", value = "组织id", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "delFlag", value = "是否删除", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/orgPersonList")
	public AjaxJson getPersonByOrgId(HttpServletRequest request, HttpServletResponse response,@ApiIgnore PersonBase personBase,
			@RequestParam String orgId){
		Page<PersonBase> pageList = personBaseService.findPageByOrgId(new Page<PersonBase>(request, response),personBase,orgId);
		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
	}

	/**
	 * 获取人员详细信息
	 * @param accountId
	 * @return
	 */
	@ApiOperation(value = "获取人员详细信息", notes = "根据id来获取人员详细信息")
	@ApiImplicitParam(name = "accountId", value = "id", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/{accountId}")
	public AjaxJson detail(@PathVariable String accountId) {
		AjaxJson json = new AjaxJson();
		PersonBaseDTO personBaseDTO = personBaseService.getDetailInfo(accountId);
		//密码策略
		AccountPolicy accountPolicy = accountPolicyService.getOne();
		json.put("accountPolicy", accountPolicy);
		json.put("accout", personBaseDTO);
		return json;
	}

	/**
	 * 获取密码策略
	 * @param
	 * @return
	 */
	@ApiOperation(value = "获取密码策略", notes = "获取密码策略")
	@GetMapping(value = "/passwordApolicy")
	public AjaxJson passwordApolicy() {
		AjaxJson json = new AjaxJson();
		//密码策略
		AccountPolicy accountPolicy = accountPolicyService.getOne();
		json.put("accountPolicy", accountPolicy);
		return json;
	}
	
	/**
	 * 保存
	 * @param
	 * @return
	 */
	@ApiOperation(value = "保存", notes = "保存对象信息")
	@ApiImplicitParam(name = "personBaseDTO", value = "实体PersonBaseDTO详情", required = true, dataType = "PersonBaseDTO")
	@PostMapping(value = "/save")
	public AjaxJson save(@RequestBody PersonBaseDTO personBaseDTO) {
		AjaxJson json = new AjaxJson();
		boolean isInsert = personBaseDTO.getAccountId() == null ? true : false;
		if(BusinessConstants.ORG_USER.equals(personBaseDTO.getUserType())) {
			if(isInsert) {
				//查询该组织下是否有组织用户
				AccoutInfo accoutInfo = accoutInfoService.findOrgUser(personBaseDTO.getOrgId());
				if(accoutInfo != null) {
					json.setSuccess(false);
					json.setMsg("该组织下已存在组织用户,不可重复添加!");
				}
			}else {
				AccoutInfo accoutInfo = accoutInfoService.get(personBaseDTO.getAccountId());
				if(BusinessConstants.ORG_USER.equals(accoutInfo.getUserType())) {
					json.setSuccess(false);
					json.setMsg("该用户是组织用户,无修改权限");
				}
			}
		}
		try {
			personBaseService.addPerson(personBaseDTO);
			json.setMsg("保存成功!");
			json.setSuccess(true);
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
		}

		//发送消息到队列
		Map<String, Object> map = new HashMap<>();
		//判断是否是新增或是修改
		map.put("isInsert", isInsert);
        map.put("person",personBaseDTO);
        MessageModel message = new MessageModel();
        message.setMap(map);
        producer.send(QUEUE_SYNC_PERSON_SAVE,message);
		return json;
	}
	
	/**
	 * 验证账号唯一
	 * @param
	 * @return
	 */
	@ApiOperation(value = "账号唯一验证", notes = "账号唯一验证")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "userId", value = "账号", required = true, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "accountId", value = "账号id", required = false, dataType = "String",paramType = "query")
	})
	@GetMapping(value = "/validateUserId")
	public AjaxJson validateUserId(@ApiIgnore AccoutInfo accoutInfo ) {
		AjaxJson json = new AjaxJson();
		boolean result = personBaseService.validateUserId(accoutInfo);
		if(result) {
			json.setMsg("该账号可用!");
			json.setSuccess(true);
		}else {
			json.setSuccess(false);
			return AjaxJson.returnExceptionInfo(LoitStatusCode.USER_ACCOUNT_EXISTED);
		}
		return json;
	}
	
	/**
	 * 验证人员名称唯一
	 * @param
	 * @return
	 */
	@ApiOperation(value = "人员名称唯一验证", notes = "人员名称唯一验证")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "personName", value = "人员名称", required = true, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "personBaseId", value = "人员基本信息id", required = false, dataType = "String",paramType = "query")
	})
	@GetMapping(value = "/validatePersonName")
	public AjaxJson validatePersonName(@ApiIgnore PersonBase personBase ) {
		AjaxJson json = new AjaxJson();
		boolean result = personBaseService.validatePersonName(personBase);
		if(result) {
			json.setMsg("该名称可用!");
			json.setSuccess(true);
		}else {
			json.setSuccess(false);
			return AjaxJson.returnExceptionInfo(LoitStatusCode.USER_NAME_EXISTED);
		}
		return json;
	}
	
	/**
	 * 物理删除
	 *@param 
	 */
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "accountIds", value = "accountIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/{accountIds}")
	public AjaxJson delete(@PathVariable String accountIds) {
		if(StringUtils.isBlank(accountIds)) {
			return AjaxJson.returnExceptionInfo("id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(accountIds)) {
			AccoutInfo accoutInfo = new AccoutInfo();
			try {
				//取出对象列表
				String[] accountArr = accountIds.split(",");
				List<PersonBase> accountList = personBaseService.findByIdList(Arrays.asList(accountArr));
                List<ApporgShortcut> apporgShortcutList = apporgShortcutService.findListByAccountIdList(Arrays.asList(accountArr));

                //删除
                accoutInfoService.deleteAllAccount(accountIds, accoutInfo);
                json.setMsg("删除成功!");
				//发送消息到队列
                if(apporgShortcutList != null) {
                    Map<String, Object> map = new HashMap<>();
//				map.put("accountIds",accountIds);
                    map.put("accountList", accountList);
                    map.put("apporgShortcutList", apporgShortcutList);
                    MessageModel message = new MessageModel();
                    message.setMap(map);
                    producer.send(QUEUE_SYNC_PERSON_DELETE, message);//发消息
                }
				json.setSuccess(true);
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
			}
		}
		return json;
	}

	/**
	 * 逻辑删除
	 * @param
	 * @return
	 */
	@ApiOperation(value = "逻辑删除", notes = "根据id来指定逻辑删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "accountIds", value = "accoutInfoIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/deleteByLogic/{accountIds}")
	public AjaxJson deleteByLogic(@PathVariable String accountIds) {
		if(StringUtils.isBlank(accountIds)) {
			return AjaxJson.returnExceptionInfo("id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(accountIds)) {
			AccoutInfo accoutInfo = new AccoutInfo();
			try {
				accoutInfoService.deleteByLogicAll(accountIds, accoutInfo);
				json.setMsg("删除成功!");
				json.setSuccess(true);
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
			}
		}
		return json;
	}
	
	/**
	 *  还原
	 * @param
	 * @return
	 */
	@ApiOperation(value = "还原", notes = "支持还原多个,用逗号隔开")
	@ApiImplicitParam(name = "accountIds", value = "账号id", required = true, dataType = "String",paramType = "path")
	@GetMapping(value = "/revert/{accountIds}")
	public AjaxJson revert(@PathVariable String accountIds) {
		AjaxJson json = new AjaxJson();
		String[] accountIdArray = accountIds.split(",");
		if(accountIdArray.length > 0) {
			for (String accountId : accountIdArray) {
				AccoutInfo accoutInfo = accoutInfoService.get(accountId);
				if(accoutInfo != null) {
					accoutInfo.setDelFlag(AccoutInfo.DEL_FLAG_NORMAL);
					accoutInfoService.save(accoutInfo, accoutInfo.getAccountId());
				}
			}
			json.setSuccess(true);
			json.setMsg("操作成功");
			return json;
		}
		json.setSuccess(false);
		json.setMsg("还原失败");
		return json;
	}
	
	/**
	 * 人员图片删除
	 *@param
	 */
	@Transactional(readOnly = false)
	@ApiOperation(value = "人员图片删除", notes = "人员图片删除")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "photoUrl", value = "图片地址", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "personBaseId", value = "基本信息id", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/deletePersonPic")
	public AjaxJson deletePersonPic(@ApiIgnore PersonBase personBase) {
		AjaxJson json = new AjaxJson();
		AjaxJson result = personApiService.photoDelete(personBase.getPhotoUrl());
		if(result.isSuccess()) {
			PersonBase personBase2 = personBaseService.get(personBase.getPersonBaseId());
			personBase2.setPhotoUrl("");
			personBaseService.save(personBase2, personBase2.getPersonBaseId());
			json.setSuccess(true);
			json.setMsg("删除成功");
		}else{
			json.setSuccess(false);
			json.setMsg(result.getMsg());
		}
		return json;
	}

	/**
	 * 设置密码
	 * @param accoutInfo
	 * @return
	 */
	@ApiOperation(value = "设置密码", notes = "设置新密码")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "passWord", value = "新密码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "accountId", value = "账号id", required = true, dataType = "Integer", paramType = "query")
	})
	@GetMapping(value = "/passwordUp")
	public AjaxJson setPassword(@ApiIgnore AccoutInfo accoutInfo) {
		AjaxJson json = new AjaxJson();
		try {
			accoutInfoService.setPassword(accoutInfo);
			json.setMsg("密码修改成功!");
			json.setSuccess(true);
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
		}
		return json;
	}
	
	
	/**
	 * 密码策略校验
	 * @param
	 * @return
	 */
	@ApiOperation(value = "密码策略校验", notes = "密码策略校验")
	@ApiImplicitParam(name = "passWord", value = "密码", required = true, dataType = "String",paramType = "query")
	@PostMapping(value = "/validatePassword")
	public AjaxJson validatePassword(String passWord) {
		AjaxJson json = accountPolicyService.pwdPolicyValidate(passWord);
		return json;
	}
	
	
	/**
	 * 获取人员详细信息
	 * @param
	 * @return
	 */
	@ApiOperation(value = "获取当前登录用户", notes = "获取当前登录用户")
	@GetMapping(value = "/getCurretUser")
	public AjaxJson getCurretUser() {
		AjaxJson json = new AjaxJson();
		json.put("accout", UserUtils.getUser());
		return json;
	}
	
	/**
	 * 设置ip策略
	 * @param
	 * @return
	 */
	@ApiOperation(value = "设置ip策略", notes = "设置ip策略")
	@PostMapping(value = "/setIpPolicy")
	public AjaxJson setIpPolicy(@RequestBody IpPolicyAccount ipPolicyAccount) {
		AjaxJson json = accoutInfoService.setIpPolicy(ipPolicyAccount);
		return json;
	}
	
	/**
	 * 获取ip策略
	 * @param
	 * @return
	 */
	@ApiOperation(value = "获取ip策略", notes = "获取ip策略")
	@GetMapping(value = "/getIpPolicy/{accountId}")
	@ApiImplicitParam(name = "accountId", value = "账号id", required = true, dataType = "String", paramType = "path")
	public AjaxJson getIpPolicy(@PathVariable int accountId) {
		AjaxJson json = new AjaxJson();
		IpPolicy ipPolicy = ipPolicyService.getByAccountId(accountId);
		json.put("ipPolicy", ipPolicy);
		return json;
	}
	
	/**
	 * @Description: TODO(开发设置cookie)
	 * @author yangwenbin
	 * @date
	 */
	@ApiOperation(value = "设置Cookie", notes = "设置Cookie")
	@ApiImplicitParam(name = "userId", value = "用户ID", required = true, dataType = "String",paramType = "path")
	@GetMapping(value = "setCookie/{userId}")
	public AjaxJson setDevCookie(HttpServletResponse response, HttpServletRequest request,@PathVariable String userId){
		AjaxJson json = new AjaxJson();
		CookieUtils.setCookie(response, LoitHeaderConstant.AUTHORIZATION, userId);
		json.put("userId",userId);
		return json;
	}

	@GetMapping(value = "/getPortalIndexUrl")
	@ApiOperation(value="获取配置信息", notes="获取配置信息")
	public AjaxJson getPortalIndexUrl(){
		AjaxJson json = new AjaxJson();
		String portalIndexUrl = clientAppConfig.getPortalIndexUrl();
		json.put("portalIndexUrl", portalIndexUrl);
		return json;
	}
	
}
