package com.loit.v1.modules.controller;

import java.util.HashMap;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.v1.api.dto.PersonExtattrDTO;
import com.loit.v1.modules.entity.PersonBase;
import com.loit.v1.modules.entity.PersonExtattr;
import com.loit.v1.modules.service.PersonExtattrService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**  
* @ClassName: PersonExtattrController  
* @Description:   
* @author zhaokz  
* @date 2019年10月26日 下午2:52:44 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/personExtattr")
@Api(tags = "人员扩展属性管理Controller",  value = "PersonExtattrController")
public class PersonExtattrController {
	
	@Autowired
	private PersonExtattrService personExtattrService;
	
	/**
	 * 获取人员属性列表
	 * @param request
	 * @param response
	 * @param personExtattr
	 * @return
	 */
	@ApiOperation(value = "获取人员属性列表", notes = "获取人员属性列表")
	@GetMapping(value = "/personExtattrs/pageList")
	public AjaxJson list(@ApiIgnore PersonExtattr personExtattr){
		List<PersonExtattr> list = personExtattrService.findList(personExtattr);
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}

	/**
	 * 获取人员扩展属性详细信息
	 * @param personExtattrId
	 * @return
	 */
	@ApiOperation(value = "获取人员扩展属性详细信息", notes = "根据id来获取PersonExtattr详细信息")
	@ApiImplicitParam(name = "personAttrId", value = "id", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/personExtattr/{personAttrId}")
	public AjaxJson detail(@PathVariable String personAttrId) {
		AjaxJson json = new AjaxJson();
		PersonExtattr newPersonExtattr = new PersonExtattr();
		newPersonExtattr.setPersonAttrId(Integer.valueOf(personAttrId));
		PersonExtattr personExtattr = personExtattrService.get(newPersonExtattr);
		PersonExtattrDTO personExtattrDTO = null;
		if(personExtattr != null) {
			personExtattrDTO = ModelMapperUtils.map(personExtattr, PersonExtattrDTO.class);
		}
		json.put("accout", personExtattrDTO);
		return json;
	}

	
	/**
	 * 保存
	 * @param personExtattr
	 * @return
	 */
	@ApiOperation(value = "保存", notes = "保存对象信息")
	@ApiImplicitParam(name = "personExtattrDTO", value = "实体PersonExtattrDTO详情", required = true, dataType = "PersonExtattrDTO")
	@PostMapping(value = "/save")
	public AjaxJson save(@RequestBody PersonExtattrDTO personExtattrDTO) {
		AjaxJson json = new AjaxJson();
		try {
			PersonExtattr personExtattr = ModelMapperUtils.map(personExtattrDTO, PersonExtattr.class);
			personExtattrService.save(personExtattr,personExtattr.getPersonAttrId());
			json.setMsg("保存成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存失败!" + e.getMessage());
		}
		return json;
	}

	/**
	 * 删除
	 *@param 
	 */
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "personAttrIds", value = "personExtattrIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/personExtattr/{personAttrIds}")
	public AjaxJson delete(@PathVariable String personAttrIds) {
		if(StringUtils.isBlank(personAttrIds)) {
			return AjaxJson.returnExceptionInfo("属性id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(personAttrIds)) {
			PersonExtattr personExtattr = new PersonExtattr();
			try {
				personExtattrService.deleteAll(personAttrIds, personExtattr);
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除失败" + e.getMessage());
			}
		}
		return json;
	}
}
