package com.loit.v1.modules.controller;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.v1.api.dto.PersonExtattrInstDTO;
import com.loit.v1.modules.entity.PersonExtattrInst;
import com.loit.v1.modules.service.PersonExtattrInstService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**  
* @ClassName: PersonExtattrInstController  
* @Description:   人员扩展实例管理
* @author zhaokz  
* @date 2019年10月26日 下午3:02:28 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/personExtattrInst")
@Api(tags = "人员扩展实例管理",  value = "PersonExtattrInstController")
public class PersonExtattrInstController {
	
	@Autowired
	private PersonExtattrInstService personExtattrInstService;
	
	/**
	 * 获取人员扩展实例列表
	 * @param request
	 * @param response
	 * @param personExtattrInst
	 * @return
	 */
	@ApiOperation(value = "获取人员扩展实例列表", notes = "获取人员扩展实例列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "personExtattrInst", value = "详细实体PersonExtattrInst的参数值可设为查询条件", required = false, dataType = "PersonExtattrInst", paramType = "query") })
	@GetMapping(value = "/personExtattrInsts/pageList")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response,@ApiIgnore PersonExtattrInst personExtattrInst){

		Page<PersonExtattrInst> pageList = personExtattrInstService.findPage(new Page<PersonExtattrInst>(request, response),personExtattrInst);
		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
	}

	/**
	 * 获取人员扩展实例详细信息
	 * @param personExtattrInstId
	 * @return
	 */
	@ApiOperation(value = "获取人员扩展实例详细信息", notes = "根据id来获取PersonExtattrInst详细信息")
	@ApiImplicitParam(name = "instId", value = "id", required = true, dataType = "String", paramType = "path")
	@GetMapping(value = "/personExtattrInst/{instId}")
	public AjaxJson detail(@PathVariable String instId) {
		AjaxJson json = new AjaxJson();
		PersonExtattrInst newPersonExtattrInst = new PersonExtattrInst();
		newPersonExtattrInst.setInstId(Integer.valueOf(instId));
		PersonExtattrInst personExtattrInst = personExtattrInstService.get(newPersonExtattrInst);
		PersonExtattrInstDTO personExtattrInstDTO = null;
		if(personExtattrInst != null) {
			personExtattrInstDTO = ModelMapperUtils.map(personExtattrInst, PersonExtattrInstDTO.class);
		}
		json.put("accout", personExtattrInstDTO);
		return json;
	}

	
	/**
	 * 保存
	 * @param personExtattrInst
	 * @return
	 */
	@ApiOperation(value = "保存", notes = "保存对象信息")
	@ApiImplicitParam(name = "personExtattrInstDTO", value = "实体PersonExtattrInstDTO详情", required = true, dataType = "PersonExtattrInstDTO")
	@PostMapping(value = "/save")
	public AjaxJson save(@RequestBody PersonExtattrInstDTO personExtattrInstDTO) {
		AjaxJson json = new AjaxJson();
		try {
			PersonExtattrInst personExtattrInst = ModelMapperUtils.map(personExtattrInstDTO, PersonExtattrInst.class);
			personExtattrInstService.save(personExtattrInst,personExtattrInst.getInstId());
			json.setMsg("保存成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存失败!" + e.getMessage());
		}
		return json;
	}
	
	/**
	 * 删除
	 *@param 
	 */
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "instIds", value = "personExtattrInstIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/personExtattrInst/{instIds}")
	public AjaxJson delete(@PathVariable String instIds) {
		if(StringUtils.isBlank(instIds)) {
			return AjaxJson.returnExceptionInfo("机构id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(instIds)) {
			PersonExtattrInst personExtattrInst = new PersonExtattrInst();
			try {
				personExtattrInstService.deleteAll(instIds, personExtattrInst);
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除失败" + e.getMessage());
			}
		}
		return json;
	}
}
