package com.loit.v1.modules.controller;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.v1.modules.entity.Rank;
import com.loit.v1.modules.service.RankService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.List;
                                                                                                                                             
                                                                                                                                             


/**
 * 职级[门户平台]接口
 * Created by DEMON on 2020-03-05.
 **/
@RestController                                                                                                                              
@RequestMapping("api/v1/rank")                                                                                                         
@Api(tags = "职级[门户平台]接口",  value = "RankController")                                                                                              
public class RankController {                                                                                                            
                                                                                                                                             
	/**                                                                                                                                      
	 * 日志工具                                                                                                                              
	 */                                                                                                                                      
	private Logger logger = LoggerFactory.getLogger(this.getClass());                                                                        
	                                                                                                                                         
	@Autowired                                                                                                                               
	private RankService rankService;

	/**                                                                                                                                      
	 * 职级列表                                                                                                                              
	 */                                                                                                                                      
	@ApiOperation(value = "职级列表", notes = "职级列表")
	@ApiImplicitParams({                                                                                                                     
	        @ApiImplicitParam(paramType = "query", name = "rankCode", value = "职级代码", required = false, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "rankName", value = "职级名称", required = false, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "rankLvel", value = "级别", required = false, dataType = "Integer"),
	        @ApiImplicitParam(paramType = "query", name = "serialIndex", value = "排序号", required = false, dataType = "java.lang.Float"),
	        @ApiImplicitParam(paramType = "query", name = "rankDesc", value = "职级说明", required = false, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "creatTime", value = "创建时间", required = false, dataType = "Date"),
	        @ApiImplicitParam(paramType = "query", name = "updateTime", value = "修改时间", required = false, dataType = "Date"),
	        @ApiImplicitParam(paramType = "query", name = "delFlag", value = "删除标记", required = false, dataType = "String")
	})
	@RequestMapping(value = "/list", method = RequestMethod.GET)                                                                           
	public AjaxJson findAll(@ApiIgnore Rank rank) {                                                                                  
		AjaxJson json = new AjaxJson();
		try {
			List<Rank> list = rankService.findList(rank);
			json.put("list", list);
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("获取列表异常", json);
		}
		return json;                                                                                                                         
	}                                                                                                                                        
                                                                                                                                             
	/**                                                                                                                                      
	 * 职级分页列表                                                                                                                          
	 */                                                                                                                                      
	@ApiOperation(value = "职级分页列表", notes = "职级分页列表")                                                                        
	@ApiImplicitParams({                                                                                                                     
	        @ApiImplicitParam(paramType = "query", name = "pageNo", value = "当前页码", required = true, dataType = "String"),  
	        @ApiImplicitParam(paramType = "query", name = "pageSize", value = "分页大小", required = true, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "rankCode", value = "职级代码", required = false, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "rankName", value = "职级名称", required = false, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "rankLvel", value = "级别", required = false, dataType = "Integer"),
	        @ApiImplicitParam(paramType = "query", name = "serialIndex", value = "排序号", required = false, dataType = "java.lang.Float"),
	        @ApiImplicitParam(paramType = "query", name = "rankDesc", value = "职级说明", required = false, dataType = "String"),
	        @ApiImplicitParam(paramType = "query", name = "creatTime", value = "创建时间", required = false, dataType = "Date"),
	        @ApiImplicitParam(paramType = "query", name = "updateTime", value = "修改时间", required = false, dataType = "Date"),
	        @ApiImplicitParam(paramType = "query", name = "delFlag", value = "删除标记", required = false, dataType = "String")
	})
	@RequestMapping(value = "/page", method = RequestMethod.GET)                                                                           
	public AjaxJson findPage(@ApiIgnore Rank rank, HttpServletRequest request, HttpServletResponse response) {                                  
		AjaxJson json = new AjaxJson();
		try {
			Page<Rank> page = rankService.findPage(new Page<Rank>(request, response), rank);
			json.put("page", page);
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("获取分页列表异常", json);
		}
		return json;                                                                                                                         
	}                                                                                                                                        
                                                                                                                                             
	/**                                                                                                                                      
	 * 职级详情                                                                                                                              
	 */                                                                                                                                    
	@ApiOperation(value = "职级详情", notes = "根据id获取职级详情信息")                                                                  
	@ApiImplicitParam(paramType = "path", name = "id", value = "职级id", required = true, dataType = "Long")                         
	@RequestMapping(value = "/{id}", method = RequestMethod.GET)                                                                           
	public AjaxJson get(@PathVariable("id") Integer id) {
		AjaxJson json = new AjaxJson();
		try {
			Rank rank = rankService.get(id);
			json.put("obj", rank);
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("获取详情异常", json);
		}
		return json;                                                                                                                         
	}                                                                                                                                        
                                                                                                                                             
	/**                                                                                                                                      
	 * 职级保存                                                                                                                              
	 */                                                                                                                                      
	@ApiOperation(value = "职级保存", notes = "职级保存")                                                                                
	@RequestMapping(value = "/save", method = RequestMethod.POST)                                                                          
	public AjaxJson save(@RequestBody Rank rank) {                                                                                   
		AjaxJson json = new AjaxJson();                                                                                                      
		try {
			// 职级code重复判断
			if(rankService.isRankCodeRepeat(rank.getRankCode())){
				return AjaxJson.returnExceptionInfo("职级编码重复");
			}

			rank.setCreateTime(new Date());
			rankService.insert(rank);                                                                                                
			json.setMsg("保存成功");                                                                                                       
		} catch (Exception e) {                                                                                                              
			e.printStackTrace();                                                                                                             
			return AjaxJson.returnExceptionInfo("保存异常");                                                                               
		}                                                                                                                                    
		return json;                                                                                                                         
	}                                                                                                                                        
	                                                                                                                                         
	/**                                                                                                                                      
	 * 职级修改                                                                                                                              
	 */                                                                                                                                      
	@ApiOperation(value = "职级修改", notes = "职级修改")                                                                                
	@RequestMapping(value = "/update", method = RequestMethod.POST)                                                                        
	public AjaxJson update(@RequestBody Rank rank) {                                                                                 
		AjaxJson json = new AjaxJson();
		try {
			// 职级code重复判断
			if(rankService.isRankCodeRepeat(rank.getRankCode(), rank.getRankId())){
				return AjaxJson.returnExceptionInfo("职级编码重复");
			}

			rankService.update(rank);                                                                                                
			json.setMsg("修改成功");                                                                                                       
		} catch (Exception e) {                                                                                                              
			e.printStackTrace();                                                                                                             
			return AjaxJson.returnExceptionInfo("修改异常");                                                                               
		}                                                                                                                                    
		return json;                                                                                                                         
	}                                                                                                                                        
	                                                                                                                                         
	/**                                                                                                                                      
	 * 职级删除                                                                                                                              
	 */                                                                                                                                      
	@ApiOperation(value = "职级删除", notes = "职级删除")                                                                                
	@ApiImplicitParam(paramType = "path", name = "id", value = "职级id", required = true, dataType = "Long")                         
	@RequestMapping(value = "/{id}", method = RequestMethod.DELETE)                                                                        
	public AjaxJson delete(@PathVariable("id") String ids) {                                                                               
		AjaxJson json = new AjaxJson();                                                                                                      
		try {                                                                                                                                
			rankService.deleteAll(ids, new Rank());                                                                                  
			json.setMsg("删除成功!");                                                                                                      
		} catch (Exception e) {                                                                                                              
			e.printStackTrace();                                                                                                             
			return AjaxJson.returnExceptionInfo("删除失败" + e.getMessage());                                                              
		}                                                                                                                                    
		return json;                                                                                                                         
	}

	/**
	 * 职级逻辑删除
	 */
	@ApiOperation(value = "职级逻辑删除", notes = "职级逻辑删除")
	@ApiImplicitParam(paramType = "path", name = "id", value = "职级id", required = true, dataType = "Long")
	@RequestMapping(value = "/logic/{id}", method = RequestMethod.DELETE)
	public AjaxJson deleteLogic(@PathVariable("id") String ids) {
		AjaxJson json = new AjaxJson();
		try {
			rankService.deleteAllByLogic(ids, new Rank());
			json.setMsg("删除成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("删除失败" + e.getMessage());
		}
		return json;
	}

}
