package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.api.dto.ResourceManageDTO;
import com.loit.v1.modules.entity.ResourceManage;
import com.loit.v1.modules.service.ResourceManageService;
import com.loit.v1.modules.service.RoleRightService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @description 资源管理Controller
 * @author xutao
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/resourceManage")
@Api(value = "ResourceManageController", tags = {"应用资源管理Controller"})
public class ResourceManageController extends BaseController {

	@Autowired
	private ResourceManageService resourceManageService;
	@Autowired
	private RoleRightService roleRightService;
	
	/**
	 * @description 获取资源的树形分页列表
	 * @author xutao
	 * @date 2019年10月18日
	 * @param resourceManage 应用资源管理实体类	
	 * @return
	 */
	@GetMapping(value = "treePage")
	@ApiOperation(value="树形分页列表", notes="获取资源的树形分页列表")	
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "resourceName", value = "资源名称", required = false, dataType = "String",paramType = "query"),
		@ApiImplicitParam(name = "appId", value = "应用ID", required = false, dataType = "Integer",paramType = "query")
	})
	public AjaxJson page(@ApiIgnore ResourceManage resourceManage, HttpServletRequest request, HttpServletResponse response) {
		Page<ResourceManage> page = resourceManageService.findPage(new Page<ResourceManage>(request, response), resourceManage);
		//logger.debug("获取树形列表成功");
		AjaxJson json = new AjaxJson();
		json.put("treePage", page);
		return json;
	}
	
	/**
	 * @description 根据应用ID获取所分配资源树状结构
	 * @author xutao
	 * @date 2019年10月18日
	 * @param resourceManage 应用资源管理实体类	
	 * @return
	 */
	@GetMapping(value = "treeList")
	@ApiOperation(value="根据应用ID获取所分配资源树状结构  - 菜单增加选择资源时用", notes="根据应用ID获取所分配资源树状结构 - 菜单增加选择资源时用")	
	@ApiImplicitParams({
		@ApiImplicitParam(name = "typeId", value = "资源类别Id", required = false, dataType = "int", paramType = "query"), 
		@ApiImplicitParam(name = "appId", value = "应用Id", required = true, dataType = "String", paramType = "query")
	})
	public AjaxJson tree(Integer appId,Integer typeId) {
		List<ResourceManage> treeList = resourceManageService.getResourceTreeList(appId,typeId);
		AjaxJson json = new AjaxJson();
		json.put("treeList", treeList);
		return json;
	}

	@GetMapping(value = "rightTreeList")
	@ApiOperation(value="根据应用ID获取所分配资源树状结构  - 角色授权时用", notes="根据应用ID获取所分配资源树状结构 - 角色授权时用")	
	@ApiImplicitParams({
		@ApiImplicitParam(name = "typeId", value = "资源类别Id", required = false, dataType = "int", paramType = "query"), 
		@ApiImplicitParam(name = "appId", value = "应用Id，type=1或2时必填值", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "roleId", value = "角色Id。type=1或3时，必须传值", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "type", value = "1:获取资源树并获取已选中资源IDs；2:仅获取资源树；3：紧获取已选中资源IDs", required = true, dataType = "String", paramType = "query")
	})
	public AjaxJson rightTreeList(String type,Integer appId,Integer typeId,Integer roleId) {
		AjaxJson json = new AjaxJson();
		//获取资源树
		if(StringUtils.equals("1", type) || StringUtils.equals("2", type)) {
			json.put("treeList", resourceManageService.getResourceTreeList(appId,typeId));
		}
		//获取选中资源IDs
		if(StringUtils.equals("1", type) || StringUtils.equals("3", type)) {
			if(roleId != null) {
				json.put("selectedIds", roleRightService.getRoleResIds(roleId, typeId));
			} else {
				return AjaxJson.returnExceptionInfo("请先选择角色！");
			}
		}
		return json;
	}
	

	/**
	 * @Description 应用资源管理详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param resourceId
	 * @return
	 */
	@GetMapping(value = "{resourceId}")
	@ApiOperation(value="详情", notes="根据resourceId获取资源详情")
	@ApiImplicitParam(name = "resourceId", value = "资源ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String resourceId) {	
		if(StringUtils.isBlank(resourceId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		ResourceManage resourceManage  = resourceManageService.get(resourceId);
		//logger.debug("获取应用资源详情成功");
		AjaxJson json = new AjaxJson();
		json.put("obj", resourceManage);
		return json;
	}


	/**
	 * @Description 新增应用资源
	 * @author xutao
	 * @date：2019年10月18日
	 * @param skinManageDTO
	 * @return
	 */
	@PostMapping(value = "save")
	@ApiOperation(value = "保存", notes = "新增应用资源")
	@ApiImplicitParam(name = "resourceManageDTO", value = "应用资源数据传输实体类", required = true, dataType = "ResourceManageDTO")
	public AjaxJson save(@RequestBody ResourceManageDTO resourceManageDTO) {
		AjaxJson json = new AjaxJson();	
		ResourceManage resourceManage = ModelMapperUtils.map(resourceManageDTO, ResourceManage.class);
		try {			
			resourceManageService.insert(resourceManage);
			json.setMsg("应用资源保存成功");
			return json;
		} catch (Exception e) {
			logger.error("info:应用资源保存失败,class:{},message:{}",this.getClass(),e.getMessage());
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);			
		}		
	}
	
	
	/**
	 * @Description  修改应用资源
	 * @param resourceId
	 * @param resourceManageDTO
	 * @return
	 */
	@PostMapping(value = "{resourceId}")
	@ApiOperation(value = "修改", notes = "修改应用资源")
	public AjaxJson update(@PathVariable int resourceId,@RequestBody ResourceManageDTO resourceManageDTO) {
		AjaxJson json = new AjaxJson();
		ResourceManage resourceManage = ModelMapperUtils.map(resourceManageDTO, ResourceManage.class);
		try {
			resourceManage.setResourceId(resourceId);
			resourceManageService.update(resourceManage);
			json.setMsg("应用资源修改成功");
			return json;
		} catch (Exception e) {
			logger.error("info:应用资源修改失败,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.UPDATE_EXCEPTION);		
		}		
	}

	/**
	 * @Description 应用资源删除
	 * @author xutao
	 * @date：2019年10月18日
	 * @param ids 多ID时，以逗号分隔
	 * @return
	 */
	@ApiOperation(value = "删除", notes = "单个或批量删除应用资源，多ID时，以逗号分隔")
	@DeleteMapping(value = "{resourceId}")
	public AjaxJson deleteAll(@PathVariable("resourceId")String ids) {
		AjaxJson json = new AjaxJson();
		if(StringUtils.isBlank(ids)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}		
		try {
			ResourceManage resourceManage = new ResourceManage();
			resourceManageService.deleteAll(ids,resourceManage);
			json.setMsg("删除成功");
			return json;
		} catch (Exception e) {
			logger.error("info:删除异常,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.DELETE_EXCEPTION);			
		}
	}
	
	/**
	 * @description 检查资源名称是否重复
	 * @param resourceManage
	 * @return
	 */
	@ApiOperation(value = "检查资源名称是否重复", notes = "检查资源名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "resourceId", value = "资源ID", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "resourceName", value = "资源名称", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkResourceNameUnique")
	public AjaxJson checkResourceNameUnique(@ApiIgnore ResourceManage resourceManage) {
		AjaxJson json = new AjaxJson();
		List<ResourceManage> list = resourceManageService.findList(resourceManage);
		if(list.size()>0) {
			json.setMsg("资源名称重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}else {
			json.setMsg("资源名称不重复");
		}
		return json;
	}

	/**
	 * @description 检查资源编码是否重复
	 * @param resourceManage
	 * @return
	 */
	@ApiOperation(value = "检查资源编码是否重复", notes = "检查资源编码是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "resourceId", value = "资源ID", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "resourceCode", value = "资源编码", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkResourceCodeUnique")
	public AjaxJson checkResourceCodeUnique(@ApiIgnore ResourceManage resourceManage) {
		AjaxJson json = new AjaxJson();
		List<ResourceManage> list = resourceManageService.findList(resourceManage);
		if(list.size()>0) {
			json.setMsg("资源编码重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}else {
			json.setMsg("资源编码可用");
		}
		return json;
	}

}
