package com.loit.v1.modules.controller;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.v1.modules.entity.ResourceOperation;
import com.loit.v1.modules.service.ResourceOperationService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**  
* @ClassName: ResourceOperationController  
* @Description:   
* @author zhaokz  
* @date 2020年4月13日 上午10:25:43 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/resourceOperation")
@Api(value = "ResourceOperationController", tags = {"资源操作Controller"})
public class ResourceOperationController {
	private static Logger logger = LoggerFactory.getLogger(ResourceOperationController.class);
	
	@Autowired
	private ResourceOperationService resourceOperationService;
	
	@GetMapping(value = "list")
	@ApiOperation(value="列表", notes="获取资源操作列表")	
	@ApiImplicitParam(name = "resId", value = "资源ID", required = false, dataType = "Integer", paramType = "query")
	public AjaxJson list(@ApiIgnore ResourceOperation resourceOperation) {
		List<ResourceOperation> list = resourceOperationService.findList(resourceOperation);
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}
	
	@GetMapping(value = "{instId}")
	@ApiOperation(value="详情", notes="获取资源操作详情")
	@ApiImplicitParam(name = "instId", value = "资源操作ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=true) String instId) {	
		if(StringUtils.isBlank(instId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		ResourceOperation resourceOperation  = resourceOperationService.get(instId);
		AjaxJson json = new AjaxJson();
		json.put("resourceOperation", resourceOperation);
		return json;
	}
	
	@PostMapping(value = "save")
	@ApiOperation(value = "保存", notes = "新增资源操作")
	@ApiImplicitParam(name = "resourceOperation", value = "资源操作数据传输实体类", required = true, dataType = "ResourceOperation")
	public AjaxJson save(@RequestBody ResourceOperation resourceOperation) {
		AjaxJson json = new AjaxJson();	
		try {			
			resourceOperationService.save(resourceOperation);
			json.setMsg("保存成功");
			return json;
		} catch (Exception e) {
			logger.error("info:资源操作保存失败,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
		}		
	}
	
	@PostMapping(value = "{instId}")
	@ApiOperation(value = "修改", notes = "修改资源操作")
	public AjaxJson update(@PathVariable int instId,@RequestBody ResourceOperation resourceOperation) {
		AjaxJson json = new AjaxJson();
		try {
			resourceOperation.setInstId(instId);;
			resourceOperationService.update(resourceOperation);
			json.setMsg("修改成功");
			return json;
		} catch (Exception e) {
			logger.error("info:应用资源类别修改失败,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.UPDATE_EXCEPTION);	
		}		
	}
	
	
	@ApiOperation(value = "删除", notes = "单个或批量删除应用资源，多ID时，以逗号分隔")
	@DeleteMapping(value = "{instIds}")
	public AjaxJson deleteAll(@PathVariable("instIds")String instIds) {
		if(StringUtils.isBlank(instIds)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		AjaxJson json = new AjaxJson();
		try {
			ResourceOperation resourceOperation = new ResourceOperation();
			resourceOperationService.deleteAll(instIds,resourceOperation);
			json.setMsg("删除成功");
			return json;
		} catch (Exception e) {
			logger.error("info:删除异常,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.DELETE_EXCEPTION);	
		}
	}
}
