package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.api.dto.ResourceTypeOperationDTO;
import com.loit.v1.modules.entity.ResourceTypeOperation;
import com.loit.v1.modules.service.ResourceTypeOperationService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @description 资源类别操作管理Controller
 * @author xutao
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/resourceTypeOperation")
@Api(value = "ResourceTypeOperationController", tags = {"应用资源类别操作管理Controller"})
public class ResourceTypeOperationController extends BaseController {

	@Autowired
	private ResourceTypeOperationService resourceTypeOperationService;
	
	/**
	 * @description 应用资源类别操作管理分页列表
	 * @author xutao
	 * @date 2019年10月18日
	 * @param resourceTypeOperation 应用资源类别操作管理实体类	
	 * @return
	 */
	@GetMapping(value = "page")
	@ApiOperation(value="分页列表", notes="获取资源类别操作管理分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "typeId", value = "资源类别ID", required = true, dataType = "String", paramType = "query")	
	})
	public AjaxJson page(@ApiIgnore ResourceTypeOperation resourceTypeOperation, HttpServletRequest request, HttpServletResponse response) {
		Page<ResourceTypeOperation> page = resourceTypeOperationService.findPage(new Page<ResourceTypeOperation>(request, response), resourceTypeOperation);
		//logger.debug("获取资源类别操作管理分页列表成功");
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}
	
	/**
	 * @description 应用资源类别操作管理列表
	 * @author xutao
	 * @date 2019年10月18日
	 * @param resourceTypeOperation 应用资源类别操作管理实体类	
	 * @return
	 */
	@GetMapping(value = "list")
	@ApiOperation(value="列表", notes="获取全部资源类别操作列表")	
	@ApiImplicitParam(name = "typeId", value = "资源类别ID", required = true, dataType = "String", paramType = "query")
	public AjaxJson list(@ApiIgnore ResourceTypeOperation resourceTypeOperation) {
		List<ResourceTypeOperation> list = resourceTypeOperationService.findAllList(resourceTypeOperation);
		//logger.debug("获取资源类别操作列表成功");
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}
	
	@GetMapping(value = "resOpList")
	@ApiOperation(value="列表", notes="获取资源类别操作列表")	
	@ApiImplicitParam(name = "resId", value = "资源类别ID", required = true, dataType = "int", paramType = "query")
	public AjaxJson resOpList(@ApiIgnore ResourceTypeOperation resourceTypeOperation) {
		List<ResourceTypeOperation> list = resourceTypeOperationService.findList(resourceTypeOperation);
		//logger.debug("获取资源类别操作列表成功");
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}

	/**
	 * @description 获取资源操作列表
	 * @param resourceTypeOperation
	 * @return
	 */
	@GetMapping(value = "actList/{resId}")
	@ApiOperation(value="获取资源操作列表", notes="获取资源操作列表")
	@ApiImplicitParam(name = "resId", value = "资源ID", required = true, dataType = "int", paramType = "path")
	public AjaxJson actList(@PathVariable(required = true) int resId) {
		ResourceTypeOperation resourceTypeOperation = new ResourceTypeOperation();
		resourceTypeOperation.setResId(resId);
		List<ResourceTypeOperation> list = resourceTypeOperationService.findList(resourceTypeOperation);
		//logger.debug("获取资源类别操作管理分页列表成功");
		AjaxJson json = new AjaxJson();
		json.put("actList", list);
		return json;
	}
	
	/**
	 * @Description 资源类别操作详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param actId
	 * @return
	 */
	@GetMapping(value = "{actId}")
	@ApiOperation(value="详情", notes="根据actId获取资源类别操作详情")
	@ApiImplicitParam(name = "actId", value = "资源类别操作ID", required = true, dataType = "int")
	public AjaxJson get(@PathVariable(required=false) String actId) {	
		if(StringUtils.isBlank(actId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		ResourceTypeOperation resourceTypeOperation  = resourceTypeOperationService.get(actId);
		//logger.debug("获取资源类别操作详情成功");
		AjaxJson json = new AjaxJson();
		json.put("obj", resourceTypeOperation);
		return json;
	}
	
	/**
	 * @description 新增或修改资源类别操作
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dict 
	 * @return
	 */
	@PostMapping(value = "")
	@ApiOperation(value = "保存", notes = "新增或修改资源类别操作")
	@ApiImplicitParam(name = "resourceTypeOperationDTO", value = "资源类别操作管理数据传输实体类", required = true, dataType = "ResourceTypeOperationDTO")
	public AjaxJson save(@RequestBody ResourceTypeOperationDTO resourceTypeOperationDTO) {
		AjaxJson json = new AjaxJson();
		ResourceTypeOperation resourceTypeOperation = ModelMapperUtils.map(resourceTypeOperationDTO, ResourceTypeOperation.class);
		try {
			List<ResourceTypeOperation> list = resourceTypeOperationService.findAllList(resourceTypeOperation);
			if(list.size()>0) {
				return AjaxJson.returnExceptionInfo(LoitStatusCode.RESOURCETYPE_OPERATION_IS_EXIST);
			}else {
				resourceTypeOperationService.save(resourceTypeOperation,resourceTypeOperation.getActId());
				json.setMsg("保存成功");
				return json;
			}
		} catch (Exception e) {
			logger.error("erroCode:{},info:保存资源类别操作异常",LoitStatusCode.FAIL.statusCode);
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
		}
	}

	/**
	 * @Description 单个或批量删除
	 * @author xutao
	 * @date：2019年10月18日
	 * @param ids 多ID时，以逗号分隔
	 * @return
	 */
	@ApiOperation(value = "删除", notes = "单个或批量删除，多ID时，以逗号分隔")
	@DeleteMapping(value = "{actId}")
	public AjaxJson deleteAll(@PathVariable("actId")String ids) {
		if(StringUtils.isBlank(ids)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		AjaxJson json = new AjaxJson();
		try {
			ResourceTypeOperation resourceTypeOperation = new ResourceTypeOperation();
			resourceTypeOperationService.deleteAll(ids,resourceTypeOperation);
			json.setMsg("删除成功");
			return json;
		} catch (Exception e) {
			logger.error("info:删除异常,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.DELETE_EXCEPTION);
		}
	}
	
	/**
	 * @description 检查资源类别名称是否重复
	 * @param actId
	 * @param actName
	 * @return
	 */
	@ApiOperation(value = "检查资源操作名称是否重复", notes = "检查资源类别操作名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "actId", value = "资源操作ID", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "actName", value = "资源操作名称", required = true, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/checkOperationNameUnique")
	public AjaxJson checkOperationNameUnique(Integer actId,String actName) {
		boolean uniqueFlag = resourceTypeOperationService.checkNameUnique(actId, actName);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("资源操作名称不重复");
		} else {
			json.setMsg("资源操作名称重复");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}
	
}
