package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.loit.v1.modules.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.web.BaseController;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.api.dto.RightDTO;
import com.loit.v1.modules.entity.PersonBase;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**  
* @ClassName: RightController  
* @Description:   
* @author zhaokz  
* @date 2019年11月11日 上午10:37:51 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/right")
@Api(value = "RightController", tags = {"权限分配Controller"})
public class RightController extends BaseController {
	
	
	@Autowired
	private AccountRoleService accountRoleService;
	@Autowired
	private OrgRoleService orgRoleService;
	@Autowired
	private GrpRoleService grpRoleService;	
	@Autowired
	private PersonBaseService personBaseService;	
	@Autowired
	private RoleService roleService;
	@Autowired
	private PositionRoleService positionRoleService;
	@Autowired
	private RankRoleService rankRoleService;

	/**
	 * 人员角色分配
	 * @param appId
	 * @return
	 */
	@ApiOperation(value = "人员角色分配", notes = "人员角色分配")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "accountId", value = "人员ID", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "roleIds", value = "角色ID列表, 多个用逗号分隔", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "appId", value = "应用ID", required = true, dataType = "String", paramType = "query")
	})
	@PostMapping(value = "/personAssign")
	public AjaxJson personAssign(@ApiIgnore RightDTO rightDTO) {
		AjaxJson json = accountRoleService.personAssign(rightDTO);
		return json;
	}	
	

	/**
	 * 用户组角色分配
	 * @param appId
	 * @return
	 */
	@ApiOperation(value = "用户组角色分配", notes = "用户组角色分配")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "grpId", value = "用户组ID", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "roleIds", value = "角色ID列表, 多个用逗号分隔", required = false, dataType = "String", paramType = "query")
	})
	@PostMapping(value = "/groupAssign")
	public AjaxJson groupAssign(@ApiIgnore RightDTO rightDTO) {
		AjaxJson json = grpRoleService.groupAssign(rightDTO);
		return json;
	}

	/**
	 * 职务角色分配
	 */
	@ApiOperation(value = "职务角色分配", notes = "职务角色分配")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "positionId", value = "职务ID", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "roleIds", value = "角色ID列表, 多个用逗号分隔", required = true, dataType = "String", paramType = "query")
	})
	@PostMapping(value = "/positionAssign")
	public AjaxJson positionAssign(@RequestBody RightDTO rightDTO) {
		AjaxJson json = positionRoleService.positionAssign(rightDTO);
		return json;
	}

	/**
	 * 职级角色分配
	 */
	@ApiOperation(value = "职级角色分配", notes = "职级角色分配")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "rankId", value = "职级ID", required = true, dataType = "String", paramType = "query"),
			@ApiImplicitParam(name = "roleIds", value = "角色ID列表, 多个用逗号分隔", required = true, dataType = "String", paramType = "query")
	})
	@PostMapping(value = "/rankAssign")
	public AjaxJson rankAssign(@RequestBody RightDTO rightDTO) {
		AjaxJson json = rankRoleService.rankAssign(rightDTO);
		return json;
	}

	/**
	 * @description 组织机构分配角色
	 * @author xutao
	 * @date：2019年10月18日
	 * @param roleIds 角色ID 多ID时，以逗号分隔 
	 * @param orgId 组织机构ID
	 * TODO 对接时需要修改相关接口
	 * @return
	 */
	@ApiOperation(value = "组织机构分配角色", notes = "组织机构分配角色")
	@PostMapping(value = "orgAssign")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "roleIds", value = "角色ID,多ID时，以逗号分隔", required = true, dataType = "String",
				paramType = "query"),
		@ApiImplicitParam(name = "orgId", value = "组织机构ID", required = true, dataType = "String",
		paramType = "query")		
	})
	public AjaxJson orgAssign(@RequestBody RightDTO rightDTO) {		
		AjaxJson json = new AjaxJson();
		try {
			orgRoleService.orgDistributeRoles(rightDTO);
			json.setMsg("组织机构分配角色成功");
			return json;
		} catch (Exception e) {
			logger.error("info:组织机构分配角色异常,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.UNKNOWN_EXCEPTION);		
		}
	}
	
	/**
	 * @description 组织机构分配角色的id数组
	 * @param appId
	 * @return
	 */
	@ApiOperation(value = "组织机构分配角色的选中数组", notes = "组织机构分配角色的选中数组")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "orgId", value = "组织机构ID", required = true, dataType = "String", paramType = "path")
	})
	@GetMapping(value = "/orgRoleIds/{orgId}")
	public AjaxJson orgRoleIds(@PathVariable Integer orgId) {
		AjaxJson json = new AjaxJson();
		List<Integer> roleIds = orgRoleService.getRoleIdsByOrgId(orgId);
		json.put("roleIds", roleIds);
		json.setSuccess(true);
		return json;
	}
	
	/**
	 * @description 查看某应用下某角色分配的人员的分页列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @param personBase 人员基础信息表数据对象实体
	 * @param request
	 * @param response
	 * @return
	 */
	@GetMapping(value = "findPersonPageByRole")
	@ApiOperation(value="查看某应用下某角色分配的人员的分页列表", notes="查看某应用下某角色分配的人员的分页列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "roleId", value = "角色ID", required = true, dataType = "String", paramType = "query")		
	})	
	public AjaxJson findPersonPageByRole(@ApiIgnore PersonBase personBase, HttpServletRequest request, HttpServletResponse response) {
		Role role = roleService.get(personBase.getRoleId());
		personBase.setAppId(role.getAppId());
		Page<PersonBase> page = personBaseService.findPersonPageByRole(new Page<PersonBase>(request, response), personBase);        
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}
	
	/**
	 * 人员角色查看
	 * @param appId
	 * @return
	 */
	@ApiOperation(value = "人员角色查看", notes = "人员角色查看")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "accountId", value = "账号ID", required = true, dataType = "String", paramType = "path")
	})
	@GetMapping(value = "/personRoleView/{accountId}")
	public AjaxJson personRoleView(@PathVariable String accountId) {
		AjaxJson json = new AjaxJson();
		List<Role> roles = roleService.getRolesByAccountId(accountId);
		json.put("roles", roles);
		json.setSuccess(true);
		return json;
	}
	
	
	/**
	 * 人员角色id数组
	 * @param appId
	 * @return
	 */
	@ApiOperation(value = "人员角色选中数组", notes = "人员角色选中数组")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "accountId", value = "账号ID", required = true, dataType = "String", paramType = "path")
	})
	@GetMapping(value = "/personRoleIds/{accountId}")
	public AjaxJson personRoleIds(@PathVariable String accountId) {
		AjaxJson json = new AjaxJson();
		List<Integer> roleIds = roleService.getRoleIdsByAccountId(accountId);
		json.put("roleIds", roleIds);
		json.setSuccess(true);
		return json;
	}
	
	/**
	 * 用户组角色列表
	 * @param appId
	 * @return
	 */
	@ApiOperation(value = "用户组角色选中数组", notes = "用户组角色选中数组")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "grpId", value = "用户组ID", required = true, dataType = "String", paramType = "path")
	})
	@GetMapping(value = "/groupRoleIds/{grpId}")
	public AjaxJson groupRoleIds(@PathVariable String grpId) {
		AjaxJson json = new AjaxJson();
		List<Integer> roleIds = roleService.getRolesByGrpId(grpId);
		json.put("roleIds", roleIds);
		json.setSuccess(true);
		return json;
	}
}
