package com.loit.v1.modules.controller;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.jms.Destination;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.loit.common.annotation.BeforeAuditAnnotation;
import org.apache.activemq.command.ActiveMQQueue;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jms.core.JmsTemplate;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Role;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.api.dto.RoleDTO;
import com.loit.v1.modules.activemq.MessageModel;
import com.loit.v1.modules.activemq.Producer;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.service.RoleService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**  
* @ClassName: RoleController  
* @Description:   
* @author zhaokz  
* @date 2019年10月26日 下午3:50:59 
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/role")
@Api(tags = "角色管理",  value = "RoleController")
public class RoleController {

	public static final Destination QUEUE_SYNC_ROLE_SAVE = new ActiveMQQueue(BusinessConstants.QUEUE_SYNC_ROLE_SAVE);
	public static final Destination QUEUE_SYNC_ROLE_DELETE = new ActiveMQQueue(BusinessConstants.QUEUE_SYNC_ROLE_DELETE);

	@Autowired
	private RoleService roleService;

	@Autowired
	private Producer producer;
	@Autowired
	private JmsTemplate jmsTemplate;
	

	/**
	 * 获取角色列表
	 * @param request
	 * @param response
	 * @param role
	 * @return
	 */
	@ApiOperation(value = "获取角色列表", notes = "获取角色列表")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "appId", value = "应用Id", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "orgId", value = "组织Id", required = false, dataType = "int", paramType = "query"),
		@ApiImplicitParam(name = "roleName", value = "角色名称", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "roleCode", value = "角色编码", required = false, dataType = "String", paramType = "query"),
		@ApiImplicitParam(name = "roleDesc", value = "角色描述", required = false, dataType = "String", paramType = "query"), 
		@ApiImplicitParam(name = "roleStatus", value = "角色状态", required = false, dataType = "String", paramType = "query") 
	})
	@GetMapping(value = "")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response,@ApiIgnore Role role){
		User user = UserUtils.getUser();
		if(BusinessConstants.ORG_USER.equals(user.getUserType())) {
			role.setRoleType(BusinessConstants.ORG_ROLE);
		}
		Page<Role> pageList = roleService.findPage(new Page<Role>(request, response),role);
		AjaxJson json = new AjaxJson();
		json.put("page", pageList);
		return json;
	}

	/**
	 * 获取角色角色详细信息
	 * @param roleId
	 * @return
	 */
	@ApiOperation(value = "获取角色详细信息", notes = "根据id来获取Role详细信息")
	@ApiImplicitParam(name = "roleId", value = "roleId", required = true, dataType = "int", paramType = "path")
	@GetMapping(value = "{roleId}")
	public AjaxJson detail(@PathVariable int roleId) {
		AjaxJson json = new AjaxJson();
		RoleDTO role = roleService.getByRoleId(roleId);
		json.put("role", role);
		return json;
	}

	
	/**
	 * 保存
	 * @param role
	 * @return
	 */
	@ApiOperation(value = "保存/修改", notes = "保存对象信息")
	@PostMapping(value = "/save")
	@BeforeAuditAnnotation
	public AjaxJson save( @RequestBody Role role) {
		AjaxJson json = new AjaxJson();
		//发送消息到队列
		Map<String, Object> map = new HashMap<>();
		map.put("isInsert", role.getRoleId() == null ? true : false);
		try {
			boolean nameUniqueFlag = roleService.checkNameUnique(role.getRoleId(), role.getRoleName());
			if(!nameUniqueFlag) {
				return AjaxJson.returnExceptionInfo("角色名称重复!");
			}
			boolean codeUniqueFlag = roleService.checkNameUnique(role.getRoleId(), role.getRoleName());
			if(!codeUniqueFlag) {
				return AjaxJson.returnExceptionInfo("角色编码重复!");
			}
			roleService.save(role,role.getRoleId());
			json.setMsg("保存成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存角色异常" + e.getMessage());
		}
		map.put("role",role);
		MessageModel message = new MessageModel();
		message.setMap(map);
		producer.send(QUEUE_SYNC_ROLE_SAVE,message);
		return json;
	}

	/**
	 * 删除
	 *@param 
	 */
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "roleIds", value = "roleIds", required = true, dataType = "String", paramType = "path")
	@DeleteMapping(value = "/{roleIds}")
	public AjaxJson delete(@PathVariable String roleIds) {
		if(StringUtils.isBlank(roleIds)) {
			return AjaxJson.returnExceptionInfo("机构id不能为空!");
		}
		AjaxJson json = new AjaxJson();
		try {
			//取出对象列表
			String[] roleArr = roleIds.split(",");
			List<Role> roleList = roleService.findByIdList(Arrays.asList(roleArr));
			//删除
			roleService.deleteRoleByIds(roleIds);
			json.setMsg("删除成功!");
			//发送消息到队列
			if( roleList != null) {
				Map<String, Object> map = new HashMap<>();
				map.put("roleIds", roleIds);
				map.put("roleList", roleList);
				MessageModel message = new MessageModel();
				message.setMap(map);
				producer.send(QUEUE_SYNC_ROLE_DELETE, message);
			}
			} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("删除角色失败" + e.getMessage());
		}
		return json;
	}
	
	
	@ApiOperation(value = "检查角色名称是否重复", notes = "检查角色名称是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "roleId", value = "角色id", required = false, dataType = "int", paramType = "path"),
		@ApiImplicitParam(name = "roleName", value = "角色名称", required = true, dataType = "String", paramType = "path")
	})
	@GetMapping(value = "/checkNameUnique")
	public AjaxJson checkNameUnique( Integer roleId,String roleName) {
		boolean uniqueFlag = roleService.checkNameUnique(roleId, roleName);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("角色名称不重复！");
		} else {
			json.setMsg("角色名称重复！");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}
	
	@ApiOperation(value = "检查角色编码是否重复", notes = "检查角色编码是否重复")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "roleId", value = "角色id", required = false, dataType = "int", paramType = "path"),
		@ApiImplicitParam(name = "roleCode", value = "角色编码", required = true, dataType = "String", paramType = "path")
	})
	@GetMapping(value = "/checkCodeUnique")
	public AjaxJson checkCodeUnique( Integer roleId,String roleCode) {
		boolean uniqueFlag = roleService.checkCodeUnique(roleId, roleCode);
		AjaxJson json = new AjaxJson();
		if(uniqueFlag) {
			json.setMsg("角色名称不重复！");
		} else {
			json.setMsg("角色名称重复！");
			json.setCode(LoitStatusCode.FAIL.statusCode);
		}
		return json;
	}
}
