package com.loit.v1.modules.controller;

import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.modules.entity.SkinManage;
import com.loit.v1.modules.service.SkinManageService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import springfox.documentation.annotations.ApiIgnore;

/**
 * @description 应用皮肤管理Controller
 * @author xutao
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/skinManage")
@Api(value = "SkinManageController", tags = {"应用皮肤管理Controller"})
public class SkinManageController extends BaseController {

	@Autowired
	private SkinManageService skinManageService;

	/**
	 * @description 应用皮肤管理列表
	 * @author xutao
	 * @date：2019年10月18日
	 * @param skinManage 应用皮肤管理实体类
	 * @param request
	 * @param response
	 * @return
	 */
	@GetMapping(value = "")
	@ApiOperation(value="列表", notes="应用皮肤管理列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "skinCode", value = "皮肤代码", required = false, dataType = "String",
					paramType = "query"),
			@ApiImplicitParam(name = "skinName", value = "皮肤名称", required = false, dataType = "String",
					paramType = "query")
	})
	public AjaxJson list(@ApiIgnore SkinManage skinManage, HttpServletRequest request, HttpServletResponse response) {
		List<SkinManage> list = skinManageService.findAllList(skinManage);
		//logger.debug("获取应用皮肤管理列表信息成功");
		AjaxJson json = new AjaxJson();
		json.put("list", list);
		return json;
	}


	/**
	 * @description 应用皮肤管理详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param skinId
	 * @return
	 */
	@GetMapping(value = "{skinId}")
	@ApiOperation(value="详情", notes="根据skinId获取应用皮肤管理详情")
	@ApiImplicitParam(name = "skinId", value = "皮肤ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String skinId) {	
		if(StringUtils.isBlank(skinId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		SkinManage skinManage = skinManageService.get(skinId);
		//logger.debug("获取应用皮肤管理详情成功");
		AjaxJson json = new AjaxJson();
		json.put("obj", skinManage);
		return json;
	}


	/**
	 * @description 新增或修改应用皮肤
	 * @author xutao
	 * @date：2019年10月18日
	 * @param dict 
	 * @return
	 */
	@PostMapping(value = "")
	@ApiOperation(value = "保存（新增或修改）", notes = "新增或修改应用皮肤")
	@ApiImplicitParam(name = "skinManage", value = "应用皮肤实体", required = true, dataType = "SkinManage")
	public AjaxJson save(@RequestBody SkinManage skinManage) {
		AjaxJson json = new AjaxJson();
		try {
			List<SkinManage> list = skinManageService.findList(skinManage);
			if(list.size()>0) {
				return AjaxJson.returnExceptionInfo(LoitStatusCode.SKIN_IS_EXIST);
			}else {
				skinManageService.save(skinManage,skinManage.getSkinId());
				json.setMsg("保存成功");
				return json;
			}
		} catch (Exception e) {
			logger.error("erroCode:{},info:保存应用皮肤异常",LoitStatusCode.FAIL.statusCode);
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
		}
	}	

	/**
	 * @description 应用皮肤删除
	 * @author xutao
	 * @date：2019年10月18日
	 * @param ids 多ID时，以逗号分隔
	 * @return
	 */
	@ApiOperation(value = "删除", notes = "单个或批量删除应用皮肤，多ID时，以逗号分隔")
	@DeleteMapping(value = "{skinId}")
	public AjaxJson deleteAll(@PathVariable("skinId")String ids) {
		if(StringUtils.isBlank(ids)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		AjaxJson json = new AjaxJson();
		try {
			SkinManage skinManage = new SkinManage();
			skinManageService.deleteAll(ids,skinManage);
			json.setMsg("删除应用皮肤成功");
			return json;
		} catch (Exception e) {
			logger.error("erroCode:{},info:删除应用皮肤异常",LoitStatusCode.REDIS_LINK_FAIL.statusCode);
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo(LoitStatusCode.REDIS_LINK_FAIL);
		}
	}


}
