package com.loit.v1.modules.controller;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.modules.entity.SynLog;
import com.loit.v1.modules.entity.SynTarget;
import com.loit.v1.modules.service.SynLogService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**  
* @ClassName: DataSyncController
* @Description:  数据同步controller
* @author zhangyl
* @date 2019年11月12日 上午15:37:51
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/syncLog")
@Api(value = "SyncLogController", tags = {"数据同步日志Controller"})
public class SyncLogController extends BaseController {
	
	
	@Autowired
	private SynLogService synLogService;

	@ApiOperation(value = "数据同步日志分页列表", notes = "获取数据同步日志分页列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "objectId", value = "同步对象ID", required = false, dataType = "String", paramType = "query") ,
			@ApiImplicitParam(name = "objectName", value = "同步对象名称", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response, @ApiIgnore SynLog synLog){
		Page<SynLog> page = synLogService.findPage(new Page<SynLog>(request, response),synLog);
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}


	/**
	 *   同步失败日志记录
	 * @param request
	 * @param response
	 * @param synLog
	 * @return
	 */
	@ApiOperation(value = "数据同步失败日志分页列表", notes = "获取数据同步失败日志分页列表")
	@ApiImplicitParams({
			@ApiImplicitParam(name = "objectId", value = "同步失败对象ID", required = false, dataType = "String", paramType = "query") ,
			@ApiImplicitParam(name = "objectName", value = "同步对象名称", required = false, dataType = "String", paramType = "query")
	})
	@GetMapping(value = "/failLog")
	public AjaxJson listFail(HttpServletRequest request, HttpServletResponse response, @ApiIgnore SynLog synLog){
		Page<SynLog> page = synLogService.findPageFailLog(new Page<SynLog>(request, response),synLog);
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}

	/**
	 * 获取同步目标详细信息
	 * @param
	 * @return
	 */
	@GetMapping(value = "/{logId}")
	@ApiOperation(value = "获取同步目标详细信息", notes = "根据id来获取同步目标信息")
	@ApiImplicitParam(name = "logId", value = "logId", required = true, dataType = "String", paramType = "path")
	public AjaxJson get(@PathVariable String logId) {
		if(StringUtils.isBlank(logId)) {
			return AjaxJson.returnExceptionInfo("获取同步目标数据详情异常，请联系管理员");
		}
		SynLog synLog = synLogService.get(logId);
		AjaxJson json = new AjaxJson();
		json.put("obj", synLog);
		return json;
	}


	/**
	 * 保存
	 * @param
	 * @return
	 */
	@PostMapping(value = "/save")
	@ApiOperation(value = "保存/修改", notes = "保存/修改同步日志数据")
	@ApiImplicitParam(name = "SynLog", value = "实体同步日志详情", required = true, dataType = "SynLog")
	public AjaxJson save(@RequestBody SynLog synLog) {
		AjaxJson json = new AjaxJson();
		try {
			synLogService.save(synLog, synLog.getTargetId());
			json.setMsg("保存成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存同步日志数据异常");
		}
		return json;
	}

	/**
	 * 删除
	 *@param
	 */
	@DeleteMapping(value = "/{logId}")
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "logId", value = "同步日志ID", required = true, dataType = "String", paramType = "path")
	public AjaxJson delete(@PathVariable String logId) {
		if(StringUtils.isBlank(logId)) {
			return AjaxJson.returnExceptionInfo("同步日志id为空,请选择相应的列表!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(logId)) {
			try {
				synLogService.deleteAll(logId, new SynLog());
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除同步日志数据失败" + e.getMessage());
			}
		}
		return json;
	}

	/**
	 * 手动同步
	 * @param logId
	 * @return AjaxJson
	 */
	@PostMapping(value = "/manualSync/{logId}")
	@ApiOperation(value = "获取同步目标详细信息", notes = "根据日志id来获取同步目标信息")
	@ApiImplicitParam(name = "logId", value = "logId", required = true, dataType = "String", paramType = "path")
	public AjaxJson manualSync(@PathVariable String logId){
		if(StringUtils.isBlank(logId)) {
			return AjaxJson.returnExceptionInfo("日志记录id为空,请选择相应的列表");
		}
		return synLogService.manualSync(logId);
	}
}
