package com.loit.v1.modules.controller;

import com.alibaba.fastjson.JSONObject;
import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.modules.entity.*;
import com.loit.v1.modules.service.*;
import com.loit.v1.modules.utils.Model.HttpUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**  
* @ClassName: DataSyncController
* @Description:  数据同步controller
* @author zhangyl
* @date 2019年11月12日 上午15:37:51
* @version V1.0  
*/

@RestController
@RequestMapping(value = "api/v1/syncTarget")
@Api(value = "DataSyncController", tags = {"数据同步Controller"})
public class SyncTargetController extends BaseController {
	
	
	@Autowired
	private SynTargetService synTargetService;


	@ApiOperation(value = "数据同步分页列表", notes = "获取数据同步分页列表")
	@ApiImplicitParams({@ApiImplicitParam(name = "targetName", value = "目标名称", required = false, dataType = "String", paramType = "query") ,
					 	@ApiImplicitParam(name = "serviceUrl", value = "服务地址", required = false, dataType = "String", paramType = "query")
						})
	@GetMapping(value = "")
	public AjaxJson list(HttpServletRequest request, HttpServletResponse response, @ApiIgnore SynTarget synTarget){
		Page<SynTarget> page = synTargetService.findPage(new Page<SynTarget>(request, response),synTarget);
		AjaxJson json = new AjaxJson();
		json.put("page", page);
		return json;
	}

	/**
	 * 获取同步目标详细信息
	 * @param
	 * @return
	 */
	@GetMapping(value = "/{targetId}")
	@ApiOperation(value = "获取同步目标详细信息", notes = "根据id来获取同步目标信息")
	@ApiImplicitParam(name = "targetId", value = "targetId", required = true, dataType = "String", paramType = "path")
	public AjaxJson get(@PathVariable String targetId) {
		if(StringUtils.isBlank(targetId)) {
			return AjaxJson.returnExceptionInfo("获取同步目标数据详情异常，请联系管理员");
		}
		SynTarget synTarget = synTargetService.get(targetId);
		AjaxJson json = new AjaxJson();
		json.put("obj", synTarget);
		return json;
	}


	/**
	 * 保
	 * @param
	 * @return
	 */
	@PostMapping(value = "/save")
	@ApiOperation(value = "保存/修改", notes = "保存/修改同步目标数据")
	@ApiImplicitParam(name = "SynTarget", value = "实体同步目标详情", required = true, dataType = "AuditAlertDTO")
	public AjaxJson save(@RequestBody SynTarget synTarget) {
		AjaxJson json = new AjaxJson();
		try {
			synTargetService.save(synTarget, synTarget.getTargetId());
			json.setMsg("保存成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存同步目标数据异常");
		}
		return json;
	}

	/**
	 * 删除
	 *@param
	 */
	@DeleteMapping(value = "/{targetId}")
	@ApiOperation(value = "删除", notes = "根据id来指定删除对象,支持删除多个,以逗号分隔")
	@ApiImplicitParam(name = "targetId", value = "同步目标ID", required = true, dataType = "String", paramType = "path")
	public AjaxJson delete(@PathVariable String targetId) {
		if(StringUtils.isBlank(targetId)) {
			return AjaxJson.returnExceptionInfo("同步目标id为空,请选择相应的列表!");
		}
		AjaxJson json = new AjaxJson();
		if(StringUtils.isNotBlank(targetId)) {
			try {
				synTargetService.deleteAll(targetId, new SynTarget());
				json.setMsg("删除成功!");
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo("删除同步目标数据失败" + e.getMessage());
			}
		}
		return json;
	}

	/**
	 * 全量同步
	 * @param targetId
	 * @return AjaxJson
	 */
	@PostMapping(value = "/allSync/{targetId}")
	@ApiOperation(value = "获取同步目标详细信息", notes = "根据同步目标id来获取同步目标信息")
	@ApiImplicitParam(name = "targetId", value = "targetId", required = true, dataType = "String", paramType = "path")
	public AjaxJson allSync(@PathVariable String targetId){
		if(StringUtils.isBlank(targetId)) {
			return AjaxJson.returnExceptionInfo("同步目标id为空,请选择相应的列表");
		}
		return synTargetService.allSync(targetId);
	}


	/**
	 * 同步消息订阅
	 * @param orgPerson
	 * @return AjaxJson
	 */

	public AjaxJson subscribeGruopAddSync(OrgPerson orgPerson){
		//TODO
		return null;
	}

}
