package com.loit.v1.modules.controller;

import com.loit.api.service.loituser.TestFeignApiService;
import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Dict;
import com.loit.modules.sys.service.DictService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.client.loadbalancer.LoadBalanced;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * Description： TODO
 * Author: yangwenbin
 * Date: Created in 2020/7/30 10:58
 * Version: 1.0
 * Modified By:
 */
@RestController
@RequestMapping(value = "test")
public class TestController {
    protected Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private DictService dictService;
    @Autowired
    @LoadBalanced
    private RestTemplate restTemplate;
    @Autowired
    private TestFeignApiService testFeignApiService;

    /**
     * 空测试用
     *
     * @return
     */
    @GetMapping(value = "json")
    public AjaxJson test() {
        AjaxJson ajaxJson = new AjaxJson();
        return ajaxJson;
    }

    @GetMapping(value = "hello")
    public String hello() {
        return "hello";
    }

    /**
     * @param dict     系统字典实体类
     * @param request
     * @param response
     * @return
     * @description 系统字典分页列表
     * @author xutao
     * @date：2019年10月18日
     */
    @GetMapping(value = "dict/page")
    @ApiOperation(value = "分页列表", notes = "获取系统字典分页列表")
    @ApiImplicitParams({
//            @ApiImplicitParam(name = "pageNo", value = "当前页码", required = true, dataType = "String", paramType = "query"),
//            @ApiImplicitParam(name = "pageSize", value = "分页大小", required = true, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "dictCode", value = "字典编码", required = false, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "dictName", value = "字典名称", required = false, dataType = "String", paramType = "query"),
            @ApiImplicitParam(name = "baseFlag", value = "1、系统内置 2、用户自定义 ", required = false, dataType = "String", paramType = "query")
    })
    public AjaxJson list(@ApiIgnore Dict dict, HttpServletRequest request, HttpServletResponse response) {
        logger.info("分页查询");
        Page<Dict> page = dictService.findTreePage(new Page<Dict>(request, response), dict);
        AjaxJson json = new AjaxJson();
        json.put("page", page);
        return json;
    }


    /**
     * @param dictId 系统字典ID
     * @return
     * @description 系统字典详情
     * @author xutao
     * @date：2019年10月18日
     */
    @GetMapping(value = "dict/{dictId}")
    @ApiOperation(value = "详情", notes = "根据dictId获取系统字典详情")
    @ApiImplicitParam(name = "dictId", value = "系统字典ID", required = true, dataType = "String")
    public AjaxJson get(@PathVariable(required = false) String dictId) {
        logger.info("获取详情");
        if (StringUtils.isBlank(dictId)) {
            return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
        }
        Dict dict = dictService.get(dictId);
        AjaxJson json = new AjaxJson();
        json.put("obj", dict);
        return json;
    }


    /**
     * @return
     * @description 系统字典新增或更新
     * @author xutao
     * @date：2019年10月18日
     */
    @PostMapping(value = "dict/save")
    @ApiOperation(value = "保存(新增或修改)", notes = "保存或更新系统字典")
//    @ApiImplicitParam(name = "dict", value = "字典实体类", required = true, dataType = "Dict")
    public AjaxJson save() {
        AjaxJson json = new AjaxJson();
        try {
//			List<Dict> list = dictService.findList(dict);
//			if(list.size()>0) {
//				return AjaxJson.returnExceptionInfo(LoitStatusCode.DICT_IS_EXIST);
//			}else {
            Dict dict = new Dict();
            dict.setDictCode("test");
            dict.setDictName("loit");
            dictService.save(dict);
            json.setMsg("字典保存成功");
//			}
            logger.info("保存");
            return json;
        } catch (Exception e) {
            logger.error("erroCode:{},info:保存字典数据异常", LoitStatusCode.FAIL.statusCode);
            e.printStackTrace();
            return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
        }
    }


    @GetMapping(value = "dict/getSave")
    @ApiOperation(value = "保存(新增或修改)", notes = "保存或更新系统字典")
    public AjaxJson getSave() {
        AjaxJson json = new AjaxJson();
        try {
            Dict dict = new Dict();
            dict.setDictCode("test");
            dict.setDictName("loit");
            dictService.save(dict);
            json.setMsg("字典保存成功");
//			}
            logger.info("保存");
            return json;
        } catch (Exception e) {
            logger.error("erroCode:{},info:保存字典数据异常", LoitStatusCode.FAIL.statusCode);
            e.printStackTrace();
            return AjaxJson.returnExceptionInfo(LoitStatusCode.SAVE_EXCEPTION);
        }
    }


    /**
     * 批量添加新的对象
     *
     * @return
     */
    @RequestMapping(value = "/batAdd", method = {RequestMethod.GET, RequestMethod.POST})
    @ResponseBody
    public void batAdd() {
        long startTime = System.currentTimeMillis();
        for (int i = 1; i <= 1000000; i++) {
            Dict dict = new Dict();
            dict.setDictCode("test");
            dict.setDictName("loit");
            dictService.save(dict);
        }
    }

    /**
     * 批量添加新的对象
     *
     * @return
     */
    @RequestMapping(value = "/batAddList", method = {RequestMethod.GET, RequestMethod.POST})
    @ResponseBody
    public String batAddList() {

        List<Dict> resultList = new ArrayList<>(1000);
        long startTime = System.currentTimeMillis();
        int count = 1000000;


        for (int i = 0; i <= 1000000; i++) {
            Dict dict = new Dict();
            dict.setDictCode("test" + i);
            dict.setDictName("loit");
            resultList.add(dict);

            if (resultList.size() == 1000) {
                dictService.insertResultBatch(resultList);
                resultList.clear();
            }
        }

        long millis = System.currentTimeMillis() - startTime;
        long sec = TimeUnit.MILLISECONDS.toSeconds(millis);
        long qps = count / sec;
        return "插入表loit_sys_dict: 插入记录数" + count + "条;  耗时" + sec + "秒; " + "QPS:" + qps;
    }


    /**
     * 验证for循环删除
     */
    @RequestMapping(value = "/batDelete", method = {RequestMethod.GET, RequestMethod.POST})
    @ResponseBody
    public String batDelete() {

        long startTime = System.currentTimeMillis();
        int count = 1000000;
        for (int i = 0; i <= 1000000; i++) {
            Dict dict = new Dict();
            Integer dictId = i + 10;
            dict.setDictId(dictId);
            dictService.delete(dict);
        }

        long millis = System.currentTimeMillis() - startTime;
        long sec = TimeUnit.MILLISECONDS.toSeconds(millis);
        long qps = count / sec;

        return "表loit_sys_dict: 删除记录数" + count + "条;  耗时" + sec + "秒; " + "QPS:" + qps;
    }


    @RequestMapping(value = "/batDeleteList", method = {RequestMethod.GET, RequestMethod.POST})
    @ResponseBody
    public String batDeleteList() {
        List<Integer> resultList = new ArrayList<>(1000);
        long startTime = System.currentTimeMillis();
        int count = 1000000;

        for (int i = 0; i <= 1000000; i++) {
            Integer dictId = i + 10;
            resultList.add(dictId);

            if (resultList.size() == 1000) {
                dictService.deleteResultBatch(resultList);
                resultList.clear();
            }
        }

        long millis = System.currentTimeMillis() - startTime;
        long sec = TimeUnit.MILLISECONDS.toSeconds(millis);
        long qps = count / sec;

        return "表loit_sys_dict: 删除记录数" + count + "条;  耗时" + sec + "秒; " + "QPS:" + qps;
    }


    /**
     * 批量更新
     *
     * @return
     */
    @RequestMapping(value = "/batUpdateList", method = {RequestMethod.GET, RequestMethod.POST})
    @ResponseBody
    public String batUpdateList() {

        List<Dict> resultList = new ArrayList<>(1000);
        long startTime = System.currentTimeMillis();
        int count = 1000000;


        for (int i = 0; i <= 1000000; i++) {
            Dict dict = new Dict();
            Integer dictId = i + 10;
            dict.setDictId(dictId);
            resultList.add(dict);
            if (resultList.size() == 1000) {
                dictService.updateResultBatch(resultList);
                resultList.clear();
            }
        }

        long millis = System.currentTimeMillis() - startTime;
        long sec = TimeUnit.MILLISECONDS.toSeconds(millis);
        long qps = count / sec;
        return "插入表loit_sys_dict: 插入记录数" + count + "条;  耗时" + sec + "秒; " + "QPS:" + qps;
    }

    /**
     * @param dictId 字典ID，多ID时，以逗号分隔
     * @return
     * @description 系统字典删除
     * @author xutao
     * @date：2019年10月18日
     */
    @ApiOperation(value = "删除", notes = "删除单个dictId")
    @DeleteMapping(value = "dict/{dictId}")
    public AjaxJson deleteAll(@PathVariable("dictId") String dictId) {
        logger.info("删除");
        if (StringUtils.isBlank(dictId)) {
            return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
        }
        AjaxJson json = new AjaxJson();
        try {

            Dict dict = new Dict();
            dict.setDictId(Integer.valueOf(dictId));
            dictService.delete(dict);
            json.setMsg("删除字典成功");
            return json;
        } catch (Exception e) {
            logger.error("erroCode:{},info:删除字典异常", LoitStatusCode.REDIS_LINK_FAIL.statusCode);
            e.printStackTrace();
            return AjaxJson.returnExceptionInfo(LoitStatusCode.REDIS_LINK_FAIL);
        }
    }


    /**
     * @param dictId
     * @param dictName
     * @return
     * @description 检查字典名称是否重复
     */
    @ApiOperation(value = "检查字典名称是否重复", notes = "检查字典名称是否重复")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "dictId", value = "字典ID", required = false, dataType = "int", paramType = "query"),
            @ApiImplicitParam(name = "dictName", value = "字典名称", required = true, dataType = "String", paramType = "query")
    })
    @GetMapping(value = "/checkDictNameUnique")
    public AjaxJson checkDictNameUnique(Integer dictId, String dictName) {
        boolean uniqueFlag = dictService.checkNameUnique(dictId, dictName);
        AjaxJson json = new AjaxJson();
        if (uniqueFlag) {
            json.setMsg("字典名称不重复");
        } else {
            json.setMsg("字典名称重复");
            json.setCode(LoitStatusCode.FAIL.statusCode);
        }
        return json;
    }

    @GetMapping(value = "fileRibbon")
    public AjaxJson testFileRibbon() {
        String url = "http://loit-file/file/test";
        AjaxJson ajaxJson = restTemplate.getForObject(url, AjaxJson.class);
        return ajaxJson;
    }

    @GetMapping(value = "testFeign")
    public AjaxJson testFeign() {
        AjaxJson ajaxJson = new AjaxJson();

        return testFeignApiService.testFeign();
    }
}
