package com.loit.v1.modules.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.utils.StringUtils;
import com.loit.common.web.BaseController;
import com.loit.v1.api.dto.ThemeManageDTO;
import com.loit.v1.api.dto.ThemeSetDTO;
import com.loit.v1.modules.entity.ThemeManage;
import com.loit.v1.modules.service.ThemeManageService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;

/**
 * @description 应用主题管理Controller
 * @author xutao
 * @date 2019-10-18
 */
@RestController
@RequestMapping(value = "api/v1/themeManage")
@Api(value = "ThemeManageController", tags = {"应用主题管理Controller"})
public class ThemeManageController extends BaseController {

	@Autowired
	private ThemeManageService themeManageService;

		
	/**
	 * @description 通过应用获取应用主题详情
	 * @author xutao
	 * @date：2019年10月18日
	 * @param appId 应用ID
	 * @return
	 */
	@GetMapping(value = "{appId}")
	@ApiOperation(value="通过应用ID获取应用主题详情", notes="通过应用ID获取应用主题详情")
	@ApiImplicitParam(name = "appId", value = "应用ID", required = true, dataType = "String")
	public AjaxJson get(@PathVariable(required=false) String appId) {	
		if(StringUtils.isBlank(appId)) {
			return AjaxJson.returnExceptionInfo(LoitStatusCode.PRIMARY_KEY_ISNULL);
		}
		ThemeManage manage = themeManageService.getThemeByAppId(appId);
		AjaxJson json = new AjaxJson();
		json.put("obj", manage);
		return json;
	}
	
	/**
	 * @description 修改应用菜单主题
	 * @param appId 应用ID
	 * @param themeManageDTO 应用主题管理数据传输实体类
	 * @return
	 */
	@PostMapping(value = "{appId}")
	@ApiOperation(value = "修改", notes = "修改应用菜单主题")
	public AjaxJson update(@PathVariable int appId,@RequestBody ThemeManageDTO themeManageDTO) {
		AjaxJson json = new AjaxJson();
		ThemeManage manage = ModelMapperUtils.map(themeManageDTO, ThemeManage.class);
		try {
			manage.setAppId(appId);
			themeManageService.update(manage);
			json.setMsg("修改成功");
			return json;
		} catch (Exception e) {
			logger.error("info:应用菜单主题修改失败,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.UPDATE_EXCEPTION);
		}		
	}
	
	/**
	 * @description 修改系统主题
	 * @param appId 应用ID
	 * @param themeSetDTO 系统主题设置数据传输实体类
	 * @return
	 */
	@PostMapping(value = "/themeSet/{appId}")
	@ApiOperation(value = "修改", notes = "修改系统主题")
	public AjaxJson themeSet(@PathVariable int appId,@RequestBody ThemeSetDTO themeSetDTO) {
		AjaxJson json = new AjaxJson();
		ThemeManage manage = ModelMapperUtils.map(themeSetDTO, ThemeManage.class);
		try {
			manage.setAppId(appId);
			themeManageService.themeSet(manage);
			json.setMsg("修改成功");
			return json;
		} catch (Exception e) {
			logger.error("info:系统主题修改失败,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.UPDATE_EXCEPTION);
		}		
	}
}
