package com.loit.v1.modules.controller;

import com.loit.loitcasclient.utils.LoitLog;
import com.loit.v1.modules.entity.UserLogLevel;
import com.loit.v1.modules.service.UserLogLevelService;
import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;




/**
 * 用户日志级别配置[门户平台]接口
 * Created by ymm on 2020-07-31.
 **/
@RestController
@RequestMapping(value = "api/v1/userLogLeveluserLogLevel")
@Api(tags = "用户日志级别配置[门户平台]接口",  value = "UserLogLevelController")
public class UserLogLevelController {

	@Autowired
	private LoitLog loitLog;
	@Autowired
	private UserLogLevelService userLogLevelService;

	/**
	 * 用户日志级别配置列表
	 */
	@ApiOperation(value = "用户日志级别配置列表", notes = "用户日志级别配置列表")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "query", name = "userid", value = "用户id", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "logLevel", value = "日志级别", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "createUser", value = "创建人id", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "createDate", value = "创建时间", required = false, dataType = "Date"),
			@ApiImplicitParam(paramType = "query", name = "updateUser", value = "更新人id", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "updateDate", value = "更新时间", required = false, dataType = "Date"),
			@ApiImplicitParam(paramType = "query", name = "descInfo", value = "描述", required = false, dataType = "String")
	})
	@RequestMapping(value = "/list", method = RequestMethod.GET)
	public AjaxJson findAll(@ApiIgnore UserLogLevel userLogLevel) {
		AjaxJson json = new AjaxJson();
		List<UserLogLevel> list = userLogLevelService.findList(userLogLevel);
		json.put("list", list);
		return json;
	}

	/**
	 * 用户日志级别配置分页列表
	 */
	@ApiOperation(value = "用户日志级别配置分页列表", notes = "用户日志级别配置分页列表")
	@ApiImplicitParams({
			@ApiImplicitParam(paramType = "query", name = "pageNo", value = "当前页码", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "pageSize", value = "分页大小", required = true, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "userid", value = "用户id", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "logLevel", value = "日志级别", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "createUser", value = "创建人id", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "createDate", value = "创建时间", required = false, dataType = "Date"),
			@ApiImplicitParam(paramType = "query", name = "updateUser", value = "更新人id", required = false, dataType = "String"),
			@ApiImplicitParam(paramType = "query", name = "updateDate", value = "更新时间", required = false, dataType = "Date"),
			@ApiImplicitParam(paramType = "query", name = "descInfo", value = "描述", required = false, dataType = "String")
	})
	@RequestMapping(value = "/page", method = RequestMethod.GET)
	public AjaxJson findPage(@ApiIgnore UserLogLevel userLogLevel, HttpServletRequest request, HttpServletResponse response) {
//		loitLog.info("测试日志info");
//		loitLog.debug("测试日志debug");
//		loitLog.warn("测试日志warn");
//		loitLog.trace("测试日志trace");
		AjaxJson json = new AjaxJson();
		Page<UserLogLevel> page = userLogLevelService.findPage(new Page<UserLogLevel>(request, response), userLogLevel);
		json.put("page", page);
		return json;
	}

	/**
	 * 用户日志级别配置详情
	 */
	@ApiOperation(value = "用户日志级别配置详情", notes = "根据id获取用户日志级别配置详情信息")
	@ApiImplicitParam(paramType = "path", name = "id", value = "用户日志级别配置id", required = true, dataType = "Long")
	@RequestMapping(value = "/{id}", method = RequestMethod.GET)
	public AjaxJson get(@PathVariable("id") Integer id) {
		UserLogLevel userLogLevel = userLogLevelService.get(id);
		AjaxJson json = new AjaxJson();
		json.put("obj", userLogLevel);
		return json;
	}

	/**
	 * 用户日志级别配置保存
	 */
	@ApiOperation(value = "用户日志级别配置保存", notes = "用户日志级别配置保存")
	@RequestMapping(value = "/save", method = RequestMethod.POST)
	public AjaxJson save(@RequestBody UserLogLevel userLogLevel) {
		AjaxJson json = new AjaxJson();
		try {
			userLogLevelService.insert(userLogLevel);
			json.setMsg("保存成功");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("保存异常");
		}
		return json;
	}

	/**
	 * 用户日志级别配置修改
	 */
	@ApiOperation(value = "用户日志级别配置修改", notes = "用户日志级别配置修改")
	@RequestMapping(value = "/update", method = RequestMethod.POST)
	public AjaxJson update(@RequestBody UserLogLevel userLogLevel) {
		AjaxJson json = new AjaxJson();
		try {
			userLogLevelService.update(userLogLevel);
			json.setMsg("修改成功");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("修改异常");
		}
		return json;
	}

	/**
	 * 用户日志级别配置删除
	 */
	@ApiOperation(value = "用户日志级别配置删除", notes = "用户日志级别配置删除")
	@ApiImplicitParam(paramType = "path", name = "id", value = "用户日志级别配置id", required = true, dataType = "Long")
	@RequestMapping(value = "/{id}", method = RequestMethod.DELETE)
	public AjaxJson delete(@PathVariable("id") String ids) {
		AjaxJson json = new AjaxJson();
		try {
			userLogLevelService.deleteAll(ids, new UserLogLevel());
			json.setMsg("删除成功!");
		} catch (Exception e) {
			e.printStackTrace();
			return AjaxJson.returnExceptionInfo("删除失败" + e.getMessage());
		}
		return json;
	}


}
