package com.loit.v1.modules.dao;

import com.loit.common.persistence.CrudDao;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.modules.sys.entity.User;
import com.loit.v1.modules.entity.PersonBase;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;


/**
 * @ClassName: AccoutInfoDao
 * @Description: 人员账号信息表
 * @author  zhaokz 
 * @date  2019-10-23 15:22:25
 * @version V1.0 
 **/
@Mapper
public interface AccoutInfoDao extends CrudDao<AccoutInfo> {

	/**
	 * 获取人员详情
	 * @param accoutInfo
	 * @return
	 */
	PersonBaseDTO getInfo(AccoutInfo accoutInfo);
	/**
	* @Description: 缓存的用户信息查询
	* @author: gzx
	* @date: 2019年11月4日 下午3:56:51 
	* @param @param userId
	* @param @return 参数 
	* @return User 返回类型 
	* @throws
	 */
	User getCacheUserInfo(@Param("userId")String userId);

	/**
	 * 根据登录账号获取账号信息
	 * @param userId
	 * @return
	 */
	AccoutInfo getAccountByUserId(@Param("userId")String userId);

	/**
	 * 获取组织下所有账号
	 * @return
	 */
	List<AccoutInfo> findListByOrgId(@Param("orgId")String orgId, @Param("delFlag")String delFlag);

	/**
	 * 获取用户组下所有账号
	 * @param grpId
	 * @return
	 */
	List<AccoutInfo> findListByGrpId(@Param("grpId")Integer grpId);

	/**
	 * 根据登录账号获取唯一账号对象(用于账号唯一性验证)
	 * @param accoutInfo
	 * @return
	 */
	AccoutInfo getUniqueAccount(AccoutInfo accoutInfo);

	/**
	 *  逻辑删除组织及下属组织下账号
	 * @param orgId
	 */
	void logicDeleteInOrg(@Param("orgId")Integer orgId);

	/**
	 * 彻底删除组织及下属组织下所有人员
	 * @param orgId
	 */
	void deletePersonInOrg(@Param("orgId")String orgId);
	
	/**
	 * 还原逻辑删除的本组织及下属组织下的人
	 * @param orgId
	 */
	void revertPersonInOrg(String orgId);

	/**
	 * 根据登录账号列表获取账号信息列表
	 * DEMON add by 2020-2-14
	 */
	List<PersonBaseDTO> getAccountByUserIds(List<String> userIds, String userIdOrPersonName);

	/**
	 * 获取登录账号总人数
	 * DEMON add by 2020-2-14
	 */
	Integer findListCount(AccoutInfo accoutInfo);
	/**
	 * 查询组织下组织用户
	 * @param orgId
	 * @return
	 */
	AccoutInfo findOrgUser(Integer orgId);

	/**
	 * 获取所有的账号信息
	 * @return
	 */
    List<AccoutInfo> getAccoutInfoList();

	/**
	 * getPersonByPersonName
	 * @param personName
	 * @return
	 */
	List<PersonBase> getPersonByPersonName(String personName);
}