package com.loit.v1.modules.dao;

import com.loit.v1.api.dto.AppOrgDTO;
import com.loit.v1.modules.entity.AppOrg;
import com.loit.common.persistence.CrudDao;

import java.util.List;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;


/**
 * @ClassName: AppOrgDao
 * @Description: 应用组织关系表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Mapper
public interface AppOrgDao extends CrudDao<AppOrg> {

	/**
	 * 获取已分配组织列表
	 * @param appOrg
	 * @return
	 */
	List<AppOrgDTO> findAppOrgList(AppOrg appOrg);

	/**
	 * 查询是否已分配权限
	 * @param appOrg
	 * @return
	 */
	AppOrg check(AppOrg appOrg);

	/**
	 * 获取应用权限
	 * @param parentId
	 * @param appId
	 * @return
	 */
	AppOrg getByOrgIdAppId(@Param("orgId")String orgId, @Param("appId")String appId);

	/**
	 * @param appId
	 * @return
	 */
	List<AppOrg> getAppOrgByAppId(@Param("appId")Integer appId);

	/**
	 * 根据组织id删除应用组织关系
	 * @param orgId
	 */
	void deleteByOrgId(@Param("orgId")Integer orgId);

	/**
	 * @description 根据组织编码获取当前应用组织树形列表
	 * @param appId
	 * @param orgCode
	 * @return
	 */
	List<AppOrg> getAppOrgByAppIdAndOrgCode(@Param("appId")String appId, @Param("orgCode")String orgCode);
	
}