package com.loit.v1.modules.dao;

import com.loit.v1.api.dto.OrgDTO;
import com.loit.v1.modules.entity.Org;
import com.loit.common.persistence.CrudDao;

import java.util.List;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;


/**
 * @ClassName: OrgDao
 * @Description: 组织机构表
 * @author  zhaokz 
 * @date  2019-10-22 18:36:40
 * @version V1.0 
 **/
@Mapper
public interface OrgDao extends CrudDao<Org> {

	/**查询组织树状列表
	 * @param org
	 * @return
	 */
	List<OrgDTO> findOrgTreeList(Org org);

	/**
	 * 获取下属组织
	 * @param orgId
	 * @return
	 */
	List<Org> getChildrenOrg(int orgId);

	/**
	 * 获取当前应用组织树
	 * @param org
	 * @return
	 */
	List<OrgDTO> findCuAppOrgList(Org org);

	/**
	 * 获取未添加用户组组织
	 * @param org
	 * @return
	 */
	List<OrgDTO> findUnAddOrgs(Org org);

	/**
	 * 已添加用户组组织
	 * @param org
	 * @return
	 */
	List<Org> findGrpOrgs(Org org);

	/**
	 * @param appId
	 * @return
	 */
	List<Org> getOrgByAppId(Integer appId);

	/**
	 * @param orgId
	 * @return
	 */
	OrgDTO getDTO(Integer orgId);


	/**
	 * 查询所有数据列表
	 * @param  appId
	 * @return
	 */
	public List<OrgDTO> findListByAppId(int appId);

	/**
	 * 根据组织名称和父组织id获取组织
	 * @param org
	 * @return
	 */
	Org getByOrgName(Org org);

	/**
	 * 还原逻辑删除的本组织及下属组织
	 * @param org
	 */
	void revert(Org org);

	/**
	 * 删除本组织及下属组织
	 * @param org
	 */
	void deleteOrgAndChildren(Org org);

	/**
	 * DTO接受返回数据，用于数据同步
	 * @param orgId
	 * @return
	 */
	OrgDTO getByOrgId(long orgId);

	/**
	 * @descrtption 根据组织编码获取组织
	 * @param orgCode
	 * @return
	 */
	Org getByOrgCode(String orgCode);

	/**
	 * @description 获取所有组织
	 * @param org
	 * @return
	 */
	List<OrgDTO> findAll(Org org);

	/**
	 * @description 获取本组织及所有下属组织
	 * @param orgId
	 * @return
	 */
	List<Org> getAllChildrenOrg(Integer orgId);

	/**
	 * @description 根据扩展唯一编码获取组织
	 * @param extCode
	 * @return
	 */
	Org getByExtCode(String extCode);

	/**
	 * 根据机构层次代码查询机构（非部门）
	 * @param orgLevelCode
	 * @return
	 */
	List<Org> findOryByOrgLevelCode(@Param("orgLevelCode") String orgLevelCode);

}