package com.loit.v1.modules.dao;

import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.entity.PersonBase;
import com.loit.common.persistence.CrudDao;

import java.util.List;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.hibernate.validator.constraints.Length;


/**
 * @ClassName: PersonBaseDao
 * @Description: 人员基础信息表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Mapper
public interface PersonBaseDao extends CrudDao<PersonBase> {

	/**
	 * 根据orgId获取人员列表
	 * @param orgId
	 * @param delFlag 
	 * @return
	 */
	List<PersonBase> findListByOrgId(@Param("orgId")String orgId, @Param("delFlag")String delFlag);

	/**
	 * 获取用户组内人员
	 * @param personBase 
	 * @return
	 */
	List<PersonBase> findGrpPersonPage(PersonBase personBase);

	/**
	 * 获取职务内人员
	 * @param personBase
	 * @return
	 */
	List<PersonBase> findPositionPersonPage(PersonBase personBase);

	/**
	 * 获取职级内人员
	 * @param personBase
	 * @return
	 */
	List<PersonBase> findRankPersonPage(PersonBase personBase);

	/**
	 *  未添加人员列表-用户组
	 * @param personBase
	 * @return
	 */
	List<PersonBase> findUnAddPersons(PersonBase personBase);

	/**
	 *  未添加人员列表-职务
	 * @param personBase
	 * @return
	 */
	List<PersonBase> findPositionUnAddPersons(PersonBase personBase);

	/**
	 *  未添加人员列表-职级
	 * @param personBase
	 * @return
	 */
	List<PersonBase> findRankUnAddPersons(PersonBase personBase);
	
	/**
	 * @Description: 查看某应用下的某角色分配的人员列表
	 * @param personBase 
	 * @return
	 */
	List<PersonBase> findPersonListByRole(PersonBase personBase);


	/**
	 * 通过APPID查询AccountInfo
	 * @param  appId
	 * @return
	 */
	public List<PersonBaseDTO> getAccountInfoByAppId(int appId);

	/**
	 * 根据人员名称查询人员基本信息
	 * @param personBase
	 * @return
	 */
	List<PersonBase> findUniquePersonName(PersonBase personBase);

	/**
	 * @description 根据用户id获取该用户所在组织下的人员列表
	 * @param accountId
	 * @return
	 */
	List<PersonBase> findPageByAccountId(@Param("accountId")Integer accountId);

	/**
	 * @description 根据组织编码获取组织及下属组织所有人
	 * @param appId
	 * @return
	 */
	List<PersonBaseDTO> findPersonsByOrgCode(@Param("appId")Integer appId);

	/**
	 * @desctiption 根据唯一扩展编码获取人员
	 * @param extCode
	 * @return
	 */
	PersonBase getByExtCode(String extCode);
}