package com.loit.v1.modules.dao;

import java.util.List;
import java.util.Map;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import com.loit.common.persistence.CrudDao;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.modules.entity.RightShortcut;


/**
 * @ClassName: RightShortcutDao
 * @Description: 权限快捷存取表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:07
 * @version V1.0 
 **/
@Mapper
public interface RightShortcutDao extends CrudDao<RightShortcut> {
	
	/**
	* @Description: 当应用管理员从应用门户进入权限模块，查询该应用管理员负责的应用列表，权限下拉框使用。map存appId，appName。
	* @author: gzx
	* @date: 2019年11月8日 下午2:23:59 
	* @param @return 参数 
	* @return List<Map<String,Object>> 返回类型 
	* @throws
	 */
	public List<Map<String,Object>> listMapAppsByAccountId(@Param("accountId")Integer accountId,@Param("appId")Integer appId,@Param("type")String type);
	
	/**
	* @Description: 获取当前用户的右侧应用列表(超级管理员除外)
	* @author: xutao
	* @date: 2019年11月8日 下午2:23:59 
	* @param @return 参数 
	* @return List<Map<String,Object>> 返回类型 
	* @throws
	 */
	public List<Map<String,Object>> getAppList(@Param("accountId")Integer accountId);


	/**
	 * 
	 * @param fromId
	 * @param fromType
	 */
	public void deleteByFrom(@Param("fromId")String fromId, @Param("fromType")String fromType);


	/**
	 * 根据账号id删除权限
	 * @param accountId
	 */
	public void deleteByAccountId(@Param("accountId")String accountId);


	/**
	 * @param rightShortcut
	 */
	public void deleteRightShortCut(RightShortcut rightShortcut);
	
	/**
	* @Description: 批量删除角色和快捷表对应关系数据 - 角色删除时调用
	* @author: gzx
	* @date: 2019年11月22日 下午1:42:58 
	* @param @param list 参数 
	* @return void 返回类型 
	* @throws
	 */
	void deleteBatchByRoleIds(List<Role> list);

	/**
	 * 删除当前应用下的人员权限
	 * @param accountId
	 * @param appId
	 * @param fromType
	 */
	public void deletePersonRight(@Param("fromId")String fromId, @Param("appId")String appId, @Param("fromType")String fromType);

	/**
	 * @param accountId
	 * @return 
	 */
	public List<RightShortcut> getRightByAccountId(Integer accountId);
}