package com.loit.v1.modules.dao;

import java.util.List;

import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import com.loit.common.persistence.CrudDao;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.api.dto.RoleDTO;


/**
 * @ClassName: RoleDao
 * @Description: 角色表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:07
 * @version V1.0 
 **/
@Mapper
public interface RoleDao extends CrudDao<Role> {
	
	/**
	* @Description: 批量删除
	* @author: gzx
	* @date: 2019年11月9日 上午11:41:35 
	* @param @param roleList 参数 
	* @return void 返回类型 
	* @throws
	 */
	void deleteBatch(List<Role> roleList);

	/**
	 * 通过账号id查询角色列表
	 * @param accountId
	 * @return
	 */
	List<Role> getRolesByAccountId(String accountId);
	/**
	* @Description: 检查角色名称的唯一性
	* @author: gzx
	* @date: 2019年11月20日 上午10:40:16 
	* @param @param roleName
	* @param @param roleId
	* @param @return 参数 
	* @return int 返回类型 
	* @throws
	 */
	int checkNameUnique(@Param("roleName")String roleName,@Param("roleId")Integer roleId);
	/**
	* @Description: 检查角色编码的唯一性
	* @author: gzx
	* @date: 2019年11月20日 上午10:40:16 
	* @param @param roleCode
	* @param @param roleId
	* @param @return 参数 
	* @return int 返回类型 
	* @throws
	 */
	int checkCodeUnique(@Param("roleCode")String roleCode,@Param("roleId")Integer roleId);

	/**
	 * 查询所有数据列表
	 * @param  appId
	 * @return
	 */
	public List<RoleDTO> findListByAppId(int appId);

	/**
	 * 获取用户组角色列表
	 * @param grpId
	 * @return
	 */
	List<Role> getRolesByGrpId(String grpId);

	/**
	 * DTO接受返回数据，用于数据同步
	 * @param roleId
	 * @return
	 */
	RoleDTO getByRoleId(long roleId);

	/**
	 * @description 通过角色id获取拥有该角色的人员
	 * @param roleId
	 * @return
	 */
	List<Integer> getAccountsByRoleId(@Param("roleId")Integer roleId);

	/**
	 * @description 根据用户id获取角色列表
	 * @param accountId
	 * @return
	 */
	List<Role> findByAccountId(@Param("accountId")Integer accountId);

	/**
	 * 根据角色编码获取角色
	 * @param roleCode
	 * @return
	 */
	Role getRoleByRoleCode(@Param("roleCode")String roleCode);
}