package com.loit.v1.modules.dataPush.service;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Optional;

import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.loit.v1.api.service.DutyService;
import com.loit.v1.modules.constant.KafkaConstantCode;
import com.loit.v1.modules.dataPush.entity.DutyUserInfo;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.entity.Org;
import com.loit.v1.modules.entity.OrgPerson;
import com.loit.v1.modules.entity.PersonBase;
import com.loit.v1.modules.service.AccountRoleService;
import com.loit.v1.modules.service.AccoutInfoService;
import com.loit.v1.modules.service.GrpUserService;
import com.loit.v1.modules.service.OrgPersonService;
import com.loit.v1.modules.service.OrgService;
import com.loit.v1.modules.service.PersonBaseService;
import com.loit.v1.modules.service.RightShortcutService;

/**  
* @ClassName: DutyPersonDataPushService  
* @Description:   接收勤务推送人员数据
* @author zhaokz  
* @date 2020年6月4日 上午9:08:23 
* @version V1.0  
*/

//@Component
public class DutyPersonDataPushService implements DutyService{
	
	private static Logger logger = LoggerFactory.getLogger(DutyPersonDataPushService.class);
	
	@Autowired
	PersonBaseService personBaseService;
	@Autowired
	AccoutInfoService accoutInfoService;
	@Autowired
	OrgService orgService;
	@Autowired
	OrgPersonService orgPersonService;
	@Autowired
	AccountRoleService accountRoleService;
	@Autowired
	RightShortcutService rightShortcutService;
	@Autowired
	GrpUserService grpUserService;
	
	@Transactional(readOnly = false)
    @KafkaListener(topics = KafkaConstantCode.TOPIC_DUTY_PERSON_ADD)
    @Override
    public void addPerson(ConsumerRecord<?, ?> record) {
        Optional<?> kafkaMessage = Optional.ofNullable(record.value());
        if (kafkaMessage.isPresent()) {
            Object message = kafkaMessage.get();
            JSONObject jsonObject = JSONObject.parseObject(message.toString());
            DutyUserInfo dutyUser = JSON.parseObject(jsonObject.getString("user"), new TypeReference<DutyUserInfo>() {});
            PersonBase personBase = new PersonBase();
			personBase.setExtCode(dutyUser.getId());
			personBase.setPersonName(dutyUser.getName());
			personBaseService.save(personBase, personBase.getPersonBaseId());
			AccoutInfo accountInfo = new AccoutInfo();
			accountInfo.setUserId(dutyUser.getLoginName());
			accountInfo.setPassWord(dutyUser.getPassword());
			accountInfo.setPersonBaseId(personBase.getPersonBaseId());
			accountInfo.setUserStat("0");
			accountInfo.setUserType("1");
			accountInfo.setAccountType("0");
			accoutInfoService.save(accountInfo, accountInfo.getAccountId());
			OrgPerson orgPerson = new OrgPerson();
			Org org = orgService.getByExtCode(dutyUser.getOfficeId());
			orgPerson.setIsbelong("1");
			orgPerson.setAccountId(accountInfo.getAccountId());
			orgPerson.setOrgId(org.getOrgId());
			orgPersonService.save(orgPerson, orgPerson.getOrgAccountId());
            
            logger.info("-----------------同步勤务新增人员 : " + personBase.getPersonName());
            SimpleDateFormat formatter= new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            Date date = new Date(System.currentTimeMillis());
            logger.info("同步时间:" + formatter.format(date));
        }

    }

	@Transactional(readOnly = false)
    @KafkaListener(topics = KafkaConstantCode.TOPIC_DUTY_PERSON_UPDATE)
	@Override
	public void updatePerson(ConsumerRecord<?, ?> record) {
		// TODO Auto-generated method stub
		
	}

	@Transactional(readOnly = false)
    @KafkaListener(topics = KafkaConstantCode.TOPIC_DUTY_PERSON_DELETE)
	@Override
	public void deletePerson(ConsumerRecord<?, ?> record) {
		Optional<?> kafkaMessage = Optional.ofNullable(record.value());
        if (kafkaMessage.isPresent()) {
            Object message = kafkaMessage.get();
            JSONObject jsonObject = JSONObject.parseObject(message.toString());
            String userId = jsonObject.getString("userId");
            PersonBase personBase = personBaseService.getByExtCode(userId);
            if(personBase != null) {
            	AccoutInfo accoutInfo = accoutInfoService.get(personBase.getAccountId());
            	accoutInfoService.delete(accoutInfo);
            	personBaseService.delete(personBase);
            	//删除人员角色关系
            	accountRoleService.deleteByAccountId(personBase.getAccountId());
            	//删除快捷表
            	rightShortcutService.deleteByAccountId(personBase.getAccountId());
            	//删除组织人员关系
            	orgPersonService.deleteByAccountId(personBase.getAccountId().toString());
            	//删除用户组人员关系
            	grpUserService.deleteByAccountId(personBase.getAccountId().toString());
            }
            
        }
		
	}
}
