package com.loit.v1.modules.enums;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.google.common.collect.Lists;

import java.util.Arrays;
import java.util.List;

/**
 * @Description: 警告类型枚举类
 * @author: zhangyl
 * @date: 2019年11月9日 下午14:50:56
 * @version: V1.0
 */

@JsonFormat(shape = JsonFormat.Shape.OBJECT)
public enum AlertTypeEnum {

	ALERT_TYPE_NOTE("1", "短信"),
	ALERT_TYPE_EMAIL("2", "邮件"),
	ALERT_TYPE_MESSAGE("3", "消息")
	;

	private AlertTypeEnum(String code, String name) {
		this.code = code;
		this.name = name;
	}

	private String code;// 编码
	private String name;// 名称

	public String getCode() {
		return code;
	}

	public String getName() {
		return name;
	}

	public static AlertTypeEnum queryByCode(String code) {
		for (AlertTypeEnum en : AlertTypeEnum.values()) {
			if (en.getCode() == code)
				return en;
		}

		return null;
	}

	public static String codeToName(String code) {
		AlertTypeEnum en = queryByCode(code);
		return en == null ? String.valueOf(code) : en.getName();
	}

	/**
	 * 获取全部枚举
	 */
	public static List<AlertTypeEnum> getAllEnum() {
		List<AlertTypeEnum> list = Lists.newArrayList();
		list.addAll(Arrays.asList(values()));
		return list;
	}

	/**
	 * 获取全部枚举值
	 *
	 * @return List<String>
	 */
	public static List<String> getAllEnumCode() {
		List<String> list = Lists.newArrayList();
		for (AlertTypeEnum en : values()) {
			list.add(en.getCode());
		}
		return list;
	}

	/**
	 * 通过Code获取枚举
	 */
	public static AlertTypeEnum getEnumByCode(String code) {
		AlertTypeEnum result = null;
		for (AlertTypeEnum en : getAllEnum()) {
			if (en.code.equals(code)) {
				result = en;
				break;
			}
		}
		return result;
	}
}
