package com.loit.v1.modules.handler;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.http.server.ServletServerHttpRequest;
import org.springframework.web.socket.WebSocketHandler;
import org.springframework.web.socket.server.support.HttpSessionHandshakeInterceptor;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * @description: WebSocket握手请求的拦截器. 检查握手请求和响应, 对WebSocketHandler传递属性
 * @date: 2020/4/26 15:25
 * @author: wubj
 * @version v1.0
 */
public class HandshakeInterceptor extends HttpSessionHandshakeInterceptor {

    private static Logger LOGGER = LoggerFactory.getLogger(HandshakeInterceptor.class);

    public static String address = null;

    // 在握手之前执行该方法, 继续握手返回true, 中断握手返回false. 通过attributes参数设置WebSocketSession的属性
    @Override
    public boolean beforeHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler, Map<String, Object> attributes) throws Exception {
        LOGGER.info("=========================HandshakeInterceptor: beforeHandshake=========================" + attributes);
        if (request instanceof ServletServerHttpRequest) {
            HttpServletRequest servletRequest = ((ServletServerHttpRequest) request).getServletRequest();
            String userId = servletRequest.getParameter("userId");
            if (address == null) {
                address = servletRequest.getLocalAddr()+":"+servletRequest.getLocalPort();
            }

            LOGGER.info("=========================WebSocket: " + address + ":" + userId + "=========================");

            // 使用userName区分WebSocketHandler，以便定向发送消息
            attributes.put("userId", userId);
            if (StringUtils.isBlank(userId)){
                return false;
            }
        }
        return super.beforeHandshake(request, response, wsHandler, attributes);
    }

    // 在握手之后执行该方法. 无论是否握手成功都指明了响应状态码和相应头.
    @Override
    public void afterHandshake(ServerHttpRequest request, ServerHttpResponse response, WebSocketHandler wsHandler, Exception ex) {
        LOGGER.info("=========================HandshakeInterceptor: afterHandshake=========================");
        super.afterHandshake(request, response, wsHandler, ex);
    }
}