package com.loit.v1.modules.service;

import com.loit.common.json.AjaxJson;
import com.loit.common.utils.ListUtil;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.modules.sys.utils.DictDataUtils;
import com.loit.v1.modules.utils.Model.PasswordCheckUtils;
import org.springframework.stereotype.Service;
import com.loit.common.service.CrudService;
import com.loit.v1.modules.dao.AccountPolicyDao;
import com.loit.v1.modules.entity.AccountPolicy;

import java.util.List;

import static com.loit.v1.modules.utils.Model.PasswordCheckUtils.checkPasswordLength;


/**
 * @ClassName: AccountPolicyService
 * @Description: 密码策略表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Service
public class AccountPolicyService extends CrudService<AccountPolicyDao, AccountPolicy> {

    /**
     * 用户密码策略验证
     * @return 验证通过true, 验证不通过false
     *
     */
    public  AjaxJson  pwdPolicyValidate(String password) {
        AjaxJson json = new AjaxJson();
        if( password == null || password =="" ){
            json.put("message", LoitStatusCode.POLICY_PASSWORD_NOT_NULL.statusDesc);
            return json;
        }
        List<AccountPolicy> list = this.dao.findList(null);
        if (ListUtil.isNotEmpty(list)) {
            AccountPolicy accountPolicy = list.get(0);
            if (checkPasswordLength(password,accountPolicy.getMinPass())) {
                while (PasswordCheckUtils.checkPasswordMixLevel(password, accountPolicy.getMixLevel())) {
                    json.setSuccess(true);
                    json.put("message", LoitStatusCode.POLICY_PASSWORD_SUCCESS.statusDesc);
                    return json;
                }
                json.setSuccess(false);
                json.put("message", LoitStatusCode.POLICY_PASSWORD_MIXLEVEL_ERR.statusDesc);
                return json;
            }
            json.setSuccess(false);
            json.put("message", LoitStatusCode.POLICY_PASSWORD_LENGTH_ERR.statusDesc);
            return json;
        }
        json.setSuccess(true);
        return json;
    }
    /**
     * 获取第一条密码策略, 业务逻辑只有一条密码策略
     * DEMON add by 2019-11-5
     */
    public AccountPolicy getOne(){
        List<AccountPolicy> list = this.dao.findList(null);
        if(ListUtil.isNotEmpty(list)){
        	AccountPolicy accountPolicy = list.get(0);
        	DictDataUtils.transfor(accountPolicy);
            return accountPolicy;
        }
        return null;
    }
}