package com.loit.v1.modules.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.json.AjaxJson;
import com.loit.common.service.CrudService;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.spring.timeloit.utils.RedisConstants;
import com.loit.common.utils.RedisUtil;
import com.loit.common.utils.StringReplaceUtil;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.api.dto.RightDTO;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.AccountRoleDao;
import com.loit.v1.modules.dao.AccoutInfoDao;
import com.loit.v1.modules.dao.RoleDao;
import com.loit.v1.modules.entity.AccountRole;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.entity.RightShortcut;



/**
 * @ClassName: AccountRoleService
 * @Description: 人员角色分配表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Service
public class AccountRoleService extends CrudService<AccountRoleDao, AccountRole> {

	@Autowired
	RightShortcutService rightShortcutService;
	@Autowired
	AccoutInfoDao accoutInfoDao;
	@Autowired
	RoleDao roleDao;
	
	/**
	 * 人员角色分配
	 * @param rightDTO
	 * @return
	 */
	@Transactional(readOnly = false)
	public AjaxJson personAssign(RightDTO rightDTO) {
		AjaxJson json = new AjaxJson();
		String roleIds = rightDTO.getRoleIds();
		if(StringUtils.isBlank(roleIds)) {
			//判断该用户有无其他角色
			List<RightShortcut> list = rightShortcutService.getRightByAccountId(rightDTO.getAccountId());
			if(list.size() < 1) {
				return AjaxJson.returnExceptionInfo(LoitStatusCode.ROLE_IS_NOT_NULL);
			}
		}
		AccoutInfo accoutInfo = accoutInfoDao.get(rightDTO.getAccountId());
		dao.deleteAccountRole(rightDTO.getAccountId(),rightDTO.getAppId());
		rightShortcutService.deletePersonRight(rightDTO,BusinessConstants.SOURCE_FROM_USER);
		if(StringUtils.isNotBlank(roleIds)) {
			String[] roleIdArray = roleIds.split(",");
			if(roleIdArray.length > 0 ) {
				for (String roleId : roleIdArray) {
					if(BusinessConstants.ORG_USER.equals(accoutInfo.getUserType())) {
						Role role = roleDao.get(Integer.valueOf(roleId));
						if(BusinessConstants.COMMON_ROLE.equals(role.getRoleType())) {
							json.setSuccess(false);
							json.setMsg("该角色:  " + role.getRoleName() +"无法分配给该用户,请联系管理员");
							return json;
						}
					}
					//保存账号角色关系
					AccountRole accountRole = new AccountRole();
					accountRole.setRoleId(Integer.valueOf(roleId));
					accountRole.setAccountId(rightDTO.getAccountId());
					this.save(accountRole, accountRole.getAccountRoleId());
					//更新快捷表
					RightShortcut rightShortcut  = new RightShortcut();
					rightShortcut.setAccountId(rightDTO.getAccountId());
					rightShortcut.setRoleId(Integer.valueOf(roleId));
					rightShortcut.setFromType(BusinessConstants.SOURCE_FROM_USER);
					rightShortcut.setFromId(rightDTO.getAccountId());
					rightShortcutService.save(rightShortcut, rightShortcut.getRscId());
				}
			}
		}
		String key = new StringReplaceUtil(RedisConstants.PORTAL_USER_KEY).replace("userId",  accoutInfo.getUserId()).toString();
		RedisUtil.remove(key);
		json.setSuccess(true);
		json.setMsg("分配成功!");
		return json;
	}

	/**
	 * @param accountId
	 */
	public void deleteByAccountId(Integer accountId) {
		dao.deleteByAccountId(accountId);
	}


}