package com.loit.v1.modules.service;

import com.loit.common.json.AjaxJson;
import com.loit.common.service.CrudService;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.AccountRoleDao;
import com.loit.v1.modules.dao.AccoutInfoDao;
import com.loit.v1.modules.dao.RightShortcutDao;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.entity.IpPolicy;
import com.loit.v1.modules.entity.IpPolicyAccount;
import com.loit.v1.modules.entity.PersonBase;
import com.loit.v1.modules.utils.Model.PasswordCheckUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;


/**
 * @ClassName: AccoutInfoService
 * @Description: 人员账号信息表
 * @author  zhaokz 
 * @date  2019-10-23 15:22:25
 * @version V1.0 
 **/
@Service
public class AccoutInfoService extends CrudService<AccoutInfoDao, AccoutInfo> {

	@Autowired
	private PersonBaseService personBaseService;
	@Autowired
	AccountRoleDao accountRoleDao;
	@Autowired
	RightShortcutDao rightShortcutDao;
	@Autowired
	OrgPersonService orgPersonService;
	@Autowired
	GrpUserService grpUserService;
	@Autowired
	IpPolicyService ipPolicyService;
	@Autowired
	IpPolicyAccountService ipPolicyAccountService;
	
	/**
	 * 批量逻辑删除
	 * @param accoutInfoIds
	 * @param accoutInfo
	 */
	@Transactional(readOnly = false)
	public void deleteByLogicAll(String accoutInfoIds, AccoutInfo accoutInfo) {
		String idArray[] =accoutInfoIds.split(",");
		if(idArray.length >0) {
			for(String id : idArray){
				accoutInfo.setAccountId(Integer.valueOf(id));
				accoutInfo.setDelFlag(AccoutInfo.DEL_FLAG_DELETE);
				dao.deleteByLogic(accoutInfo);
			}
		}
	}

	/**
	 * 获取人员详情
	 * @param accoutInfo
	 * @return
	 */
	public PersonBaseDTO getInfo(AccoutInfo accoutInfo) {
		return dao.getInfo(accoutInfo);
	}
	
	/**
	 * 根据登录账号获取账号信息
	 * @param userId
	 * @return
	 */
	public AccoutInfo getAccountByUserId(String userId) {
		return dao.getAccountByUserId(userId);
	}

	/**
	 * 删除人员
	 * @param accountIds
	 * @param accoutInfo
	 */
	@Transactional(readOnly = false)
	public void deleteAllAccount(String accountIds, AccoutInfo accoutInfo) {
		String idArray[] =accountIds.split(",");
        for(String accountId : idArray){
        	accoutInfo = this.get(accountId);
        	PersonBase personBase = personBaseService.get(accoutInfo.getPersonBaseId());
            dao.delete(accoutInfo);
            if(personBase != null) {
            	personBaseService.delete(personBase);
            }
            //删除人员角色关系
            accountRoleDao.deleteByAccountId(Integer.valueOf(accountId));
            //删除快捷表
            rightShortcutDao.deleteByAccountId(accountId);
            //删除组织人员关系
            orgPersonService.deleteByAccountId(accountId);
            //删除用户组人员关系
            grpUserService.deleteByAccountId(accountId);
        }
		
	}

	/**
	 * 密码修改
	 * @param accoutInfo
	 */
	@Transactional(readOnly = false)
	public void setPassword(AccoutInfo accoutInfo) {
		accoutInfo.setUnPassword(accoutInfo.getPassWord());
		String password = PasswordCheckUtils.encrypt(accoutInfo.getPassWord());
		accoutInfo.setPassWord(password);
		accoutInfo.setPassUpdateTime(new Date());
		this.save(accoutInfo,accoutInfo.getAccountId());
	}

	/**
	 * 密码验证
	 * @param userId 用户id，登录用户名
	 * @param password 明文密码
	 */
	public boolean validatePassword(String userId, String password) {
		AccoutInfo user = this.getAccountByUserId(userId);
		return PasswordCheckUtils.matches(password, user.getPassWord());
	}

	/**
	 * 账号唯一验证
	 * @param accoutInfo
	 * @return
	 */
	public AccoutInfo getUniqueAccount(AccoutInfo accoutInfo) {
		return dao.getUniqueAccount(accoutInfo);
	}

	/**
	 * 设置ip策略
	 * @param ipPolicyAccount
	 * @return
	 */
	@Transactional(readOnly = false)
	public AjaxJson setIpPolicy(IpPolicyAccount ipPolicyAccount) {
		AjaxJson json = new AjaxJson();
		// 获取策略信息
		IpPolicy ipPolicy = ipPolicyService.get(ipPolicyAccount.getIpPolicyId());
		IpPolicy ipPolicy1 = ipPolicyService.getByAccountId(ipPolicyAccount.getAccountId());
		if(ipPolicy1 == null && ipPolicy != null) {
			// 保存人员ip策略关系
			ipPolicyAccount.setIpType(ipPolicy.getIpType());
			ipPolicyAccount.setPolicyIp(ipPolicy.getPolicyIp());
			ipPolicyAccountService.save(ipPolicyAccount, ipPolicyAccount.getPaId());
		}
		json.setSuccess(true);
		json.setMsg("设置ip策略成功");
		return json;
	}

    /**
     * 根据登录账号列表获取账号信息列表
     * DEMON add by 2020-2-14
     */
    public List<PersonBaseDTO> getAccountByUserIds(List<String> userIds, String userIdOrPersonName){
        return dao.getAccountByUserIds(userIds, userIdOrPersonName);
    }

    /**
     * 获取登录账号总人数
     * DEMON add by 2020-2-14
     */
    public Integer findListCount(AccoutInfo accoutInfo){
        return dao.findListCount(accoutInfo);
    }

	/**
	 * 查询组织下组织用户
	 * @param orgId
	 * @return
	 */
	public AccoutInfo findOrgUser(Integer orgId) {
		return dao.findOrgUser(orgId);
	}

	/**
	 * 获取所有的账号信息
	 * @return
	 */
	public List<AccoutInfo> getAccoutInfoList() {
		return dao.getAccoutInfoList();
	}

	/**
	 * 根据人员名称获取人员信息
	 * @param personName
	 * @return
	 */
    public List<PersonBase> getPersonByPersonName(String personName) {
    	return dao.getPersonByPersonName(personName);
    }
}