package com.loit.v1.modules.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.service.CrudService;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.modules.dao.ActRightDao;
import com.loit.v1.modules.entity.ActRight;
import com.loit.v1.modules.entity.ResourceTypeOperation;



/**
 * @ClassName: ActRightService
 * @Description: 资源操作权限表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Service
public class ActRightService extends CrudService<ActRightDao, ActRight> {

	/**
	* @Description: 保存角色资源操作，先删除原有资源操作，再批量插入现有选择
	* @author: gzx
	* @date: 2019年11月11日 下午3:17:37 
	* @param @param roleId
	* @param @param resId
	* @param @param actIds 参数 
	* @return void 返回类型 
	* @throws
	 */
	@Transactional(readOnly = false)
	public void dealActRight(Map<String,Object> map) {
		String roleId = map.get("roleId").toString();
		Integer iRoleId = Integer.valueOf(roleId);
		String resId =  map.get("resId").toString();
		Integer iResId = Integer.valueOf(resId);
		String actIds =  map.get("actIds").toString();
		ActRight actRightTemp = new ActRight();
		actRightTemp.setRoleId(iRoleId);
		actRightTemp.setResId(iResId);
		super.delete(actRightTemp);
		if (actIds != null && actIds.length() > 0) {
			List<ActRight> actRightList = new ArrayList<ActRight>();
			String[] actRightIdArr = actIds.split(",");
			for (int i=0,len=actRightIdArr.length;i<len;i++) {
				ActRight actRight = new ActRight();
				actRight.setActId(Integer.valueOf(actRightIdArr[i]));
				actRight.setResId(Integer.valueOf(iResId));
				actRight.setRoleId(Integer.valueOf(iRoleId));
				actRight.setCreateTime(new Date());
				actRightList.add(actRight);
			}
			dao.insertBatch(actRightList);
		}
	}
	
	
	public List<Integer> getIdsByRoleIdAndResId(ActRight actRight) {
		List<ActRight> actRightList = dao.findAllList(actRight);
		if(actRightList != null && actRightList.size()>0) {
			List<Integer>  list = new ArrayList<Integer>();
			for(int i=0,len=actRightList.size();i<len;i++) {
				list.add(actRightList.get(i).getActId());
			}
			return list;
		} else {
			return null;
		}
	}
	/**
	* @Description: 根据菜单ID获取当前用户对本资源的操作权限
	* @author: Administrator
	* @date: 2019年11月19日 上午11:44:56 
	* @param @param menuId
	* @param @return 参数 
	* @return Map<String,String> 返回类型 
	* @throws
	 */
	public Map<String,Boolean> getActRightByMenuId(Integer menuId){
		User user = UserUtils.getUser();
		List<ResourceTypeOperation> list = dao.getActRightByMenuId(menuId, user.getAccountId());
		if(list!=null && list.size()>0) {
			Map<String,Boolean> map = new HashMap<String, Boolean>();
			for(int i=0,len=list.size();i<len;i++) {
				map.put(list.get(i).getActCode(), true);
			}
			return map;
		}
		return null;
	}
}