package com.loit.v1.modules.service;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.service.CrudService;
import com.loit.common.utils.StringUtils;
import com.loit.v1.api.dto.AppOrgDTO;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.AppOrgDao;
import com.loit.v1.modules.dao.ApporgShortcutDao;
import com.loit.v1.modules.dao.OrgDao;
import com.loit.v1.modules.entity.AppOrg;
import com.loit.v1.modules.entity.ApporgShortcut;
import com.loit.v1.modules.entity.Org;


/**
 * @ClassName: AppOrgService
 * @Description: 应用组织关系
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/

@Service
public class AppOrgService extends CrudService<AppOrgDao, AppOrg> {

	@Autowired
	private ApporgShortcutService apporgShortcutService;
	@Autowired
	private ApporgShortcutDao apporgShortcutDao;
	@Autowired
	OrgDao orgDao;
	
	/**
	 * 应用组织权限设置
	 * @param appOrg
	 */
	@Transactional(readOnly = false)
	public void insertAppOrg(AppOrg appOrg) {
		String orgFlag = appOrg.getOrgFlag();
		//删除应用权限表已设置数据
		appOrg.setOrgFlag("");
		AppOrg assignedAppOrg = dao.getByOrgIdAppId(appOrg.getOrgId().toString(), appOrg.getAppId().toString());
		dao.delete(assignedAppOrg);
		//递归删除快捷表本组织及所有子组织应用权限数据
		deleteChildrenAppOrg(appOrg.getAppId(),appOrg.getOrgId());
		//重新设置应用权限
		if(orgFlag.equals(BusinessConstants.ORG_SELF)) {
			//仅本组织
			ApporgShortcut apporgShortcut = new ApporgShortcut();
			apporgShortcut.setOrgId(appOrg.getOrgId());
			apporgShortcut.setAppId(appOrg.getAppId());
			apporgShortcutService.save(apporgShortcut,apporgShortcut.getAppOrgId());
		}else if(orgFlag.equals(BusinessConstants.ORG_CHILDREN)) {
			//仅下属组织 递归添加应用组织关系
			List<ApporgShortcut> list = new ArrayList<ApporgShortcut>();
 			list = addChildrenAppOrg(list,appOrg.getAppId(),appOrg.getOrgId());
			apporgShortcutService.batchInsertAppOrg(list);
		}else if(orgFlag.equals(BusinessConstants.ORG_SELF_CHILDREN)) {
			//本组织及下属组织
			List<ApporgShortcut> list = new ArrayList<ApporgShortcut>();
			list = addChildrenAppOrg(list,appOrg.getAppId(),appOrg.getOrgId());
			ApporgShortcut apporgShortcut = new ApporgShortcut();
			apporgShortcut.setOrgId(appOrg.getOrgId());
			apporgShortcut.setAppId(appOrg.getAppId());
			list.add(apporgShortcut);
			apporgShortcutService.batchInsertAppOrg(list);
		}
		appOrg.setOrgFlag(orgFlag);
		this.save(appOrg,appOrg.getRelId());
	}

	/**
	 * @apiNote 删除子组织应用权限数据(包括本组织)
	 * @param pid
	 */
	@Transactional(readOnly = false)
	public void deleteChildrenAppOrg(Integer appid,Integer orgid) {
		List<Integer> orgIds = new ArrayList<Integer>();
		List<Org> allChildrenList = orgDao.getAllChildrenOrg(orgid);
		for (Org org : allChildrenList) {
			orgIds.add(org.getOrgId());
		}
		apporgShortcutDao.deleteByOrgIds(orgIds,appid.toString());
	}
	
	/**
	 * 递归添加子组织应用权限数据(包括本组织)
	 * @param list 
	 * @param pid
	 */
	@Transactional(readOnly = false)
	private List<ApporgShortcut> addChildrenAppOrg(List<ApporgShortcut> list, int appid,int orgid) {
		List<Org> childrenList = orgDao.getAllChildrenOrg(orgid);
		//判断是否有子组织
		if(childrenList.size() > 0) {
			for (Org org : childrenList) {
				ApporgShortcut apporgShortcut = new ApporgShortcut();
				apporgShortcut.setOrgId(org.getOrgId());
				apporgShortcut.setAppId(appid);
				list.add(apporgShortcut);
			}
		}
		return list;
	}
	
	/**
	 * 获取已分配组织列表
	 * @param appOrg
	 * @return
	 */
	public List<AppOrgDTO> findAppOrgList(AppOrg appOrg) {
		return dao.findAppOrgList(appOrg);
	}

	/**
	 * 应用组织关系删除
	 * @param relIds
	 * @param appOrg
	 */
	@Transactional(readOnly = false)
	public void deleteAppOrg(String relIds, AppOrg appOrg) {
		String idArray[] =relIds.split(",");
        for(String id : idArray){
        	appOrg = this.get(id);
        	if(appOrg != null) {
        		String orgFlag = appOrg.getOrgFlag();
        		if(orgFlag.equals(BusinessConstants.ORG_SELF)) {
        			//仅本组织
        			apporgShortcutDao.deleteAppOrg(appOrg.getOrgId(),appOrg.getAppId());
        		}else if(orgFlag.equals(BusinessConstants.ORG_CHILDREN)) {
        			//仅下属组织
        			deleteChildrenAppOrg(appOrg.getAppId(), appOrg.getOrgId());
        			ApporgShortcut apporgShortcut = new ApporgShortcut();
        			apporgShortcut.setOrgId(appOrg.getOrgId());
        			apporgShortcut.setAppId(appOrg.getAppId());
        			apporgShortcutService.save(apporgShortcut,apporgShortcut.getAppOrgId());
        		}else if(orgFlag.equals(BusinessConstants.ORG_SELF_CHILDREN)) {
        			//本组织及下属组织
        			deleteChildrenAppOrg(appOrg.getAppId(), appOrg.getOrgId());
        		}
                dao.delete(appOrg);
        	}
        }
	}

	/**
	 * 校验组织是否分配权限
	 * @param appOrg
	 * @return
	 */
	public boolean check(AppOrg appOrg) {
		AppOrg appOrg2 = dao.check(appOrg);
		if(appOrg2 == null) {
			return true;
		}
		return false;
	}

	/**
	 * 新增/删除组织更新组织权限快捷表
	 * @param org
	 * @param string 
	 */
	@Transactional(readOnly = false)
	public void updateAppOrgShortCut(Org org) {
		String orgLevelCode = org.getOrgLevelCode();
		if(StringUtils.isNotBlank(orgLevelCode)) {
			String[] parentIds = orgLevelCode.split(",");
			if(parentIds.length > 0 ) {
				for (String parentId : parentIds) {
					AppOrg appOrg = dao.getByOrgIdAppId(parentId,org.getAppId().toString());
					if(appOrg != null) {
						if(appOrg.getOrgFlag().equals(BusinessConstants.ORG_CHILDREN) || appOrg.getOrgFlag().equals(BusinessConstants.ORG_SELF_CHILDREN)) {
							ApporgShortcut apporgShortcut = new ApporgShortcut();
		        			apporgShortcut.setOrgId(org.getOrgId());
		        			apporgShortcut.setAppId(org.getAppId());
		        			apporgShortcutService.save(apporgShortcut,apporgShortcut.getAppOrgId());
		        			break;
						}
					}
				}
			}
		}
	}

	/**
	 * @param appId
	 * @return
	 */
	public List<AppOrg> getAppOrgByAppId(Integer appId) {
		return dao.getAppOrgByAppId(appId);
	}

	/**
	 * 根据组织id删除应用组织关系
	 * @param orgId
	 */
	public void deleteAppOrgByOrgId(Integer orgId) {
		// 删除应用组织关系表及快捷表相关数据
		dao.deleteByOrgId(orgId);
		apporgShortcutDao.deleteByOrgId(orgId);
	}

	/**
	 * @description 根据组织编码获取当前应用组织树形列表
	 * @param appId
	 * @param orgCode
	 * @return
	 */
	public List<AppOrg> getAppOrgByAppIdAndOrgCode(String appId, String orgCode) {
		return dao.getAppOrgByAppIdAndOrgCode(appId,orgCode);
	}


}