package com.loit.v1.modules.service;

import java.net.URI;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.loit.common.utils.CacheUtils;
import com.loit.loitcasclient.config.ClientAppConfig;
import com.loit.loitcasclient.utils.ClientBase64Util;
import org.springframework.aop.framework.AopContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONObject;
import com.loit.common.json.AjaxJson;
import com.loit.common.service.CrudService;
import com.loit.common.spring.timeloit.utils.LogUtil;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.RedisUtil;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Role;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.AppResourceRelationDao;
import com.loit.v1.modules.dao.ApplicationManageDao;
import com.loit.v1.modules.entity.AppResourceRelation;
import com.loit.v1.modules.entity.ApplicationManage;
import com.loit.v1.modules.entity.MenuManage;
import com.loit.v1.modules.entity.RoleResMenu;
import com.loit.v1.modules.entity.ThemeManage;
import org.springframework.util.Base64Utils;

/**
 * @description 应用管理Service
 * @author xutao
 * @date 2019-10-18
 */
@Service
public class ApplicationManageService extends CrudService<ApplicationManageDao, ApplicationManage> {

	@Value("${loit.portal.indexUrlConf:}")
	private  String indexUrlConf;   // （1）前端上下文是否是门户的上下文。  （2）当前appId是内嵌还是独立系统。

	@Value("${loit.portal.vue_default_url:}")
	private String vue_default_url;   //vue默认的配置信息

	@Value("${loit.cas.app.client_url:}")
	private String clientUrl;     //前端默认配置的url


	@Autowired
	private ThemeManageService themeManageService;		
	@Autowired
	private AppResourceRelationDao appResourceRelationDao;
	@Autowired
	private RightShortcutService rightShortcutService;
	@Autowired
	private MenuManageService menuManageService;
	@Autowired
	private  ApplicationManageService applicationManageService;
	//用于获得配置的信息
	@Autowired
	private ClientAppConfig config = new ClientAppConfig();

	/**
	 * @description保存数据（新增应用）
	 * @param entity 应用实体
	 */
	@Transactional(readOnly = false)
	public void insert(ApplicationManage entity) {
		entity.preInsert();
//		if(StringUtils.isNotEmpty(entity.getDefaultHomePage()) &&  entity.getDefaultHomePage().startsWith("/") == false){
//			entity.setDefaultHomePage("/"+entity.getDefaultHomePage());	
//		}
		dao.insert(entity);
		ThemeManage themeManage = new ThemeManage();
		themeManage.setAppId(entity.getAppId());
		themeManageService.insert(themeManage);
	}
	
	/**
	 * @description 保存数据（更新应用）
	 * @param entity 应用实体
	 */
	@Transactional(readOnly = false)
	public void update(ApplicationManage entity) {
		entity.preUpdate();
//		if(StringUtils.isNotEmpty(entity.getDefaultHomePage()) &&  entity.getDefaultHomePage().startsWith("/") == false){
//			entity.setDefaultHomePage("/"+entity.getDefaultHomePage());	
//		}
		dao.update(entity);		
	}
	
	/**
	 * @description 删除全部数据（逻辑删除）
	 * @param ids 需要删除的应用的id集合 例：1,2,3
	 * @return 返回应用删除的信息，哪些成功，哪些失败
	 */
	@Transactional(readOnly = false)
	public String deleteAllByLogic(String ids) {
		String message;
		StringBuffer message1 = new StringBuffer();
		ApplicationManage entity = new ApplicationManage();
		List<String> list =new ArrayList<String>();
		String idArray[] =ids.split(",");
        for(String id : idArray){
            entity = get(id);
            //1代表是管理用的应用，无法做删除（如门户平台）
            if("1".equals(entity.getAdminFlag())) {
            	message = entity.getAppName() +"为系统应用，删除失败！";
            }else {
            	dao.deleteByLogic(entity);
            	//删除应用对应的主题表
            	ThemeManage themeManage = new ThemeManage();
            	themeManage.setAppId(Integer.parseInt(id));
            	themeManageService.delete(themeManage);
            	message = entity.getAppName() +"删除成功！";
            }
            list.add(message);
        }
        for (String string : list) {
        	message1 = message1.append(string);
		}
		return message1.toString();
	}
	
	/**
	 * @description 给应用分配单个或者多个资源
	 * @param mapStr(包含需要分配资源的应用appId及分配资源时勾选的资源的id和该资源所属应用的id 例如： appId:"1" resourceIds:"1$1,2$1")
	 */
	@Transactional(readOnly = false)
	public void distributeResources(String mapStr) {
		JSONObject jsonObject = JSONObject.parseObject(mapStr);
		//获取需要分配的应用的ID
		String appId = jsonObject.getString("appId");
		//获取已勾选的资源的id及该资源所属应用的id，中间用$符号隔开，例如（"1$1,2$1"）
		String resourceIds = jsonObject.getString("resourceIds");
		//先删除该应用之前分配的所有资源（应用和资源的关系表）
		if(StringUtils.isNotEmpty(appId)) {
			appResourceRelationDao.deleteByAppId(Integer.parseInt(appId));
		}
		if(StringUtils.isNotEmpty(resourceIds)){
			String[] idArray =resourceIds.split(",");
			for(String id : idArray){
				AppResourceRelation appResourceRelation = new AppResourceRelation();
				String[] splitId = id.split("\\$");
				//遍历出的单个资源id
				String splitResourceId = splitId[0];
				//遍历出的单个资源所属的应用id
				String splitAppId = splitId[1];
				//判断单个资源所属的应用id和需要分配资源的应用id是否相等，如果相等，说明该资源隶属于需要分配资源的应用，不相等的话，说明应用是引用了该资源
				if(appId.equals(splitAppId)) {
					appResourceRelation.setStatus("1");//隶属  
				}else {
					appResourceRelation.setStatus("2");//引用
				}
				appResourceRelation.setAppId(Integer.parseInt(appId));
				appResourceRelation.setResId(Integer.parseInt(splitResourceId));
				appResourceRelation.setCreateTime(new Date());
				//插入到应用和资源的关系表中
				appResourceRelationDao.insert(appResourceRelation);	    	
			}
		}
	}

	/**
	 * 获取应用信息
	 * @param appId
	 * @return
	 */
	public ApplicationManage getApplicationManage(Integer appId) {
		return this.getApplicationManage(appId, null);
	}

	/**
	 * 获取应用信息
	 * @param appId
	 * @return
	 */
	public ApplicationManage getApplicationManage(Integer appId, String appCode) {
		ApplicationManage applicationManage;
		if(appId != null || StringUtils.isNotEmpty(appCode)) {
			applicationManage = new ApplicationManage();
			applicationManage.setAppId(appId);
			applicationManage.setAppCode(appCode);
			applicationManage = dao.getByAppId(applicationManage);
		}else {
			applicationManage = null;
		}
		return applicationManage;
	}


	/**
	 * 获取菜单列表
	 */
	public List<MenuManage> getMenuListForUser(User user, int appId){
		RoleResMenu roleResMenu = new RoleResMenu().accountId(user.getAccountId()).accountType(user.getAccountType()).appId(appId);
		return this.getMenuList(roleResMenu, appId);
	}

	/**
	 * 获取菜单列表
	 * @param roleResMenu
	 * @param appId
	 * @return
	 */
	public List<MenuManage> getMenuList(RoleResMenu roleResMenu, Integer appId){
		long startTime2=System.currentTimeMillis();
		List<MenuManage> menuList = new ArrayList<MenuManage>();
		if(StringUtils.equals(roleResMenu.getAccountType(),BusinessConstants.ACCOUNT_TYPE_SUPERADMIN)) {
			MenuManage manage = new MenuManage();
			manage.setAppId(appId);
			manage.setIsEnabled("1");//可用菜单
			menuList = menuManageService.findAllTreeList(manage);
		}else {
			//根据登录人员的角色权限，获取当前登录人所拥有的叶子节点的菜单
			List<MenuManage> mapList = menuManageService.findLeafMenuList(roleResMenu.getAccountId(),appId);
			//通过子菜单查询菜单的树形列表集合
			if(mapList != null && mapList.size() > 0) {
				menuList = menuManageService.getMenuTreeList(mapList);
			}
		}
		long excTime2=System.currentTimeMillis()-startTime2;
		logger.info(LogUtil.print("获取菜单列表，执行用时： "+excTime2+"ms"));
		return menuList;
	}
	
	/**
	 * TODO DEMON 全局设置登录用户当前操作的appid
	 * @param appId
	 */
	public void setCurrentAppId(Integer accountId, Integer appId){

	}

	/**
	 * TODO DEMON 获取登录用户当前操作的appid
	 * @return
	 */
	public Integer getCurrentAppId(Integer accountId){
		return null;
	}
	
	/**
	* @Description: 判断名称的唯一性
	* @author: xutao
	* @date: 2019年11月19日 下午6:59:32 
	* @param @param appId
	* @param @param appName
	* @param @return 参数 
	* @return boolean 返回类型  名称重复，返回false；不重复，返回true
	* @throws
	 */
	public boolean checkNameUnique(Integer appId,String appName) {
		int count = dao.checkNameUnique(appName, appId);
		if(count>0) {
			return false;
		}
		return true;
	}
	
	/**
	* @description: 获取应用所分配的资源，并将资源ID及资源和应用的对应关系返回，供前端给应用分配资源时选中
	* @author: xutao
	* @date: 2019年11月13日 下午3:35:48 
	* @param @param appId	
	* @return List<AppResourceRelation> 返回类型 
	 */
	public List<AppResourceRelation> getAppResIds(Integer appId){
		AppResourceRelation appResourceRelation = new AppResourceRelation();
		appResourceRelation.setAppId(appId);
		List<AppResourceRelation> appResList = appResourceRelationDao.findList(appResourceRelation);		
		return appResList;
	}
	/**
	* @Description: 获取嵌入式展示的应用列表（接入系统门户中展示）
	* @author: gzx
	* @date: 2019年12月2日 下午2:42:31 
	* @param @param applicationManage
	* @param @return 参数 
	* @return List<ApplicationManage> 返回类型 
	* @throws
	 */
	public List<ApplicationManage> findEmbedAppList(ApplicationManage applicationManage){
		return dao.findEmbedAppList(applicationManage);
	}

	/**
	 * @description 检查应用系统代码是否重复
	 * @param appId
	 * @param appCode
	 * @return
	 */
	public boolean checkCodeUnique(Integer appId, String appCode) {
		int count = dao.checkCodeUnique(appCode, appId);
		if(count>0) {
			return false;
		}
		return true;
	}

	/**
	 * 根据appCode获取应用信息
	 * @param appCode
	 * @return
	 */
	public ApplicationManage getByAppCode(String appCode){
		return dao.getByAppCode(appCode);
	}

	/**
	 * @description 获取当前用户所拥有的应用列表及菜单树形列表及当前应用的主题，皮肤，菜单布局方式等
	 * @param roleResMenu 用户角色资源菜单应用关系对象
	 *
	 * Demon add by 2020-05-11: 从controller提取到service，被多个不同场景的接口调用
	 */
	@SuppressWarnings({ "rawtypes", "unchecked" })
	public AjaxJson getCurrentUserAppMenu(RoleResMenu roleResMenu, User user) {
		try {

			if(roleResMenu.getAccountId() == null) {
				roleResMenu.setAccountId(user.getAccountId());
			}

			if(roleResMenu.getAccountType() == null) {
				roleResMenu.setAccountType(user.getAccountType());
			}

			//获取应用列表
			Map<String, Object> map = new HashMap<String, Object>();
			//getAppList 被其他很多地方引用，此处为了根据角色所对应的应用做缓存，所以将user角色设置到roleResMenu中
			roleResMenu.setAccountId(user.getAccountId());
			List<Role> roleList = user.getRoleList();
			List<Integer> roleIdList = new ArrayList<>();
			for (Role role:roleList){
				roleIdList.add(role.getRoleId());
			}
			roleResMenu.setRoleList(roleIdList);
			//获取当前人员的应用列表
			long startTime=System.currentTimeMillis();
			List<Map<String,Object>> appList = rightShortcutService.getAppList(roleResMenu);
			long endTime=System.currentTimeMillis();
			float excTime=(float)(endTime-startTime)/1000;
			logger.info(LogUtil.print("获取应用列表，执行用时："+excTime+"秒"));
			map.put("appList", appList);

			Integer defaultAppId = null;

			// 判断使用appCode转换的appId还是使用参数appId还是使用第一个应用的appId来作为查询条件
			if(roleResMenu != null && StringUtils.isNotEmpty(roleResMenu.getAppCode())) {
				String appCode = roleResMenu.getAppCode();

				// Demon modify by 2020-05-11： 根据appCode获取appId，下面逻辑不能直接使用appCode查询菜单，查询菜单方法被切面植入缓存操作，缓存逻辑对参数敏感
				ApplicationManage application = this.getByAppCode(appCode);
				if(application == null || application.getAppId() == null){
					return AjaxJson.returnExceptionInfo(LoitStatusCode.APPLICATION_APP_CODE_ERRO);
				}

				defaultAppId = application.getAppId();
			} else if(roleResMenu != null && roleResMenu.getAppId() != null) {
				defaultAppId = roleResMenu.getAppId();
			}else if(appList != null && appList.size() > 0) {
				//获取当前应用（appId或appCode不传时，默认获取第一个应用）
				defaultAppId = (Integer) appList.get(0).get("appId");
			}

			// Demon add by 2020-05-11：切面缓存里需要从对象里取appId
			roleResMenu.setAppId(defaultAppId);
            //因为调用方法需要切面拦截,service方法内部调用需要获取代理类不能使用this直接调用，否则无法拦截到
            ApplicationManageService applicationManageService = (ApplicationManageService) AopContext.currentProxy();
			//应用的主题，皮肤，布局方式等信息
			ApplicationManage applicationManage = applicationManageService.getApplicationManage(defaultAppId);
			map.put("applicationManage", applicationManage);

			//菜单树形列表
			List<MenuManage> menuList = applicationManageService.getMenuList(roleResMenu, defaultAppId);
			map.put("menuList", menuList);

			//设置缓存（当前用户操作的应用id设置到全局里面, 共其他接口使用）
			this.setCurrentAppId(roleResMenu.getAccountId(), defaultAppId);

			AjaxJson json = new AjaxJson();
			json.put("map", map);
			return json;
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("info:获取当前用户所拥有的应用列表及菜单树形列表及当前应用的主题，皮肤，菜单布局方式等,class:{},message:{}",this.getClass(),e.getMessage());
			return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
		}
	}

	/**
	 * 获取appId
	 *
	 * 获取appId逻辑
	 * 	1.appCode不为空：使用appCode换取appId并返回appId
	 * 	2.appId不为空：返回appId
	 * 	3.appCode和appId都为空：则获取应用列表里的第一个应用，并返回应用的appId
	 *
	 * @param appCode app编码
	 * @param appId   appId
	 * @return appId
	 */
	public Integer defaultAppId(String appCode, Integer appId){
		Integer defaultAppId = null;

		// appCode不为空：使用appCode换取appId并返回appId
		if(StringUtils.isNotEmpty(appCode)) {
			String _appCode = appCode;

			// Demon modify by 2020-05-11： 根据appCode获取appId
			ApplicationManage application = this.getByAppCode(_appCode);
			if(application == null || application.getAppId() == null){
				return null;
			}

			defaultAppId = application.getAppId();
		} else if(appId != null) {
			// appId不为空：返回appId
			defaultAppId = appId;
		} else {
			// 获取应用列表里的第一个应用，并返回应用的appId
			User user = UserUtils.getUser();
			List<Map<String,Object>> appList = rightShortcutService.getAppList(
					new RoleResMenu().accountId(user.getAccountId()).accountType(user.getAccountType())
			);

			if(appList != null && appList.size() > 0) {
				//获取当前应用（appId或appCode不传时，默认获取第一个应用）
				defaultAppId = (Integer) appList.get(0).get("appId");
			}
		}

		return defaultAppId;
	}

	/**
	 * @desc 重置资源权限缓存
	 * @param roleIdList 角色列表
	 * @author yangwenbin
	 * @date
	 */
	public void resetRoleApplicationCache(List<Role> roleIdList) {
		for(Role role:roleIdList){
			Integer roleId = role.getRoleId();
			RoleResMenu roleResMenu = new RoleResMenu();
			List<Integer> roleIntegerList = new ArrayList<>();
			roleIntegerList.add(roleId);
			roleResMenu.setRoleList(roleIntegerList);
			//清除角色资源操作缓存
			CacheUtils.remove(BusinessConstants.ROLE_APPLICATION_LIST + roleId);
			//如果是超级管理员查询所有应用并缓存
			if(roleId.compareTo(1)== 0 ) {
				roleResMenu.setAccountType(BusinessConstants.ACCOUNT_TYPE_SUPERADMIN);
			}else{
				roleResMenu.setAccountType(BusinessConstants.ACCOUNT_TYPE_NORMAL);
			}
			List<Map<String,Object>> appList = rightShortcutService.getAppList(roleResMenu);
			//由于getAppList切面中往redis中进行填充值
			CacheUtils.set(BusinessConstants.ROLE_APPLICATION_LIST+roleId,appList,0L);
			//更新应用信息 由于在切面中更新应用时调用这个方法，故需要更新缓存的信息
			Integer appId = role.getAppId();
			CacheUtils.remove(BusinessConstants.APPLICATION_THEME_CACHE + appId);
			ApplicationManage applicationManage  = applicationManageService.getApplicationManage(appId);
			if(applicationManage!=null){
				CacheUtils.set(BusinessConstants.APPLICATION_THEME_CACHE + appId,applicationManage,0L);
			}
		}



	}

	/**
	 * ClientLoginController的accessToken方法中，
	 * 通过restTemplate调用api/v1/feign/applicationManage/getCurrentUserAppMenu获取applicationManage对象和menuList列表(参考ClientAppConfig的getWebUrl()方法)
	 *
	 * 1. applicationManage对象，判断应用是否配置首页菜单：defaultHomePage（ 默认首页菜单路径），contextValue （后端上下文），clientContextValue（前端上下文） 如果defaultHomePage为空，则去menuList列表继续查找。
	 * 2. menuList列表，判断第0个菜单：
	 * （1）entranceUrl （入口路径），menuContextValue （入口上下文值 ），menuClientContextValue（入口前端上下文）
	 * （2）resourceUrl（资源url），resContextValue（资源后端上下文），resClientContextValue（资源前端上下文），
	 * 	如果entranceUrl 入口路径为空，则查找resourceUrl，如果resourceUrl为空，递归查询其children。
	 * 3. 找到资源url和后端上下文，分为两种情况：
	 * （1）规则1，拼接规则：门户前端上下文+接资源url。
	 * （2）规则2，拼接规则：门户前端上下文+"/dist/#/myiframe/urlPath?src="+后端上下文+"/cas/login?url="+base64编码的资源url+"&appShow=1"。
	 *
	 * 注：规则1或者规则2的判断，有两种判断方式：
	 * （1）前端上下文是否是门户的上下文。   如果是前端的地址，那就调用规则1，否则调用规则2
	 * （2）当前appId是内嵌还是独立系统。  如果是1调用规则1，如果是2调用规则2
	 * （
	 *        @ApiModelProperty("应用展现方式（1、门户展示（内嵌） 2、独立展示），默认为1")
	 * 	private String appDisplayMode;
	 *  ）
	 * 这个要做成yml可配置的，增加一个loit.indexUrlConf参数来切换两种判断方式。
	 *
	 * 配置为1或者2，用于判断调用第几种判断规则
	 * @return
	 */
	public  AjaxJson getConfUrl(){
		AjaxJson result = new AjaxJson();
		User user =  UserUtils.getUser();
		RoleResMenu roleResMenu = new RoleResMenu();
		AjaxJson ajax = getCurrentUserAppMenu(roleResMenu,user);   //获得当前用户有权限的菜单
		if(ajax!=null&&ajax.isSuccess()){
			Map ajaxBody =  ajax.getBody();
			Map<String, Object> map = (Map<String, Object>)ajaxBody.get("map");  //获得返回的内容
			if(map!=null&&!map.isEmpty()){
				ApplicationManage applicationManage = (ApplicationManage)map.get("applicationManage");  //获得应用管理
				List<MenuManage> menuList  = (List<MenuManage>)map.get("menuList");   //获得菜单的信息
				//获得门户上下文中的serviceId，用于下面进行判断是不是前端地址
				String serviceId = "";
				Map contextMap  = config.contextConfig();
				if(contextMap!=null){
					serviceId =String.valueOf(contextMap.get("serviceId"));
				}else{
					logger.info("获取门户上下文配置异常，请重新获取");
				}
				if(applicationManage!=null){
					if(StringUtils.isBlank(applicationManage.getDefaultHomePage())
//							||StringUtils.isBlank(applicationManage.getContextValue())
//							||StringUtils.isBlank(applicationManage.getClientContextValue())
					){
						MenuManage  menuManage = recursionMenu(menuList,serviceId);   //递归循环菜单的信息
						if(menuManage!=null){
							String url ="";
							//判断入口的地址
							if(StringUtils.isNotBlank(menuManage.getEntranceUrl())){
								url=urlInfo(menuManage.getEntranceUrl(),menuManage.getMenuContextValue(),
										menuManage.getMenuClientContextValue(),applicationManage.getThemeManage().getAppDisplayMode(),serviceId);
								if(StringUtils.isBlank(url)){
									logger.info("菜单配置URL异常，请检查menuContextValue （入口上下文值）或者 menuClientContextValue（入口前端上下文）是否为空，具体的菜单信息是"+JSONObject.toJSON(menuManage));
									result = AjaxJson.returnExceptionInfo("菜单配置URL异常，请检查menuContextValue （入口上下文值）或者 menuClientContextValue（入口前端上下文）是否为空，具体的菜单信息是"+JSONObject.toJSON(menuManage));
								}
							}else{  //判断资源的地址
								url=urlInfo(menuManage.getResourceUrl(),menuManage.getResContextValue(),
										menuManage.getResClientContextValue(),applicationManage.getThemeManage().getAppDisplayMode(),serviceId);
								if(StringUtils.isBlank(url)){
									logger.info("菜单配置URL异常，请检查resContextValue（资源后端上下文），resClientContextValue（资源前端上下文）是否为空，具体的菜单信息是"+JSONObject.toJSON(menuManage));
									result = AjaxJson.returnExceptionInfo("菜单配置URL异常，请检查resContextValue（资源后端上下文），resClientContextValue（资源前端上下文）是否为空，具体的菜单信息是"+JSONObject.toJSON(menuManage));
								}
							}
							result.put("url",url);
						}else{
							logger.info("菜单配置不正确，请检查有权限的第一个菜单中entranceUrl （入口路径），menuContextValue （入口上下文值 ），menuClientContextValue（入口前端上下文）或者" +
									"resourceUrl（资源url），resContextValue（资源后端上下文），resClientContextValue（资源前端上下文）是否都已配置");
							result =AjaxJson.returnExceptionInfo("菜单配置不正确，请检查有权限的第一个菜单中entranceUrl （入口路径），menuContextValue （入口上下文值 ），menuClientContextValue（入口前端上下文）或者" +
									"resourceUrl（资源url），resContextValue（资源后端上下文），resClientContextValue（资源前端上下文）是否都已配置");
						}
					}else{
						String url = urlInfo(applicationManage.getDefaultHomePage(),applicationManage.getContextValue(),
								applicationManage.getClientContextValue(),applicationManage.getThemeManage().getAppDisplayMode(),serviceId);
						if(StringUtils.isBlank(url)){
							logger.info("应用配置URL异常，请检查contextValue （后端上下文），clientContextValue（前端上下文）是否都已配置，具体应用的信息是"+JSONObject.toJSON(applicationManage));
							result = AjaxJson.returnExceptionInfo("应用配置URL异常，请检查contextValue （后端上下文），clientContextValue（前端上下文）是否都已配置，具体应用的信息是"+JSONObject.toJSON(applicationManage));
						}
						result.put("url",url);
					}
				}else{
					logger.info("获取应用管理为空，请联系管理员");
					result =AjaxJson.returnExceptionInfo("获取应用管理为空，请联系管理员");
				}
			}
		}
		return result;
	}

	/**
	 * 用于递归获得配置正确的菜单信息
	 * @param menuList
	 * @return
	 */
	public MenuManage recursionMenu(List<MenuManage> menuList,String serviceId){
		MenuManage manage = null;
		if(menuList!=null&&!menuList.isEmpty()){
			MenuManage menuManage = menuList.get(0);
			String entranceUrl = menuManage.getEntranceUrl();   //入口路径
			String menuContextValue = menuManage.getMenuContextValue();  //入口上下文值
			String menuClientContextValue = menuManage.getMenuClientContextValue();  //入口前端上下文
			String resourceUrl = menuManage.getResourceUrl();  //资源url
			String resContextValue = menuManage.getResContextValue();   //资源后端上下文
			String resClientContextValue  = menuManage.getResClientContextValue(); //资源前端上下文
			//1、先判断入口的url信息，再判断资源的url信息，最后进入递归循环的信息
			if(config.getPortalServiceId().equals(serviceId)){    //如果是门户的情况调用以下的判断方式
				if(StringUtils.isNotBlank(entranceUrl)){
					manage = menuManage;
				}else if(StringUtils.isNotBlank(resourceUrl)){
					manage = menuManage;
				}else{
					manage = recursionMenu(menuManage.getChildren(),serviceId);
				}
			}else{    //调用不是门户的接口信息
				if(StringUtils.isNotBlank(entranceUrl)
						&&StringUtils.isNotBlank(menuContextValue)
						&&StringUtils.isNotBlank(menuClientContextValue)){
					manage = menuManage;
				}else if(StringUtils.isNotBlank(resourceUrl)
						&&StringUtils.isNotBlank(resContextValue)
						&&StringUtils.isNotBlank(resClientContextValue)){
					manage = menuManage;
				}else{
					manage = recursionMenu(menuManage.getChildren(),serviceId);
				}
			}
		}
		return manage;
	}

	/**
	 * 用于拼装所需的url信息
	 * @param resourceUrl  资源url
	 * @param resContextValue  资源后端上下文
	 * @param resClientContextValue  资源前端上下文
	 * @param appDisplayMode 应用展现方式（1、门户展示（内嵌） 2、独立展示），默认为1
	 * @param serviceId 服务的id
	 * @return
	 */
	public String urlInfo(String resourceUrl,String resContextValue,String resClientContextValue,String appDisplayMode,String serviceId){
		logger.info("resourceUrl="+resourceUrl+"====resContextValue="+resContextValue+"===resClientContextValue"+resClientContextValue+"===appDisplayMode="+appDisplayMode);
		String result = "";
//		String clientUrl = config.gePortalUrl();
		if(StringUtils.isNotBlank(indexUrlConf)){
			if(indexUrlConf.equals("1")){  //调用  前端上下文是否是门户的上下文。 的判断方法
				if(StringUtils.isNotBlank(clientUrl)){
					try{
						URI uri = new URI(clientUrl);
						if(config.getPortalServiceId().equals(serviceId)){
							if(StringUtils.isBlank(resourceUrl)){
								logger.info("资源url为空，请重新配置！！");
							}else{
								result="http://"+uri.getAuthority()+vue_default_url+resourceUrl;
							}
						}else{   // 门户前端上下文+"/dist/#/myiframe/urlPath?src="+后端上下文+"/cas/login?url="+base64编码的资源url+"&appShow=1"。
							if (StringUtils.isBlank(resClientContextValue)||StringUtils.isBlank(resContextValue)||StringUtils.isBlank(resourceUrl)){
								logger.info("资源前端上下文或者资源url或者资源后端上下文为空，请重新配置！！");
							}else {
								result=resClientContextValue+"/dist/#/myiframe/urlPath?src="+resContextValue+"/cas/login?url="+ ClientBase64Util.encoder(vue_default_url+resourceUrl) +"&appShow=1";
							}
						}
					}catch (Exception e){
						logger.error("url转换失败，请检查",e);
					}
				}
			}else{     //当前appId是内嵌还是独立系统。
				if("1".equals(appDisplayMode)){
					if(StringUtils.isBlank(resClientContextValue)||StringUtils.isBlank(resourceUrl)){
						logger.info("资源前端上下文或者资源url为空，请重新配置！！");
					}else{
						result=resClientContextValue+vue_default_url+resourceUrl;
					}
				}else{
					if (StringUtils.isBlank(resClientContextValue)||StringUtils.isBlank(resContextValue)||StringUtils.isBlank(resourceUrl)){
						logger.info("资源前端上下文或者资源url或者资源后端上下文为空，请重新配置！！");
					}else {
						result=resClientContextValue+"/dist/#/myiframe/urlPath?src="+resContextValue+"/cas/login?url="+ ClientBase64Util.encoder(vue_default_url+resourceUrl) +"&appShow=1";
					}
				}
			}
		}
		return result;
	}

}
