package com.loit.v1.modules.service;

import com.loit.common.service.CrudService;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.modules.dao.CommonEntranceDao;
import com.loit.v1.modules.entity.ApplicationManage;
import com.loit.v1.modules.entity.CommonEntrance;
import com.loit.v1.modules.entity.MenuManage;
import com.loit.v1.modules.entity.ThemeManage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;


/**
 * 常用入口管理表服务实现.
 * Created by wubj on 2020-06-15.
 **/
@Service
@Transactional(readOnly = true)
public class CommonEntranceService extends CrudService<CommonEntranceDao, CommonEntrance> {

    @Autowired
    private ApplicationManageService applicationManageService;

    @Autowired
    private MenuManageService menuManageService;

    @Autowired
    private ThemeManageService themeManageService;

    /**
     * 保存或更新
     * @param entity
     */
    @Transactional(readOnly = false)
    public void save(CommonEntrance entity) {
        // 应用信息
        if (entity.getAppId() != null) {
            ApplicationManage applicationManage = applicationManageService.get(entity.getAppId());
            entity.setAppName(applicationManage.getAppName());
            entity.setMenuContextValue(applicationManage.getContextValue());
            entity.setResContextValue(applicationManage.getContextValue());

            ThemeManage manage = themeManageService.get(entity.getAppId());
            if (manage != null && manage.getLayoutId() != null) {
                entity.setLayoutId(manage.getLayoutId());
            }
        }
        // 菜单信息
        if (entity.getMenuId() != null) {
            MenuManage menuManage = menuManageService.get(entity.getMenuId());
            entity.setEntranceName(menuManage.getEntranceName());
            entity.setMenuName(menuManage.getMenuName());
            entity.setEntranceUrl(menuManage.getEntranceUrl());
            entity.setResourceUrl(menuManage.getResourceUrl());

            String parentIds = menuManage.getParentIds();
            if (parentIds != null && !"".equals(parentIds)) {
                String[] arr = parentIds.split(",");
                if (arr.length > 1) {
                    entity.setMenuParentId(Integer.parseInt(arr[1]));
                }
            }
        }

        if (entity.getEntranceId() != null) {
            this.update(entity);
        } else {
            this.insert(entity);
        }
    }

    /**
     * 根据用户登录账号获取常用入口列表
     * @param userId
     * @return
     */
    public List<CommonEntrance> getListByUserId(String userId) {
        // 没传userId，就获取当前登录用户
        if (!StringUtils.isNotEmpty(userId)) {
            User user = UserUtils.getUser();
            userId = user != null ? user.getUserId() : null;
        }
        if(userId == null){
            return null;
        }
        List<CommonEntrance> list = new ArrayList<>();
        if ("admin".equals(userId)) {
            CommonEntrance entrance = new CommonEntrance();
            entrance.setDelFlag(CommonEntrance.DEL_FLAG_NORMAL);
            list = dao.findList(entrance);
        } else {
            list = dao.getListByUserId(userId);
        }
        return list;
    }

    /**
     * 常用入口管理菜单名称校验
     * @param commonEntrance
     * @return
     */
    public boolean validateMenuId(CommonEntrance commonEntrance) {
        CommonEntrance entrance = dao.validateMenuId(commonEntrance);
        if(entrance != null) {
            return false;
        }
        return true;
    }
}