package com.loit.v1.modules.service;

import com.loit.api.service.loituser.FileApiService;
import com.loit.common.json.AjaxJson;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.v1.modules.entity.FileResponseData;
import feign.Response;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.List;

/**
 * @description 文件上传下载管理Service
 * @author xutao
 * @date 2019-10-28 
 * @version V1.0
 */
@Service
public class FileRestService{
	
	protected Logger logger = LoggerFactory.getLogger(getClass());
	
    //private String uploadPath = Global.getConfig("file.uploadDir"); //文件上传的地址
    
    private static final int MAX_F_SIZE = 10 * 1024 * 1024;//文件大小设置
	@Autowired
	private FileApiService fileApiService;
    
	/**
	 * @description 上传图标文件 
	 * @param file
	 */
	public AjaxJson uploadIcon(@RequestParam(value = "file", required = true)MultipartFile file){
        String suffix = null;
        String fileName = null;
        FileResponseData fileResponseData = new FileResponseData();
        try {
	    	if(file!=null) {
	    		logger.info("开始上传文件");
	    		suffix = file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")); //后缀名
	    		//System.out.println(file.getSize());
	    		if(file.getSize() > MAX_F_SIZE) {
	    			logger.info("上传文件失败，文件超过大小");
	    			return AjaxJson.returnExceptionInfo(LoitStatusCode.FILE_OUT_SIZE);
	    		}else {
	    			if(".gif".equals(suffix)||".png".equals(suffix)||".jpg".equals(suffix)){
						return fileApiService.fileUpload(file);

		    		}else {
		    			logger.info("上传文件失败，不是允许的文件类型");
		        		return AjaxJson.returnExceptionInfo(LoitStatusCode.FILE_TYPE_ERROR_ALLOW);
		    		}
	    		}	    		
	    	}else{
	    		return AjaxJson.returnExceptionInfo(LoitStatusCode.FILE_ISNULL);
			}
        }catch (Exception e){
            logger.error("上传图标文件异常:",e);
            return AjaxJson.returnExceptionInfo(LoitStatusCode.UNKNOWN_EXCEPTION);
        }

    }
	
	/**
	 * @description 删除已上传的图标文件
	 * @param fileName
	 * @return
	 * @throws IOException 
	 */
	public AjaxJson delFile(String fileName) throws IOException {
		AjaxJson json = new AjaxJson();
		return fileApiService.deleteFile(fileName);
	}
	
	/**
	 * @description 图标文件下载
	 * @param response
	 * @param fileName
	 * @return
	 */
	public AjaxJson downLoad(HttpServletResponse response,String fileName){
		AjaxJson ajaxJson = new AjaxJson();
		//文件下载
		Response feignResonse=fileApiService.downloadFile(fileName);
		Response.Body body = feignResonse.body();
		for(Object key : feignResonse.headers().keySet()){
			List<String> kList = (List)feignResonse.headers().get(key);
			for(String val : kList){
				response.setHeader(String.valueOf(key), val);
			}
		}
		try{
			InputStream inputStream = body.asInputStream();
			OutputStream outputStream = response.getOutputStream();
			byte[] b = new byte[inputStream.available()];
			inputStream.read(b);
			outputStream.write(b);
			outputStream.flush();
			return ajaxJson;
		}catch (IOException e){
			return  AjaxJson.returnExceptionInfo(LoitStatusCode.FILE_DOWNLOAD_FAILED);
		}

	}
	
}
