package com.loit.v1.modules.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.json.AjaxJson;
import com.loit.common.service.CrudService;
import com.loit.common.utils.StringUtils;
import com.loit.v1.api.dto.RightDTO;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.GrpRoleDao;
import com.loit.v1.modules.dao.PersonBaseDao;
import com.loit.v1.modules.entity.GrpRole;
import com.loit.v1.modules.entity.PersonBase;
import com.loit.v1.modules.entity.RightShortcut;



/**
 * @ClassName: GrpRoleService
 * @Description: 用户组角色分配表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Service
public class GrpRoleService extends CrudService<GrpRoleDao, GrpRole> {

	@Autowired
	RightShortcutService rightShortcutService;
	@Autowired
	PersonBaseDao persBaseDao;
	
	/**
	 * 用户组角色分配
	 * @param rightDTO
	 * @return
	 */
	@Transactional(readOnly = false)
	public AjaxJson groupAssign(RightDTO rightDTO) {
		AjaxJson json = new AjaxJson();
		dao.deleteByGrpId(rightDTO.getGrpId());
		rightShortcutService.deleteByFrom(rightDTO.getGrpId().toString(),BusinessConstants.SOURCE_FROM_GRP);
		String roleIds = rightDTO.getRoleIds();
		if(StringUtils.isNotBlank(roleIds)) {
			String[] roleIdArray = roleIds.split(",");
			if(roleIdArray.length > 0 ) {
				PersonBase personBase = new PersonBase();
				personBase.setGrpId(rightDTO.getGrpId());
				for (String roleId : roleIdArray) {
					//保存用户组角色关系
					GrpRole grpRole = new GrpRole();
					grpRole.setRoleId(Integer.valueOf(roleId));
					grpRole.setGrpId(rightDTO.getGrpId());
					this.save(grpRole, grpRole.getGrpRoleId());
					//保存权限快捷表
					List<PersonBase> findGrpPersonPage = persBaseDao.findGrpPersonPage(personBase);
					if(findGrpPersonPage.size() > 0) {
						for (PersonBase person : findGrpPersonPage) {
							RightShortcut rightShortcut  = new RightShortcut();
							rightShortcut.setAccountId(person.getAccountId());
							rightShortcut.setRoleId(Integer.valueOf(roleId));
							rightShortcut.setFromType(BusinessConstants.SOURCE_FROM_GRP);
							rightShortcut.setFromId(rightDTO.getGrpId());
							rightShortcutService.save(rightShortcut, rightShortcut.getRscId());
						}
					}
				}
			}
		}
		json.setSuccess(true);
		json.setMsg("分配成功!");
		return json;
	}
}