package com.loit.v1.modules.service;

import org.springframework.stereotype.Service;

import com.loit.common.service.CrudService;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.IpPolicyAccountDao;
import com.loit.v1.modules.entity.IpPolicyAccount;
import com.loit.v1.modules.utils.Model.IpUtil;



/**
 * @ClassName: IpPolicyAccountService
 * @Description: IP人员策略分配表 - ip白名单
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Service
public class IpPolicyAccountService extends CrudService<IpPolicyAccountDao, IpPolicyAccount> {
	
	/**
     * 用户IP策略验证
     * @return 验证通过true, 验证不通过false
     */
    public Boolean ipValidate(String userId, String ip){

    	IpPolicyAccount ipPolicy = dao.get(userId);
		if(ipPolicy == null){
			return true;
		}

		// 策略类型 1、任意 IP 2、指定IP 3、IP段
    	String ipType = ipPolicy.getIpType();
    	if(ipType == BusinessConstants.IP_ANY) {
    		return true;
    	}
    	else if(ipType == BusinessConstants.IP_APPOINT) {
    		if(ipPolicy.getPolicyIp().equals(ip)) {
    			return true;
    		}
    		return false;
    	}
    	else if(ipType == BusinessConstants.IP_AREA) {
    		if(IpUtil.ipExistsInRange(ip,ipPolicy.getPolicyIp())) {
    			return true;
    		}
    		return false;
    	}
        return false;
    }
}