package com.loit.v1.modules.service;

import cn.jiguang.common.resp.APIConnectionException;
import cn.jiguang.common.resp.APIRequestException;
import cn.jpush.api.push.PushResult;
import cn.jpush.api.push.model.Message;
import cn.jpush.api.push.model.Platform;
import cn.jpush.api.push.model.PushPayload;
import cn.jpush.api.push.model.audience.Audience;
import cn.jpush.api.push.model.notification.Notification;
import com.loit.v1.modules.config.JPushConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @description: 极光推送service
 * @date: 2020/5/11 16:55
 * @author: wubj
 * @version v1.0
 */
@Service
public class JPushService {

    /**
     * 日志工具
     */
    private static Logger logger = LoggerFactory.getLogger(JPushService.class);

    // 一次推送最大数量 (极光限制1000)
    private static final int max_size = 800;

    @Autowired
    private JPushConfig jpushConfig;

    /**
     * 推送全部android
     * @param message
     * @return
     */
    public boolean pushAndroid(String message){
        PushPayload payload =  PushPayload.newBuilder()
                .setPlatform(Platform.android())//指定android平台的用户
                .setAudience(Audience.all())//你项目中的所有用户
                .setNotification(Notification.alert(message))
                //.setMessage(Message.content(message))  //内部消息不显示
                .build();
        return this.sendPush(payload);
    }

    /**
     * 推送android 指定id
     * @param message
     * @param userIds
     * @return
     */
    public boolean pushAndroid(String message, List<String> userIds){
        userIds = checkuserIds(userIds); // 剔除无效用户
        while (userIds.size() > max_size) { // 每次推送max_size个
            this.pushAndroidData(message, userIds.subList(0, max_size));
            userIds = userIds.subList(max_size, userIds.size());
        }
        return this.pushAndroidData(message, userIds);
    }

    /**
     * 推送android 指定id
     * @param message
     * @param userIds
     * @return
     */
    private boolean pushAndroidData(String message, List<String> userIds) {
        PushPayload payload = PushPayload.newBuilder()
                .setPlatform(Platform.android())//指定android平台的用户
                .setAudience(Audience.alias(userIds))//别名list
                .setNotification(Notification.alert(message))
                //.setMessage(Message.content(message))  //内部消息不显示
                .build();
        return this.sendPush(payload);
    }

    /**
     * 调用api推送
     * @param pushPayload 推送实体
     * @return
     */
    public boolean sendPush(PushPayload pushPayload){
        logger.info("发送极光推送请求: {}", pushPayload);
        PushResult result = null;
        try{
            if (jpushConfig.getJPushClient() != null) {
                result = jpushConfig.getJPushClient().sendPush(pushPayload);
            }
        } catch (APIConnectionException e) {
            logger.error("极光推送连接异常: ", e);
        } catch (APIRequestException e) {
            logger.error("极光推送请求异常: ", e);
        }
        if (result!=null && result.isResultOK()) {
            logger.info("极光推送请求成功: {}", result);
            return true;
        }else {
            logger.info("极光推送请求失败: {}", result);
            return false;
        }
    }

    /**
     * 剔除无效registed
     * @param userIds
     * @return
     */
    public List<String> checkuserIds(List<String> userIds) {
        List<String> regList = new ArrayList<String>(userIds.size());
        for (String registid : userIds) {
            if (registid!=null && !"".equals(registid.trim())) {
                regList.add(registid);
            }
        }
        return regList;
    }
}
