package com.loit.v1.modules.service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.loit.common.spring.timeloit.utils.LogUtil;

import com.loit.common.utils.CacheUtils;
import org.hibernate.validator.constraints.Length;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.persistence.Page;
import com.loit.common.service.CrudService;
import com.loit.common.utils.RedisUtil;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.MenuManageDao;
import com.loit.v1.modules.dao.ResMenuDao;
import com.loit.v1.modules.entity.MenuManage;
import com.loit.v1.modules.entity.MenuResourceRelation;
import com.loit.v1.modules.entity.RoleResMenu;
import com.loit.v1.modules.utils.Model.IndexComparator;
/**
 * @description 应用菜单管理Service
 * @author xutao
 * @date 2019-10-18
 */
@Service
//@Transactional(readOnly = true)
public class MenuManageService extends CrudService<MenuManageDao, MenuManage> {
	
	@Autowired
	private ResMenuDao resMenuDao;
	@Autowired
	private RoleService roleService;
	@Autowired
	private RightShortcutService rightShortcutService;
	
	/**
	 * @description 保存数据（新增菜单）
	 * @param entity 菜单实体
	 */
	@Transactional(readOnly = false)
	public void insert(MenuManage entity) {
		entity.preInsert();
		//如果不传父ID或者父ID为空时，默认为1级菜单，设置一级菜单的父ID为0
		if(null == entity.getParentId()) {
			entity.setParentId(0);
			entity.setParentIds("0,");
		}
		//如果不传父ID或者父ID为空时，默认为1级菜单，一级菜单的菜单级别为1
		if(StringUtils.isEmpty(entity.getMenuLevel())){
			entity.setMenuLevel("1");
		}
		dao.insert(entity);
		//如果选择资源的话，向菜单和资源的关联表中插入数据
		if(entity.getResId()!= null && entity.getResId()!= 0) {
			MenuResourceRelation menuResourceRelation = new MenuResourceRelation();
			menuResourceRelation.preInsert();
			menuResourceRelation.setResId(entity.getResId());
			menuResourceRelation.setMenuId(entity.getMenuId());
			resMenuDao.insertResMenu(menuResourceRelation);
		}
	}
	
	/**
	 * @description 保存菜单数据（更新菜单）
	 * @param entity
	 */
	@Transactional(readOnly = false)
	public void update(MenuManage entity) {
		//查询菜单和资源的关联表中是否有数据
		MenuResourceRelation menuResourceRelation = resMenuDao.getResMenu(entity.getMenuId());
		//没有关联数据
		if(menuResourceRelation == null) {
			//是否选择了资源
			if(entity.getResId()!= null && entity.getResId()!= 0) {
				//选择了资源，向菜单和资源的关联表中插入数据
				MenuResourceRelation menuResourceRelation1 = new MenuResourceRelation();
				menuResourceRelation1.preInsert();
				menuResourceRelation1.setResId(entity.getResId());
				menuResourceRelation1.setMenuId(entity.getMenuId());
				resMenuDao.insertResMenu(menuResourceRelation1);
			}
		//有关联数据	
		}else {
			//是否有资源数据
			if(entity.getResId()!= null && entity.getResId()!= 0) {
				//判断修改的资源数据和原来关联的资源数据是否一致，不一致时，根据菜单修改关联表数据
				if(menuResourceRelation.getResId() != entity.getResId()) {
					MenuResourceRelation menuResourceRelation2 = new MenuResourceRelation();
					menuResourceRelation2.setMenuId(entity.getMenuId());
					menuResourceRelation2.setResId(entity.getResId());
					menuResourceRelation2.preUpdate();
					resMenuDao.updateResMenu(menuResourceRelation2);
				}
			}else {
				//未选择资源数据时，将原来菜单和资源的关联关系删除
				resMenuDao.deleteResMenu(menuResourceRelation);
			}
		}
		//更新菜单数据
		entity.preUpdate();
		dao.update(entity);		
	}
	
	/**
	 * @description 批量删除菜单数据（物理删除）
	 * @param ids 菜单ID，多ID时，以逗号进行分割
	 */
	@Transactional(readOnly = false)
	public void deleteAll(String ids,MenuManage entity) {
		String idArray[] =ids.split(",");
		MenuResourceRelation menuResourceRelation = new MenuResourceRelation();
        for(String id : idArray){
            entity = get(id);
            dao.delete(entity);
            MenuManage manage = new MenuManage();
            manage.setParentId(Integer.valueOf(id));
            List<MenuManage> list = dao.findList(manage);
            if(list != null && list.size() > 0) {
            	for (MenuManage m : list) {
            		dao.delete(m);
				}
            }
            menuResourceRelation.setMenuId(Integer.valueOf(id));
            resMenuDao.deleteResMenu(menuResourceRelation);
        }
	}
	
	/**
	 * @description 获取菜单树状列表（）
	 * @param menuManage 菜单实体
	 * @return
	 */
	public List<MenuManage> getMenuTreeList(MenuManage menuManage){		
		return dao.getMenuTreeList(menuManage);
	}
	
	/**
	 * @description 查询菜单树形分页数据
	 * @param page 分页对象
	 * @param entity 菜单实体
	 * @return
	 */
	public Page<MenuManage> findTreePage(Page<MenuManage> page, MenuManage entity) {	
		entity.setPage(page);
		page.setList(dao.getMenuTreeList(entity));
		return page;
	}

	/**
	 * @description 根据登录人员的角色权限，获取当前登录人所拥有的叶子节点的菜单
	 * @return
	 */
	public List<MenuManage> findLeafMenuList(Integer accountId,Integer appId){
		return this.findLeafMenuList(accountId, appId, null);
	}
	
	/**
	 * @description 根据登录人员的角色权限，获取当前登录人所拥有的叶子节点的菜单
	 * @return
	 */
	public List<MenuManage> findLeafMenuList(Integer accountId,Integer appId, String appCode){
		List<MenuManage> list = dao.findLeafMenuList(accountId,appId,appCode);
		for (MenuManage menuManage : list) {
			menuManage.setChildren(new ArrayList<MenuManage>());
		}
		return list;
	}
	/**
	 * 将菜单按照顶层为parentId整理为树形结构
	 * @date: 2020年7月10日
	 * add by wbh
	 * @param menuList 菜单列表（包含所有父级菜单）
	 * @param parentId 父菜单ID
	 * @return
	 */
	public List<MenuManage> getMenuTreeList(List<MenuManage> menuList){
		List<MenuManage> resultMenuList = new ArrayList<>();
		Map<Integer, MenuManage> topMap = new HashMap<>();
		Map<Integer, Map<Integer, MenuManage>> map = new HashMap<>();//map集合，key为父ID，value为子节点集合
		for (MenuManage menuManage : menuList) {//整理出一个map集合，key为父ID，value为子节点集合
			if(menuManage.getParentId() == 0) {	//找出1级菜单
				topMap.put(menuManage.getMenuId(), menuManage);
			}else {//子菜单整理后放入map集合
				Integer pId = menuManage.getParentId();
				Map<Integer, MenuManage> mlist = map.get(pId);
				if(mlist==null) {
					mlist = new HashMap<>();
					mlist.put(menuManage.getMenuId(), menuManage);
					map.put(pId, mlist);
				}else{
					mlist.put(menuManage.getMenuId(), menuManage);
				}
			}
		}
		if(topMap.keySet().size()>0) {
			resultMenuList = topMap.values().stream().collect(Collectors.toList());
			Collections.sort(resultMenuList, new IndexComparator());
			for (MenuManage menuManage : resultMenuList) {//为1级菜单设置子菜单集合
				menuManage.setChildren(getTreeMenu(menuManage, map));
			}
		}
		
		return resultMenuList;
	}
	/**
	 * 递归设置菜单树
	 * add by wbh 2018-08-28
	 * @param menu 父菜单
	 * @param map 菜单关联map
	 * @return
	 */
	public List<MenuManage> getTreeMenu(MenuManage menu,Map<Integer, Map<Integer, MenuManage>> map){
		Integer pId = menu.getMenuId();
		Map<Integer, MenuManage> objMap = map.get(pId);
		if(objMap==null||objMap.keySet().size()==0) return new ArrayList<>();
		List<MenuManage> menuList = objMap.values().stream().collect(Collectors.toList());
		Collections.sort(menuList, new IndexComparator());
		for (MenuManage m : menuList) {
			m.setChildren(getTreeMenu(m, map));
		}
		return menuList;
	}
	
	/**
	 * @description 通过子菜单查询菜单的树形列表集合
	 * @param menuList 各级子菜单集合（从叶子菜单到根菜单，初始传入的为叶子菜单集合）
	 * @param map 用于控制父菜单的唯一性
	 * @param resultMenuList  树形菜单集合
	 * @return resultMenuList 树形菜单集合
	 */
	public List<MenuManage> getMenuTreeList(List<MenuManage> menuList,HashMap<String, Object> map,List<MenuManage> resultMenuList){
		if(menuList.size() > 0) {
			//初始化父菜单集合
			List<MenuManage> pList = new ArrayList<MenuManage>();
			//遍历各级子菜单（从叶子菜单到根菜单）
			for (MenuManage menuManage : menuList) {
				//已遍历到根菜单
				if(menuManage.getParentId() == 0) {					
					resultMenuList.add(menuManage);
					continue;
				}
				//获取父菜单对象
				MenuManage pMenuManage = this.get(menuManage.getParentId());
				//获取父菜单的子菜单集合
				List<MenuManage> childrenlist = pMenuManage.getChildren();
				if(childrenlist == null) {
					childrenlist = new ArrayList<MenuManage>();
				}
				//将子菜单集合添加到父菜单
				childrenlist.add(menuManage);
				pMenuManage.setChildren(childrenlist);
				//控制父菜单的唯一性
				if(!map.containsKey(menuManage.getParentId().toString())) {
					map.put(menuManage.getParentId().toString(), pMenuManage);
					pList.add(pMenuManage);
				}
			}
			//递归调用
			getMenuTreeList(pList,map,resultMenuList);
		}
		return resultMenuList;
	}
	
	/**
	 * @description （可根据状态（可用或者不可用））获取所有的菜单的树形列表
	 * @param manage 菜单实体
	 * @return
	 */
    public List<MenuManage> findAllTreeList(MenuManage manage){
        // 加载所有可用菜单列表  
        List<MenuManage> rootList = this.findAllList(manage);
        List<MenuManage> list = new ArrayList<>();
        list = getMenuTreeList(rootList);
        return list;
    }
    
    private List<MenuManage> getChildNodes(Integer menuId, List<MenuManage> rootList){
        List<MenuManage> childList = new ArrayList<>();
        for (MenuManage entity : rootList) {
            if (entity.getParentId() != null) {
                if (menuId.equals(entity.getParentId())) {
                    childList.add(entity);
                }
            }
        }
        if (childList.size() == 0) {
            return new ArrayList<MenuManage>();
        }
        for (MenuManage entity : childList) {
            entity.setChildren(getChildNodes(entity.getMenuId(), rootList));
        }
        return childList;
    }

	/**
	 *    根据角色查询菜单树形列表
	 * @param defaultAppId 
	 * @param roleResMenu
	 * @return
	 */
	public List<MenuManage> getMenuByroleIdsAndAppID(RoleResMenu roleResMenu, Integer defaultAppId) {
		List<MenuManage> menuList = new ArrayList<MenuManage>();
		if(StringUtils.equals(roleResMenu.getAccountType(),BusinessConstants.ACCOUNT_TYPE_SUPERADMIN)) {
			MenuManage manage = new MenuManage();
			manage.setAppId(defaultAppId);
			manage.setIsEnabled("1");//可用菜单
			long startTime2=System.currentTimeMillis();
			menuList = this.findAllTreeList(manage);
			long endTime2=System.currentTimeMillis();
			float excTime2=(float)(endTime2-startTime2)/1000;
			logger.info(LogUtil.print("获取菜单列表，执行用时："+excTime2+"秒"));
		}else {
			long startTime2=System.currentTimeMillis();
			if(roleResMenu.getRoleList() != null && roleResMenu.getRoleList().size() > 0) {
				List<MenuManage> mapList = new ArrayList<MenuManage>();
				for (int roleId : roleResMenu.getRoleList()) {
					//根据登录人员的角色权限，获取当前登录人所拥有的叶子节点的菜单
					List<MenuManage> list = this.findLeafMenuListByRoleId(roleId,defaultAppId);
					//缓存角色资源菜单
					RedisUtil.set(BusinessConstants.USER_MENU_CACHE + roleId + defaultAppId, list);
					mapList.addAll(list);
				}
				//通过子菜单查询菜单的树形列表集合
				if(mapList != null && mapList.size() > 0) {
					menuList = this.getMenuTreeList(menuList);
				}
			}
			long endTime2=System.currentTimeMillis();
			float excTime2=(float)(endTime2-startTime2)/1000;
			logger.info(LogUtil.print("获取菜单列表，执行用时："+excTime2+"秒"));
		}
		
		return menuList;
	}

	/**
	 * @param roleId
	 * @param defaultAppId
	 * @return
	 */
	public List<MenuManage> findLeafMenuListByRoleId(int roleId, Integer defaultAppId) {
		List<MenuManage> list = dao.findLeafMenuListByRoleId(roleId,defaultAppId);
//		for (MenuManage menuManage : list) {
//			menuManage.setChildren(new ArrayList<MenuManage>());
//		}
		return list;
	}

	/**
	 * @description 通过菜单id获取拥有该菜单的角色
	 * @param menuId
	 * @return 
	 */
	public List<Role> getRoleIdsByMenuId(Integer menuId) {
		return dao.getRoleIdsByMenuId(menuId);
	}

	/**
	 * @description 获取根菜单目录
	 * @return
	 */
	public List<MenuManage> getRootMenuList() {
		return dao.getRootMenuList();
	}

	/**
	 * @description 重置redis缓存
	 */
	public void resetMenuCache() {
		Role role = new Role();
		List<Role> roleList = roleService.findList(role);
		//清空缓存
//		RedisUtil.removePattern(BusinessConstants.USER_MENU_CACHE);
		CacheUtils.remove(BusinessConstants.APP_CACHE);
		CacheUtils.remove(BusinessConstants.ROOT_MENU_CACHE);
		//缓存应用列表
		RoleResMenu roleResMenu = new RoleResMenu();
		roleResMenu.setAccountId(1);
		List<Map<String,Object>> appList = rightShortcutService.getAppList(roleResMenu);
//		RedisUtil.set(BusinessConstants.APP_CACHE, appList);
		//缓存根菜单
		List<MenuManage> rootMenuList = this.getRootMenuList();
//		RedisUtil.set(BusinessConstants.ROOT_MENU_CACHE, rootMenuList);
		CacheUtils.set(BusinessConstants.ROOT_MENU_CACHE, rootMenuList,0L);
		for (Role role2 : roleList) {
			//如果是超级管理员,则缓存所有应用的菜单
			if(role2.getRoleId() == 1 ) {
				for (Map<String, Object> app : appList) {
					Integer appId = (Integer) app.get("appId");
					MenuManage manage = new MenuManage();
					manage.setAppId(appId);
					manage.setIsEnabled("1");//可用菜单
					List<MenuManage> menuList = this.findAllTreeList(manage);
					CacheUtils.set(BusinessConstants.USER_MENU_CACHE + role2.getRoleId() + appId, menuList.stream().distinct().collect(Collectors.toList()),0L);
				}
			}else {
				//目前其他角色只属于某一个应用
				List<MenuManage> menuList = this.findLeafMenuListByRoleId(role2.getRoleId(), role2.getAppId());
				CacheUtils.set(BusinessConstants.USER_MENU_CACHE + role2.getRoleId() + role2.getAppId(), menuList,0L);
			}
		}
	}

}
