package com.loit.v1.modules.service;

import com.loit.common.persistence.Page;
import com.loit.common.service.CrudService;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.DictData;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.DictDataUtils;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.NoticeReceiveDao;
import com.loit.v1.modules.entity.Notice;
import com.loit.v1.modules.entity.NoticeReceive;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 通知接收表服务实现.
 * Created by WBJ on 2020-04-23.
 **/
@Service
@Transactional(readOnly = true)
public class NoticeReceiveService extends CrudService<NoticeReceiveDao, NoticeReceive> {

    // 字典map
    private Map<String, String> dictMap = new HashMap<>();

    /**
     * 根据noticeIds删除数据
     */
    @Transactional(readOnly = false)
    public void deleteByNoticeIds(List<Long> noticeIds) {
        dao.deleteByNoticeIds(noticeIds);
    }

    @Transactional(readOnly = false)
    public void insertBatch(List<NoticeReceive> receiveList) {
        dao.insertBatch(receiveList);
    }

    @Transactional(readOnly = false)
    public void updateStatus(List<Long> list, String receiveId) {
        if (StringUtils.isEmpty(receiveId)) {
            User user = UserUtils.getUser();
            receiveId = user.getUserId();
        }
        dao.updateStatus(list, receiveId);
    }

    /**
     * 通知接收分页列表
     * @param page 分页对象
     * @param noticeReceive
     * @return
     */
    public Page<NoticeReceive> findPage(Page<NoticeReceive> page, NoticeReceive noticeReceive) {
        noticeReceive.setPage(page);
        if (!StringUtils.isNotEmpty(noticeReceive.getReceiveId())) {
            // 当前登录用户名
            User user = UserUtils.getUser();
            noticeReceive.setReceiveId(user.getUserId());
        }
        noticeReceive.setPublishStatus("1");
        List<NoticeReceive> list = dao.findList(noticeReceive);
        if (ListUtil.isNotEmpty(list)) {
            this.getDictMap();

            for (NoticeReceive entity : list) {
                if (StringUtils.isNotEmpty(entity.getNoticeType()) && StringUtils.isNotEmpty(dictMap.get(entity.getNoticeType()))) {
                    entity.setNoticeTypeName(dictMap.get(entity.getNoticeType()));
                }
            }
        }
        page.setList(list);
        return page;
    }

    /**
     * 通知详情
     * @param id
     * @return
     */
    public NoticeReceive get(Integer id) {
        NoticeReceive noticeReceive = dao.get(id);
        if (noticeReceive != null) {
            this.getDictMap();
            noticeReceive.setNoticeTypeName(dictMap.get(noticeReceive.getNoticeType()));
        }
        return noticeReceive;
    }

    /**
     * 获取字典数据
     */
    private void getDictMap() {
        if (dictMap == null || dictMap.size() == 0) {
            // 获取字典信息
            List<DictData> typeDict = DictDataUtils.getDictDataList(BusinessConstants.NOTICE_TYPE);
            for (DictData dictData : typeDict) {
                dictMap.put(dictData.getDataCode(), dictData.getDataName());
            }
        }
    }
}