package com.loit.v1.modules.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.loit.v1.api.dto.OrgExtattrDTO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.service.CrudService;
import com.loit.v1.modules.dao.OrgExtattrDao;
import com.loit.v1.modules.dao.OrgExtattrInstDao;
import com.loit.v1.modules.entity.OrgExtattr;
import com.loit.v1.modules.utils.Model.ModelReflectUtils;



/**
 * @ClassName: OrgExtattrService
 * @Description: 组织扩展属性表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Service
public class OrgExtattrService extends CrudService<OrgExtattrDao, OrgExtattr> {
	
	@Autowired
	private OrgExtattrInstDao orgExtattrInstDao;
	

	/**
	 * 获取组织属性列表
	 */
	public List<HashMap<String, Object>> findOrgPropertiesList(OrgExtattr orgExtattr) {
		List<HashMap<String, Object>> list = ModelReflectUtils.getPropertiesList(orgExtattr);
        List<OrgExtattr> orgExtattrList = dao.findList(orgExtattr);
        if(orgExtattrList.size() > 0) {
        	for (OrgExtattr orgExtattr2 : orgExtattrList) {
        		HashMap<String, Object> map = new HashMap<String, Object>();
				map.put("name", orgExtattr2.getAttrName());
				map.put("type", orgExtattr2.getAttrType());
				list.add(map);
			}
        }
		return list;
	}

	/**
	 * @param orgExtattr
	 * @return
	 */
	public List<OrgExtattr> findExtattrList(OrgExtattr orgExtattr) {
		return dao.findExtattrList(orgExtattr);
	}
	/**
	 * @param orgExtattr
	 * @return
	 */
	public List<OrgExtattrDTO> findExtattrListDTO(OrgExtattrDTO orgExtattr) {
		return dao.findExtattrListDTO(orgExtattr);
	}
	/**
	* @Description: 批量删除
	* @author: gzx
	* @date: 2019年11月19日 下午3:23:05 
	* @param @param orgAttrIds 参数 
	* @return void 返回类型 
	* @throws
	 */
	@Transactional(readOnly = false)
	public void deleteBatch(String orgAttrIds) {
		if(orgAttrIds != null && orgAttrIds.length() >0) {
			String[] orgArrtIdArr = orgAttrIds.split(",");
			List<OrgExtattr> extattrList = new ArrayList<OrgExtattr>();
			for(int i=0,len=orgArrtIdArr.length;i<len;i++) {
				OrgExtattr orgExtattr = new OrgExtattr();
				orgExtattr.setOrgAttrId(Integer.valueOf(orgArrtIdArr[i]));
				extattrList.add(orgExtattr);
			}
			dao.deleteBatch(extattrList);
			orgExtattrInstDao.deleteBatchByExtattrIds(extattrList);
		}
	}

}