package com.loit.v1.modules.service;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.service.CrudService;
import com.loit.v1.api.dto.RightDTO;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.OrgRoleDao;
import com.loit.v1.modules.dao.PersonBaseDao;
import com.loit.v1.modules.entity.OrgRole;
import com.loit.v1.modules.entity.PersonBase;
import com.loit.v1.modules.entity.RightShortcut;

import cn.jiguang.common.utils.StringUtils;



/**
 * @ClassName: OrgRoleService
 * @Description: 组织机构角色分配表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Service
public class OrgRoleService extends CrudService<OrgRoleDao, OrgRole> {
	
	@Autowired
	RightShortcutService rightShortcutService;
	@Autowired
	PersonBaseDao personBaseDao;
	
	/**
	 * @Description 组织机构分配角色
	 * @param rightDTO 权限分配数据传输类
	 */
	@Transactional(readOnly = false)
	public void orgDistributeRoles(RightDTO rightDTO) {		
		Integer orgId = rightDTO.getOrgId();
		if(orgId != null) {
			//根据组织ID删除组织机构角色分配表数据
			dao.deleteByOrgId(rightDTO.getOrgId());
			//根据来源唯一ID及来源类型删除权限快捷存储表里的数据
			rightShortcutService.deleteByFrom(orgId.toString(),BusinessConstants.SOURCE_FROM_ORG);
			String roleIds = rightDTO.getRoleIds();			
			if(StringUtils.isNotEmpty(roleIds)) {
				String idArray[] =roleIds.split(",");
		        for(String roleId : idArray){
		        	//向组织机构角色分配表插入数据
		        	OrgRole orgRole = new OrgRole();
		        	orgRole.setRoleId(Integer.valueOf(roleId));
		        	orgRole.setOrgId(orgId);
		            this.insert(orgRole);
		            //获取该组织下的所有人员账号
		            List<PersonBase> list = personBaseDao.findListByOrgId(orgId.toString(), new PersonBase().getDelFlag());
		            if(list.size() > 0) {
		            	for (PersonBase personBase : list) {
		            		//向权限快捷存储表插入数据
							RightShortcut rightShortcut  = new RightShortcut();
							rightShortcut.setAccountId(personBase.getAccountId());
							rightShortcut.setRoleId(Integer.valueOf(Integer.valueOf(roleId)));
							rightShortcut.setFromType(BusinessConstants.SOURCE_FROM_ORG);
							rightShortcut.setFromId(orgId);
							rightShortcutService.insert(rightShortcut);
				        }
		            }
		        }
			}			
		}	
	}
	
	/**
	 * @description 组织机构分配角色的id数组
	 * @param orgId
	 * @return
	 */
	public List<Integer> getRoleIdsByOrgId(Integer orgId) {
		List<Integer> list = new ArrayList<Integer>();
		OrgRole orgRole = new OrgRole();
		orgRole.setOrgId(orgId);
		List<OrgRole> roles = dao.findList(orgRole);
		if(roles.size() > 0) {
			for (OrgRole role : roles) {
				list.add(role.getRoleId());
			}
		}
		return list;
	}
	
}