package com.loit.v1.modules.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.service.CrudService;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.v1.api.dto.OrgDTO;
import com.loit.v1.api.dto.OrgExtattrDTO;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.api.util.TreeUtils;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.AccoutInfoDao;
import com.loit.v1.modules.dao.OrgDao;
import com.loit.v1.modules.dao.OrgPersonDao;
import com.loit.v1.modules.dao.OrgRoleDao;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.entity.AppOrg;
import com.loit.v1.modules.entity.Org;
import com.loit.v1.modules.entity.OrgExtattr;
import com.loit.v1.modules.entity.OrgExtattrInst;
import com.loit.v1.modules.utils.Model.IotUUIDUtils;



/**
 * @ClassName: OrgService
 * @Description: 组织机构表
 * @author  zhaokz 
 * @date  2019-10-22 18:36:40
 * @version V1.0 
 **/
@Service
public class OrgService extends CrudService<OrgDao, Org> {

	@Autowired
	AppOrgService appOrgService;
	@Autowired
	OrgExtattrInstService orgExtattrInstService;
	@Autowired
	RightShortcutService rightShortcutService;
	@Autowired
	OrgRoleDao orgRoleDao;
	@Autowired
	AccoutInfoDao accoutInfoDao;
	@Autowired
	OrgExtattrService orgExtattrService;
	@Autowired
	OrgPersonDao orgPersonDao;
	@Autowired
	ApporgShortcutService apporgShortcutService;
	@Autowired
	PersonBaseService personBaseService;
	
	
	/**
	 * 逻辑删除
	 * @param orgIds
	 * @param org
	 */
	@Transactional(readOnly = false)
	public void deleteByLogicAll(String orgIds, Org org) {
		String[] idArray = orgIds.split(",");
		if(idArray.length >0) {
			for(String id : idArray){
				org.setOrgId(Integer.valueOf(id));
				org.setDeltag(Org.DEL_FLAG_DELETE); 
				//逻辑删除本组织及下属组织
				dao.deleteByLogic(org);
				//逻辑删除本组织及下属组织下的人
				accoutInfoDao.logicDeleteInOrg(Integer.valueOf(id));
			}
		}
	}

	/** 查询组织树状列表
	 * @param org
	 * @return
	 */
	public List<OrgDTO> findOrgTreeList(Org org) {
		long startTime = System.currentTimeMillis();
		List<OrgDTO> allList = dao.findAll(org);
		List<OrgDTO> tree = new ArrayList<OrgDTO>();
		try {
			tree = TreeUtils.getTree(allList, "orgId");
		} catch (Exception e) {
			e.printStackTrace();
		}
		long endTime = System.currentTimeMillis();
		float excTime=(float)(endTime-startTime)/1000;
		logger.info("执行耗时----"+ excTime +"秒");
		return tree;
	}

	/**
	 * 获取下属组织
	 * @param orgId 
	 * @return
	 */
	public List<Org> getChildrenOrg(int orgId) {
		return dao.getChildrenOrg(orgId);
	}

	/**
	 * 获取当前应用组织树
	 * @param org
	 * @return
	 */
	public List<OrgDTO> findCuAppOrgTree(Org org) {
		List<OrgDTO> list = new ArrayList<OrgDTO>();
		List<AppOrg> appOrgs = appOrgService.getAppOrgByAppId(org.getAppId());
		if(appOrgs == null) {
			return list;
		}
		List<OrgDTO> tree = new ArrayList<OrgDTO>();
		for (AppOrg appOrg : appOrgs) {
			OrgDTO org3 = dao.getDTO(appOrg.getOrgId());
			Org org2 = dao.get(appOrg.getOrgId());
			org2.setAppId(org.getAppId());
			if(appOrg.getOrgFlag().equals(BusinessConstants.ORG_SELF)) {
				//仅本组织
				list.add(org3);
			}else {
				List<OrgDTO> treeList = dao.findCuAppOrgList(org2);
				if(appOrg.getOrgFlag().equals(BusinessConstants.ORG_SELF_CHILDREN)){
					treeList.add(org3);
				}
				try {
					tree = TreeUtils.getTree(treeList, "orgId");
				} catch (Exception e) {
					e.printStackTrace();
				}
				list.addAll(tree);
			}
		}
		return list;
	}
	
	/**
	 * 保存组织
	 * @param org
	 */
	@Transactional(readOnly = false)
	public void saveOrg(Org org) {
		int flag = 0;
		if(org.getOrgId()==null) {
			//物联子平台需要uuid
			org.setSid(IotUUIDUtils.getUUID());
			//添加组织，parentIds处理下
			org.setOrgLevelCode(org.getOrgLevelCode()+org.getParentId()+",");
			flag = 1;
		}else {
			//判断组织是否禁用
			if(org.getOrgStatus().equals(BusinessConstants.ORG_STATUS_FORBID)) {
				List<AccoutInfo> findListByOrgId = accoutInfoDao.findListByOrgId(org.getOrgId().toString(), Org.DEL_FLAG_NORMAL);
				if(findListByOrgId.size() > 0 ) {
					for (AccoutInfo accoutInfo : findListByOrgId) {
						accoutInfo.setUserStat(BusinessConstants.ACCOUNT_STATUS_FORBID);
						accoutInfoDao.update(accoutInfo);
					}
				}
			}
			if(org.getOrgStatus().equals(BusinessConstants.ORG_STATUS_NORMAL)) {
				List<AccoutInfo> findListByOrgId = accoutInfoDao.findListByOrgId(org.getOrgId().toString(), Org.DEL_FLAG_NORMAL);
				if(findListByOrgId.size() > 0 ) {
					for (AccoutInfo accoutInfo : findListByOrgId) {
						accoutInfo.setUserStat(BusinessConstants.ACCOUNT_STATUS_NORMAL);
						accoutInfoDao.update(accoutInfo);
					}
				}
			}
		}
		super.save(org, org.getOrgId());
		// 扩展字段保存
		List<OrgExtattr> orgExtattrs = org.getOrgExtattrs();
		if(orgExtattrs.size() > 0 ) {
			for (OrgExtattr orgExtattr : orgExtattrs) {
				if(StringUtils.isNotBlank(orgExtattr.getOrgAttrValue())) {
					OrgExtattrInst orgExtattrInst = orgExtattrInstService.getUnique(org.getOrgId(),orgExtattr.getOrgAttrId());
					if(orgExtattrInst != null) {
						orgExtattrInst.setOrgAttrId(orgExtattr.getOrgAttrId());
						orgExtattrInst.setOrgAttrValue(orgExtattr.getOrgAttrValue());
						orgExtattrInst.setOrgId(org.getOrgId());
						orgExtattrInstService.save(orgExtattrInst, orgExtattrInst.getInstId());
					}else {
						OrgExtattrInst orgExtattrInst1 = new OrgExtattrInst();
						orgExtattrInst1.setOrgAttrId(orgExtattr.getOrgAttrId());
						orgExtattrInst1.setOrgAttrValue(orgExtattr.getOrgAttrValue());
						orgExtattrInst1.setOrgId(org.getOrgId());
						orgExtattrInstService.save(orgExtattrInst1, orgExtattrInst1.getInstId());
					}
				}
			}
		}
		//应用权限控制
		if(flag == 1) {
			appOrgService.updateAppOrgShortCut(org);
		}
	}

	/**
	 * 物理删除组织
	 * @param orgIds
	 * @param appId 
	 */
	@Transactional(readOnly = false)
	public void deleteOrg(String orgIds) {
		String[] orgIdArray = orgIds.split(",");
		Org org = new Org();
		org.setDeltag("1");
		for (String orgId : orgIdArray) {
			org.setOrgId(Integer.valueOf(orgId));
			dao.deleteOrgAndChildren(org);
			//删除应用组织关系
			appOrgService.deleteAppOrgByOrgId(Integer.valueOf(orgId));
			//根据组织ID删除组织机构角色分配表数据
			orgRoleDao.deleteByOrgId(Integer.valueOf(orgId));
			//根据来源唯一ID及来源类型删除权限快捷存储表里的数据
			rightShortcutService.deleteByFrom(orgId.toString(),BusinessConstants.SOURCE_FROM_ORG);
			//删除人员组织关系
			orgPersonDao.deleteByOrgId(orgId);
			//删除组织及下属组织下所有人员
			accoutInfoDao.deletePersonInOrg(orgId);
		}
		
	}

	/**
	 * 获取未添加用户组组织列表
	 * @param org
	 * @return
	 */
	public List<OrgDTO> findUnAddOrgs(Org org) {
		List<OrgDTO> list = new ArrayList<OrgDTO>();
		List<AppOrg> appOrgs = appOrgService.getAppOrgByAppId(org.getAppId());
		if(appOrgs == null) {
			return list;
		}
		List<OrgDTO> tree = new ArrayList<OrgDTO>();
		for (AppOrg appOrg : appOrgs) {
			OrgDTO org3 = dao.getDTO(appOrg.getOrgId());
			if(org3 != null) {
				Org org2 = dao.get(appOrg.getOrgId());
				org2.setAppId(org.getAppId());
				if(appOrg.getOrgFlag().equals(BusinessConstants.ORG_SELF)) {
					//仅本组织
					list.add(org3);
				}else {
					List<OrgDTO> treeList = dao.findCuAppOrgList(org2);
					if(appOrg.getOrgFlag().equals(BusinessConstants.ORG_SELF_CHILDREN)){
						treeList.add(org3);
					}
					try {
						tree = TreeUtils.getTree(treeList, "orgId");
					} catch (Exception e) {
						e.printStackTrace();
					}
					list.addAll(tree);
				}
			}
		}
		return list;
	}

	/**
	 * 已添加用户组组织列表
	 * @param page
	 * @param org
	 * @return
	 */
	public Page<Org> findGrpOrgs(Page<Org> page, Org org) {
		org.setPage(page);
		List<Org> list = dao.findGrpOrgs(org);
		page.setList(list);
		return page;
	}
	
	public static void main(String[] args) {
		List<String> list = new ArrayList<String>();
		String a = "a";
		list.add(a);
		List<String> bs = new ArrayList<String>();
		bs.add("b");
		bs.add("c");
		list.addAll(bs);
		System.out.println(list);
	}
	/**
	 * 查询列表数据
	 * @param appId
	 * @return
	 */
	public List<OrgDTO> findListByAppId(int appId) {
		List<OrgDTO> orgDTOList = dao.findListByAppId(appId);

		if(orgDTOList.size() > 0){
			for(OrgDTO orgDTO : orgDTOList) {
				OrgExtattrDTO orgExtattr = new OrgExtattrDTO();
				orgExtattr.setOrgId(orgDTO.getOrgId());
				List<OrgExtattrDTO> orgExtattrs  = orgExtattrService.findExtattrListDTO(orgExtattr);
				orgDTO.setOrgExtattrs(orgExtattrs);
			}
		}
		return orgDTOList;
	}

	/**
	 * 组织还原
	 * @param orgIds
	 * @return
	 */
	@Transactional(readOnly = false)
	public AjaxJson revert(String orgIds) {
		AjaxJson json = new AjaxJson();
		String[] orgIdArray = orgIds.split(",");
		if(orgIdArray.length > 0) {
			Org org = new Org();
			for (String orgId : orgIdArray) {
				org.setOrgId(Integer.valueOf(orgId));
				org.setDeltag(Org.DEL_FLAG_NORMAL); 
				//还原逻辑删除的本组织及下属组织
				dao.revert(org);
				//还原逻辑删除的本组织及下属组织下的人
				accoutInfoDao.revertPersonInOrg(orgId);
			}
			json.setSuccess(true);
			json.setMsg("操作成功");
		}
		return json;
	}
	
	/**
	 * 组织名称唯一性验证
	 * @param org
	 * @return
	 */
	public boolean validateOrgName(Org org) {
		Org org2 = dao.getByOrgName(org);
		if(org2 != null) {
			return false;
		}
		return true;
	}

	/**
	 *  根据ID列表获取多条数据
	 * @param ids
	 * @return
	 */
	public List<Org> findByIdList(List<String> ids){
		return dao.findByIdList(ids);
	}
	/**
	 *  根据Id查询
	 * @param orgId
	 * @return
	 */
	public OrgDTO getByOrgId(long orgId){
		return dao.getByOrgId(orgId);
	}

	/**
	 * 获取当前组织及下属组织树
	 * @return
	 */
	public List<OrgDTO> getOwnOrgTree(Org org) {
		List<OrgDTO> list = new ArrayList<OrgDTO>();
		OrgDTO org2 = dao.getDTO(org.getOrgId());
		//本组织及下属组织
		List<OrgDTO> treeList = dao.findCuAppOrgList(org);
		list.add(org2);
		try {
			list = TreeUtils.getTree(treeList, "orgId");
		} catch (Exception e) {
			e.printStackTrace();
		}
		return list;
	}

	/**
	 * @description 根据组织编码获取当前应用组织树形列表
	 * @param org
	 * @return
	 */
	public List<OrgDTO> findOrgTreeByOrgCode(Org org) {
		//获取组织树列表
		List<OrgDTO> list = new ArrayList<OrgDTO>();
		List<AppOrg> appOrgs = null;
		if (StringUtils.isNotEmpty(org.getOrgCode())) {
			appOrgs = appOrgService.getAppOrgByAppIdAndOrgCode(org.getAppId().toString(), org.getOrgCode());
		} else {
			appOrgs = appOrgService.getAppOrgByAppId(org.getAppId());
		}

		if(appOrgs == null || appOrgs.size() < 1 ) {
			return list;
		}
		List<OrgDTO> tree = new ArrayList<OrgDTO>();
		for (AppOrg appOrg : appOrgs) {
			OrgDTO org3 = dao.getDTO(appOrg.getOrgId());
			if (StringUtils.isNotEmpty(org.getOrgCode())) {
				org3.setParentId(0);
			}
			Org org2 = dao.get(appOrg.getOrgId());
			org2.setAppId(org.getAppId());
			if(appOrg.getOrgFlag().equals(BusinessConstants.ORG_SELF)) {
				//仅本组织
				list.add(org3);
			}else {
				List<OrgDTO> treeList = dao.findCuAppOrgList(org2);
				if(appOrg.getOrgFlag().equals(BusinessConstants.ORG_SELF_CHILDREN)){
					treeList.add(org3);
				}
				try {
					tree = TreeUtils.getTree(treeList, "orgId");
				} catch (Exception e) {
					e.printStackTrace();
				}
				list.addAll(tree);
			}
		}
		//根据组织编码获取组织及下属组织所有人
//		Org org2 = dao.getByOrgCode(org.getOrgCode());
		List<PersonBaseDTO> personList = personBaseService.findPersonsByOrgCode(org.getAppId());
		HashMap<Integer, List<PersonBaseDTO>> personMap = personBaseGroupByOrgId(personList);
		//递归将人员放入组织树中
		for (OrgDTO orgDTO : list) {
			addPersonsToOrgTree(orgDTO,personMap);
		}
		return list;
	}
	
	/**
	 * @description 人员列表根据组织id分组
	 * @param personBaseDTOs
	 * @return
	 */
	private HashMap<Integer, List<PersonBaseDTO>> personBaseGroupByOrgId(List<PersonBaseDTO> personBaseDTOs) {
		HashMap<Integer, List<PersonBaseDTO>> map = new HashMap<>();
		for (PersonBaseDTO personBaseDTO : personBaseDTOs) {
			Integer orgId = personBaseDTO.getOrgId();
			List<PersonBaseDTO> oldList = map.get(orgId);
			if(ListUtil.isEmpty(oldList)) {
				oldList = new ArrayList<PersonBaseDTO>();
			}
			oldList.add(personBaseDTO);
			map.put(orgId, oldList);
		}
		return map;
	}

	/**
	 * @description 递归把人员添加到组织树中
	 * @param list
	 * @param personMap
	 */
	private void addPersonsToOrgTree(OrgDTO orgDTO, HashMap<Integer, List<PersonBaseDTO>> personMap) {
		orgDTO.setPersons(personMap.get(orgDTO.getOrgId()));
		if(orgDTO.getChildren() != null && orgDTO.getChildren().size() > 0) {
			List<OrgDTO> childOrgs = orgDTO.getChildren();
			for (OrgDTO orgDTO2 : childOrgs) {
				addPersonsToOrgTree(orgDTO2,personMap);
			}
		}
	}

	/**
	 * @param orgCode
	 * @return
	 */
	public Org getByOrgCode(String orgCode) {
		return dao.getByOrgCode(orgCode);
	}

	/**
	 * @description 根据扩展唯一编码获取组织
	 * @param extCode
	 * @return
	 */
	public Org getByExtCode(String extCode) {
		return dao.getByExtCode(extCode);
	}

	 /**
	 * 根据机构层次代码查询机构（非部门）
	 * @param orgLevelCode
	 * @return
	 */
	 public List<Org> findOryByOrgLevelCode(String orgLevelCode){
		return dao.findOryByOrgLevelCode(orgLevelCode);
	 }
}