package com.loit.v1.modules.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.loit.common.json.AjaxJson;
import com.loit.v1.api.dto.PersonExtattrDTO;
import com.loit.v1.modules.activemq.MessageModel;
import com.loit.v1.modules.activemq.Producer;
import com.loit.v1.modules.utils.Model.IotUUIDUtils;
import org.apache.activemq.command.ActiveMQQueue;
import org.apache.activemq.filter.function.regexMatchFunction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.persistence.Page;
import com.loit.common.service.CrudService;
import com.loit.common.utils.CacheUtils;
import com.loit.common.spring.timeloit.utils.ModelMapperUtils;
import com.loit.common.spring.timeloit.utils.RedisConstants;
import com.loit.common.utils.StringReplaceUtil;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.utils.DictDataUtils;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.AreaCodeDao;
import com.loit.v1.modules.dao.OrgRoleDao;
import com.loit.v1.modules.dao.PersonBaseDao;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.entity.OrgPerson;
import com.loit.v1.modules.entity.OrgRole;
import com.loit.v1.modules.entity.PersonBase;
import com.loit.v1.modules.entity.PersonExtattr;
import com.loit.v1.modules.entity.PersonExtattrInst;
import com.loit.v1.modules.entity.RightShortcut;
import com.loit.v1.modules.utils.Model.PasswordCheckUtils;

import javax.jms.Destination;


/**
 * @ClassName: PersonBaseService
 * @Description: 人员基础信息表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:06
 * @version V1.0 
 **/
@Service
public class PersonBaseService extends CrudService<PersonBaseDao, PersonBase> {


	public static final Destination QUEUE_SYNC_PERSON_SAVE = new ActiveMQQueue(BusinessConstants.QUEUE_SYNC_PERSON_SAVE);


	@Autowired
	Producer producer;
	@Autowired
	AccoutInfoService accoutInfoService;
	@Autowired
	OrgPersonService orgPersonService;
	@Autowired
	PersonExtattrService personExtattrService;
	@Autowired
	PersonExtattrInstService personExtattrInstService;
	@Autowired
	OrgRoleDao orgRoleDao;
	@Autowired
	RightShortcutService rightShortcutService;
	@Autowired
	AreaCodeDao areaCodeDao;
	
	/**
	 * 获取组织下的人员列表
	 * @param page
	 * @param personBase
	 * @param orgId
	 * @return
	 */
	public Page<PersonBase> findPageByOrgId(Page<PersonBase> page, PersonBase personBase, String orgId) {
		personBase.setPage(page);
		List<PersonBase> list = dao.findListByOrgId(orgId,personBase.getDelFlag());
		page.setList(list);
		page.setCount(list.size());
		return page;
	}

	/**
	 * 人员信息新增
	 * @param personBaseDTO
	 */
	@Transactional(readOnly = false)
	public void addPerson(PersonBaseDTO personBaseDTO) {
		PersonBase personBase = ModelMapperUtils.map(personBaseDTO, PersonBase.class);
		AccoutInfo accoutInfo = ModelMapperUtils.map(personBaseDTO, AccoutInfo.class);
		//基本信息保存
		this.save(personBase, personBase.getPersonBaseId());
		// 扩展字段保存
//		List<PersonExtattr> personExtattrs = personBaseDTO.getPersonExtattr();
		List<PersonExtattrDTO> personExtattrs = personBaseDTO.getPersonExtattr();
		if(personExtattrs.size() > 0 ) {
//			for (PersonExtattr personExtattr : personExtattrs) {
			for (PersonExtattrDTO personExtattr : personExtattrs) {
				if(StringUtils.isNotBlank(personExtattr.getPersonAttrValue())) {
					PersonExtattrInst personExtattrInst = personExtattrInstService.getUnique(personBase.getPersonBaseId(),personExtattr.getPersonAttrId());
					if(personExtattrInst != null) {
						personExtattrInst.setPersonAttrId(personExtattr.getPersonAttrId());
						personExtattrInst.setPersonAttrValue(personExtattr.getPersonAttrValue());
						personExtattrInst.setPersonBaseId(personBase.getPersonBaseId());
						personExtattrInstService.save(personExtattrInst, personExtattrInst.getInstId());
					}else {
						PersonExtattrInst personExtattrInst1 = new PersonExtattrInst();
						personExtattrInst1.setPersonAttrId(personExtattr.getPersonAttrId());
						personExtattrInst1.setPersonAttrValue(personExtattr.getPersonAttrValue());
						personExtattrInst1.setPersonBaseId(personBase.getPersonBaseId());
						personExtattrInstService.save(personExtattrInst1, personExtattrInst1.getInstId());
					}
				}
			}
		}
		//密码加密
		String password = PasswordCheckUtils.encrypt(accoutInfo.getUnPassword());
		accoutInfo.setPassWord(password);
		//账号保存
		accoutInfo.setPersonBaseId(personBase.getPersonBaseId());
		//物联子平台用
		accoutInfo.setSid(IotUUIDUtils.getUUID());

		accoutInfoService.save(accoutInfo, accoutInfo.getAccountId());
		String key = new StringReplaceUtil(RedisConstants.PORTAL_USER_KEY).replace("userId",  accoutInfo.getUserId()).toString();
		CacheUtils.remove(key);
		//组织人员关系维护
		OrgPerson orgPerson =orgPersonService.getByAccountId(accoutInfo.getAccountId());
		if(orgPerson != null) {
			if(!personBaseDTO.getOrgId().equals(orgPerson.getOrgId())) {
				RightShortcut rightShortcut = new RightShortcut();
				rightShortcut.setFromType(BusinessConstants.SOURCE_FROM_ORG);
				rightShortcut.setFromId(orgPerson.getOrgId());
				rightShortcut.setAccountId(accoutInfo.getAccountId());
				rightShortcutService.deleteRightShortCut(rightShortcut);
			}
			orgPerson.setOrgId(personBaseDTO.getOrgId());
			orgPersonService.save(orgPerson, orgPerson.getOrgAccountId());
			
		}else {
			OrgPerson orgPerson1 = new OrgPerson();
			orgPerson1.setAccountId(accoutInfo.getAccountId());
			orgPerson1.setOrgId(personBaseDTO.getOrgId());
			orgPerson1.setIsbelong(BusinessConstants.IS);
			orgPersonService.save(orgPerson1, orgPerson1.getOrgAccountId());
		}
		//若有组织角色,则需要给人员添加相关角色
		OrgRole orgRole = new OrgRole();
		orgRole.setOrgId(personBaseDTO.getOrgId());
		List<OrgRole> orgRoles = orgRoleDao.findList(orgRole);
		if(orgRoles.size() > 0) {
			for (OrgRole orgRole2 : orgRoles) {
				RightShortcut rightShortcut = new RightShortcut();
				rightShortcut.setFromType(BusinessConstants.SOURCE_FROM_ORG);
				rightShortcut.setFromId(orgRole2.getOrgId());
				rightShortcut.setAccountId(accoutInfo.getAccountId());
				rightShortcut.setRoleId(orgRole2.getRoleId());
				rightShortcutService.save(rightShortcut, rightShortcut.getRscId());
			}
		}
		personBaseDTO.setAccountId(accoutInfo.getAccountId());
	}

	/**
	 * 获取人员详情
	 * @param accountId
	 * @return
	 */
	public PersonBaseDTO getDetailInfo(String accountId) {
		AccoutInfo accoutInfo = new AccoutInfo();
		accoutInfo.setAccountId(Integer.valueOf(accountId));
		PersonBaseDTO personBaseDTO = accoutInfoService.getInfo(accoutInfo);
//		PersonExtattr personExtattr = new PersonExtattr();
//		personExtattr.setPersonBaseId(personBaseDTO.getPersonBaseId());
//		List<PersonExtattr> personExtattrs = personExtattrService.findExtattrList(personExtattr);
		PersonExtattrDTO personExtattr = new PersonExtattrDTO();
		personExtattr.setPersonBaseId(personBaseDTO.getPersonBaseId());
		List<PersonExtattrDTO> personExtattrs = personExtattrService.findExtattrListDTO(personExtattr);
		personBaseDTO.setPersonExtattr(personExtattrs);
		DictDataUtils.transfor(personBaseDTO);
		return personBaseDTO;
	}

	/**
	 * 获取用户组内人员
	 * @param page
	 * @param personBase
	 * @return
	 */
	public Page<PersonBase> findGrpPersonPage(Page<PersonBase> page, PersonBase personBase) {
		personBase.setPage(page);
		List<PersonBase> list= dao.findGrpPersonPage(personBase);
		page.setList(list);
		return page;
	}

	/**
	 * 获取职务内人员
	 * @param personBase
	 * @return
	 */
	public Page<PersonBase> findPositionPersonPage(Page<PersonBase> page, PersonBase personBase) {
		personBase.setPage(page);
		List<PersonBase> list= dao.findPositionPersonPage(personBase);
		page.setList(list);
		return page;
	}

	/**
	 * 获取职级内人员
	 * @param personBase
	 * @return
	 */
	public Page<PersonBase> findRankPersonPage(Page<PersonBase> page, PersonBase personBase) {
		personBase.setPage(page);
		List<PersonBase> list= dao.findRankPersonPage(personBase);
		page.setList(list);
		return page;
	}

	/**
	 * 未添加人员列表-用户组
	 * @param page
	 * @param personBase
	 * @return
	 */
	public Page<PersonBase> findUnAddPersons(Page<PersonBase> page, PersonBase personBase) {
		personBase.setPage(page);
		List<PersonBase> list= dao.findUnAddPersons(personBase);
		page.setList(list);
		return page;
	}

	/**
	 *  未添加人员列表-职务
	 * @param personBase
	 * @return
	 */
	public Page<PersonBase> findPositionUnAddPersons(Page<PersonBase> page, PersonBase personBase) {
		personBase.setPage(page);
		List<PersonBase> list= dao.findPositionUnAddPersons(personBase);
		page.setList(list);
		return page;
	}

	/**
	 *  未添加人员列表-职级
	 * @param personBase
	 * @return
	 */
	public Page<PersonBase> findRankUnAddPersons(Page<PersonBase> page, PersonBase personBase) {
		personBase.setPage(page);
		List<PersonBase> list= dao.findRankUnAddPersons(personBase);
		page.setList(list);
		return page;
	}
	
	/**
	 * @Description: 查看某应用下某角色分配的人员的分页列表
	 * @param page
	 * @param personBase
	 * @return
	 */
	public Page<PersonBase> findPersonPageByRole(Page<PersonBase> page, PersonBase personBase) {
		personBase.setPage(page);
		List<PersonBase> list= dao.findPersonListByRole(personBase);
		page.setList(list);
		return page;
	}

	/**
	 * 查询列表数据
	 * @param appId
	 * @return
	 */
	public List<PersonBaseDTO> findListByAppId(int appId) {

		List<PersonBaseDTO> personBaseDTOList = dao.getAccountInfoByAppId(appId);

		if(personBaseDTOList.size() > 0){
			for(PersonBaseDTO personBaseDTO : personBaseDTOList){
				PersonExtattrDTO personExtattr = new PersonExtattrDTO();
				personExtattr.setPersonBaseId(personBaseDTO.getPersonBaseId());
				List<PersonExtattrDTO> personExtattrs = personExtattrService.findExtattrListDTO(personExtattr);
				personBaseDTO.setPersonExtattr(personExtattrs);
			}
		}
		return personBaseDTOList;
	}

	/**
	 * 账号唯一性验证
	 * @param accoutInfo
	 * @return 
	 */
	public boolean validateUserId(AccoutInfo accoutInfo) {
		AccoutInfo accountByUserId = accoutInfoService.getUniqueAccount(accoutInfo);
		if(accountByUserId != null) {
			return false;
		}
		return true;
	}

	/**
	 *  根据ID列表获取多条数据
	 * @param ids
	 * @return
	 */
	public List<PersonBase> findByIdList(List<String> ids){
		return dao.findByIdList(ids);
	}

	/**
	 * 验证人员名称唯一性
	 * @param personBase
	 * @return
	 */
	public boolean validatePersonName(PersonBase personBase) {
		List<PersonBase> list =dao.findUniquePersonName(personBase);
		if(list.size() > 0) {
			return false;
		}
		return true;
	}

	/**
	 * @description 根据用户id获取该用户所在组织下的人员列表
	 * @param page
	 * @param personBase
	 * @return
	 */
	public Page<PersonBase> findPageByAccountId(Page<PersonBase> page, PersonBase personBase) {
		personBase.setPage(page);
		List<PersonBase> list = dao.findPageByAccountId(personBase.getAccountId());
		page.setList(list);
		page.setCount(list.size());
		return page;
	}

	/**
	 * @description 根据组织编码获取组织及下属组织所有人
	 * @param appId
	 * @param integer
	 * @return
	 */
	public List<PersonBaseDTO> findPersonsByOrgCode(Integer appId) {
		return dao.findPersonsByOrgCode(appId);
	}

	/**
	 * 获取所有的账号信息
	 * @return
	 */
	public List<AccoutInfo> getAccoutInfoList() {
		return accoutInfoService.getAccoutInfoList();
	}

	/**
	 * 根据登录账号列表获取账号信息列表
	 * @param userIds
	 * @return
	 */
    public List<PersonBaseDTO> getAccountByUserIds(List<String> userIds) {
		return accoutInfoService.getAccountByUserIds(userIds, null);
    }

	/**
	 * @desctiption 根据唯一扩展编码获取人员
	 * @param userId
	 * @return
	 */
	public PersonBase getByExtCode(String extCode) {
		return dao.getByExtCode(extCode);
	}

	/**
	 * 根据人员名称获取人员信息
	 * @param personName
	 * @return
	 */
    public List<PersonBase> getPersonByPersonName(String personName) {
		return accoutInfoService.getPersonByPersonName(personName);
    }
}