package com.loit.v1.modules.service;

import com.loit.common.service.CrudService;
import com.loit.common.utils.ListUtil;
import com.loit.common.utils.StringUtils;
import com.loit.v1.modules.dao.RankDao;
import com.loit.v1.modules.entity.Rank;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;


/**
 * 职级表 [门户平台]服务实现.
 * Created by DEMON on 2020-03-05.
 **/
@Service
@Transactional(readOnly = true)
public class RankService extends CrudService<RankDao, Rank> {

    /**
     * 判断职级编码是否重复
     *
     * @param rankCode 职级编码,空返回false
     * @return true是，false否
     */
    public Boolean isRankCodeRepeat(String rankCode) {
        if(StringUtils.isEmpty(rankCode)){
            return false;
        }

        Rank _rankParams = new Rank();
        _rankParams.setRankCode(rankCode);
        List<Rank> list = this.findList(_rankParams);
        if (ListUtil.isNotEmpty(list)) {
            return true;
        }

        return false;
    }

    /**
     * 判断职级编码是否重复
     * 放过验证自己本身code重复-一般用在修改方法中
     *
     * @param rankCode 职级编码,空返回false
     * @return true是，false否
     */
    public Boolean isRankCodeRepeat(String rankCode, Integer rankId){
        if(StringUtils.isEmpty(rankCode)){
            return false;
        }

        Rank _rankParams = new Rank();
        _rankParams.setRankCode(rankCode);
        List<Rank> list = this.findList(_rankParams);
        if (ListUtil.isNotEmpty(list)) {
            // 如果重复的code是自己本身， 测不算重复，一般用在修改方法里
            if(rankId != null && list.size() == 1 && list.get(0).getRankId().equals(rankId)){
                return false;
            }
            return true;
        }

        return false;
    }

}