package com.loit.v1.modules.service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.loit.common.persistence.Page;
import com.loit.common.service.CrudService;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.modules.dao.AppResourceRelationDao;
import com.loit.v1.modules.dao.ApplicationManageDao;
import com.loit.v1.modules.dao.ResourceManageDao;
import com.loit.v1.modules.entity.AppResourceRelation;
import com.loit.v1.modules.entity.ApplicationManage;
import com.loit.v1.modules.entity.ResourceManage;
/**
 * @description 资源管理Service
 * @author xutao
 * @date 2019-10-26
 */
@Service
public class ResourceManageService extends CrudService<ResourceManageDao, ResourceManage> {
	
	@Autowired
	private AppResourceRelationDao appResourceRelationDao;
	@Autowired
	private ApplicationManageDao applicationManageDao;
	/**
	 * @description 保存数据（插入）
	 * @param entity
	 */
	@Transactional(readOnly = false)
	public void insert(ResourceManage entity) {
		entity.preInsert();
		if(entity.getResourceParentId() == null) {
			entity.setResourceParentId(0);//无父级资源的父ID为0
		}
		if(entity.getAppId() == null) {
			entity.setAppId(0);//无资源的应用ID为0
		}
		dao.insert(entity);	
		//如果新增时选择了应用的话，说明这个资源隶属于某个应用，需要维护应用和资源关系表
		if(entity.getAppId() != null && entity.getAppId() != 0) {
			AppResourceRelation appResourceRelation = new AppResourceRelation();
			appResourceRelation.preInsert();
			appResourceRelation.setAppId(entity.getAppId());
			appResourceRelation.setResId(entity.getResourceId());
			appResourceRelation.setStatus("1");//隶属
			appResourceRelationDao.insert(appResourceRelation);
		}			
	}
	
	/**
	 * @description 保存数据（更新）
	 * @param entity
	 */
	@Transactional(readOnly = false)
	public void update(ResourceManage entity) {
		entity.preUpdate();
		if(entity.getAppId() == null) {
			entity.setAppId(0);//无资源的应用ID为0
		}
		dao.update(entity);	
		//如果修改之后的资源所属应用ID不为空
		if(entity.getAppId() != null && entity.getAppId() != 0) {			
			AppResourceRelation appResourceRelation = new AppResourceRelation();			
			appResourceRelation.setAppId(entity.getAppId());
			appResourceRelation.setResId(entity.getResourceId());
			appResourceRelation.setStatus("1");//隶属
			AppResourceRelation appResourceRelation1 = appResourceRelationDao.get(appResourceRelation);
			if(appResourceRelation1!=null) {
				appResourceRelationDao.delete(appResourceRelation1);
			}
			appResourceRelation.preInsert();
			appResourceRelationDao.insert(appResourceRelation);
		}
	}
	
	/**
	 * @description 批量删除资源数据（物理删除）
	 * @param ids 资源ID，多ID时，以逗号进行分割
	 */
	@Transactional(readOnly = false)
	public void deleteAll(String ids,ResourceManage entity) {
		String idArray[] =ids.split(",");
        for(String id : idArray){
            entity = get(id);
            dao.delete(entity);
            ResourceManage manage = new ResourceManage();
            manage.setResourceParentId(Integer.valueOf(id));
            List<ResourceManage> list = dao.findList(manage);
            if(list != null && list.size() > 0) {
            	for (ResourceManage r : list) {
            		dao.delete(r);
				}
            }
        }
	}
	
	/**
	 * @description 根据应用ID获取所分配资源树状结构 - 菜单增加界面,普通授权，高级授权用
	 * @param appId 应用ID
	 * @param typeId 资源类别ID
	 * @return
	 */
	public List<ResourceManage> getResourceTreeList(Integer appId,Integer typeId){
		List<ResourceManage> list = dao.getResourceListByAppId(appId,typeId);
		if(list != null && list.size() > 0) {
			for(ResourceManage res : list) {
				res.setChildren(dao.getUseResourceTreeList(res));
			}
		}
		return list;
	}

	/**
	 * @description 查询分页数据
	 * @param page 分页对象
	 * @param entity
	 * @return
	 */
	/*
	 * public Page<ResourceManage> findPage(Page<ResourceManage> page,
	 * ResourceManage entity) { if( null == entity.getResourceId()) {
	 * entity.setResourceId(0); } entity.setPage(page);
	 * page.setList(dao.getAllResourceTreeList(entity)); return page; }
	 */
	
	/**
	 * @description 查询分页数据
	 * @param page 分页对象
	 * @param entity
	 * @return
	 */
	public Page<ResourceManage> findPage(Page<ResourceManage> page, ResourceManage entity) {	
		entity.setPage(page);
		page.setList(dao.getResourceTreeList(entity));
		return page;
	}
	
	/**
	 * @description 按应用纬度获取资源的父级列表
	 * @return
	 */
	public List<Map<String, Object>> treeData(int appId) {
		List<Map<String, Object>> mapList = Lists.newArrayList();
		//获取所有应用列表
		ApplicationManage applicationManage = new ApplicationManage();
		List<ApplicationManage> list = applicationManageDao.findList(applicationManage);
		//定义未分配资源的应用
		applicationManage.setAppId(0);
		applicationManage.setAppName("未分配资源的应用");
		list.add(applicationManage);	
		for (int i=0; i<list.size(); i++){			
			ApplicationManage a = list.get(i);
			//根据应用ID获取所有的父级资源
			ResourceManage resourceManage = new ResourceManage();
			resourceManage.setAppId(a.getAppId());
			resourceManage.setResourceParentId(0);
			resourceManage.setResourceStatus("1");
			List<ResourceManage> rList = dao.findList(resourceManage);
			Map<String, Object> map = Maps.newHashMap();
			//
			if(rList != null && rList.size() > 0) {
				map.put("appId", a.getAppId());//应用的ID
				map.put("resourceName", a.getAppName());//应用的名称，为了方便前台展示，命名为resourceName
				map.put("children", rList);//应用下的所有父级资源
				mapList.add(map);
			}
		}		
		return mapList;
	}

	/**
	 * TODO DEMON 待实现
	 * 根据用户id和应用id获取用户权限范围内的所有资源列表
	 * @param userId
	 * @return
	 */
	public List<ResourceManage> getResourceManagesByUserId(String userId, Integer appId){
		return new ArrayList<>();
	}

	/**
	 * @description 通过资源id获取拥有该资源的角色
	 * @param resourceId
	 * @return
	 */
	public List<Role> getRolesByResourceId(Integer resourceId) {
		return dao.getRolesByResourceId(resourceId);
	}
}