package com.loit.v1.modules.service;

import com.loit.common.service.CrudService;
import com.loit.common.utils.CacheUtils;
import com.loit.common.utils.CacheUtils;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Role;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.ResourceManageDao;
import com.loit.v1.modules.dao.ResourceOperationDao;
import com.loit.v1.modules.dao.ResourceTypeOperationDao;
import com.loit.v1.modules.entity.ResourceManage;
import com.loit.v1.modules.entity.ResourceOperation;
import com.loit.v1.modules.entity.ResourceTypeOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

/**  
* @ClassName: ResourceOperationService  
* @Description:   
* @author zhaokz  
* @date 2020年4月13日 上午9:56:01 
* @version V1.0  
*/
@Service
@Transactional(readOnly = true)
public class ResourceOperationService extends CrudService<ResourceOperationDao, ResourceOperation> {
	
	@Autowired
	private ResourceManageDao resourceManageDao;
	@Autowired
	private ResourceTypeOperationDao resourceTypeOperationDao;
	@Autowired
	private RoleService roleService;

	/**
	 * @description 保存资源操作
	 * @param resourceOperation
	 */
	@Transactional(readOnly = false)
	public void save(ResourceOperation resourceOperation) {
		ResourceManage resourceManage = resourceManageDao.get(resourceOperation.getResId());
		ResourceTypeOperation resourceTypeOperation = resourceTypeOperationDao.get(resourceOperation.getActId());
		resourceOperation.setActCode(resourceTypeOperation.getActCode());
		resourceOperation.setResCode(resourceManage.getResourceCode());
		this.insert(resourceOperation);

	}
	/**
	 * @desc 根据actUrl路径获取当前用户是否具有对本资源的操作权限
	 * @param actUrl
	 * @author yangwenbin
	 * @date
	 */
	public boolean checkPermission(String userId,String actUrl,String method){
		User user = UserUtils.get(userId);
		if(user == null || StringUtils.isEmpty(actUrl)){
			return false;
		}
		//查询用户是否拥有才资源的权限
		List<Role> roleList = user.getRoleList();
		List<Map<String,Object>> operationAllList = new ArrayList<Map<String,Object>>();
		for(Role role:roleList){
			//如果是超级管理员则放行
			logger.info("用户:"+user.getUserId()+" 角色："+role.getRoleId());
			if(role.getRoleId().compareTo(1) == 0){
				return true;
			}else{
				List<Map<String,Object>> operationList = (List<Map<String,Object>>) CacheUtils.get(BusinessConstants.ROLE_OPERATOR_CACHE+role.getRoleId());
				if(operationList == null || operationList.size()<=0){
					operationList = this.dao.getOperationListByRole(role.getRoleId());
					if(operationList != null && operationList.size() > 0){
						CacheUtils.set(BusinessConstants.ROLE_OPERATOR_CACHE + role.getRoleId(), operationList,0L);
					}
				}
				operationAllList.addAll(operationList);
			}
		}
		//去除最后一个/
		String lastPath = actUrl.substring(0,actUrl.lastIndexOf("/"));
		//如果存在则验证通过
		for(Map<String,Object> map:operationAllList){
			String existActUrl = map.get("act_url").toString();
			String existReqMode = map.get("req_mode").toString();

			//如果有包含/** 转换成正则表达
			if(existActUrl.indexOf("/**")>-1 && existReqMode.equals(method)){
				String lastExistUrl = existActUrl.replaceAll("/\\*\\*","");
				if(lastPath.equals(lastExistUrl)){
					String pattern = existActUrl.replaceAll("\\*\\*",".*");
					return Pattern.matches(pattern, actUrl);
				}

			}
			if(existActUrl.equals(actUrl) &&  existReqMode.equals(method)){
				return true;
			}

		}
		return false;

	}
	/**
	 * @desc 重置资源权限缓存
	 * @param roleIdList 角色列表
	 * @author yangwenbin
	 * @date
	 */
	public void resetOperationCache(List<Role> roleIdList) {
		for(Role role:roleIdList){
			Integer roleId = role.getRoleId();
			//清除角色资源操作缓存
			CacheUtils.remove(BusinessConstants.ROLE_OPERATOR_CACHE + roleId);
			//非超级管理员才需要缓存,超级管理员因为放过所有权限验证所以无需缓存
			if(roleId.compareTo(1)!= 0 ) {
				List<Map<String,Object>> operationList = this.dao.getOperationListByRole(roleId);
				CacheUtils.set(BusinessConstants.ROLE_OPERATOR_CACHE + roleId, operationList,0L);
			}
		}

	}

}
