package com.loit.v1.modules.service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.loit.common.service.CrudService;
import com.loit.common.utils.StringUtils;
import com.loit.modules.sys.entity.Role;
import com.loit.modules.sys.entity.User;
import com.loit.modules.sys.utils.UserUtils;
import com.loit.v1.api.dto.RightDTO;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.ResourceManageDao;
import com.loit.v1.modules.dao.ResourceTypeDao;
import com.loit.v1.modules.dao.RightShortcutDao;
import com.loit.v1.modules.entity.ApplicationManage;
import com.loit.v1.modules.entity.MenuManage;
import com.loit.v1.modules.entity.ResourceManage;
import com.loit.v1.modules.entity.ResourceType;
import com.loit.v1.modules.entity.RightShortcut;
import com.loit.v1.modules.entity.RoleResMenu;



/**
 * @ClassName: RightShortcutService
 * @Description: 权限快捷存取表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:07
 * @version V1.0 
 **/
@Service
public class RightShortcutService extends CrudService<RightShortcutDao, RightShortcut> {
	
	@Autowired
	private ApplicationManageService applicationManageService;
	@Autowired
	private ResourceManageDao resourceManageDao;
	@Autowired
	private ResourceTypeDao resourceTypeDao;

	/**
	* @Description: 根据当前人员获取应用列表-下拉框用
	* @author: gzx
	* @date: 2019年11月8日 下午9:06:36 
	* @param @return 参数 
	* @return List<Map<String,Object>> 返回类型 
	* @throws
	 */
	public List<Map<String,Object>> listMapAppsByAccountId(Integer appId,String type){
		//获取当前用户
		User user = UserUtils.getUser();		
		List<Map<String,Object>> appList = new ArrayList<Map<String,Object>>();
		//超级管理员返回所有应用
		if(StringUtils.equals(user.getAccountType(),BusinessConstants.ACCOUNT_TYPE_SUPERADMIN)) {
			List<ApplicationManage> list = new ArrayList<ApplicationManage>();
			if("1".equals(type)) {
				//获取嵌入式展示的应用列表（接入系统门户中展示）
				list = applicationManageService.findEmbedAppList(new ApplicationManage());
			}else {
				//获取所有应用列表
				list = applicationManageService.findList(new ApplicationManage());
			}
			
			if(list != null && list.size() > 0) {
				for(ApplicationManage app : list) {
					Map<String,Object> map = new HashMap<String, Object>();
					map.put("appId", app.getAppId());
					map.put("appName", app.getAppName());
					map.put("appUrl", app.getAppUrl());
					appList.add(map);
				}
			}
//		}else if(StringUtils.equals(user.getAccountType(),DictDataEnum.ACCOUNT_TYPE_APPADMIN.getCode())){
//			//应用管理员返回所管理的应用  如果appId为1(集成门户)，代表应用管理员从集成门户进入配置界面，sql将只集成门户排除
//			appList = dao.listMapAppsByAccountId(user.getAccountId(),appId);		
		}else {
			//return null;
			appList = dao.listMapAppsByAccountId(user.getAccountId(),appId,type);
		}
		return appList;
	}
	
	/**
	* @Description: 获取当前用户所拥有的应用列表
	* @author: xutao
	* @date: 2019年11月8日 下午9:06:36 
	* @param @return 参数 
	* @return List<Map<String,Object>> 返回类型 
	*/
	public List<Map<String,Object>> getAppList(RoleResMenu roleResMenu){
		List<Map<String,Object>> appList = new ArrayList<Map<String,Object>>();
		//超级管理员返回所有应用列表
		if(StringUtils.equals(roleResMenu.getAccountType(),BusinessConstants.ACCOUNT_TYPE_SUPERADMIN)) {
			// Demon modify by 20200509: 查询已显示的应用列表
			List<ApplicationManage> list = applicationManageService.findList(new ApplicationManage().showFlag("1"));
			if(list != null && list.size() > 0) {
				for(ApplicationManage app : list) {
					Map<String,Object> map = new HashMap<String, Object>();
					map.put("appId", app.getAppId());
					map.put("appName", app.getAppName());
					map.put("appUrl", app.getAppUrl());
					appList.add(map);
				}
			}
		}else {
			//获取当前用户拥有权限的应用列表
			appList = dao.getAppList(roleResMenu.getAccountId());
		}
		return appList;
	}

	/**
	 * 根据来源id删除权限
	 */
	public void deleteByFrom(String fromId, String fromType) {
		dao.deleteByFrom(fromId,fromType);
	}

	/**
	 * @param rightShortcut
	 */
	public void deleteRightShortCut(RightShortcut rightShortcut) {
		dao.deleteRightShortCut(rightShortcut);
	}

	/**
	 * 删除人员权限
	 * @param rightDTO
	 */
	public void deletePersonRight(RightDTO rightDTO, String fromType) {
		dao.deletePersonRight(rightDTO.getAccountId().toString(),rightDTO.getAppId().toString(),fromType);
	}

	/**
	 * @description 根据账号id获取角色权限
	 * @param accountId
	 * @return 
	 */
	public List<RightShortcut> getRightByAccountId(Integer accountId) {
		return dao.getRightByAccountId(accountId);
	}

	/**
	 * @param accountId
	 */
	public void deleteByAccountId(Integer accountId) {
		dao.deleteByAccountId(accountId.toString());
		
	}

	/**
	 * @description 根据当前用户获取拥有的数据权限
	 * @return
	 */
	public List<ResourceManage> getCurrentUserDataResList() {
		User user = UserUtils.getUser();
		List<Role> roleList = user.getRoleList();
		List<ResourceManage> list = new ArrayList<ResourceManage>();
		if(roleList != null && roleList.size() > 0) {
			ResourceType resourceType = resourceTypeDao.getByCode("data");
			for (Role role : roleList) {
				List<ResourceManage> dataResList = resourceManageDao.getRoleDataRes(role.getRoleId(),resourceType.getTypeId());
				list.addAll(dataResList);
			}
		}
		List<ResourceManage> collect = list.stream().distinct().collect(Collectors.toList());
		return collect.stream().sorted(Comparator.comparing(ResourceManage::getSerialIndex)).collect(Collectors.toList());
	}
}