package com.loit.v1.modules.service;

import com.loit.common.service.CrudService;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.modules.dao.RoleRightDao;
import com.loit.v1.modules.entity.RoleRight;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;



/**
 * @ClassName: RoleRightService
 * @Description: 角色权限对应表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:07
 * @version V1.0 
 **/
@Service
public class RoleRightService extends CrudService<RoleRightDao, RoleRight> {
	
	@Autowired
	private RoleService roleService;
	@Autowired
	private MenuManageService menuManageService;
	@Autowired
	private ResourceOperationService resourceOperationService;
	/**
	 * @param resType 
	* @Description: 配置角色资源权限
	* @author: gzx
	* @date: 2019年11月12日 下午2:17:21 
	* @param @param type   1:增加  2：删除
	* @param @param roleId
	* @param @param resId 参数 
	* @return void 返回类型 
	* @throws
	 */
	@Transactional(readOnly = false)
	public void dealRoleRight(Integer roleId,String resIds, Integer resType) {
		//批量删除
		dao.deleteByRoleIdAndResType(roleId,resType);
		if (resIds != null && resIds.length() >0) {
			String[] resIdArr = resIds.split(",");
			List<RoleRight> roleRightList = new ArrayList<RoleRight>();
			for(int i=0,len=resIdArr.length;i<len;i++) {
				RoleRight roleRight = new RoleRight();
				roleRight.setRoleId(roleId);
				roleRight.setResId(Integer.valueOf(resIdArr[i]));
				roleRight.setCreateTime(new Date());
				roleRightList.add(roleRight);
			}
			//批量插入
			dao.insertBatch(roleRightList);
			//更新redis菜单缓存
			new Thread(new Runnable() {
				@Override
				public void run() {
					menuManageService.resetMenuCache();
					//根据受到此更改影响的角色更新资源操作缓存
					Integer tmpRoleId = roleId;
					List<Role> roleList =new ArrayList<Role>();
					Role role = new Role();
					role.setRoleId(tmpRoleId);
					roleList.add(role);
					resourceOperationService.resetOperationCache(roleList);
				}
			}).start();
		}		
	}
	
	/**
	* @Description: 获取角色所分配的资源，并将资源ID组装成list返回给前端，供前端角色授权时选中
	* @author: gzx
	* @date: 2019年11月13日 下午3:35:48 
	* @param @param roleId
	* @param @param typeId
	* @param @return 参数 
	* @return List<Integer> 返回类型 
	* @throws
	 */
	public List<Integer> getRoleResIds(Integer roleId,Integer typeId){
		List<Integer> list = new ArrayList<Integer>();
		RoleRight roleRight = new RoleRight();
		roleRight.setRoleId(roleId);
		roleRight.setTypeId(typeId);
		List<RoleRight> roleRightList = this.findList(roleRight);
		if (roleRightList != null && roleRightList.size() >0) {
			for(RoleRight temp : roleRightList) {
				list.add(temp.getResId());
			}
		}
		return list;
	}
}