package com.loit.v1.modules.service;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.service.CrudService;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.api.dto.RoleDTO;
import com.loit.v1.modules.dao.AccountRoleDao;
import com.loit.v1.modules.dao.ActRightDao;
import com.loit.v1.modules.dao.GrpRoleDao;
import com.loit.v1.modules.dao.OrgRoleDao;
import com.loit.v1.modules.dao.RightShortcutDao;
import com.loit.v1.modules.dao.RoleDao;
import com.loit.v1.modules.dao.RoleRightDao;



/**
 * @ClassName: RoleService
 * @Description: 角色表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:07
 * @version V1.0 
 **/
@Service
public class RoleService extends CrudService<RoleDao, Role> {
	@Autowired
	private RoleRightDao roleRightDao;
	@Autowired
	private ActRightDao actRightDao;
	@Autowired
	private OrgRoleDao orgRoleDao;
	@Autowired
	private AccountRoleDao accountRoleDao;
	@Autowired
	private GrpRoleDao grpRoleDao;
	@Autowired
	private RightShortcutDao rightShortcutDao;

	/**
	* @Description: 批量删除角色 TODO 角色删除时，相关关系表需要删除
	* @author: gzx
	* @date: 2019年11月9日 上午11:35:25 
	* @param @param roleIds 参数 
	* @return void 返回类型 
	* @throws
	 */
	@Transactional(readOnly = false)
	public void deleteRoleByIds(String roleIds) {
		String[] roleArr = roleIds.split(",");
		List<Role> roleList = new ArrayList<Role>();
		for(int i=0,len=roleArr.length;i<len;i++) {
			roleList.add(new Role(Integer.valueOf(roleArr[i])));
		}
		if(roleList!=null && roleList.size() > 0 ) {
			dao.deleteBatch(roleList);
			roleRightDao.deleteBatchByRoleIds(roleList);
			actRightDao.deleteBatchByRoleIds(roleList);
			orgRoleDao.deleteBatchByRoleIds(roleList);
			accountRoleDao.deleteBatchByRoleIds(roleList);
			grpRoleDao.deleteBatchByRoleIds(roleList);
			rightShortcutDao.deleteBatchByRoleIds(roleList);
		}

	}

	/**
	 * 通过账号id查询角色列表
	 * @param accountId
	 * @return
	 */
	public List<Role> getRolesByAccountId(String accountId) {
		return dao.getRolesByAccountId(accountId);
	}
	
	/**
	* @Description: 检查角色名称的唯一性
	* @author: gzx
	* @date: 2019年11月20日 上午10:43:03 
	* @param @param roleId
	* @param @param roleName
	* @param @return 参数 
	* @return boolean 返回类型   不唯一，返回false；唯一返回true
	* @throws
	 */
	public boolean checkNameUnique(Integer roleId,String roleName) {
		int count = dao.checkNameUnique(roleName, roleId);
		if(count>0) {
			return false;
		}
		return true;
	}
	
	/**
	* @Description: 检查角色编码的唯一性
	* @author: gzx
	* @date: 2019年11月20日 上午10:43:41 
	* @param @param roleId
	* @param @param roleCode
	* @param @return 参数 
	* @return boolean 返回类型    不唯一，返回false；唯一返回true
	* @throws
	 */
	public boolean checkCodeUnique(Integer roleId,String roleCode) {
		int count = dao.checkCodeUnique(roleCode, roleId);
		if(count>0) {
			return false;
		}
		return true;
	}
	
	/**
	 * 查询列表数据
	 * @param appId
	 * @return
	 */
	public List<RoleDTO> findListByAppId(int appId) {
		List<RoleDTO> list = dao.findListByAppId(appId);
		return list;
	}

	/**
	 * 获取用户组角色id数组
	 * @param grpId
	 * @return
	 */
	public List<Integer> getRolesByGrpId(String grpId) {
		List<Integer> list = new ArrayList<Integer>();
		List<Role> roles = dao.getRolesByGrpId(grpId);
		if(roles.size() > 0) {
			for (Role role : roles) {
				list.add(role.getRoleId());
			}
		}
		return list;
	}

	/**
	 *  获取人员角色id数组
	 * @param accountId
	 * @return
	 */
	public List<Integer> getRoleIdsByAccountId(String accountId) {
		List<Integer> list = new ArrayList<Integer>();
		List<Role> roles = dao.getRolesByAccountId(accountId);
		if(roles.size() > 0) {
			for (Role role : roles) {
				list.add(role.getRoleId());
			}
		}
		return list;
	}
	/**
	 *  根据ID列表获取多条数据
	 * @param ids
	 * @return
	 */
    public List<Role> findByIdList(List<String> ids){
		return dao.findByIdList(ids);
	}

	/**
	 *  根据Id查询
	 * @param roleId
	 * @return
	 */
	public RoleDTO getByRoleId(long roleId){
		return dao.getByRoleId(roleId);
	}

	/**
	 * @description 通过角色id获取拥有该角色的人员
	 * @param roleId
	 * @return
	 */
	public List<Integer> getAccountsByRoleId(Integer roleId) {
		return dao.getAccountsByRoleId(roleId);
	}
	
	
	/**
	 * @description 根据用户id获取角色列表
	 * @return
	 */
    public List<Role> findByAccountId(Integer accountId){
		return dao.findByAccountId(accountId);
	}

	/**
	 * 根据角色编码获取角色
	 * @param roleCode
	 * @return
	 */
	public Role getRoleByRoleCode(String roleCode){
    	return dao.getRoleByRoleCode(roleCode);
	}
}