package com.loit.v1.modules.service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONObject;
import com.loit.common.json.AjaxJson;
import com.loit.common.persistence.Page;
import com.loit.common.service.CrudService;
import com.loit.v1.api.dto.OrgDTO;
import com.loit.v1.api.dto.PersonBaseDTO;
import com.loit.v1.api.dto.RoleDTO;
import com.loit.v1.api.dto.SynLogDTO;
import com.loit.v1.modules.dao.SynLogDao;
import com.loit.v1.modules.entity.SynLog;
import com.loit.v1.modules.entity.SynTarget;
import com.loit.v1.modules.utils.Model.HttpUtil;


/**
 * @author zhangyl
 * @version V1.0
 * @ClassName: SynLogService
 * @Description: 同步日志记录表
 * @date 2019-11-23 15:24:07
 **/
@Service
public class SynLogService extends CrudService<SynLogDao, SynLog> {

    @Autowired
    private RoleService roleService;
    @Autowired
    private OrgService orgService;
    @Autowired
    private PersonBaseService personBaseService;
    @Autowired
    private SynLogService synLogService;
    @Autowired
    private SynTargetService synTargetService;

    /**
     * 手动同步
     *
     * @param logId
     * @return AjaxJson
     */
    @Transactional(readOnly = false)
    public AjaxJson manualSync(String logId) {

        AjaxJson ajaxJson = new AjaxJson();
        Map<String, Object> syncMap = new HashMap<>();
            SynLog synLog = dao.get(logId);
        if (synLog == null) {
            return AjaxJson.returnExceptionInfo("数据异常，请联系管理员");
        }
        int targetId = synLog.getTargetId();
        if (synLog.getSynStatus().equals("1")) {
            return AjaxJson.returnExceptionInfo("该条数据已同步成功，不需要手动同步！");
        }
        SynTarget synTarget = synTargetService.get(targetId);
        //同步组织
        if (synTarget.getObjectType().equals("1")) {
            OrgDTO org = orgService.getByOrgId(synLog.getObjectId());
            syncMap.put("org", org);
            boolean falg = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), JSONObject.toJSONString(ajaxJson));
            if (!falg) {
                synLog.setSynStatus("2"); //更新状态
                synLogService.update(synLog);//更新日志记录
                return AjaxJson.returnExceptionInfo("手动同步数据【组织】失败");
            }
            synLog.setSynStatus("1");//更新状态
            synLogService.update(synLog);//更新日志记录

        }
        //同步人员
        else if (synTarget.getObjectType().equals("2")) {
            PersonBaseDTO personBase = personBaseService.getDetailInfo(String.valueOf(synLog.getObjectId()));
            syncMap.put("personBase", personBase);
            //更新日志记录
            boolean falg = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), JSONObject.toJSONString(ajaxJson));
            if (!falg) {
                synLog.setSynStatus("2"); //更新状态
                synLogService.update(synLog);//更新日志记录
                return AjaxJson.returnExceptionInfo("手动同步数据【人员】失败");
            }
            synLog.setSynStatus("1");//更新状态
            synLogService.update(synLog);//插入日志

        }
        //同步角色
        else if (synTarget.getObjectType().equals("3")) {
            RoleDTO role = roleService.getByRoleId(synLog.getObjectId());
            syncMap.put("role", role);
            //更新日志记录
            boolean falg = HttpUtil.doPost4Boolean(synTarget.getServiceUrl(), JSONObject.toJSONString(ajaxJson));
            if (!falg) {
                synLog.setSynStatus("2"); //更新状态
                synLogService.update(synLog);//更新日志记录
                return AjaxJson.returnExceptionInfo("手动同步数据【角色】失败");
            }
            synLog.setSynStatus("1");//更新状态
            synLogService.update(synLog);//更新日志记录
        }
        return ajaxJson;
    }


    /**
     * 查询失败日志分页数据
     *
     * @param page   分页对象
     * @param entity
     * @return
     */
    public Page<SynLog> findPageFailLog(Page<SynLog> page, SynLog entity) {
        entity.setPage(page);
        page.setList(findListFailLog(entity));
        return page;
    }

    /**
     * 查询数据同步失败日志列表数据
     *
     * @param entity
     * @return
     */
    public List<SynLog> findListFailLog(SynLog entity) {
        List<SynLog> list = dao.findListFailLog(entity);
        return list;
    }

    public List<SynLog> findListByTargetId(long targetId){
        List<SynLog> list = dao.findListByTargetId(targetId);
        return list;
    }

    /**
     * 插入数据 （用于实时同步消息时日志的插入方法）
     * @param entity
     */
    @Transactional(readOnly = false)
    public void insertLog(SynLogDTO entity) {
        dao.insertLog(entity);
    }
}