package com.loit.v1.modules.service;

import com.loit.common.utils.RedisUtil;
import com.loit.modules.sys.entity.Role;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.entity.ApplicationManage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.loit.common.service.CrudService;
import com.loit.v1.modules.dao.ThemeManageDao;
import com.loit.v1.modules.entity.ThemeManage;

import java.util.List;

/**
 * @description 应用主题管理Service
 * @author xutao
 * @date 2019-10-18
 */
@Service
public class ThemeManageService extends CrudService<ThemeManageDao, ThemeManage> {
	@Autowired
	private ApplicationManageService applicationManageService;
	/**
	 * @description 系统的主题皮肤及导航栏的位置设置
	 * @param entity
	 * @return
	 */
	@Transactional(readOnly = false)
	public void themeSet(ThemeManage entity) {
		entity.preUpdate();
		dao.themeSet(entity);
	}
	/**
	 * @description 根据应用Id获取系统主题 为了切面做缓存
	 * @param appId 应用ID
	 * @return
	 */
	public ThemeManage getThemeByAppId(String appId){
		return this.get(appId);
	}

	/**
	 * @description 根据应用id重置系统主题缓存
	 * @param appId
	 */
	public void resetApplicationThemeCache(Integer appId) {
		RedisUtil.remove(BusinessConstants.APPLICATION_THEME_CACHE+appId);
		ApplicationManage applicationManage = applicationManageService.getApplicationManage(appId,"");
		RedisUtil.set(BusinessConstants.APPLICATION_THEME_CACHE+appId,applicationManage);

	}
}
