package com.loit.v1.modules.service;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONObject;
import com.loit.common.json.AjaxJson;
import com.loit.common.service.CrudService;
import com.loit.common.spring.timeloit.utils.LoitStatusCode;
import com.loit.common.utils.StringUtils;
import com.loit.v1.modules.constant.BusinessConstants;
import com.loit.v1.modules.dao.AccoutInfoDao;
import com.loit.v1.modules.dao.GrpRoleDao;
import com.loit.v1.modules.dao.PersonBaseDao;
import com.loit.v1.modules.dao.UserGroupDao;
import com.loit.v1.modules.entity.AccoutInfo;
import com.loit.v1.modules.entity.GrpFrom;
import com.loit.v1.modules.entity.GrpRole;
import com.loit.v1.modules.entity.GrpUser;
import com.loit.v1.modules.entity.PersonBase;
import com.loit.v1.modules.entity.RightShortcut;
import com.loit.v1.modules.entity.UserGroup;



/**
 * @ClassName: UserGroupService
 * @Description: 用户组表
 * @author  zhaokz 
 * @date  2019-10-23 15:24:07
 * @version V1.0 
 **/
@Service
public class UserGroupService extends CrudService<UserGroupDao, UserGroup> {
	
	@Autowired
	GrpUserService grpUserService;
	@Autowired
	PersonBaseDao personBaseDao;
	@Autowired
	GrpFromService grpFromService;
	@Autowired
	GrpRoleDao grpRoleDao;
	@Autowired
	OrgRoleService orgRoleService;
	@Autowired
	RightShortcutService rightShortcutService;
	@Autowired
	AccoutInfoDao accoutInfoDao;

	/** 用户组批量添加人员
	 * @param grpId
	 * @param userIds
	 */
	@Transactional(readOnly = false)
	public void addperson(String grpId,String userIds) {
		if(StringUtils.isNotBlank(userIds)) {
			String[] userIdArray = userIds.split(",");
			if(userIdArray.length > 0) {
				for (String userId : userIdArray) {
					GrpUser unique = grpUserService.getUnique(grpId, userId);
					if(unique != null) {
						unique.setGrpId(Integer.valueOf(grpId));
						unique.setAccoutId(Integer.valueOf(userId));
						grpUserService.save(unique,unique.getGrpPersonId());
					}else {
						GrpUser grpUser = new GrpUser();
						grpUser.setGrpId(Integer.valueOf(grpId));
						grpUser.setAccoutId(Integer.valueOf(userId));
						grpUserService.save(grpUser,grpUser.getGrpPersonId());
					}
					GrpFrom grFrom = new GrpFrom();
					grFrom.setFromId(userId);
					grFrom.setFromType(BusinessConstants.SOURCE_FROM_USER);
					grFrom.setGrpId(Integer.valueOf(grpId));
					grpFromService.save(grFrom, grFrom.getGrpFromId());
					//获取用户组角色
					GrpRole grpRole = new GrpRole();
					grpRole.setGrpId(Integer.valueOf(grpId));
					List<GrpRole> grpRoles = grpRoleDao.findList(grpRole);
					if(grpRoles.size() > 0) {
						for (GrpRole grpRole1 : grpRoles) {
							//给用户添加用户组角色
							RightShortcut rightShortcut = new RightShortcut();
							rightShortcut.setFromType(BusinessConstants.SOURCE_FROM_GRP);
							rightShortcut.setFromId(grpRole1.getGrpId());
							rightShortcut.setAccountId(Integer.valueOf(userId));
							rightShortcut.setRoleId(grpRole1.getRoleId());
							rightShortcutService.save(rightShortcut, rightShortcut.getRscId());
						}
					}
				}
			}
		}
	}

	/**
	 * 用户组添加组织
	 * @param grpId
	 */
	@Transactional(readOnly = false)
	public AjaxJson addOrg(String params) {
		AjaxJson json = new AjaxJson();
		JSONObject jsonObject = JSONObject.parseObject(params);
	    String grpId = jsonObject.getString("grpId");
		String orgIds = jsonObject.getString("orgIds");
		UserGroup userGroup = dao.get(grpId);
		if(userGroup != null) {
			try {
				if(StringUtils.isNotBlank(orgIds)) {
					String[] orgIdArray = orgIds.split(",");
					if(orgIdArray.length > 0) {
						for (String orgId : orgIdArray) {
							List<PersonBase> personList = personBaseDao.findListByOrgId(orgId,BusinessConstants.IS);
							for (PersonBase person : personList) {
								GrpUser unique = grpUserService.getUnique(grpId,person.getAccountId().toString());
								if(unique != null) {
									unique.setGrpId(Integer.valueOf(grpId));
									unique.setAccoutId(person.getAccountId());
									grpUserService.save(unique,unique.getGrpPersonId());
								}else {
									GrpUser grpUser = new GrpUser();
									grpUser.setGrpId(Integer.valueOf(grpId));
									grpUser.setAccoutId(person.getAccountId());
									grpUserService.save(grpUser,grpUser.getGrpPersonId());
								}
							}
							//判断是否已添加
							GrpFrom grFrom = new GrpFrom();
							grFrom.setFromId(orgId);
							grFrom.setFromType(BusinessConstants.SOURCE_FROM_ORG);
							grFrom.setGrpId(Integer.valueOf(grpId));
							grpFromService.save(grFrom, grFrom.getGrpFromId());
							//给该组织分配用户组所拥有的的角色
							GrpRole grpRole = new GrpRole();
							grpRole.setGrpId(Integer.valueOf(grpId));
							List<GrpRole> grpRoles = grpRoleDao.findList(grpRole);
							if(grpRoles.size() > 0) {
								for (GrpRole grpRole1 : grpRoles) {
						            //获取该组织下的所有人员账号
						            List<PersonBase> list = personBaseDao.findListByOrgId(orgId, new PersonBase().getDelFlag());
						            if(list.size() > 0) {
						            	for (PersonBase personBase : list) {
						            		//向权限快捷存储表插入数据
											RightShortcut rightShortcut  = new RightShortcut();
											rightShortcut.setAccountId(personBase.getAccountId());
											rightShortcut.setRoleId(grpRole1.getRoleId());
											rightShortcut.setFromType(BusinessConstants.SOURCE_FROM_GRP_ADD_ORG);
											rightShortcut.setFromId(grpRole1.getGrpId());
											rightShortcutService.save(rightShortcut,rightShortcut.getRscId());
								        }
						            }
								}
							}
						}
					}
				}
				json.setMsg("添加成功!");
				json.setSuccess(true);
			} catch (Exception e) {
				e.printStackTrace();
				return AjaxJson.returnExceptionInfo(LoitStatusCode.FAIL);
			}
		}else {
			json.setSuccess(false);
			json.setMsg(LoitStatusCode.GRP_NOT_EXIST.statusDesc);
		}
		return json;
		
	}

	/**
	 * 用户组移除组织
	 * @param orgIds
	 * @param grpId 
	 */
	@Transactional(readOnly = false)
	public void removeOrg(String orgIds, String grpId) {
		String[] orgIdArray = orgIds.split(",");
		if(orgIdArray.length > 0) {
			for (String orgId : orgIdArray) {
				//删除用户组来源表数据
				grpFromService.deleteGrpFrom(grpId, orgId,BusinessConstants.SOURCE_FROM_ORG);
				//删除该组织下在该用户组人员
				List<PersonBase> personList = personBaseDao.findListByOrgId(orgId,BusinessConstants.IS);
				for (PersonBase person : personList) {
					GrpUser grpUser = grpUserService.getUnique(grpId,person.getAccountId().toString());
					grpUserService.delete(grpUser);
				}
				// 删除角色权限快捷表
				rightShortcutService.deleteByFrom(orgId, BusinessConstants.SOURCE_FROM_GRP_ADD_ORG);
			}
		}
	}

	/**
	 * 用户组移除人员
	 * @param accountIds
	 * @param grpId
	 */
	@Transactional(readOnly = false)
	public void removePerson(String accountIds, String grpId) {
		String[] accountIdArray = accountIds.split(",");
		if(accountIdArray.length > 0) {
			for (String accountId : accountIdArray) {
				GrpUser grpUser = grpUserService.getUnique(grpId,accountId);
				grpUserService.delete(grpUser);
				//删除用户组来源表数据
				grpFromService.deleteGrpFrom(grpId, accountId,BusinessConstants.SOURCE_FROM_USER);
				//移除该人员拥有的用户组角色
				GrpRole grpRole = new GrpRole();
				grpRole.setGrpId(Integer.valueOf(grpId));
				List<GrpRole> grpRoles = grpRoleDao.findList(grpRole);
				if(grpRoles.size() > 0) {
					RightShortcut rightShortcut = new RightShortcut();
					rightShortcut.setFromType(BusinessConstants.SOURCE_FROM_GRP);
					rightShortcut.setAccountId(Integer.valueOf(accountId));
					rightShortcut.setFromId(Integer.valueOf(grpId));
					for (GrpRole grpRole1 : grpRoles) {
						rightShortcut.setRoleId(grpRole1.getRoleId());
						rightShortcutService.deleteRightShortCut(rightShortcut);
					}
				}
			}
		}
	}

	/**
	 * 删除用户组
	 * @param grpIds
	 */
	@Transactional(readOnly = false)
	public void deleteGrp(String grpIds) {
		UserGroup userGroup = new UserGroup();
		String idArray[] =grpIds.split(",");
        for(String grpId : idArray){
        	userGroup = get(grpId);
            dao.delete(userGroup);
            //删除用户组角色
            grpRoleDao.deleteByGrpId(Integer.valueOf(grpId));
            //删除快捷表数据
            rightShortcutService.deleteByFrom(grpId, BusinessConstants.SOURCE_FROM_GRP);
            //删除用户组来源表数据
            grpFromService.deleteByGrpId(grpId);
            //删除用户组人员关系表中数据
            grpUserService.deleteByGrpId(grpId);
        }
		
	}

	/**
	 * 保存/修改用户组
	 * @param userGroup
	 */
	@Transactional(readOnly = false)
	public void saveUserGroup(UserGroup userGroup) {
		if(userGroup.getGrpId() != null) {
			//如果用户组已禁用,则修改用户组下所有账号状态为禁用
			if(userGroup.getGrpState().toString().equals(BusinessConstants.GRP_STATUS_FORBID)) {
				List<AccoutInfo> list = accoutInfoDao.findListByGrpId(userGroup.getGrpId());
				for (AccoutInfo accoutInfo : list) {
					accoutInfo.setUserStat(BusinessConstants.ACCOUNT_STATUS_FORBID);
					accoutInfoDao.update(accoutInfo);
				}
			}
			if(userGroup.getGrpState().toString().equals(BusinessConstants.GRP_STATUS_NORMAL)) {
				List<AccoutInfo> list = accoutInfoDao.findListByGrpId(userGroup.getGrpId());
				for (AccoutInfo accoutInfo : list) {
					accoutInfo.setUserStat(BusinessConstants.ACCOUNT_STATUS_NORMAL);
					accoutInfoDao.update(accoutInfo);
				}
			}
		}
		this.save(userGroup, userGroup.getGrpId());
	}

	/**
	 * 验证用户组名称
	 * @param userGroup
	 * @return
	 */
	public boolean validateGrpName(UserGroup userGroup) {
		UserGroup userGroup2 = dao.getUniqueGrpName(userGroup);
		if(userGroup2 != null) {
			return false;
		}
		return true;
	}

}