package com.loit.v1.modules.utils.Model;

import com.alibaba.fastjson.JSONObject;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URI;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * 模拟http请求工具类
 * @author zhangyinglu
 *
 */
public class HttpUtil {

	private final static Logger logger = LoggerFactory.getLogger(HttpUtil.class);
	/**
	 * Get请求String
	 * @author zhangyinglu
	 * @param url   请求地址
	 * @return String
	 */
	public static String doGet4String(String url){
		try {
			HttpClient httpClient = HttpClients.createDefault();
			HttpGet httpGet = new HttpGet(url);
			HttpResponse httpResponse = httpClient.execute(httpGet);
			if(HttpStatus.SC_OK == httpResponse.getStatusLine().getStatusCode()){
				HttpEntity httpEntity = httpResponse.getEntity();
				String strResult = EntityUtils.toString(httpEntity, "UTF-8");
				
				return strResult;
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		
		return null;
	}

	/**
	 * Get请求String
	 * @author zhangyinglu
	 * @param url   请求地址
	 * @param map   请求参数
	 * @return String
	 */
	public static String doGet4String(String url, Map<String, String> map){
		try {
			List<NameValuePair> params = map.entrySet().stream().map(entry -> new BasicNameValuePair(entry.getKey(), entry.getValue())).collect(Collectors.toList());
			URI uri = new URI(url + "?" + URLEncodedUtils.format(params, "UTF-8" ));
			HttpClient httpClient = HttpClients.createDefault();
			HttpGet httpGet = new HttpGet(uri);
			HttpResponse httpResponse = httpClient.execute(httpGet);
			if(HttpStatus.SC_OK == httpResponse.getStatusLine().getStatusCode()){
				HttpEntity httpEntity = httpResponse.getEntity();
				String strResult = EntityUtils.toString(httpEntity, "UTF-8");

				return strResult;
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		return null;
	}
	
	/**
	 * Post请求返回String
	 * @author zhangyinglu
	 * @param url   请求地址
	 * @return String
	 */
	public static String doPost4String(String url){
		try {
			HttpClient httpClient = HttpClients.createDefault();
			HttpPost httpPost = new HttpPost(url);
			HttpResponse httpResponse = httpClient.execute(httpPost);
			if(HttpStatus.SC_OK == httpResponse.getStatusLine().getStatusCode()){
				HttpEntity httpEntity = httpResponse.getEntity();
				String strResult = EntityUtils.toString(httpEntity, "UTF-8");
				return strResult;
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		
		return null;
	}

	/**
	 * Post请求返回String
	 * @author zhangyinglu
	 * @param url           请求地址
	 * @param jsonString    请求参数
	 * @return String
	 */
	public static String doPost4String(String url, String jsonString){
		try {
			HttpClient httpClient = HttpClients.createDefault();
			HttpPost httpPost = new HttpPost(url);
			StringEntity stringEntity = new StringEntity(jsonString, "UTF-8");
			httpPost.setEntity(stringEntity);
			HttpResponse httpResponse = httpClient.execute(httpPost);
			if(HttpStatus.SC_OK == httpResponse.getStatusLine().getStatusCode()){
				HttpEntity httpEntity = httpResponse.getEntity();
				String strResult = EntityUtils.toString(httpEntity, "UTF-8");
				logger.info("请求结果：" + strResult);
				return strResult;
			}
		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		return null;
	}

	
	/**
	 * Get请求返回Json
	 * @author zhangyinglu
	 * @param url   请求地址
	 * @return JSONObject
	 */
	public static JSONObject doGet4Json(String url){
		try {
			return JSONObject.parseObject(doGet4String(url));
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		
		return null;
	}

	/**
	 * Get请求返回Json
	 * @author zhangyinglu
	 * @param url   请求地址
	 * @param map   请求参数
	 * @return JSONObject
	 */
	public static JSONObject doGet4Json(String url, Map<String, String> map){
		try {
			return JSONObject.parseObject(doGet4String(url, map));
		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		return null;
	}
	
	/**
	 * Post请求返回Json
	 * @author zhangyinglu
	 * @param url   请求地址
	 * @return JSONObject
	 */
	public static JSONObject doPost4Json(String url){
		try {
			return JSONObject.parseObject(doPost4String(url));
		} catch (Exception e) {
			logger.error(e.getMessage());
		}
		
		return null;
	}

	/**
	 * Post请求返回Json
	 * @author zhangyinglu
	 * @param url           请求地址
	 * @param jsonString    请求参数
	 * @return JSONObject
	 */
	public static JSONObject doPost4Json(String url, String jsonString){
			try {
			return JSONObject.parseObject(doPost4String(url, jsonString));
		} catch (Exception e) {
			logger.error(e.getMessage());
		}

		return null;
	}

	/**
	 * Post请求返回boolean
	 * @author zhangyinglu
	 * @param url           请求地址
	 * @param jsonString    请求参数
	 * @return boolean
	 */
	public static boolean doPost4Boolean(String url, String jsonString){
		String result = doPost4String(url,jsonString);
		return Boolean.valueOf(result);
	}


	/**
	 * Get请求返回Xml<b>暂不支持</b>
	 * @author zhangyinglu
	 * @param url   请求地址
	 * @return JSONObject
	 */
	public static String doGet4Xml(String url){
		//TODO
		return null;
	}
	
	/**
	 * Post请求返回Xml<b>暂不支持</b>
	 * @author zhangyinglu
	 * @param url   请求地址
	 * @return JSONObject
	 */
	public static String doPost4Xml(String url){
		//TODO
		return null;
	}
}
