package com.loit.v1.modules.utils.Model;


import com.loit.common.spring.timeloit.constant.CommonConstant;
import com.loit.common.spring.timeloit.utils.LogUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;

/**
 * @author zhangyl
 * @version V1.0
 * @ClassName: IpUtil
 * @Description: 验证IP是否属于某个IP段
 * @date 2019年11月4日 下午19:29:08
 */

public class IpUtil {
    public static Logger LOGGER = LoggerFactory.getLogger(IpUtil.class);

    /**
     * @param ipSection Ip段(以"-"分隔)
     * @param ip 所验证的IP号
     */

    public static boolean ipExistsInRange(String ip, String ipSection) {
        ipSection = ipSection.trim();
        ip = ip.trim();
        int idx = ipSection.indexOf('-');
        String beginIP = ipSection.substring(0, idx);
        String endIP = ipSection.substring(idx + 1);
        return getIp2long(beginIP) <= getIp2long(ip) && getIp2long(ip) <= getIp2long(endIP);
    }

    public static long getIp2long(String ip) {
        ip = ip.trim();
        String[] ips = ip.split("\\.");
        long ip2long = 0L;
        for (int i = 0; i < 4; ++i) {
            ip2long = ip2long << 8 | Integer.parseInt(ips[i]);
        }
        return ip2long;
    }

    public static long getIp2long2(String ip) {
        ip = ip.trim();
        String[] ips = ip.split("\\.");
        long ip1 = Integer.parseInt(ips[0]);
        long ip2 = Integer.parseInt(ips[1]);
        long ip3 = Integer.parseInt(ips[2]);
        long ip4 = Integer.parseInt(ips[3]);
        long ip2long = 1L * ip1 * 256 * 256 * 256 + ip2 * 256 * 256 + ip3 * 256 + ip4;
        return ip2long;
    }

    /**
     * 获取客户端ip
     * 统一管理ip的获取，方便后期修改
     *
     * @return
     */
    public static String clientIp() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = attributes.getRequest();

        String ip = request.getHeader(CommonConstant.CLIENT_IP_REQUEST_HEADER);
        IpUtil.LOGGER.debug(LogUtil.print("客户端ip：{}，门户获取从请求头：{}"), ip, CommonConstant.CLIENT_IP_REQUEST_HEADER);
        return ip;
    }

}
