package com.loit.v1.modules.utils.Model;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.crypto.factory.PasswordEncoderFactories;
import org.springframework.security.crypto.password.PasswordEncoder;

import com.loit.v1.modules.constant.BusinessConstants;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @ClassName: PasswordCheckUtils
 * @Description:  验证密码是否符合规则
 * @author zhangyl
 * @date 2019年11月5日 下午19:29:08
 * @version V1.0
 */
public  class PasswordCheckUtils {

    private static int digit = 0;     //数字
    private static int lowerCase = 0; //小写字母
    private static int upperCase = 0; //大写字母
    private static int special = 0;   //特殊字符


    private final static Logger logger = LoggerFactory.getLogger(PasswordCheckUtils.class);
    /**
     * 检验密码长度合法性
     * @Parm password
     * @return boolean
     */
    public static boolean checkPasswordLength(String password, int minLength) {
        if ( password == null || password.length() < minLength) {
            //密码长度不正确，返回false
            return false;
        }
        return true;
    }

    /**
     * 检验密码复杂度级别
     * @Parm password
     * @return boolean
     */
    public  static boolean  checkPasswordMixLevel(String password,String mixLevel) {

        if(mixLevel.equals(BusinessConstants.PASSWORD_ONLY_DIGIT)) {
        	return validPassword1(password);
        }
        else if(mixLevel.equals(BusinessConstants.PASSWORD_ONLY_LOWERCASE) ) {
        	return validPassword2(password);
        }
        else if(mixLevel.equals(BusinessConstants.PASSWORD_ONLY_UPPERCASE)) {
        	return validPassword3(password);
        }
        else if(mixLevel.equals(BusinessConstants.PASSWORD_DIGIT_ADN_UPPERCASE)) {
        	return validPassword4(password);
        }
        else if(mixLevel.equals(BusinessConstants.PASSWORD_SYMBOL_AND_DIGIT_AND_UPPERCASE)) {
        	return validPassword5(password);
        }
        return false;
    }

    //级别1  数字组成
    public static boolean validPassword1(String password){
        for(Character ch : password.toCharArray()) {
            if(Character.isDigit(ch)) {
                digit = 1;
            }
            else if (Character.isLowerCase(ch)) {
                return false;
            }
            else if (Character.isUpperCase(ch)) {
                return false;
            }
            else if (checkSpecial(password)) {
                return false;
            }
        }
        if (digit == 1) {
            return true;
        }
        return false;
    }
    //级别2  小写字母组成
    public static boolean validPassword2(String password){
        for(Character ch : password.toCharArray()) {
            if(Character.isDigit(ch)) {
                return false;
            }
            else if (Character.isLowerCase(ch)) {
                lowerCase = 1;
            }
            else if (Character.isUpperCase(ch)) {
                return false;
            }
            else if (checkSpecial(password)) {
                return false;
            }
        }
        if (lowerCase == 1) {
            return true;
        }
        return false;
    }

    //级别3  大写字母组成
    public static boolean validPassword3(String password){
        for(Character ch : password.toCharArray()) {
            if(Character.isDigit(ch)) {
                return false;
            }
            else if (Character.isLowerCase(ch)) {
                return false;
            }
            else if (Character.isUpperCase(ch)) {
                upperCase = 1;
            }
            else if (checkSpecial(password)) {
                return false;
            }
        }
        if (upperCase  == 1) {
            return true;
        }
        return false;
    }

    //级别4  数字+大写字母组成
    public static boolean validPassword4(String password){
        for(Character ch : password.toCharArray()) {
            if(Character.isDigit(ch)) {
                digit = 1;
            }
            else if (Character.isLowerCase(ch)) {
                return false;
            }
            else if (Character.isUpperCase(ch)) {
                upperCase = 1;
            }
            else if (checkSpecial(password)) {
                return false;
            }
        }
        if (upperCase  + digit  == 2) {
            return true;
        }
        return false;
    }

    //级别5  数字+大写字母+符号组成
    public static boolean validPassword5(String password){

        for(Character ch : password.toCharArray()) {
            if (Character.isDigit(ch)) {
                digit = 1;
            }
            else if(Character.isLowerCase(ch)){
                return false;
            }
            else if (Character.isUpperCase(ch)) {
                upperCase = 1;
            }
            else if(checkSpecial(password)){
                special = special;
            }
        }
        if (upperCase + digit + special == 3) {
            return true;
        }
        return false;
    }

    /**
     * 特殊字符正则
     * @Parm password
     * @return boolean
     */
    public static boolean checkSpecial(String password){
        String regEx = "[`~!@#$%^&*()+=|{}':;',\\[\\].<>/?~！@#￥%……&*（）——+|{}【】‘；：”“’。，、？]";
        Pattern p = Pattern.compile(regEx);
        Matcher m = p.matcher(password);
        while(m.find()){
            special = 1;
            return true;
        }
        special = 0;
        return false;
    }


    /**
     *  BCrypt 密码加密
     * @Parm password 明文密码
     * @return String 密码密文
     */
    public static String encrypt(String password) {
        PasswordEncoder passwordEncoder = PasswordEncoderFactories.createDelegatingPasswordEncoder();
        return passwordEncoder.encode(password);
    }

    /**
     *  matches 验证密文
     * @Parm password 明文密码 ，encode 密码密文
     * @return boolean
     */
    public static boolean matches(String password , String encode){
        PasswordEncoder passwordEncoder = PasswordEncoderFactories.createDelegatingPasswordEncoder();
        if(passwordEncoder.matches(password,encode)){
            return true;
        }
        return false;
    }
}
